@@ -29,33 +29,28 @@ const UNITS_COUNT = Ci.nsIMemoryReporter.UNITS_COUNT;
 const UNITS_COUNT_CUMULATIVE = Ci.nsIMemoryReporter.UNITS_COUNT_CUMULATIVE;
 const UNITS_PERCENTAGE = Ci.nsIMemoryReporter.UNITS_PERCENTAGE;
 
-const { Services } = ChromeUtils.import("resource://gre/modules/Services.jsm");
-const { XPCOMUtils } = ChromeUtils.import(
-  "resource://gre/modules/XPCOMUtils.jsm"
+const { XPCOMUtils } = ChromeUtils.importESModule(
+  "resource://gre/modules/XPCOMUtils.sys.mjs"
 );
-const { NetUtil } = ChromeUtils.import("resource://gre/modules/NetUtil.jsm");
-ChromeUtils.defineModuleGetter(
-  this,
-  "Downloads",
-  "resource://gre/modules/Downloads.jsm"
-);
-ChromeUtils.defineModuleGetter(
-  this,
-  "FileUtils",
-  "resource://gre/modules/FileUtils.jsm"
+const { NetUtil } = ChromeUtils.importESModule(
+  "resource://gre/modules/NetUtil.sys.mjs"
 );
+ChromeUtils.defineESModuleGetters(this, {
+  Downloads: "resource://gre/modules/Downloads.sys.mjs",
+  FileUtils: "resource://gre/modules/FileUtils.sys.mjs",
+});
 
-XPCOMUtils.defineLazyGetter(this, "nsBinaryStream", () =>
+ChromeUtils.defineLazyGetter(this, "nsBinaryStream", () =>
   CC(
     "@mozilla.org/binaryinputstream;1",
     "nsIBinaryInputStream",
     "setInputStream"
   )
 );
-XPCOMUtils.defineLazyGetter(this, "nsFile", () =>
+ChromeUtils.defineLazyGetter(this, "nsFile", () =>
   CC("@mozilla.org/file/local;1", "nsIFile", "initWithPath")
 );
-XPCOMUtils.defineLazyGetter(this, "nsGzipConverter", () =>
+ChromeUtils.defineLazyGetter(this, "nsGzipConverter", () =>
   CC(
     "@mozilla.org/streamconv;1?from=gzip&to=uncompressed",
     "nsIStreamConverter"
@@ -147,7 +142,7 @@ function stringMatchesFilter(aString, aFilter) {
 
 // ---------------------------------------------------------------------------
 
-window.onunload = function() {};
+window.onunload = function () {};
 
 // ---------------------------------------------------------------------------
 
@@ -305,38 +300,57 @@ function appendHiddenFileInput(aP, aId, aChangeListener) {
   return input;
 }
 
-window.onload = function() {
+window.onload = function () {
   // Generate the header.
 
   let header = appendElement(document.body, "div", "ancillary");
 
   // A hidden file input element that can be invoked when necessary.
-  let fileInput1 = appendHiddenFileInput(header, "fileInput1", function() {
+  let fileInput1 = appendHiddenFileInput(header, "fileInput1", function () {
     let file = this.files[0];
     let filename = file.mozFullPath;
     updateAboutMemoryFromFile(filename);
   });
 
   // Ditto.
-  let fileInput2 = appendHiddenFileInput(header, "fileInput2", function(aElem) {
-    let file = this.files[0];
-    // First time around, we stash a copy of the filename and reinvoke.  Second
-    // time around we do the diff and display.
-    if (!this.filename1) {
-      this.filename1 = file.mozFullPath;
+  let fileInput2 = appendHiddenFileInput(
+    header,
+    "fileInput2",
+    function (aElem) {
+      let file = this.files[0];
+      // First time around, we stash a copy of the filename and reinvoke.  Second
+      // time around we do the diff and display.
+      if (!this.filename1) {
+        this.filename1 = file.mozFullPath;
 
-      // aElem.skipClick is only true when testing -- it allows fileInput2's
-      // onchange handler to be re-called without having to go via the file
-      // picker.
-      if (!aElem.skipClick) {
-        this.click();
+        // aElem.skipClick is only true when testing -- it allows fileInput2's
+        // onchange handler to be re-called without having to go via the file
+        // picker.
+        if (!aElem.skipClick) {
+          // Attempts to reopen the picker immediately might fail because the
+          // focus may not have switched back on some platform, so explicitly
+          //  wait for focus here.
+          if (!this.ownerDocument.hasFocus()) {
+            let input = this;
+            this.ownerDocument.addEventListener(
+              "focus",
+              () => {
+                input.click();
+              },
+              { once: true }
+            );
+            return;
+          }
+          this.click();
+        }
+        return;
       }
-    } else {
+
       let filename1 = this.filename1;
       delete this.filename1;
       updateAboutMemoryFromTwoFiles(filename1, file.mozFullPath);
     }
-  });
+  );
 
   const CuDesc = "Measure current memory reports and show.";
   const LdDesc = "Load memory reports from file and show.";
@@ -381,6 +395,8 @@ window.onload = function() {
     "opsRowLabel",
     "Show memory reports"
   );
+  labelDiv1.setAttribute("role", "heading");
+  labelDiv1.setAttribute("aria-level", "1");
   let label1 = appendElementWithText(labelDiv1, "label", "");
   gVerbose = appendElement(label1, "input", "");
   gVerbose.type = "checkbox";
@@ -400,6 +416,8 @@ window.onload = function() {
     "opsRowLabel",
     "Save memory reports"
   );
+  labelDiv2.setAttribute("role", "heading");
+  labelDiv2.setAttribute("aria-level", "1");
   appendButton(row2, SvDesc, saveReportsToFile, "Measure and save…");
 
   // XXX: this isn't a great place for this checkbox, but I can't think of
@@ -411,14 +429,28 @@ window.onload = function() {
 
   let row3 = appendElement(ops, "div", "opsRow");
 
-  appendElementWithText(row3, "div", "opsRowLabel", "Free memory");
+  let labelDiv3 = appendElementWithText(
+    row3,
+    "div",
+    "opsRowLabel",
+    "Free memory"
+  );
+  labelDiv3.setAttribute("role", "heading");
+  labelDiv3.setAttribute("aria-level", "1");
   appendButton(row3, GCDesc, doGC, "GC");
   appendButton(row3, CCDesc, doCC, "CC");
   appendButton(row3, MMDesc, doMMU, "Minimize memory usage");
 
   let row4 = appendElement(ops, "div", "opsRow");
 
-  appendElementWithText(row4, "div", "opsRowLabel", "Save GC & CC logs");
+  let labelDiv4 = appendElementWithText(
+    row4,
+    "div",
+    "opsRowLabel",
+    "Save GC & CC logs"
+  );
+  labelDiv4.setAttribute("role", "heading");
+  labelDiv4.setAttribute("aria-level", "1");
   appendButton(
     row4,
     GCAndCCLogDesc,
@@ -441,7 +473,14 @@ window.onload = function() {
   if (gMgr.isDMDEnabled) {
     let row5 = appendElement(ops, "div", "opsRow");
 
-    appendElementWithText(row5, "div", "opsRowLabel", "Save DMD output");
+    let labelDiv5 = appendElementWithText(
+      row5,
+      "div",
+      "opsRowLabel",
+      "Save DMD output"
+    );
+    labelDiv5.setAttribute("role", "heading");
+    labelDiv5.setAttribute("aria-level", "1");
     let enableButtons = gMgr.isDMDRunning;
 
     let dmdButton = appendButton(
@@ -462,14 +501,17 @@ window.onload = function() {
   // Generate the footer.  It's hidden at first.
 
   gFooter = appendElement(document.body, "div", "ancillary hidden");
+  gFooter.setAttribute("role", "contentinfo");
 
-  let a = appendElementWithText(
-    gFooter,
-    "a",
-    "option",
-    "Troubleshooting information"
-  );
-  a.href = "about:support";
+  if (Services.policies.isAllowed("aboutSupport")) {
+    let a = appendElementWithText(
+      gFooter,
+      "a",
+      "option",
+      "Troubleshooting information"
+    );
+    a.href = "about:support";
+  }
 
   let legendText1 =
     "Click on a non-leaf node in a tree to expand ('++') " +
@@ -581,7 +623,7 @@ function dumpGCLogAndCCLog(aVerbose) {
   );
   let section = appendElement(gMain, "div", "section");
 
-  function displayInfo(aGCLog, aCCLog, aIsParent) {
+  function displayInfo(aGCLog, aCCLog) {
     appendElementWithText(section, "div", "", "Saved GC log to " + aGCLog.path);
 
     let ccLogType = aVerbose ? "verbose" : "concise";
@@ -615,7 +657,7 @@ function updateAboutMemoryFromReporters() {
     gFilter = "";
 
     // Record the reports from the live memory reporters then process them.
-    let handleReport = function(
+    let handleReport = function (
       aProcess,
       aUnsafePath,
       aKind,
@@ -633,7 +675,7 @@ function updateAboutMemoryFromReporters() {
       });
     };
 
-    let displayReports = function() {
+    let displayReports = function () {
       updateTitleMainAndFooter(
         "live measurement",
         "",
@@ -797,7 +839,7 @@ function loadMemoryReportsFromFile(aFilename, aTitleNote, aFn) {
       "uncompressed",
       {
         data: [],
-        onStartRequest(aR, aC) {},
+        onStartRequest() {},
         onDataAvailable(aR, aStream, aO, aCount) {
           let bi = new nsBinaryStream(aStream);
           this.data.push(bi.readBytes(aCount));
@@ -857,8 +899,8 @@ function updateAboutMemoryFromFile(aFilename) {
  */
 function updateAboutMemoryFromTwoFiles(aFilename1, aFilename2) {
   let titleNote = `diff of ${aFilename1} and ${aFilename2}`;
-  loadMemoryReportsFromFile(aFilename1, titleNote, function(aStr1) {
-    loadMemoryReportsFromFile(aFilename2, titleNote, function(aStr2) {
+  loadMemoryReportsFromFile(aFilename1, titleNote, function (aStr1) {
+    loadMemoryReportsFromFile(aFilename2, titleNote, function (aStr2) {
       try {
         let obj1 = parseAndUnwrapIfCrashDump(aStr1);
         let obj2 = parseAndUnwrapIfCrashDump(aStr2);
@@ -935,15 +977,31 @@ DReport.PRESENT_IN_FIRST_ONLY = 1;
 DReport.PRESENT_IN_SECOND_ONLY = 2;
 DReport.ADDED_FOR_BALANCE = 3;
 
+/**
+ * Return true if the report contains a webIsolated process,
+ * which is a good indication that Fission is enabled.
+ */
+function hasWebIsolatedProcess(aJSONReports) {
+  for (let jr of aJSONReports) {
+    assert(jr.process !== undefined, "Missing process");
+    if (jr.process.startsWith("webIsolated")) {
+      return true;
+    }
+  }
+  return false;
+}
+
 /**
  * Make a report map, which has combined path+process strings for keys, and
  * DReport objects for values.
  *
  * @param aJSONReports
  *        The |reports| field of a JSON object.
+ * @param aForgetIsolation
+          If this is true, treat webIsolated processes like web processes.
  * @return The constructed report map.
  */
-function makeDReportMap(aJSONReports) {
+function makeDReportMap(aJSONReports, aForgetIsolation) {
   let dreportMap = {};
   for (let jr of aJSONReports) {
     assert(jr.process !== undefined, "Missing process");
@@ -963,11 +1021,16 @@ function makeDReportMap(aJSONReports) {
     // Strip PIDs:
     // - pid 123
     // - pid=123
-    let pidRegex = /pid([ =])\d+/g;
+    // - pid: 123
+    let pidRegex = /pid([ =]|: )\d+/g;
     let pidSubst = "pid$1NNN";
     let process = jr.process.replace(pidRegex, pidSubst);
     let path = jr.path.replace(pidRegex, pidSubst);
 
+    if (aForgetIsolation && process.startsWith("webIsolated")) {
+      process = "web (pid NNN)";
+    }
+
     // Strip TIDs and threadpool IDs.
     path = path.replace(/\(tid=(\d+)\)/, "(tid=NNN)");
     path = path.replace(/#\d+ \(tid=NNN\)/, "#N (tid=NNN)");
@@ -984,8 +1047,9 @@ function makeDReportMap(aJSONReports) {
 
     // Strip top window IDs:
     // - explicit/window-objects/top(<URL>, id=123)/...
+    // - event-counts/window-objects/top(<URL>, id=123)/...
     path = path.replace(
-      /^(explicit\/window-objects\/top\(.*, id=)\d+\)/,
+      /^((?:explicit|event-counts)\/window-objects\/top\(.*, id=)\d+\)/,
       "$1NNN)"
     );
 
@@ -996,6 +1060,11 @@ function makeDReportMap(aJSONReports) {
       "moz-nullprincipal:{NNNNNNNN-NNNN-NNNN-NNNN-NNNNNNNNNNNN}"
     );
 
+    // Strip segment counts from address-space.
+    if (path.startsWith("address-space")) {
+      path = path.replace(/\(segments=\d+\)/g, "(segments=NNNN)");
+    }
+
     // Normalize omni.ja! paths.
     path = path.replace(
       /jar:file:\\\\\\(.+)\\omni.ja!/,
@@ -1102,6 +1171,16 @@ function diffJSONObjects(aJson1, aJson2) {
     return aJson1[aProp];
   }
 
+  // If one report we're diffing contains webIsolated processes, but the other
+  // does not, then we're probably comparing a report with Fission enabled with
+  // one where it is not enabled. In this case, we want to make all of the
+  // webIsolated processes look like plain old web processes to get a better
+  // diff.
+  let hasIsolated1 = hasWebIsolatedProcess(aJson1.reports);
+  let hasIsolated2 = hasWebIsolatedProcess(aJson2.reports);
+  let eitherIsolated = hasIsolated1 || hasIsolated2;
+  let forgetIsolation = hasIsolated1 != hasIsolated2 && eitherIsolated;
+
   return {
     version: simpleProp("version"),
 
@@ -1109,8 +1188,8 @@ function diffJSONObjects(aJson1, aJson2) {
 
     reports: makeJSONReports(
       diffDReportMaps(
-        makeDReportMap(aJson1.reports),
-        makeDReportMap(aJson2.reports)
+        makeDReportMap(aJson1.reports, forgetIsolation),
+        makeDReportMap(aJson2.reports, forgetIsolation)
       )
     ),
   };
@@ -1257,7 +1336,7 @@ function appendAboutMemoryMain(
   function displayReports() {
     // Sort the processes.
     let processes = Object.keys(infoByProcess);
-    processes.sort(function(aProcessA, aProcessB) {
+    processes.sort(function (aProcessA, aProcessB) {
       assert(
         aProcessA != aProcessB,
         `Elements of Object.keys() should be unique, but ` +
@@ -1313,6 +1392,7 @@ function appendAboutMemoryMain(
 
     // Generate the main process sections.
     let sections = newElement("div", "sections");
+    sections.setAttribute("role", "main");
 
     for (let [i, process] of processes.entries()) {
       let pcolls = pcollsByProcess[process];
@@ -1370,6 +1450,7 @@ function appendAboutMemoryMain(
     outputContainer.appendChild(sections);
 
     let sidebar = appendElement(outputContainer, "div", "sidebar");
+    sidebar.setAttribute("role", "navigation");
     let sidebarContents = appendElement(sidebar, "div", "sidebarContents");
 
     // Generate the filter input and checkbox.
@@ -1379,6 +1460,7 @@ function appendAboutMemoryMain(
 
     let filterInput = appendElement(filterItem, "input", "filterInput");
     filterInput.placeholder = "Memory report path filter";
+    filterInput.setAttribute("type", "text");
 
     let filterOptions = appendElement(filterItem, "div");
     let filterRegExLabel = appendElement(filterOptions, "label");
@@ -1389,11 +1471,11 @@ function appendAboutMemoryMain(
     // Set up event handlers to update the display if the filter input or
     // checkbox changes.
     let filterUpdateTimeout;
-    let filterUpdate = function() {
+    let filterUpdate = function () {
       if (filterUpdateTimeout) {
         window.clearTimeout(filterUpdateTimeout);
       }
-      filterUpdateTimeout = window.setTimeout(function() {
+      filterUpdateTimeout = window.setTimeout(function () {
         try {
           gFilter =
             filterRegExCheckbox.checked && filterInput.value != ""
@@ -1503,7 +1585,7 @@ TreeNode.prototype = {
 // Sort TreeNodes first by size, then by name.  The latter is important for the
 // about:memory tests, which need a predictable ordering of reporters which
 // have the same amount.
-TreeNode.compareAmounts = function(aA, aB) {
+TreeNode.compareAmounts = function (aA, aB) {
   let a, b;
   if (gIsDiff) {
     a = aA.maxAbsDescendant();
@@ -1521,7 +1603,7 @@ TreeNode.compareAmounts = function(aA, aB) {
   return TreeNode.compareUnsafeNames(aA, aB);
 };
 
-TreeNode.compareUnsafeNames = function(aA, aB) {
+TreeNode.compareUnsafeNames = function (aA, aB) {
   return aA._unsafeName.localeCompare(aB._unsafeName);
 };
 
@@ -1821,7 +1903,7 @@ function appendProcessAboutMemoryElements(
   aHasMozMallocUsableSize,
   aFiltered
 ) {
-  let appendLink = function(aHere, aThere, aArrow) {
+  let appendLink = function (aHere, aThere, aArrow) {
     let link = appendElementWithText(aP, "a", "upDownArrow", aArrow);
     link.href = "#" + aThere + aN;
     link.id = aHere + aN;
@@ -2108,8 +2190,12 @@ function toggle(aEvent) {
   // right nodes.  And the span containing the children of this line must
   // immediately follow.  Assertions check this.
 
-  // |aEvent.target| will be one of the spans.  Get the outer span.
-  let outerSpan = aEvent.target.parentNode;
+  // We want the outer span. |aEvent.target| will normally be one of the inner
+  // spans. However, if the click was dispatched via a11y, it might be the outer
+  // span because some of the inner spans are pruned from the a11y tree.
+  let outerSpan = aEvent.target.classList.contains("hasKids")
+    ? aEvent.target
+    : aEvent.target.parentNode;
   assertClassListContains(outerSpan, "hasKids");
 
   // Toggle the '++'/'--' separator.
@@ -2119,9 +2205,11 @@ function toggle(aEvent) {
   if (sepSpan.textContent === kHideKidsSep) {
     isExpansion = true;
     sepSpan.textContent = kShowKidsSep;
+    outerSpan.setAttribute("aria-expanded", "true");
   } else if (sepSpan.textContent === kShowKidsSep) {
     isExpansion = false;
     sepSpan.textContent = kHideKidsSep;
+    outerSpan.setAttribute("aria-expanded", "false");
   } else {
     assert(false, "bad sepSpan textContent");
   }
@@ -2150,7 +2238,8 @@ function expandPathToThisElement(aElement) {
     let sepSpan = aElement.childNodes[2];
     assertClassListContains(sepSpan, "mrSep");
     sepSpan.textContent = kShowKidsSep;
-    expandPathToThisElement(aElement.parentNode); // kids or pre.entries
+    aElement.setAttribute("aria-expanded", "true");
+    expandPathToThisElement(aElement.parentNode.parentNode); // kids or pre.entries
   } else {
     assertClassListContains(aElement, "entries");
   }
@@ -2210,6 +2299,12 @@ function appendTreeElements(aP, aRoot, aProcess, aPadText) {
       return aS;
     }
 
+    // The entire entry including children needs to be treated as a list item
+    // for a11y purposes.
+    let p = document.createElement("span");
+    p.setAttribute("role", "listitem");
+    aP.appendChild(p);
+
     // The tree line.  Indent more if this entry is narrower than its parent.
     let valueText = aT.toString();
     let extraTlLength = Math.max(aParentStringLength - valueText.length, 0);
@@ -2217,7 +2312,8 @@ function appendTreeElements(aP, aRoot, aProcess, aPadText) {
       aTlThis = appendN(aTlThis, "─", extraTlLength);
       aTlKids = appendN(aTlKids, " ", extraTlLength);
     }
-    appendElementWithText(aP, "span", "treeline", aTlThis);
+    let treeLine = appendElementWithText(p, "span", "treeline", aTlThis);
+    treeLine.setAttribute("aria-hidden", "true");
 
     // Detect and record invalid values.  But not if gIsDiff is true, because
     // we expect negative values in that case.
@@ -2250,14 +2346,16 @@ function appendTreeElements(aP, aRoot, aProcess, aPadText) {
       if (gShowSubtreesBySafeTreeId[safeTreeId] !== undefined) {
         showSubtrees = gShowSubtreesBySafeTreeId[safeTreeId];
       }
-      d = appendElement(aP, "span", "hasKids");
+      d = appendElement(p, "span", "hasKids");
       d.id = safeTreeId;
       d.onclick = toggle;
+      d.setAttribute("role", "button");
       sep = showSubtrees ? kShowKidsSep : kHideKidsSep;
+      d.setAttribute("aria-expanded", showSubtrees ? "true" : "false");
     } else {
       assert(!aT._hideKids, "leaf node with _hideKids set");
       sep = kNoKidsSep;
-      d = aP;
+      d = p;
     }
 
     // The value.
@@ -2290,13 +2388,14 @@ function appendTreeElements(aP, aRoot, aProcess, aPadText) {
     // In non-verbose mode, invalid nodes can be hidden in collapsed sub-trees.
     // But it's good to always see them, so force this.
     if (!gVerbose.checked && tIsInvalid) {
-      expandPathToThisElement(d);
+      expandPathToThisElement(aT._kids ? d : aP);
     }
 
     // Recurse over children.
     if (aT._kids) {
       // The 'kids' class is just used for sanity checking in toggle().
-      d = appendElement(aP, "span", showSubtrees ? "kids" : "kids hidden");
+      d = appendElement(p, "span", showSubtrees ? "kids" : "kids hidden");
+      d.setAttribute("role", "list");
 
       let tlThisForMost, tlKidsForMost;
       if (aT._kids.length > 1) {
@@ -2341,7 +2440,9 @@ function appendTreeElements(aP, aRoot, aProcess, aPadText) {
 
 function appendSectionHeader(aP, aText) {
   appendElementWithText(aP, "h2", "", aText + "\n");
-  return appendElement(aP, "pre", "entries");
+  let entries = appendElement(aP, "pre", "entries");
+  entries.setAttribute("role", "list");
+  return entries;
 }
 
 // ---------------------------------------------------------------------------
@@ -2354,7 +2455,7 @@ function saveReportsToFile() {
   fp.addToRecentDocs = true;
   fp.defaultString = "memory-report.json.gz";
 
-  let fpFinish = function(aFile) {
+  let fpFinish = function (aFile) {
     let dumper = Cc["@mozilla.org/memory-info-dumper;1"].getService(
       Ci.nsIMemoryInfoDumper
     );
@@ -2369,11 +2470,12 @@ function saveReportsToFile() {
       aFile.path,
       finishDumping,
       null,
-      gAnonymize.checked
+      gAnonymize.checked,
+      /* minimize memory usage = */ false
     );
   };
 
-  let fpCallback = function(aResult) {
+  let fpCallback = function (aResult) {
     if (
       aResult == Ci.nsIFilePicker.returnOK ||
       aResult == Ci.nsIFilePicker.returnReplace
@@ -2383,11 +2485,15 @@ function saveReportsToFile() {
   };
 
   try {
-    fp.init(window, "Save Memory Reports", Ci.nsIFilePicker.modeSave);
+    fp.init(
+      window.browsingContext,
+      "Save Memory Reports",
+      Ci.nsIFilePicker.modeSave
+    );
   } catch (ex) {
     // This will fail on Android, since there is no Save as file picker there.
     // Just save to the default downloads dir if it does.
-    Downloads.getSystemDownloadsDirectory().then(function(aDirPath) {
+    Downloads.getSystemDownloadsDirectory().then(function (aDirPath) {
       let file = FileUtils.File(aDirPath);
       file.append(fp.defaultString);
       fpFinish(file);
