@@ -11,13 +11,13 @@
 #include "imgIRequest.h"
 #include "imgLoader.h"
 #include "nsAlertsUtils.h"
+#include "nsAppDirectoryServiceDefs.h"
 #include "nsComponentManagerUtils.h"
 #include "nsContentUtils.h"
+#include "nsDirectoryServiceUtils.h"
 #include "nsNetUtil.h"
 #include "nsServiceManagerUtils.h"
 
-#include "mozilla/Unused.h"
-
 namespace mozilla {
 
 NS_IMPL_ISUPPORTS(AlertNotification, nsIAlertNotification)
@@ -35,6 +35,10 @@ AlertNotification::Init(const nsAString& aName, const nsAString& aImageURL,
                         const nsAString& aData, nsIPrincipal* aPrincipal,
                         bool aInPrivateBrowsing, bool aRequireInteraction,
                         bool aSilent, const nsTArray<uint32_t>& aVibrate) {
+  if (!mId.IsEmpty()) {
+    return NS_ERROR_ALREADY_INITIALIZED;
+  }
+
   mName = aName;
   mImageURL = aImageURL;
   mTitle = aTitle;
@@ -49,12 +53,82 @@ AlertNotification::Init(const nsAString& aName, const nsAString& aImageURL,
   mRequireInteraction = aRequireInteraction;
   mSilent = aSilent;
   mVibrate = aVibrate.Clone();
+
+  return InitId();
+}
+
+nsresult AlertNotification::InitId() {
+  nsAutoString id;
+
+  // Multiple profiles might overwrite each other's toast messages when a
+  // common name is used for a given origin. We prevent this by including
+  // the profile directory as part of the toast hash.
+  nsCOMPtr<nsIFile> profDir;
+  MOZ_TRY(NS_GetSpecialDirectory(NS_APP_USER_PROFILE_50_DIR,
+                                 getter_AddRefs(profDir)));
+  MOZ_TRY(profDir->Normalize());
+  MOZ_TRY(profDir->GetPath(id));
+
+  if (mPrincipal && mPrincipal->GetIsContentPrincipal()) {
+    // Notification originated from a web notification.
+    nsAutoCString origin;
+    MOZ_TRY(mPrincipal->GetOrigin(origin));
+    id += NS_ConvertUTF8toUTF16(origin);
+  } else {
+    id += u"chrome";
+  }
+
+  if (mName.IsEmpty()) {
+    // No associated name, append a UUID to prevent reuse of the same tag.
+    nsIDToCString uuidString(nsID::GenerateUUID());
+    size_t len = strlen(uuidString.get());
+    MOZ_ASSERT(len == NSID_LENGTH - 1);
+    nsAutoString uuid;
+    CopyASCIItoUTF16(nsDependentCSubstring(uuidString.get(), len), uuid);
+
+    id += u"#notag:"_ns;
+    id += uuid;
+  } else {
+    id += u"#tag:"_ns;
+    id += mName;
+  }
+
+  // Windows notification tags are limited to 16 characters, or 64 characters
+  // after the Creators Update; therefore we hash the tag to fit the minimum
+  // range.
+  HashNumber hash = HashString(id);
+  mId.AppendPrintf("%010u", hash);
+  return NS_OK;
+}
+
+NS_IMETHODIMP AlertNotification::GetId(nsAString& aId) {
+  if (mId.IsEmpty()) {
+    return NS_ERROR_NOT_INITIALIZED;
+  }
+  aId = mId;
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+AlertNotification::SetActions(
+    const nsTArray<RefPtr<nsIAlertAction>>& aActions) {
+  mActions = aActions.Clone();
   return NS_OK;
 }
 
 NS_IMETHODIMP
 AlertNotification::GetName(nsAString& aName) {
-  aName = mName;
+  if (mPrincipal && mPrincipal->GetIsContentPrincipal()) {
+    // mName is no longer unique, but there has been a long assumption
+    // throughout the codebase that GetName will be unique. So we return mId for
+    // GetName for web triggered notifications to keep uniqueness without
+    // accidentially causing subtle breakage in other modules.
+    aName = mId;
+  } else {
+    // System callers has always been expected to provide unique names
+    // themselves, so it's fine to return mName as is.
+    aName = mName;
+  }
   return NS_OK;
 }
 
@@ -152,12 +226,30 @@ AlertNotification::GetVibrate(nsTArray<uint32_t>& aVibrate) {
   return NS_OK;
 }
 
+NS_IMETHODIMP
+AlertNotification::GetActions(nsTArray<RefPtr<nsIAlertAction>>& aActions) {
+  aActions = mActions.Clone();
+  return NS_OK;
+}
+
 NS_IMETHODIMP
 AlertNotification::GetSource(nsAString& aSource) {
   nsAlertsUtils::GetSourceHostPort(mPrincipal, aSource);
   return NS_OK;
 }
 
+NS_IMETHODIMP
+AlertNotification::GetOpaqueRelaunchData(nsAString& aOpaqueRelaunchData) {
+  aOpaqueRelaunchData = mOpaqueRelaunchData;
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+AlertNotification::SetOpaqueRelaunchData(const nsAString& aOpaqueRelaunchData) {
+  mOpaqueRelaunchData = aOpaqueRelaunchData;
+  return NS_OK;
+}
+
 NS_IMETHODIMP
 AlertNotification::LoadImage(uint32_t aTimeout,
                              nsIAlertNotificationImageListener* aListener,
@@ -183,6 +275,23 @@ AlertNotification::LoadImage(uint32_t aTimeout,
   return NS_OK;
 }
 
+NS_IMETHODIMP
+AlertNotification::GetAction(const nsAString& aName,
+                             nsIAlertAction** aAlertAction) {
+  NS_ENSURE_ARG_POINTER(aAlertAction);
+  for (const auto& action : mActions) {
+    nsString name;
+    MOZ_TRY(action->GetAction(name));
+    if (name.Equals(aName)) {
+      RefPtr<nsIAlertAction> match = action;
+      match.forget(aAlertAction);
+      return NS_OK;
+    }
+  }
+  *aAlertAction = nullptr;
+  return NS_OK;
+}
+
 NS_IMPL_CYCLE_COLLECTION(AlertImageRequest, mURI, mPrincipal, mListener,
                          mUserData)
 
@@ -345,4 +454,39 @@ void AlertImageRequest::NotifyComplete() {
   }
 }
 
+NS_IMPL_ISUPPORTS(AlertAction, nsIAlertAction)
+
+AlertAction::AlertAction(const nsAString& aAction, const nsAString& aTitle)
+    : mAction(aAction), mTitle(aTitle) {}
+
+NS_IMETHODIMP
+AlertAction::GetAction(nsAString& aAction) {
+  aAction = mAction;
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+AlertAction::GetTitle(nsAString& aTitle) {
+  aTitle = mTitle;
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+AlertAction::GetIconURL(nsAString& aTitle) {
+  aTitle.Truncate();
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+AlertAction::GetWindowsSystemActivationType(bool* aType) {
+  *aType = false;
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+AlertAction::GetOpaqueRelaunchData(nsAString& aData) {
+  aData.Truncate();
+  return NS_OK;
+}
+
 }  // namespace mozilla
