@@ -36,7 +36,8 @@ this.AntiTracking = {
       .STATE_COOKIES_BLOCKED_TRACKER,
     iframeSandbox = null,
     accessRemoval = null,
-    callbackAfterRemoval = null
+    callbackAfterRemoval = null,
+    iframeAllow = null
   ) {
     // Normal mode
     this.runTest(
@@ -51,7 +52,8 @@ this.AntiTracking = {
       false,
       iframeSandbox,
       accessRemoval,
-      callbackAfterRemoval
+      callbackAfterRemoval,
+      iframeAllow
     );
 
     // Private mode
@@ -67,7 +69,8 @@ this.AntiTracking = {
       true,
       iframeSandbox,
       accessRemoval,
-      callbackAfterRemoval
+      callbackAfterRemoval,
+      iframeAllow
     );
   },
 
@@ -84,7 +87,8 @@ this.AntiTracking = {
     runInPrivateWindow = false,
     iframeSandbox = null,
     accessRemoval = null,
-    callbackAfterRemoval = null
+    callbackAfterRemoval = null,
+    iframeAllow = null
   ) {
     let runExtraTests = true;
     let options = {};
@@ -128,6 +132,7 @@ this.AntiTracking = {
       iframeSandbox,
       accessRemoval,
       callbackAfterRemoval,
+      iframeAllow,
     });
     this._createCleanupTask(cleanupFunction);
 
@@ -152,6 +157,7 @@ this.AntiTracking = {
           iframeSandbox,
           accessRemoval: null, // only passed with non-blocking callback
           callbackAfterRemoval: null,
+          iframeAllow,
         });
         this._createCleanupTask(cleanupFunction);
 
@@ -166,6 +172,7 @@ this.AntiTracking = {
           iframeSandbox,
           accessRemoval: null, // only passed with non-blocking callback
           callbackAfterRemoval: null,
+          iframeAllow,
         });
         this._createCleanupTask(cleanupFunction);
 
@@ -182,6 +189,7 @@ this.AntiTracking = {
           iframeSandbox,
           accessRemoval: null, // only passed with non-blocking callback
           callbackAfterRemoval: null,
+          iframeAllow,
         });
         this._createCleanupTask(cleanupFunction);
 
@@ -196,23 +204,7 @@ this.AntiTracking = {
           iframeSandbox,
           accessRemoval: null, // only passed with non-blocking callback
           callbackAfterRemoval: null,
-        });
-        this._createCleanupTask(cleanupFunction);
-
-        this._createTask({
-          name: name + " reject foreign without exception",
-          cookieBehavior: BEHAVIOR_REJECT_FOREIGN,
-          allowList: true,
-          callback: callbackNonTracking,
-          extraPrefs: [
-            ["network.cookie.rejectForeignWithExceptions.enabled", false],
-            ...(extraPrefs || []),
-          ],
-          expectedBlockingNotifications: 0,
-          runInPrivateWindow,
-          iframeSandbox,
-          accessRemoval: null, // only passed with non-blocking callback
-          callbackAfterRemoval: null,
+          iframeAllow,
         });
         this._createCleanupTask(cleanupFunction);
 
@@ -221,15 +213,13 @@ this.AntiTracking = {
           cookieBehavior: BEHAVIOR_REJECT_FOREIGN,
           allowList: true,
           callback: callbackNonTracking,
-          extraPrefs: [
-            ["network.cookie.rejectForeignWithExceptions.enabled", true],
-            ...(extraPrefs || []),
-          ],
+          extraPrefs: [...(extraPrefs || [])],
           expectedBlockingNotifications: 0,
           runInPrivateWindow,
           iframeSandbox,
           accessRemoval,
           callbackAfterRemoval,
+          iframeAllow,
         });
         this._createCleanupTask(cleanupFunction);
 
@@ -238,10 +228,7 @@ this.AntiTracking = {
           cookieBehavior: BEHAVIOR_REJECT_FOREIGN,
           allowList: false,
           callback: callbackTracking,
-          extraPrefs: [
-            ["network.cookie.rejectForeignWithExceptions.enabled", false],
-            ...(extraPrefs || []),
-          ],
+          extraPrefs: [...(extraPrefs || [])],
           expectedBlockingNotifications: expectedBlockingNotifications
             ? Ci.nsIWebProgressListener.STATE_COOKIES_BLOCKED_FOREIGN
             : 0,
@@ -249,24 +236,7 @@ this.AntiTracking = {
           iframeSandbox,
           accessRemoval,
           callbackAfterRemoval,
-        });
-        this._createCleanupTask(cleanupFunction);
-
-        this._createTask({
-          name,
-          cookieBehavior: BEHAVIOR_REJECT_FOREIGN,
-          allowList: false,
-          callback: callbackNonTracking,
-          extraPrefs: [
-            ["network.cookie.rejectForeignWithExceptions.enabled", true],
-            ...(extraPrefs || []),
-          ],
-          expectedBlockingNotifications: false,
-          runInPrivateWindow,
-          iframeSandbox,
-          accessRemoval: null, // only passed with non-blocking callback
-          callbackAfterRemoval: null,
-          thirdPartyPage: TEST_ANOTHER_3RD_PARTY_PAGE,
+          iframeAllow,
         });
         this._createCleanupTask(cleanupFunction);
 
@@ -281,6 +251,7 @@ this.AntiTracking = {
           iframeSandbox,
           accessRemoval,
           callbackAfterRemoval,
+          iframeAllow,
         });
         this._createCleanupTask(cleanupFunction);
 
@@ -296,6 +267,7 @@ this.AntiTracking = {
           accessRemoval: null, // only passed with non-blocking callback
           callbackAfterRemoval: null,
           thirdPartyPage: TEST_ANOTHER_3RD_PARTY_PAGE,
+          iframeAllow,
         });
         this._createCleanupTask(cleanupFunction);
       } else {
@@ -311,6 +283,7 @@ this.AntiTracking = {
           iframeSandbox,
           accessRemoval: options.accessRemoval,
           callbackAfterRemoval: options.callbackAfterRemoval,
+          iframeAllow,
         });
         this._createCleanupTask(cleanupFunction);
       }
@@ -326,22 +299,8 @@ this.AntiTracking = {
           runInPrivateWindow,
           iframeSandbox,
           false,
-          extraPrefs
-        );
-        this._createCleanupTask(cleanupFunction);
-
-        this._createWindowOpenTask(
-          name,
-          BEHAVIOR_REJECT_FOREIGN,
-          callbackTracking,
-          callbackNonTracking,
-          runInPrivateWindow,
-          iframeSandbox,
-          false,
-          [
-            ["network.cookie.rejectForeignWithExceptions.enabled", true],
-            ...(extraPrefs || []),
-          ]
+          extraPrefs,
+          iframeAllow
         );
         this._createCleanupTask(cleanupFunction);
 
@@ -354,22 +313,8 @@ this.AntiTracking = {
           runInPrivateWindow,
           iframeSandbox,
           true,
-          extraPrefs
-        );
-        this._createCleanupTask(cleanupFunction);
-
-        this._createWindowOpenTask(
-          name,
-          BEHAVIOR_REJECT_FOREIGN,
-          callbackTracking,
-          callbackNonTracking,
-          runInPrivateWindow,
-          iframeSandbox,
-          true,
-          [
-            ["network.cookie.rejectForeignWithExceptions.enabled", true],
-            ...(extraPrefs || []),
-          ]
+          extraPrefs,
+          iframeAllow
         );
         this._createCleanupTask(cleanupFunction);
       }
@@ -385,22 +330,8 @@ this.AntiTracking = {
           runInPrivateWindow,
           iframeSandbox,
           false,
-          extraPrefs
-        );
-        this._createCleanupTask(cleanupFunction);
-
-        this._createUserInteractionTask(
-          name,
-          BEHAVIOR_REJECT_FOREIGN,
-          callbackTracking,
-          callbackNonTracking,
-          runInPrivateWindow,
-          iframeSandbox,
-          false,
-          [
-            ["network.cookie.rejectForeignWithExceptions.enabled", true],
-            ...(extraPrefs || []),
-          ]
+          extraPrefs,
+          iframeAllow
         );
         this._createCleanupTask(cleanupFunction);
 
@@ -413,22 +344,8 @@ this.AntiTracking = {
           runInPrivateWindow,
           iframeSandbox,
           true,
-          extraPrefs
-        );
-        this._createCleanupTask(cleanupFunction);
-
-        this._createUserInteractionTask(
-          name,
-          BEHAVIOR_REJECT_FOREIGN,
-          callbackTracking,
-          callbackNonTracking,
-          runInPrivateWindow,
-          iframeSandbox,
-          true,
-          [
-            ["network.cookie.rejectForeignWithExceptions.enabled", true],
-            ...(extraPrefs || []),
-          ]
+          extraPrefs,
+          iframeAllow
         );
         this._createCleanupTask(cleanupFunction);
       }
@@ -439,7 +356,7 @@ this.AntiTracking = {
     let cnt = 0;
 
     return new Promise(resolve => {
-      Services.obs.addObserver(function observer(subject, topic, data) {
+      Services.obs.addObserver(function observer(subject, topic) {
         if (topic != targetTopic) {
           return;
         }
@@ -473,10 +390,10 @@ this.AntiTracking = {
     let win = await BrowserTestUtils.openNewBrowserWindow();
     await BrowserTestUtils.withNewTab(
       { gBrowser: win.gBrowser, url: TEST_3RD_PARTY_PAGE },
-      async function(browser) {
+      async function (browser) {
         info("Let's interact with the tracker");
 
-        await SpecialPowers.spawn(browser, [], async function() {
+        await SpecialPowers.spawn(browser, [], async function () {
           SpecialPowers.wrap(content.document).userInteractionForTesting();
         });
       }
@@ -555,7 +472,7 @@ this.AntiTracking = {
   },
 
   _createTask(options) {
-    add_task(async function() {
+    add_task(async function () {
       info(
         "Starting " +
           (options.cookieBehavior != BEHAVIOR_ACCEPT
@@ -572,6 +489,8 @@ this.AntiTracking = {
           " window " +
           " with iframe sandbox set to " +
           options.iframeSandbox +
+          " and iframe allow set to " +
+          options.iframeAllow +
           " and access removal set to " +
           options.accessRemoval +
           (typeof options.thirdPartyPage == "string"
@@ -636,10 +555,16 @@ this.AntiTracking = {
       }
 
       let cookieBlocked = 0;
+      let { expectedBlockingNotifications } = options;
+      if (!Array.isArray(expectedBlockingNotifications)) {
+        expectedBlockingNotifications = [expectedBlockingNotifications];
+      }
       let listener = {
         onContentBlockingEvent(webProgress, request, event) {
-          if (event & options.expectedBlockingNotifications) {
-            ++cookieBlocked;
+          for (const notification of expectedBlockingNotifications) {
+            if (event & notification) {
+              ++cookieBlocked;
+            }
           }
         },
       };
@@ -676,6 +601,33 @@ this.AntiTracking = {
         consoleWarningPromise = Promise.resolve();
       }
 
+      let { expectedPartitioningNotifications } = options;
+      if (!Array.isArray(expectedPartitioningNotifications)) {
+        expectedPartitioningNotifications = [expectedPartitioningNotifications];
+      }
+
+      let consolePartitioningWarningPromise;
+      if (options.expectedPartitioningNotifications) {
+        consolePartitioningWarningPromise = new Promise(resolve => {
+          let consoleListener = {
+            observe(msg) {
+              if (
+                msg
+                  .QueryInterface(Ci.nsIScriptError)
+                  .category.startsWith("cookiePartitioned")
+              ) {
+                Services.console.unregisterListener(consoleListener);
+                resolve();
+              }
+            },
+          };
+
+          Services.console.registerListener(consoleListener);
+        });
+      } else {
+        consolePartitioningWarningPromise = Promise.resolve();
+      }
+
       info("Creating a new tab");
       let tab = BrowserTestUtils.addTab(win.gBrowser, topPage);
       win.gBrowser.selectedTab = tab;
@@ -721,16 +673,17 @@ this.AntiTracking = {
               : null,
             accessRemoval: options.accessRemoval,
             iframeSandbox: options.iframeSandbox,
+            iframeAllow: options.iframeAllow,
             allowList: options.allowList,
             doAccessRemovalChecks,
           },
         ],
-        async function(obj) {
+        async function (obj) {
           let id = "id" + Math.random();
           await new content.Promise(resolve => {
             let ifr = content.document.createElement("iframe");
             ifr.id = id;
-            ifr.onload = function() {
+            ifr.onload = function () {
               info("Sending code to the 3rd party content");
               let callback = obj.allowList + "!!!" + obj.callback;
               ifr.contentWindow.postMessage(callback, "*");
@@ -738,6 +691,9 @@ this.AntiTracking = {
             if (typeof obj.iframeSandbox == "string") {
               ifr.setAttribute("sandbox", obj.iframeSandbox);
             }
+            if (typeof obj.iframeAllow == "string") {
+              ifr.setAttribute("allow", obj.iframeAllow);
+            }
 
             content.addEventListener("message", function msg(event) {
               if (event.data.type == "finish") {
@@ -770,13 +726,16 @@ this.AntiTracking = {
                 await new content.Promise(resolve => {
                   let ifr = content.document.getElementById(id);
                   let oldWindow = ifr.contentWindow;
-                  ifr.onload = function() {
+                  ifr.onload = function () {
                     info("Sending code to the old 3rd party content");
                     oldWindow.postMessage(obj.callbackAfterRemoval, "*");
                   };
                   if (typeof obj.iframeSandbox == "string") {
                     ifr.setAttribute("sandbox", obj.iframeSandbox);
                   }
+                  if (typeof obj.iframeAllow == "string") {
+                    ifr.setAttribute("allow", obj.iframeAllow);
+                  }
 
                   content.addEventListener("message", function msg(event) {
                     if (event.data.type == "finish") {
@@ -819,7 +778,7 @@ this.AntiTracking = {
         doAccessRemovalChecks &&
         options.accessRemoval == "navigate-topframe"
       ) {
-        BrowserTestUtils.loadURI(browser, TEST_4TH_PARTY_PAGE);
+        BrowserTestUtils.startLoadingURIString(browser, TEST_4TH_PARTY_PAGE);
         await BrowserTestUtils.browserLoaded(browser);
 
         let pageshow = BrowserTestUtils.waitForContentEvent(
@@ -838,11 +797,12 @@ this.AntiTracking = {
                 ? options.callbackAfterRemoval.toString()
                 : null,
               iframeSandbox: options.iframeSandbox,
+              iframeAllow: options.iframeAllow,
             },
           ],
-          async function(obj) {
+          async function (obj) {
             let ifr = content.document.createElement("iframe");
-            ifr.onload = function() {
+            ifr.onload = function () {
               info(
                 "Sending code to the 3rd party content to verify accessRemoval"
               );
@@ -851,6 +811,9 @@ this.AntiTracking = {
             if (typeof obj.iframeSandbox == "string") {
               ifr.setAttribute("sandbox", obj.iframeSandbox);
             }
+            if (typeof obj.iframeAllow == "string") {
+              ifr.setAttribute("allow", obj.iframeAllow);
+            }
 
             content.addEventListener("message", function msg(event) {
               if (event.data.type == "finish") {
@@ -879,36 +842,58 @@ this.AntiTracking = {
 
       // Wait until the message appears on the console.
       await consoleWarningPromise;
+      await consolePartitioningWarningPromise;
 
       let allMessages = Services.console.getMessageArray().filter(msg => {
         try {
           // Select all messages that the anti-tracking backend could generate.
-          return msg
-            .QueryInterface(Ci.nsIScriptError)
-            .category.startsWith("cookieBlocked");
+          let category = msg.QueryInterface(Ci.nsIScriptError).category;
+
+          let isBlocking = category.startsWith("cookieBlocked");
+
+          // Only look for partitioning notifications if we expect them.
+          let isPartitioning =
+            options.expectedPartitioningNotifications &&
+            category.startsWith("cookiePartitioned");
+
+          return isBlocking || isPartitioning;
         } catch (e) {
           return false;
         }
       });
-      let expectedCategory = "";
       // When changing this list, please make sure to update the corresponding
       // code in ReportBlockingToConsole().
-      switch (options.expectedBlockingNotifications) {
-        case Ci.nsIWebProgressListener.STATE_COOKIES_BLOCKED_BY_PERMISSION:
-          expectedCategory = "cookieBlockedPermission";
-          break;
-        case Ci.nsIWebProgressListener.STATE_COOKIES_BLOCKED_TRACKER:
-          expectedCategory = "cookieBlockedTracker";
-          break;
-        case Ci.nsIWebProgressListener.STATE_COOKIES_BLOCKED_ALL:
-          expectedCategory = "cookieBlockedAll";
-          break;
-        case Ci.nsIWebProgressListener.STATE_COOKIES_BLOCKED_FOREIGN:
-          expectedCategory = "cookieBlockedForeign";
-          break;
+      let expectedCategories = [];
+      let rawExpectedCategories =
+        options.expectedBlockingNotifications ??
+        options.expectedPartitioningNotifications;
+      if (!Array.isArray(rawExpectedCategories)) {
+        // if given a single value to match, expect each message to match it
+        rawExpectedCategories = Array(allMessages.length).fill(
+          rawExpectedCategories
+        );
+      }
+      for (let category of rawExpectedCategories) {
+        switch (category) {
+          case Ci.nsIWebProgressListener.STATE_COOKIES_BLOCKED_BY_PERMISSION:
+            expectedCategories.push("cookieBlockedPermission");
+            break;
+          case Ci.nsIWebProgressListener.STATE_COOKIES_BLOCKED_TRACKER:
+            expectedCategories.push("cookieBlockedTracker");
+            break;
+          case Ci.nsIWebProgressListener.STATE_COOKIES_BLOCKED_ALL:
+            expectedCategories.push("cookieBlockedAll");
+            break;
+          case Ci.nsIWebProgressListener.STATE_COOKIES_BLOCKED_FOREIGN:
+            expectedCategories.push("cookieBlockedForeign");
+            break;
+          case Ci.nsIWebProgressListener.STATE_COOKIES_PARTITIONED_TRACKER:
+            expectedCategories.push("cookiePartitionedForeign");
+            break;
+        }
       }
 
-      if (expectedCategory == "") {
+      if (!expectedCategories.length) {
         is(allMessages.length, 0, "No console messages should be generated");
       } else {
         ok(!!allMessages.length, "Some console message should be generated");
@@ -924,8 +909,8 @@ this.AntiTracking = {
       for (let msg of allMessages) {
         is(
           msg.category,
-          expectedCategory,
-          "Message should be of expected category"
+          expectedCategories[index],
+          `Message ${index} should be of expected category`
         );
 
         if (options.errorMessageDomains) {
@@ -947,6 +932,10 @@ this.AntiTracking = {
 
       win.gBrowser.removeProgressListener(listener);
 
+      if (!!cookieBlocked != !!options.expectedBlockingNotifications) {
+        ok(false, JSON.stringify(cookieBlocked));
+        ok(false, JSON.stringify(options.expectedBlockingNotifications));
+      }
       is(
         !!cookieBlocked,
         !!options.expectedBlockingNotifications,
@@ -963,7 +952,7 @@ this.AntiTracking = {
   },
 
   _createCleanupTask(cleanupFunction) {
-    add_task(async function() {
+    add_task(async function () {
       info("Cleaning up.");
       if (cleanupFunction) {
         await cleanupFunction();
@@ -983,9 +972,10 @@ this.AntiTracking = {
     runInPrivateWindow,
     iframeSandbox,
     testInSubIFrame,
-    extraPrefs
+    extraPrefs,
+    iframeAllow
   ) {
-    add_task(async function() {
+    add_task(async function () {
       info(
         `Starting window-open${
           testInSubIFrame ? " sub iframe" : ""
@@ -998,6 +988,15 @@ this.AntiTracking = {
         await TestUtils.topicObserved("browser-delayed-startup-finished");
       }
 
+      // Enable SA heuristics for trackers because the test depends on it.
+      extraPrefs = [
+        ...(extraPrefs || []),
+        [
+          "privacy.restrict3rdpartystorage.heuristic.exclude_third_party_trackers",
+          false,
+        ],
+      ];
+
       await AntiTracking._setupTest(
         win,
         cookieBehavior,
@@ -1017,7 +1016,7 @@ this.AntiTracking = {
         let iframeBrowsingContext = await SpecialPowers.spawn(
           browser,
           [{ page: TEST_IFRAME_PAGE }],
-          async function(obj) {
+          async function (obj) {
             // Add an iframe.
             let ifr = content.document.createElement("iframe");
             let loading = new content.Promise(resolve => {
@@ -1048,18 +1047,22 @@ this.AntiTracking = {
             blockingCallback: blockingCallback.toString(),
             nonBlockingCallback: nonBlockingCallback.toString(),
             iframeSandbox,
+            iframeAllow,
           },
         ],
-        async function(obj) {
+        async function (obj) {
           await new content.Promise(resolve => {
             let ifr = content.document.createElement("iframe");
-            ifr.onload = function() {
+            ifr.onload = function () {
               info("Sending code to the 3rd party content");
               ifr.contentWindow.postMessage(obj, "*");
             };
             if (typeof obj.iframeSandbox == "string") {
               ifr.setAttribute("sandbox", obj.iframeSandbox);
             }
+            if (typeof obj.iframeAllow == "string") {
+              ifr.setAttribute("allow", obj.iframeAllow);
+            }
 
             content.addEventListener("message", function msg(event) {
               if (event.data.type == "finish") {
@@ -1104,9 +1107,10 @@ this.AntiTracking = {
     runInPrivateWindow,
     iframeSandbox,
     testInSubIFrame,
-    extraPrefs
+    extraPrefs,
+    iframeAllow
   ) {
-    add_task(async function() {
+    add_task(async function () {
       info(
         `Starting user-interaction${
           testInSubIFrame ? " sub iframe" : ""
@@ -1119,6 +1123,15 @@ this.AntiTracking = {
         await TestUtils.topicObserved("browser-delayed-startup-finished");
       }
 
+      // Enable SA heuristics for trackers because the test depends on it.
+      extraPrefs = [
+        ...(extraPrefs || []),
+        [
+          "privacy.restrict3rdpartystorage.heuristic.exclude_third_party_trackers",
+          false,
+        ],
+      ];
+
       await AntiTracking._setupTest(
         win,
         cookieBehavior,
@@ -1138,7 +1151,7 @@ this.AntiTracking = {
         let iframeBrowsingContext = await SpecialPowers.spawn(
           browser,
           [{ page: TEST_IFRAME_PAGE }],
-          async function(obj) {
+          async function (obj) {
             // Add an iframe.
             let ifr = content.document.createElement("iframe");
             let loading = new content.Promise(resolve => {
@@ -1169,9 +1182,10 @@ this.AntiTracking = {
             popup: TEST_POPUP_PAGE,
             blockingCallback: blockingCallback.toString(),
             iframeSandbox,
+            iframeAllow,
           },
         ],
-        async function(obj) {
+        async function (obj) {
           let ifr = content.document.createElement("iframe");
           let loading = new content.Promise(resolve => {
             ifr.onload = resolve;
@@ -1179,6 +1193,9 @@ this.AntiTracking = {
           if (typeof obj.iframeSandbox == "string") {
             ifr.setAttribute("sandbox", obj.iframeSandbox);
           }
+          if (typeof obj.iframeAllow == "string") {
+            ifr.setAttribute("allow", obj.iframeAllow);
+          }
           content.document.body.appendChild(ifr);
           ifr.src = obj.page;
           await loading;
@@ -1213,22 +1230,24 @@ this.AntiTracking = {
           });
 
           let windowClosed = new content.Promise(resolve => {
-            Services.ww.registerNotification(function notification(
-              aSubject,
-              aTopic,
-              aData
-            ) {
-              if (aTopic == "domwindowclosed") {
-                Services.ww.unregisterNotification(notification);
-                resolve();
+            Services.ww.registerNotification(
+              function notification(aSubject, aTopic) {
+                if (aTopic == "domwindowclosed") {
+                  Services.ww.unregisterNotification(notification);
+                  resolve();
+                }
               }
-            });
+            );
           });
 
           info("Opening a window from the iframe.");
-          SpecialPowers.spawn(ifr, [{ popup: obj.popup }], async function(obj) {
-            content.open(obj.popup);
-          });
+          SpecialPowers.spawn(
+            ifr,
+            [{ popup: obj.popup }],
+            async function (obj) {
+              content.open(obj.popup);
+            }
+          );
 
           info("Let's wait for the window to be closed");
           await windowClosed;
@@ -1295,9 +1314,10 @@ this.AntiTracking = {
             popup: TEST_POPUP_PAGE,
             nonBlockingCallback: nonBlockingCallback.toString(),
             iframeSandbox,
+            iframeAllow,
           },
         ],
-        async function(obj) {
+        async function (obj) {
           let ifr = content.document.createElement("iframe");
           let loading = new content.Promise(resolve => {
             ifr.onload = resolve;
@@ -1305,27 +1325,32 @@ this.AntiTracking = {
           if (typeof obj.iframeSandbox == "string") {
             ifr.setAttribute("sandbox", obj.iframeSandbox);
           }
+          if (typeof obj.iframeAllow == "string") {
+            ifr.setAttribute("allow", obj.iframeAllow);
+          }
           content.document.body.appendChild(ifr);
           ifr.src = obj.page;
           await loading;
 
           let windowClosed = new content.Promise(resolve => {
-            Services.ww.registerNotification(function notification(
-              aSubject,
-              aTopic,
-              aData
-            ) {
-              if (aTopic == "domwindowclosed") {
-                Services.ww.unregisterNotification(notification);
-                resolve();
+            Services.ww.registerNotification(
+              function notification(aSubject, aTopic) {
+                if (aTopic == "domwindowclosed") {
+                  Services.ww.unregisterNotification(notification);
+                  resolve();
+                }
               }
-            });
+            );
           });
 
           info("Opening a window from the iframe.");
-          SpecialPowers.spawn(ifr, [{ popup: obj.popup }], async function(obj) {
-            content.open(obj.popup);
-          });
+          SpecialPowers.spawn(
+            ifr,
+            [{ popup: obj.popup }],
+            async function (obj) {
+              content.open(obj.popup);
+            }
+          );
 
           info("Let's wait for the window to be closed");
           await windowClosed;
@@ -1395,12 +1420,12 @@ this.AntiTracking = {
 
       function Listener() {}
       Listener.prototype = {
-        onStartRequest(request) {},
+        onStartRequest() {},
         onDataAvailable(request, stream, off, cnt) {
           // Consume the data to prevent hitting the assertion.
           NetUtil.readInputStreamToString(stream, cnt);
         },
-        onStopRequest(request, st) {
+        onStopRequest(request) {
           let status = request.QueryInterface(Ci.nsIHttpChannel).responseStatus;
           if (status == 200) {
             resolve();
