@@ -26,7 +26,6 @@
 #include "nsIURI.h"
 #include "nsIWebBrowserPersist.h"
 #include "nsFocusManager.h"
-#include "Layers.h"
 #include "nsILoadContext.h"
 #include "nsComponentManagerUtils.h"
 #include "nsDocShell.h"
@@ -45,7 +44,7 @@
 // Printing Includes
 #ifdef NS_PRINTING
 #  include "nsIWebBrowserPrint.h"
-#  include "nsIContentViewer.h"
+#  include "nsIDocumentViewer.h"
 #endif
 
 // PSM2 includes
@@ -60,7 +59,6 @@ nsWebBrowser::nsWebBrowser(int aItemType)
     : mContentType(aItemType),
       mShouldEnableHistory(true),
       mWillChangeProcess(false),
-      mParentNativeWindow(nullptr),
       mProgressListener(nullptr),
       mWidgetListenerDelegate(this),
       mBackgroundColor(0),
@@ -84,14 +82,13 @@ nsIWidget* nsWebBrowser::EnsureWidget() {
     return nullptr;
   }
 
-  nsWidgetInitData widgetInit;
-  widgetInit.clipChildren = true;
-  widgetInit.mWindowType = eWindowType_child;
+  widget::InitData widgetInit;
+  widgetInit.mClipChildren = true;
+  widgetInit.mWindowType = widget::WindowType::Child;
   LayoutDeviceIntRect bounds(0, 0, 0, 0);
 
   mInternalWidget->SetWidgetListener(&mWidgetListenerDelegate);
-  NS_ENSURE_SUCCESS(mInternalWidget->Create(nullptr, mParentNativeWindow,
-                                            bounds, &widgetInit),
+  NS_ENSURE_SUCCESS(mInternalWidget->Create(mParentWidget, bounds, &widgetInit),
                     nullptr);
 
   return mInternalWidget;
@@ -150,8 +147,7 @@ already_AddRefed<nsWebBrowser> nsWebBrowser::Create(
   // events from subframes. To solve that we install our own chrome event
   // handler that always gets called (even for subframes) for any bubbling
   // event.
-
-  nsresult rv = docShell->InitWindow(nullptr, docShellParentWidget, 0, 0, 0, 0);
+  nsresult rv = docShell->InitWindow(docShellParentWidget, 0, 0, 0, 0);
   if (NS_WARN_IF(NS_FAILED(rv))) {
     return nullptr;
   }
@@ -160,7 +156,7 @@ already_AddRefed<nsWebBrowser> nsWebBrowser::Create(
   docShellTreeOwner->AddChromeListeners();
 
   if (aInitialWindowChild) {
-    docShell->CreateContentViewerForActor(aInitialWindowChild);
+    docShell->CreateDocumentViewerForActor(aInitialWindowChild);
   }
 
   return browser.forget();
@@ -214,8 +210,8 @@ nsWebBrowser::GetInterface(const nsIID& aIID, void** aSink) {
   if (mDocShell) {
 #ifdef NS_PRINTING
     if (aIID.Equals(NS_GET_IID(nsIWebBrowserPrint))) {
-      nsCOMPtr<nsIContentViewer> viewer;
-      mDocShell->GetContentViewer(getter_AddRefs(viewer));
+      nsCOMPtr<nsIDocumentViewer> viewer;
+      mDocShell->GetDocViewer(getter_AddRefs(viewer));
       if (!viewer) {
         return NS_NOINTERFACE;
       }
@@ -449,6 +445,13 @@ nsWebBrowser::GetCanGoBack(bool* aCanGoBack) {
   return mDocShell->GetCanGoBack(aCanGoBack);
 }
 
+NS_IMETHODIMP
+nsWebBrowser::GetCanGoBackIgnoringUserInteraction(bool* aCanGoBack) {
+  NS_ENSURE_STATE(mDocShell);
+
+  return mDocShell->GetCanGoBackIgnoringUserInteraction(aCanGoBack);
+}
+
 NS_IMETHODIMP
 nsWebBrowser::GetCanGoForward(bool* aCanGoForward) {
   NS_ENSURE_STATE(mDocShell);
@@ -460,17 +463,19 @@ NS_IMETHODIMP
 nsWebBrowser::GoBack(bool aRequireUserInteraction, bool aUserActivation) {
   NS_ENSURE_STATE(mDocShell);
 
-  return mDocShell->GoBack(aRequireUserInteraction, aUserActivation);
+  RefPtr<nsDocShell> docShell = mDocShell;
+  return docShell->GoBack(aRequireUserInteraction, aUserActivation);
 }
 
 NS_IMETHODIMP
 nsWebBrowser::GoForward(bool aRequireUserInteraction, bool aUserActivation) {
   NS_ENSURE_STATE(mDocShell);
 
-  return mDocShell->GoForward(aRequireUserInteraction, aUserActivation);
+  RefPtr<nsDocShell> docShell = mDocShell;
+  return docShell->GoForward(aRequireUserInteraction, aUserActivation);
 }
 
-nsresult nsWebBrowser::LoadURI(const nsAString& aURI,
+nsresult nsWebBrowser::LoadURI(nsIURI* aURI,
                                const dom::LoadURIOptions& aLoadURIOptions) {
 #ifndef ANDROID
   MOZ_ASSERT(aLoadURIOptions.mTriggeringPrincipal,
@@ -478,11 +483,12 @@ nsresult nsWebBrowser::LoadURI(const nsAString& aURI,
 #endif
   NS_ENSURE_STATE(mDocShell);
 
-  return mDocShell->LoadURI(aURI, aLoadURIOptions);
+  RefPtr<nsDocShell> docShell = mDocShell;
+  return docShell->LoadURI(aURI, aLoadURIOptions);
 }
 
 NS_IMETHODIMP
-nsWebBrowser::LoadURIFromScript(const nsAString& aURI,
+nsWebBrowser::LoadURIFromScript(nsIURI* aURI,
                                 JS::Handle<JS::Value> aLoadURIOptions,
                                 JSContext* aCx) {
   // generate dictionary for loadURIOptions and forward call
@@ -493,6 +499,31 @@ nsWebBrowser::LoadURIFromScript(const nsAString& aURI,
   return LoadURI(aURI, loadURIOptions);
 }
 
+nsresult nsWebBrowser::FixupAndLoadURIString(
+    const nsAString& aURI, const dom::LoadURIOptions& aLoadURIOptions) {
+#ifndef ANDROID
+  MOZ_ASSERT(
+      aLoadURIOptions.mTriggeringPrincipal,
+      "nsWebBrowser::FixupAndLoadURIString - Need a valid triggeringPrincipal");
+#endif
+  NS_ENSURE_STATE(mDocShell);
+
+  RefPtr<nsDocShell> docShell = mDocShell;
+  return docShell->FixupAndLoadURIString(aURI, aLoadURIOptions);
+}
+
+NS_IMETHODIMP
+nsWebBrowser::FixupAndLoadURIStringFromScript(
+    const nsAString& aURI, JS::Handle<JS::Value> aLoadURIOptions,
+    JSContext* aCx) {
+  // generate dictionary for loadURIOptions and forward call
+  dom::LoadURIOptions loadURIOptions;
+  if (!loadURIOptions.Init(aCx, aLoadURIOptions)) {
+    return NS_ERROR_INVALID_ARG;
+  }
+  return FixupAndLoadURIString(aURI, loadURIOptions);
+}
+
 NS_IMETHODIMP
 nsWebBrowser::ResumeRedirectedLoad(uint64_t aIdentifier,
                                    int32_t aHistoryIndex) {
@@ -505,14 +536,16 @@ NS_IMETHODIMP
 nsWebBrowser::Reload(uint32_t aReloadFlags) {
   NS_ENSURE_STATE(mDocShell);
 
-  return mDocShell->Reload(aReloadFlags);
+  RefPtr<nsDocShell> docShell = mDocShell;
+  return docShell->Reload(aReloadFlags);
 }
 
 NS_IMETHODIMP
 nsWebBrowser::GotoIndex(int32_t aIndex, bool aUserActivation) {
   NS_ENSURE_STATE(mDocShell);
 
-  return mDocShell->GotoIndex(aIndex, aUserActivation);
+  RefPtr<nsDocShell> docShell = mDocShell;
+  return docShell->GotoIndex(aIndex, aUserActivation);
 }
 
 NS_IMETHODIMP
@@ -696,20 +729,7 @@ nsWebBrowser::SaveURI(nsIURI* aURI, nsIPrincipal* aPrincipal,
                       nsICookieJarSettings* aCookieJarSettings,
                       nsIInputStream* aPostData, const char* aExtraHeaders,
                       nsISupports* aFile,
-                      nsContentPolicyType aContentPolicyType,
-                      nsILoadContext* aPrivacyContext) {
-  return SavePrivacyAwareURI(
-      aURI, aPrincipal, aCacheKey, aReferrerInfo, aCookieJarSettings, aPostData,
-      aExtraHeaders, aFile, aContentPolicyType,
-      aPrivacyContext && aPrivacyContext->UsePrivateBrowsing());
-}
-
-NS_IMETHODIMP
-nsWebBrowser::SavePrivacyAwareURI(
-    nsIURI* aURI, nsIPrincipal* aPrincipal, uint32_t aCacheKey,
-    nsIReferrerInfo* aReferrerInfo, nsICookieJarSettings* aCookieJarSettings,
-    nsIInputStream* aPostData, const char* aExtraHeaders, nsISupports* aFile,
-    nsContentPolicyType aContentPolicyType, bool aIsPrivate) {
+                      nsContentPolicyType aContentPolicyType, bool aIsPrivate) {
   if (mPersist) {
     uint32_t currentState;
     mPersist->GetCurrentState(&currentState);
@@ -739,9 +759,9 @@ nsWebBrowser::SavePrivacyAwareURI(
   mPersist->SetPersistFlags(mPersistFlags);
   mPersist->GetCurrentState(&mPersistCurrentState);
 
-  rv = mPersist->SavePrivacyAwareURI(
-      uri, aPrincipal, aCacheKey, aReferrerInfo, aCookieJarSettings, aPostData,
-      aExtraHeaders, aFile, aContentPolicyType, aIsPrivate);
+  rv = mPersist->SaveURI(uri, aPrincipal, aCacheKey, aReferrerInfo,
+                         aCookieJarSettings, aPostData, aExtraHeaders, aFile,
+                         aContentPolicyType, aIsPrivate);
   if (NS_FAILED(rv)) {
     mPersist = nullptr;
   }
@@ -844,12 +864,11 @@ nsWebBrowser::Cancel(nsresult aReason) {
 //*****************************************************************************
 
 NS_IMETHODIMP
-nsWebBrowser::InitWindow(nativeWindow aParentNativeWindow,
-                         nsIWidget* aParentWidget, int32_t aX, int32_t aY,
+nsWebBrowser::InitWindow(nsIWidget* aParentWidget, int32_t aX, int32_t aY,
                          int32_t aCX, int32_t aCY) {
   // nsIBaseWindow::InitWindow and nsIBaseWindow::Create
   // implementations have been merged into nsWebBrowser::Create
-  MOZ_DIAGNOSTIC_ASSERT(false);
+  MOZ_DIAGNOSTIC_CRASH("Superceded by nsWebBrowser::Create()");
   return NS_ERROR_NULL_POINTER;
 }
 
@@ -860,10 +879,8 @@ nsWebBrowser::Destroy() {
   return NS_OK;
 }
 
-NS_IMETHODIMP
-nsWebBrowser::GetUnscaledDevicePixelsPerCSSPixel(double* aScale) {
-  *aScale = mParentWidget ? mParentWidget->GetDefaultScale().scale : 1.0;
-  return NS_OK;
+double nsWebBrowser::GetWidgetCSSToDeviceScale() {
+  return mParentWidget ? mParentWidget->GetDefaultScale().scale : 1.0;
 }
 
 NS_IMETHODIMP
@@ -959,12 +976,22 @@ nsWebBrowser::GetPositionAndSize(int32_t* aX, int32_t* aY, int32_t* aCX,
       *aCY = bounds.Height();
     }
     return NS_OK;
-  } else {
-    // Can directly return this as it is the
-    // same interface, thus same returns.
-    return mDocShell->GetPositionAndSize(aX, aY, aCX, aCY);
   }
-  return NS_OK;
+
+  // Can directly return this as it is the
+  // same interface, thus same returns.
+  return mDocShell->GetPositionAndSize(aX, aY, aCX, aCY);
+}
+
+NS_IMETHODIMP
+nsWebBrowser::SetDimensions(DimensionRequest&& aRequest) {
+  return NS_ERROR_NOT_IMPLEMENTED;
+}
+
+NS_IMETHODIMP
+nsWebBrowser::GetDimensions(DimensionKind aDimensionKind, int32_t* aX,
+                            int32_t* aY, int32_t* aCX, int32_t* aCY) {
+  return NS_ERROR_NOT_IMPLEMENTED;
 }
 
 NS_IMETHODIMP
@@ -989,32 +1016,7 @@ nsWebBrowser::GetParentWidget(nsIWidget** aParentWidget) {
 NS_IMETHODIMP
 nsWebBrowser::SetParentWidget(nsIWidget* aParentWidget) {
   NS_ENSURE_STATE(!mDocShell);
-
   mParentWidget = aParentWidget;
-  if (mParentWidget) {
-    mParentNativeWindow = mParentWidget->GetNativeData(NS_NATIVE_WIDGET);
-  } else {
-    mParentNativeWindow = nullptr;
-  }
-
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsWebBrowser::GetParentNativeWindow(nativeWindow* aParentNativeWindow) {
-  NS_ENSURE_ARG_POINTER(aParentNativeWindow);
-
-  *aParentNativeWindow = mParentNativeWindow;
-
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsWebBrowser::SetParentNativeWindow(nativeWindow aParentNativeWindow) {
-  NS_ENSURE_STATE(!mDocShell);
-
-  mParentNativeWindow = aParentNativeWindow;
-
   return NS_OK;
 }
 
