@@ -5,10 +5,8 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 "use strict";
 
-/* globals EventEmitter */
-
-XPCOMUtils.defineLazyModuleGetters(this, {
-  PrivateBrowsingUtils: "resource://gre/modules/PrivateBrowsingUtils.jsm",
+ChromeUtils.defineESModuleGetters(this, {
+  PrivateBrowsingUtils: "resource://gre/modules/PrivateBrowsingUtils.sys.mjs",
 });
 
 XPCOMUtils.defineLazyPreferenceGetter(
@@ -17,12 +15,8 @@ XPCOMUtils.defineLazyPreferenceGetter(
   "privacy.userContext.enabled"
 );
 
-var {
-  DefaultMap,
-  DefaultWeakMap,
-  ExtensionError,
-  parseMatchPatterns,
-} = ExtensionUtils;
+var { DefaultMap, DefaultWeakMap, ExtensionError, parseMatchPatterns } =
+  ExtensionUtils;
 
 var { defineLazyGetter } = ExtensionCommon;
 
@@ -30,11 +24,11 @@ var { defineLazyGetter } = ExtensionCommon;
  * The platform-specific type of native tab objects, which are wrapped by
  * TabBase instances.
  *
- * @typedef {Object|XULElement} NativeTab
+ * @typedef {object | XULElement} NativeTab
  */
 
 /**
- * @typedef {Object} MutedInfo
+ * @typedef {object} MutedInfo
  * @property {boolean} muted
  *        True if the tab is currently muted, false otherwise.
  * @property {string} [reason]
@@ -73,38 +67,6 @@ class TabBase {
     }
   }
 
-  /**
-   * Sends a message, via the given context, to the ExtensionContent running in
-   * this tab. The tab's current innerWindowID is automatically added to the
-   * recipient filter for the message, and is used to ensure that the message is
-   * not processed if the content process navigates to a different content page
-   * before the message is received.
-   *
-   * @param {BaseContext} context
-   *        The context through which to send the message.
-   * @param {string} messageName
-   *        The name of the message to send.
-   * @param {object} [data = {}]
-   *        Arbitrary, structured-clonable message data to send.
-   * @param {object} [options]
-   *        An options object, as accepted by BaseContext.sendMessage.
-   *
-   * @returns {Promise}
-   */
-  sendMessage(context, messageName, data = {}, options = null) {
-    let { browser, innerWindowID } = this;
-
-    options = Object.assign({}, options);
-    options.recipient = Object.assign({ innerWindowID }, options.recipient);
-
-    return context.sendMessage(
-      browser.messageManager,
-      messageName,
-      data,
-      options
-    );
-  }
-
   /**
    * Capture the visible area of this tab, and return the result as a data: URI.
    *
@@ -112,7 +74,7 @@ class TabBase {
    *        The extension context for which to perform the capture.
    * @param {number} zoom
    *        The current zoom for the page.
-   * @param {Object} [options]
+   * @param {object} [options]
    *        The options with which to perform the capture.
    * @param {string} [options.format = "png"]
    *        The image format in which to encode the captured data. May be one of
@@ -132,19 +94,39 @@ class TabBase {
     let scale = options?.scale || win.devicePixelRatio;
     let rect = options?.rect && win.DOMRect.fromRect(options.rect);
 
+    // We only allow mozilla addons to use the resetScrollPosition option,
+    // since it's not standardized.
+    let resetScrollPosition = false;
+    if (!context.extension.restrictSchemes) {
+      resetScrollPosition = !!options?.resetScrollPosition;
+    }
+
     let wgp = this.browsingContext.currentWindowGlobal;
-    let image = await wgp.drawSnapshot(rect, scale * zoom, "white");
+    let image = await wgp.drawSnapshot(
+      rect,
+      scale * zoom,
+      "white",
+      resetScrollPosition
+    );
 
-    let doc = Services.appShell.hiddenDOMWindow.document;
-    let canvas = doc.createElement("canvas");
-    canvas.width = image.width;
-    canvas.height = image.height;
+    let canvas = new OffscreenCanvas(image.width, image.height);
 
-    let ctx = canvas.getContext("2d", { alpha: false });
-    ctx.drawImage(image, 0, 0);
-    image.close();
+    let ctx = canvas.getContext("bitmaprenderer", { alpha: false });
+    ctx.transferFromImageBitmap(image);
 
-    return canvas.toDataURL(`image/${options?.format}`, options?.quality / 100);
+    let blob = await canvas.convertToBlob({
+      type: `image/${options?.format ?? "png"}`,
+      quality: options?.quality / 100,
+    });
+
+    let dataURL = await new Promise((resolve, reject) => {
+      let reader = new FileReader();
+      reader.onload = () => resolve(reader.result);
+      reader.onerror = () => reject(reader.error);
+      reader.readAsDataURL(blob);
+    });
+
+    return dataURL;
   }
 
   /**
@@ -187,7 +169,8 @@ class TabBase {
    */
   get hasActiveTabPermission() {
     return (
-      this.extension.hasPermission("activeTab") &&
+      (this.extension.originControls ||
+        this.extension.hasPermission("activeTab")) &&
       this.activeTabWindowID != null &&
       this.activeTabWindowID === this.innerWindowID
     );
@@ -222,15 +205,16 @@ class TabBase {
   }
 
   /**
-   * @property {string | null} url
+   * @property {string | undefined} url
    *        Returns the current URL of this tab if the extension has permission
-   *        to read it, or null otherwise.
+   *        to read it, or undefined otherwise.
    *        @readonly
    */
   get url() {
     if (this.hasTabPermission) {
       return this._url;
     }
+    return undefined;
   }
 
   /**
@@ -253,15 +237,16 @@ class TabBase {
   }
 
   /**
-   * @property {nsIURI | null} title
+   * @property {nsIURI | undefined} title
    *        Returns the current title of this tab if the extension has permission
-   *        to read it, or null otherwise.
+   *        to read it, or undefined otherwise.
    *        @readonly
    */
   get title() {
     if (this.hasTabPermission) {
       return this._title;
     }
+    return undefined;
   }
 
   /**
@@ -276,15 +261,16 @@ class TabBase {
   }
 
   /**
-   * @property {nsIURI | null} faviconUrl
+   * @property {nsIURI | undefined} faviconUrl
    *        Returns the current faviron URL of this tab if the extension has permission
-   *        to read it, or null otherwise.
+   *        to read it, or undefined otherwise.
    *        @readonly
    */
   get favIconUrl() {
     if (this.hasTabPermission) {
       return this._favIconUrl;
     }
+    return undefined;
   }
 
   /**
@@ -308,6 +294,16 @@ class TabBase {
     throw new Error("Not implemented");
   }
 
+  /**
+   * @property {boolean} autoDiscardable
+   *        Returns true if the tab can be discarded on memory pressure, false otherwise.
+   *        @readonly
+   *        @abstract
+   */
+  get autoDiscardable() {
+    throw new Error("Not implemented");
+  }
+
   /**
    * @property {XULElement} browser
    *        Returns the XUL browser for the given tab.
@@ -528,6 +524,15 @@ class TabBase {
     throw new Error("Not implemented");
   }
 
+  /**
+   * @property {integer} groupId
+   *        @readonly
+   *        @abstract
+   */
+  get groupId() {
+    throw new Error("Not implemented");
+  }
+
   /**
    * Returns true if this tab matches the the given query info object. Omitted
    * or null have no effect on the match.
@@ -538,6 +543,8 @@ class TabBase {
    *        Matches against the exact value of the tab's `active` attribute.
    * @param {boolean} [queryInfo.audible]
    *        Matches against the exact value of the tab's `audible` attribute.
+   * @param {boolean} [queryInfo.autoDiscardable]
+   *        Matches against the exact value of the tab's `autoDiscardable` attribute.
    * @param {string} [queryInfo.cookieStoreId]
    *        Matches against the exact value of the tab's `cookieStoreId` attribute.
    * @param {boolean} [queryInfo.discarded]
@@ -567,6 +574,8 @@ class TabBase {
    *        than an exact value match, and will do so in the future.
    * @param {MatchPattern} [queryInfo.url]
    *        Requires the tab's URL to match the given MatchPattern object.
+   * @param {integer} [queryInfo.groupId]
+   *        Matches against the exact value of the tab's `groupId` attribute.
    *
    * @returns {boolean}
    *        True if the tab matches the query.
@@ -575,7 +584,7 @@ class TabBase {
     const PROPS = [
       "active",
       "audible",
-      "cookieStoreId",
+      "autoDiscardable",
       "discarded",
       "hidden",
       "highlighted",
@@ -583,6 +592,7 @@ class TabBase {
       "openerTabId",
       "pinned",
       "status",
+      "groupId",
     ];
 
     function checkProperty(prop, obj) {
@@ -611,6 +621,13 @@ class TabBase {
         return false;
       }
     }
+
+    if (queryInfo.cookieStoreId) {
+      if (!queryInfo.cookieStoreId.includes(this.cookieStoreId)) {
+        return false;
+      }
+    }
+
     if (queryInfo.url || queryInfo.title) {
       if (!this.hasTabPermission) {
         return false;
@@ -632,7 +649,7 @@ class TabBase {
    * of its properties which the extension is permitted to access, in the format
    * required to be returned by WebExtension APIs.
    *
-   * @param {Object} [fallbackTabSize]
+   * @param {object} [fallbackTabSize]
    *        A geometry data if the lazy geometry data for this tab hasn't been
    *        initialized yet.
    * @returns {object}
@@ -654,11 +671,13 @@ class TabBase {
       height: this.height,
       lastAccessed: this.lastAccessed,
       audible: this.audible,
+      autoDiscardable: this.autoDiscardable,
       mutedInfo: this.mutedInfo,
       isArticle: this.isArticle,
       isInReaderMode: this.isInReaderMode,
       sharingState: this.sharingState,
       successorTabId: this.successorTabId,
+      groupId: this.groupId,
       cookieStoreId: this.cookieStoreId,
     };
 
@@ -690,30 +709,48 @@ class TabBase {
 
   /**
    * Query each content process hosting subframes of the tab, return results.
+   *
    * @param {string} message
    * @param {object} options
-   * @param {number} options.frameID
-   * @param {boolean} options.allFrames
+   *        These options are also sent to the message handler in the
+   *        `ExtensionContentChild`.
+   * @param {number[]} options.frameIds
+   *        When omitted, all frames will be queried.
+   * @param {boolean} options.returnResultsWithFrameIds
    * @returns {Promise[]}
    */
   async queryContent(message, options) {
-    let { allFrames, frameID } = options;
+    let { frameIds } = options;
 
     /** @type {Map<nsIDOMProcessParent, innerWindowId[]>} */
     let byProcess = new DefaultMap(() => []);
+    // We use this set to know which frame IDs are potentially invalid (as in
+    // not found when visiting the tab's BC tree below) when frameIds is a
+    // non-empty list of frame IDs.
+    let frameIdsSet = new Set(frameIds);
 
     // Recursively walk the tab's BC tree, find all frames, group by process.
     function visit(bc) {
       let win = bc.currentWindowGlobal;
-      if (win?.domProcess && (!frameID || frameID === bc.id)) {
+      let frameId = bc.parent ? bc.id : 0;
+
+      if (win?.domProcess && (!frameIds || frameIdsSet.has(frameId))) {
         byProcess.get(win.domProcess).push(win.innerWindowId);
+        frameIdsSet.delete(frameId);
       }
-      if (allFrames || (frameID && !byProcess.size)) {
+
+      if (!frameIds || frameIdsSet.size > 0) {
         bc.children.forEach(visit);
       }
     }
     visit(this.browsingContext);
 
+    if (frameIdsSet.size > 0) {
+      throw new ExtensionError(
+        `Invalid frame IDs: [${Array.from(frameIdsSet).join(", ")}].`
+      );
+    }
+
     let promises = Array.from(byProcess.entries(), ([proc, windows]) =>
       proc.getActor("ExtensionContent").sendQuery(message, { windows, options })
     );
@@ -729,15 +766,15 @@ class TabBase {
     results = results.flat();
 
     if (!results.length) {
-      if (frameID) {
-        throw new ExtensionError("Frame not found, or missing host permission");
+      let errorMessage = "Missing host permission for the tab";
+      if (!frameIds || frameIds.length > 1 || frameIds[0] !== 0) {
+        errorMessage += " or frames";
       }
 
-      let frames = allFrames ? ", and any iframes" : "";
-      throw new ExtensionError(`Missing host permission for the tab${frames}`);
+      throw new ExtensionError(errorMessage);
     }
 
-    if (!allFrames && results.length > 1) {
+    if (frameIds && frameIds.length === 1 && results.length > 1) {
       throw new ExtensionError("Internal error: multiple windows matched");
     }
 
@@ -801,12 +838,15 @@ class TabBase {
       }
       options[`${kind}Paths`].push(url);
     }
+
     if (details.allFrames) {
-      options.allFrames = details.allFrames;
-    }
-    if (details.frameId !== null) {
-      options.frameID = details.frameId;
+      options.allFrames = true;
+    } else if (details.frameId !== null) {
+      options.frameIds = [details.frameId];
+    } else if (!details.allFrames) {
+      options.frameIds = [0];
     }
+
     if (details.matchAboutBlank) {
       options.matchAboutBlank = details.matchAboutBlank;
     }
@@ -822,6 +862,10 @@ class TabBase {
     }
 
     options.wantReturnValue = true;
+
+    // The scripting API (defined in `parent/ext-scripting.js`) has its own
+    // `execute()` function that calls `queryContent()` as well. Make sure to
+    // keep both in sync when relevant.
     return this.queryContent("Execute", options);
   }
 
@@ -879,7 +923,7 @@ class TabBase {
   }
 }
 
-defineLazyGetter(TabBase.prototype, "incognito", function() {
+defineLazyGetter(TabBase.prototype, "incognito", function () {
   return this._incognito;
 });
 
@@ -1143,9 +1187,9 @@ class WindowBase {
   }
 
   /**
-   * @property {nsIURI | null} title
+   * @property {nsIURI | undefined} title
    *        Returns the current title of this window if the extension has permission
-   *        to read it, or null otherwise.
+   *        to read it, or undefined otherwise.
    *        @readonly
    */
   get title() {
@@ -1154,6 +1198,7 @@ class WindowBase {
     if (this.activeTab && this.activeTab.hasTabPermission) {
       return this._title;
     }
+    return undefined;
   }
 
   // The JSDoc validator does not support @returns tags in abstract functions or
@@ -1162,7 +1207,7 @@ class WindowBase {
   /**
    * Returns the window state of the given window.
    *
-   * @param {DOMWindow} window
+   * @param {DOMWindow} _window
    *        The window for which to return a state.
    *
    * @returns {string}
@@ -1171,7 +1216,7 @@ class WindowBase {
    * @static
    * @abstract
    */
-  static getState(window) {
+  static getState(_window) {
     throw new Error("Not implemented");
   }
 
@@ -1203,12 +1248,12 @@ class WindowBase {
   /**
    * Returns the window's tab at the specified index.
    *
-   * @param {integer} index
+   * @param {integer} _index
    *        The index of the desired tab.
    *
    * @returns {TabBase|undefined}
    */
-  getTabAtIndex(index) {
+  getTabAtIndex(_index) {
     throw new Error("Not implemented");
   }
   /* eslint-enable valid-jsdoc */
@@ -1220,7 +1265,7 @@ Object.assign(WindowBase, { WINDOW_ID_NONE, WINDOW_ID_CURRENT });
  * The parameter type of "tab-attached" events, which are emitted when a
  * pre-existing tab is attached to a new window.
  *
- * @typedef {Object} TabAttachedEvent
+ * @typedef {object} TabAttachedEvent
  * @property {NativeTab} tab
  *        The native tab object in the window to which the tab is being
  *        attached. This may be a different object than was used to represent
@@ -1238,7 +1283,7 @@ Object.assign(WindowBase, { WINDOW_ID_NONE, WINDOW_ID_CURRENT });
  * pre-existing tab is detached from a window, in order to be attached to a new
  * window.
  *
- * @typedef {Object} TabDetachedEvent
+ * @typedef {object} TabDetachedEvent
  * @property {NativeTab} tab
  *        The native tab object in the window from which the tab is being
  *        detached. This may be a different object than will be used to
@@ -1260,7 +1305,7 @@ Object.assign(WindowBase, { WINDOW_ID_NONE, WINDOW_ID_CURRENT });
  * The parameter type of "tab-created" events, which are emitted when a
  * new tab is created.
  *
- * @typedef {Object} TabCreatedEvent
+ * @typedef {object} TabCreatedEvent
  * @property {NativeTab} tab
  *        The native tab object for the tab which is being created.
  */
@@ -1269,7 +1314,7 @@ Object.assign(WindowBase, { WINDOW_ID_NONE, WINDOW_ID_CURRENT });
  * The parameter type of "tab-removed" events, which are emitted when a
  * tab is removed and destroyed.
  *
- * @typedef {Object} TabRemovedEvent
+ * @typedef {object} TabRemovedEvent
  * @property {NativeTab} tab
  *        The native tab object for the tab which is being removed.
  * @property {integer} tabId
@@ -1284,7 +1329,7 @@ Object.assign(WindowBase, { WINDOW_ID_NONE, WINDOW_ID_CURRENT });
  * An object containing basic, extension-independent information about the window
  * and tab that a XUL <browser> belongs to.
  *
- * @typedef {Object} BrowserData
+ * @typedef {object} BrowserData
  * @property {integer} tabId
  *        The numeric ID of the tab that a <browser> belongs to, or -1 if it
  *        does not belong to a tab.
@@ -1332,23 +1377,23 @@ class TabTrackerBase extends EventEmitter {
   /**
    * Returns the numeric ID for the given native tab.
    *
-   * @param {NativeTab} nativeTab
+   * @param {NativeTab} _nativeTab
    *        The native tab for which to return an ID.
    *
    * @returns {integer}
    *        The tab's numeric ID.
    * @abstract
    */
-  getId(nativeTab) {
+  getId(_nativeTab) {
     throw new Error("Not implemented");
   }
 
   /**
    * Returns the native tab with the given numeric ID.
    *
-   * @param {integer} tabId
+   * @param {integer} _tabId
    *        The numeric ID of the tab to return.
-   * @param {*} default_
+   * @param {*} _default
    *        The value to return if no tab exists with the given ID.
    *
    * @returns {NativeTab}
@@ -1357,7 +1402,7 @@ class TabTrackerBase extends EventEmitter {
    *       provided.
    * @abstract
    */
-  getTab(tabId, default_ = undefined) {
+  getTab(_tabId, _default) {
     throw new Error("Not implemented");
   }
 
@@ -1372,7 +1417,7 @@ class TabTrackerBase extends EventEmitter {
    * @abstract
    */
   /* eslint-enable valid-jsdoc */
-  getBrowserData(browser) {
+  getBrowserData() {
     throw new Error("Not implemented");
   }
 
@@ -1392,7 +1437,7 @@ class TabTrackerBase extends EventEmitter {
  * A browser progress listener instance which calls a given listener function
  * whenever the status of the given browser changes.
  *
- * @param {function(Object)} listener
+ * @param {function(object): void} listener
  *        A function to be called whenever the status of a tab's top-level
  *        browser. It is passed an object with a `browser` property pointing to
  *        the XUL browser, and a `status` property with a string description of
@@ -1428,7 +1473,7 @@ class StatusListener {
     }
   }
 
-  onLocationChange(browser, webProgress, request, locationURI, flags) {
+  onLocationChange(browser, webProgress, request, locationURI) {
     if (webProgress.isTopLevel) {
       let status = webProgress.isLoadingDocument ? "loading" : "complete";
       this.listener({ browser, status, url: locationURI.spec });
@@ -1617,7 +1662,7 @@ class WindowTrackerBase extends EventEmitter {
    * Register the given listener function to be called whenever a new browser
    * window is opened.
    *
-   * @param {function(DOMWindow)} listener
+   * @param {function(DOMWindow): void} listener
    *        The listener function to register.
    */
   addOpenListener(listener) {
@@ -1638,7 +1683,7 @@ class WindowTrackerBase extends EventEmitter {
    * Unregister a listener function registered in a previous addOpenListener
    * call.
    *
-   * @param {function(DOMWindow)} listener
+   * @param {function(DOMWindow): void} listener
    *        The listener function to unregister.
    */
   removeOpenListener(listener) {
@@ -1653,7 +1698,7 @@ class WindowTrackerBase extends EventEmitter {
    * Register the given listener function to be called whenever a browser
    * window is closed.
    *
-   * @param {function(DOMWindow)} listener
+   * @param {function(DOMWindow): void} listener
    *        The listener function to register.
    */
   addCloseListener(listener) {
@@ -1668,7 +1713,7 @@ class WindowTrackerBase extends EventEmitter {
    * Unregister a listener function registered in a previous addCloseListener
    * call.
    *
-   * @param {function(DOMWindow)} listener
+   * @param {function(DOMWindow): void} listener
    *        The listener function to unregister.
    */
   removeCloseListener(listener) {
@@ -1749,7 +1794,7 @@ class WindowTrackerBase extends EventEmitter {
    *           window.
    *        - "domwindowopened": Acts as an alias for addOpenListener.
    *        - "domwindowclosed": Acts as an alias for addCloseListener.
-   * @param {function|object} listener
+   * @param {Function | object} listener
    *        The listener to invoke in response to the given events.
    *
    * @returns {undefined}
@@ -1783,7 +1828,7 @@ class WindowTrackerBase extends EventEmitter {
    *
    * @param {string} type
    *        The type of event to stop listening for.
-   * @param {function|object} listener
+   * @param {Function | object} listener
    *        The listener to remove.
    *
    * @returns {undefined}
@@ -1829,7 +1874,7 @@ class WindowTrackerBase extends EventEmitter {
    * @param {string} eventType
    *        The type of DOM event to listen for, or "progress" to add a tab
    *        progress listener.
-   * @param {function|object} listener
+   * @param {Function | object} listener
    *        The listener to add.
    * @private
    */
@@ -1862,26 +1907,26 @@ class WindowTrackerBase extends EventEmitter {
   /**
    * Adds a tab progress listener to the given browser window.
    *
-   * @param {DOMWindow} window
+   * @param {DOMWindow} _window
    *        The browser window to which to add the listener.
-   * @param {object} listener
+   * @param {object} _listener
    *        The tab progress listener to add.
    * @abstract
    */
-  addProgressListener(window, listener) {
+  addProgressListener(_window, _listener) {
     throw new Error("Not implemented");
   }
 
   /**
    * Removes a tab progress listener from the given browser window.
    *
-   * @param {DOMWindow} window
+   * @param {DOMWindow} _window
    *        The browser window from which to remove the listener.
-   * @param {object} listener
+   * @param {object} _listener
    *        The tab progress listener to remove.
    * @abstract
    */
-  removeProgressListener(window, listener) {
+  removeProgressListener(_window, _listener) {
     throw new Error("Not implemented");
   }
 }
@@ -1908,12 +1953,16 @@ class TabManagerBase {
    *        The native tab for which to grant permissions.
    */
   addActiveTabPermission(nativeTab) {
-    if (this.extension.hasPermission("activeTab")) {
+    let tab = this.getWrapper(nativeTab);
+    if (
+      this.extension.hasPermission("activeTab") ||
+      (this.extension.originControls &&
+        this.extension.optionalOrigins.matches(tab._uri))
+    ) {
       // Note that, unlike Chrome, we don't currently clear this permission with
       // the tab navigates. If the inner window is revived from BFCache before
       // we've granted this permission to a new inner window, the extension
       // maintains its permissions for it.
-      let tab = this.getWrapper(nativeTab);
       tab.activeTabWindowID = tab.innerWindowID;
     }
   }
@@ -1942,6 +1991,26 @@ class TabManagerBase {
     return this.getWrapper(nativeTab).hasActiveTabPermission;
   }
 
+  /**
+   * Activate MV3 content scripts if the extension has activeTab or an
+   * (ungranted) host permission.
+   *
+   * @param {NativeTab} nativeTab
+   */
+  activateScripts(nativeTab) {
+    let tab = this.getWrapper(nativeTab);
+    if (
+      this.extension.originControls &&
+      !tab.matchesHostPermission &&
+      (this.extension.optionalOrigins.matches(tab._uri) ||
+        this.extension.hasPermission("activeTab")) &&
+      (this.extension.contentScripts.length ||
+        this.extension.registeredContentScripts.size)
+    ) {
+      tab.queryContent("ActivateScripts", { id: this.extension.id });
+    }
+  }
+
   /**
    * Returns true if the extension has permissions to access restricted
    * properties of the given native tab. In practice, this means that it has
@@ -1984,14 +2053,14 @@ class TabManagerBase {
   /**
    * Determines access using extension context.
    *
-   * @param {NativeTab} nativeTab
+   * @param {NativeTab} _nativeTab
    *        The tab to check access on.
    * @returns {boolean}
    *        True if the extension has permissions for this tab.
    * @protected
    * @abstract
    */
-  canAccessTab(nativeTab) {
+  canAccessTab(_nativeTab) {
     throw new Error("Not implemented");
   }
 
@@ -2002,11 +2071,11 @@ class TabManagerBase {
    *
    * @param {NativeTab} nativeTab
    *        The native tab to convert.
-   * @param {Object} [fallbackTabSize]
+   * @param {object} [fallbackTabSize]
    *        A geometry data if the lazy geometry data for this tab hasn't been
    *        initialized yet.
    *
-   * @returns {Object}
+   * @returns {object}
    */
   convert(nativeTab, fallbackTabSize = null) {
     return this.getWrapper(nativeTab).convert(fallbackTabSize);
@@ -2018,7 +2087,7 @@ class TabManagerBase {
   /**
    * Returns an iterator of TabBase objects which match the given query info.
    *
-   * @param {Object|null} [queryInfo = null]
+   * @param {object | null} [queryInfo = null]
    *        An object containing properties on which to filter. May contain any
    *        properties which are recognized by {@link TabBase#matches} or
    *        {@link WindowBase#matches}. Unknown properties will be ignored.
@@ -2036,6 +2105,10 @@ class TabManagerBase {
         });
       }
 
+      if (queryInfo.cookieStoreId !== null) {
+        queryInfo.cookieStoreId = [].concat(queryInfo.cookieStoreId);
+      }
+
       if (queryInfo.title !== null) {
         try {
           queryInfo.title = new MatchGlob(queryInfo.title);
@@ -2081,7 +2154,7 @@ class TabManagerBase {
   /**
    * Returns a TabBase wrapper for the tab with the given ID.
    *
-   * @param {integer} id
+   * @param {integer} _tabId
    *        The ID of the tab for which to return a wrapper.
    *
    * @returns {TabBase}
@@ -2089,22 +2162,21 @@ class TabManagerBase {
    *        If no tab exists with the given ID.
    * @abstract
    */
-  get(tabId) {
+  get(_tabId) {
     throw new Error("Not implemented");
   }
 
   /**
    * Returns a new TabBase instance wrapping the given native tab.
    *
-   * @param {NativeTab} nativeTab
+   * @param {NativeTab} _nativeTab
    *        The native tab for which to return a wrapper.
    *
    * @returns {TabBase}
    * @protected
    * @abstract
    */
-  /* eslint-enable valid-jsdoc */
-  wrapTab(nativeTab) {
+  wrapTab(_nativeTab) {
     throw new Error("Not implemented");
   }
 }
@@ -2132,7 +2204,7 @@ class WindowManagerBase {
    * @param {*} args
    *        Additional arguments to be passed to {@link WindowBase#convert}.
    *
-   * @returns {Object}
+   * @returns {object}
    */
   convert(window, ...args) {
     return this.getWrapper(window).convert(...args);
@@ -2178,7 +2250,7 @@ class WindowManagerBase {
   /**
    * Returns an iterator of WindowBase objects which match the given query info.
    *
-   * @param {Object|null} [queryInfo = null]
+   * @param {object | null} [queryInfo = null]
    *        An object containing properties on which to filter. May contain any
    *        properties which are recognized by {@link WindowBase#matches}.
    *        Unknown properties will be ignored.
@@ -2222,18 +2294,18 @@ class WindowManagerBase {
   /**
    * Returns a WindowBase wrapper for the browser window with the given ID.
    *
-   * @param {integer} id
+   * @param {integer} _windowId
    *        The ID of the browser window for which to return a wrapper.
-   * @param {BaseContext} context
+   * @param {BaseContext} _context
    *        The extension context for which the matching is being performed.
    *        Used to determine the current window for relevant properties.
    *
-   * @returns{WindowBase}
+   * @returns {WindowBase}
    * @throws {ExtensionError}
    *        If no window exists with the given ID.
    * @abstract
    */
-  get(windowId, context) {
+  get(_windowId, _context) {
     throw new Error("Not implemented");
   }
 
@@ -2251,14 +2323,14 @@ class WindowManagerBase {
   /**
    * Returns a new WindowBase instance wrapping the given browser window.
    *
-   * @param {DOMWindow} window
+   * @param {DOMWindow} _window
    *        The browser window for which to return a wrapper.
    *
    * @returns {WindowBase}
    * @protected
    * @abstract
    */
-  wrapWindow(window) {
+  wrapWindow(_window) {
     throw new Error("Not implemented");
   }
   /* eslint-enable valid-jsdoc */
