@@ -1,20 +1,30 @@
 "use strict";
 
-let { ExtensionTestCommon } = ChromeUtils.import(
-  "resource://testing-common/ExtensionTestCommon.jsm"
+let { ExtensionTestCommon } = ChromeUtils.importESModule(
+  "resource://testing-common/ExtensionTestCommon.sys.mjs"
 );
 
-let bundle;
-if (AppConstants.MOZ_APP_NAME == "thunderbird") {
-  bundle = Services.strings.createBundle(
-    "chrome://messenger/locale/addons.properties"
-  );
-} else {
-  bundle = Services.strings.createBundle(
-    "chrome://browser/locale/browser.properties"
-  );
-}
-const DUMMY_APP_NAME = "Dummy brandName";
+const {
+  PERMISSION_L10N,
+  PERMISSION_L10N_ID_OVERRIDES,
+  PERMISSIONS_WITH_MESSAGE,
+  permissionToL10nId,
+} = ChromeUtils.importESModule(
+  "resource://gre/modules/ExtensionPermissionMessages.sys.mjs"
+);
+
+const EXTENSION_L10N_PATHS = [
+  "toolkit/global/extensions.ftl",
+  "toolkit/global/extensionPermissions.ftl",
+  "branding/brand.ftl",
+];
+
+// For Android, these strings are only used in tests. In the actual UI, the
+// warnings are in Android-Components, as explained in bug 1671453.
+const l10n = new Localization(EXTENSION_L10N_PATHS, true);
+
+// nativeMessaging is in PRIVILEGED_PERMS on Android.
+const IS_NATIVE_MESSAGING_PRIVILEGED = AppConstants.platform == "android";
 
 const { createAppInfo } = AddonTestUtils;
 
@@ -23,32 +33,34 @@ AddonTestUtils.overrideCertDB();
 AddonTestUtils.usePrivilegedSignatures = id => id.startsWith("privileged");
 createAppInfo("xpcshell@tests.mozilla.org", "XPCShell", "1", "42");
 
+Services.prefs.setBoolPref("extensions.manifestV3.enabled", true);
+
 async function getManifestPermissions(extensionData) {
   let extension = ExtensionTestCommon.generate(extensionData);
   // Some tests contain invalid permissions; ignore the warnings about their invalidity.
   ExtensionTestUtils.failOnSchemaWarnings(false);
   await extension.loadManifest();
   ExtensionTestUtils.failOnSchemaWarnings(true);
-  const { manifestPermissions } = extension;
+  let result = extension.getRequiredPermissions();
+
+  if (extension.manifest.manifest_version >= 3) {
+    // In MV3, host permissions are optional by default.
+    deepEqual(result.origins, [], "No origins by default in MV3");
+    let optional = extension.manifestOptionalPermissions;
+    deepEqual(optional.permissions, [], "No tests use optional_permissions");
+    result.origins = optional.origins;
+  }
+
   await extension.cleanupGeneratedFile();
-  return manifestPermissions;
+  return result;
 }
 
-function getPermissionWarnings(
-  manifestPermissions,
-  options,
-  stringBundle = bundle
-) {
-  let info = {
-    permissions: manifestPermissions,
-    appName: DUMMY_APP_NAME,
-  };
-  let { msgs } = ExtensionData.formatPermissionStrings(
-    info,
-    stringBundle,
+function getPermissionWarnings(permissions, options) {
+  let { msgs, fullDomainsList } = ExtensionData.formatPermissionStrings(
+    { permissions },
     options
   );
-  return msgs;
+  return { warnings: msgs, fullDomainsList };
 }
 
 async function getPermissionWarningsForUpdate(
@@ -61,47 +73,90 @@ async function getPermissionWarningsForUpdate(
   return getPermissionWarnings(difference);
 }
 
-// Tests that the callers of ExtensionData.formatPermissionStrings can customize the
-// mapping between the permission names and related localized strings.
+// Tests that ExtensionData.formatPermissionStrings supports customized mappings
+// between permission names and related localized strings. Also test registering
+// additional fluent files so ExtensionData.formatPermissionStrings works for
+// permissions of APIs defined outside of toolkit.
 add_task(async function customized_permission_keys_mapping() {
-  const mockBundle = {
-    // Mocked nsIStringBundle getStringFromName to returns a fake localized string.
-    GetStringFromName: key => `Fake localized ${key}`,
-    formatStringFromName: (name, params) => "Fake formatted string",
-  };
+  // Mock a fluent file.
+  const l10nReg = L10nRegistry.getInstance();
+  const source = L10nFileSource.createMock(
+    "mock",
+    "app",
+    ["en-US"],
+    "/localization/",
+    [
+      {
+        path: "/localization/mock.ftl",
+        source: `
+webext-perms-description-test-downloads = Custom description for the downloads permission
 
-  // Define a non-default mapping for permission names -> locale keys.
-  const getKeyForPermission = perm => `customWebExtPerms.description.${perm}`;
+webext-perms-description-test-proxy = Custom description for the proxy permission
+`,
+      },
+    ]
+  );
+  l10nReg.registerSources([source]);
+
+  // Add the mocked fluent file to PERMISSION_L10N and override downloads and
+  // proxy permission to use the alternative string. In a real world use-case,
+  // this would be used to be able to change a localized string after release
+  // or add non-toolkit fluent files with permission strings of APIs defined
+  // outside of toolkit.
+  PERMISSION_L10N.addResourceIds(["mock.ftl"]);
+  PERMISSION_L10N_ID_OVERRIDES.set(
+    "downloads",
+    "webext-perms-description-test-downloads"
+  );
+  PERMISSION_L10N_ID_OVERRIDES.set(
+    "proxy",
+    "webext-perms-description-test-proxy"
+  );
+
+  let mockCleanup = () => {
+    // Make sure cleanup is executed only once.
+    mockCleanup = () => {};
+
+    // Remove the permission string mapping.
+    PERMISSION_L10N.removeResourceIds(["mock.ftl"]);
+    PERMISSION_L10N_ID_OVERRIDES.delete("downloads");
+    PERMISSION_L10N_ID_OVERRIDES.delete("proxy");
+    l10nReg.removeSources(["mock"]);
+  };
+  registerCleanupFunction(mockCleanup);
 
   const manifest = {
     permissions: ["downloads", "proxy"],
   };
-  const expectedWarnings = manifest.permissions.map(k =>
-    mockBundle.GetStringFromName(getKeyForPermission(k))
-  );
-  const manifestPermissions = await getManifestPermissions({ manifest });
 
-  // Pass the callback function for the non-default key mapping to
-  // ExtensionData.formatPermissionStrings() and verify it being used.
-  const warnings = getPermissionWarnings(
-    manifestPermissions,
-    { getKeyForPermission },
-    mockBundle
-  );
+  const manifestPermissions = await getManifestPermissions({ manifest });
+  let expectedWarnings = [
+    "Custom description for the downloads permission",
+    "Custom description for the proxy permission",
+  ];
+  const { warnings } = getPermissionWarnings(manifestPermissions);
   deepEqual(
     warnings,
     expectedWarnings,
     "Got the expected string from customized permission mapping"
   );
+
+  mockCleanup();
+});
+
+// Tests that permission description data is internally consistent
+add_task(async function permission_message_consistence() {
+  for (let perm of PERMISSIONS_WITH_MESSAGE) {
+    ok(permissionToL10nId(perm), `Message is provided for ${perm}`);
+  }
+  for (let [perm] of PERMISSION_L10N_ID_OVERRIDES) {
+    ok(permissionToL10nId(perm), `Message is provided for ${perm}`);
+  }
 });
 
 // Tests that the expected permission warnings are generated for various
 // combinations of host permissions.
 add_task(async function host_permissions() {
-  let { PluralForm } = ChromeUtils.import(
-    "resource://gre/modules/PluralForm.jsm"
-  );
-
   let permissionTestCases = [
     {
       description: "Empty manifest without permissions",
@@ -151,7 +206,7 @@ add_task(async function host_permissions() {
       },
       expectedOrigins: ["<all_urls>"],
       expectedWarnings: [
-        bundle.GetStringFromName("webextPerms.hostDescription.allUrls"),
+        l10n.formatValueSync("webext-perms-host-description-all-urls"),
       ],
     },
     {
@@ -161,7 +216,7 @@ add_task(async function host_permissions() {
       },
       expectedOrigins: ["file://*/"],
       expectedWarnings: [
-        bundle.GetStringFromName("webextPerms.hostDescription.allUrls"),
+        l10n.formatValueSync("webext-perms-host-description-all-urls"),
       ],
     },
     {
@@ -171,7 +226,7 @@ add_task(async function host_permissions() {
       },
       expectedOrigins: ["http://*/"],
       expectedWarnings: [
-        bundle.GetStringFromName("webextPerms.hostDescription.allUrls"),
+        l10n.formatValueSync("webext-perms-host-description-all-urls"),
       ],
     },
     {
@@ -181,7 +236,7 @@ add_task(async function host_permissions() {
       },
       expectedOrigins: ["*://*/"],
       expectedWarnings: [
-        bundle.GetStringFromName("webextPerms.hostDescription.allUrls"),
+        l10n.formatValueSync("webext-perms-host-description-all-urls"),
       ],
     },
     {
@@ -198,7 +253,7 @@ add_task(async function host_permissions() {
       },
       expectedOrigins: ["https://*/"],
       expectedWarnings: [
-        bundle.GetStringFromName("webextPerms.hostDescription.allUrls"),
+        l10n.formatValueSync("webext-perms-host-description-all-urls"),
       ],
     },
     {
@@ -207,21 +262,27 @@ add_task(async function host_permissions() {
         permissions: ["http://a/", "http://*.b/", "http://c/*"],
       },
       expectedOrigins: ["http://a/", "http://*.b/", "http://c/*"],
-      expectedWarnings: [
+      expectedWarnings: l10n.formatValuesSync([
         // Wildcard hosts take precedence in the permission list.
-        bundle.formatStringFromName("webextPerms.hostDescription.wildcard", [
-          "b",
-        ]),
-        bundle.formatStringFromName("webextPerms.hostDescription.oneSite", [
-          "a",
-        ]),
-        bundle.formatStringFromName("webextPerms.hostDescription.oneSite", [
-          "c",
-        ]),
-      ],
+        {
+          id: "webext-perms-host-description-wildcard",
+          args: { domain: "b" },
+        },
+        {
+          id: "webext-perms-host-description-one-site",
+          args: { domain: "a" },
+        },
+        {
+          id: "webext-perms-host-description-one-site",
+          args: { domain: "c" },
+        },
+      ]),
     },
     {
-      description: "many host permission",
+      // This test covers the expected strings for many host permissions
+      // as shown in permissions dialog (install, update or optional
+      // permission requests).
+      description: "many host permission (as shown in install dialog)",
       manifest: {
         permissions: [
           "http://a/",
@@ -246,39 +307,18 @@ add_task(async function host_permissions() {
         "http://*.3/",
         "http://*.4/",
       ],
-      expectedWarnings: [
-        // Wildcard hosts take precedence in the permission list.
-        bundle.formatStringFromName("webextPerms.hostDescription.wildcard", [
-          "1",
-        ]),
-        bundle.formatStringFromName("webextPerms.hostDescription.wildcard", [
-          "2",
-        ]),
-        bundle.formatStringFromName("webextPerms.hostDescription.wildcard", [
-          "3",
-        ]),
-        bundle.formatStringFromName("webextPerms.hostDescription.wildcard", [
-          "4",
-        ]),
-        bundle.formatStringFromName("webextPerms.hostDescription.oneSite", [
-          "a",
-        ]),
-        bundle.formatStringFromName("webextPerms.hostDescription.oneSite", [
-          "b",
-        ]),
-        bundle.formatStringFromName("webextPerms.hostDescription.oneSite", [
-          "c",
-        ]),
-        PluralForm.get(
-          2,
-          bundle.GetStringFromName("webextPerms.hostDescription.tooManySites")
-        ).replace("#1", "2"),
-      ],
-      options: {
-        collapseOrigins: true,
-      },
+      expectedWarnings: l10n.formatValuesSync([
+        {
+          id: "webext-perms-host-description-multiple-domains",
+          args: { domainCount: 9 },
+        },
+      ]),
+      expectedDomainsList: ["1", "2", "3", "4", "a", "b", "c", "d", "e"],
+      options: { fullDomainsList: true },
     },
     {
+      // This test covers the expected strings for many host permissions
+      // as shown in about:addons addon card permissions view.
       description:
         "many host permissions without item limit in the warning list",
       manifest: {
@@ -307,64 +347,78 @@ add_task(async function host_permissions() {
         "http://*.4/",
         "http://*.5/",
       ],
-      expectedWarnings: [
-        bundle.formatStringFromName("webextPerms.hostDescription.wildcard", [
-          "1",
-        ]),
-        bundle.formatStringFromName("webextPerms.hostDescription.wildcard", [
-          "2",
-        ]),
-        bundle.formatStringFromName("webextPerms.hostDescription.wildcard", [
-          "3",
-        ]),
-        bundle.formatStringFromName("webextPerms.hostDescription.wildcard", [
-          "4",
-        ]),
-        bundle.formatStringFromName("webextPerms.hostDescription.wildcard", [
-          "5",
-        ]),
-        bundle.formatStringFromName("webextPerms.hostDescription.oneSite", [
-          "a",
-        ]),
-        bundle.formatStringFromName("webextPerms.hostDescription.oneSite", [
-          "b",
-        ]),
-        bundle.formatStringFromName("webextPerms.hostDescription.oneSite", [
-          "c",
-        ]),
-        bundle.formatStringFromName("webextPerms.hostDescription.oneSite", [
-          "d",
-        ]),
-        bundle.formatStringFromName("webextPerms.hostDescription.oneSite", [
-          "e",
-        ]),
-      ],
+      expectedWarnings: l10n.formatValuesSync([
+        { id: "webext-perms-host-description-wildcard", args: { domain: "1" } },
+        { id: "webext-perms-host-description-wildcard", args: { domain: "2" } },
+        { id: "webext-perms-host-description-wildcard", args: { domain: "3" } },
+        { id: "webext-perms-host-description-wildcard", args: { domain: "4" } },
+        { id: "webext-perms-host-description-wildcard", args: { domain: "5" } },
+        { id: "webext-perms-host-description-one-site", args: { domain: "a" } },
+        { id: "webext-perms-host-description-one-site", args: { domain: "b" } },
+        { id: "webext-perms-host-description-one-site", args: { domain: "c" } },
+        { id: "webext-perms-host-description-one-site", args: { domain: "d" } },
+        { id: "webext-perms-host-description-one-site", args: { domain: "e" } },
+      ]),
     },
   ];
-  for (let {
-    description,
-    manifest,
-    expectedOrigins,
-    expectedWarnings,
-    options,
-  } of permissionTestCases) {
-    let manifestPermissions = await getManifestPermissions({
+  for (let manifest_version of [2, 3]) {
+    for (let {
+      description,
       manifest,
-    });
-
-    deepEqual(
-      manifestPermissions.origins,
       expectedOrigins,
-      `Expected origins (${description})`
-    );
-    deepEqual(
-      manifestPermissions.permissions,
-      [],
-      `Expected no non-host permissions (${description})`
-    );
+      expectedWarnings,
+      expectedDomainsList,
+      options,
+    } of permissionTestCases) {
+      manifest = Object.assign({}, manifest, { manifest_version });
+      if (manifest_version > 2) {
+        manifest.host_permissions = manifest.permissions;
+        manifest.permissions = [];
+      }
 
-    let warnings = getPermissionWarnings(manifestPermissions, options);
-    deepEqual(warnings, expectedWarnings, `Expected warnings (${description})`);
+      let manifestPermissions = await getManifestPermissions({ manifest });
+
+      deepEqual(
+        manifestPermissions.origins,
+        expectedOrigins,
+        `Expected origins (${description})`
+      );
+      deepEqual(
+        manifestPermissions.permissions,
+        [],
+        `Expected no non-host permissions (${description})`
+      );
+
+      let { warnings, fullDomainsList } = getPermissionWarnings(
+        manifestPermissions,
+        options
+      );
+      deepEqual(
+        warnings,
+        expectedWarnings,
+        `Expected warnings (${description})`
+      );
+      if (fullDomainsList) {
+        deepEqual(
+          Array.from(fullDomainsList.domainsSet),
+          expectedDomainsList,
+          `Expected full domains list (${description})`
+        );
+        // Verify also that the warning message at the fullDomainsList.msgIdIndex
+        // position is the one expected to be associated to the list of domains
+        // in fullDomainsList.domainsSet.
+        equal(
+          warnings[fullDomainsList.msgIdIndex],
+          l10n.formatValueSync(
+            "webext-perms-host-description-multiple-domains",
+            {
+              domainCount: expectedDomainsList.length,
+            }
+          ),
+          "Got warning associated to the full domains list set to the expected fluent string."
+        );
+      }
+    }
   }
 });
 
@@ -372,6 +426,7 @@ add_task(async function host_permissions() {
 // permissions and API permissions.
 add_task(async function api_permissions() {
   let manifestPermissions = await getManifestPermissions({
+    isPrivileged: IS_NATIVE_MESSAGING_PRIVILEGED,
     manifest: {
       permissions: [
         "activeTab",
@@ -384,6 +439,7 @@ add_task(async function api_permissions() {
       ],
     },
   });
+
   deepEqual(
     manifestPermissions,
     {
@@ -394,29 +450,103 @@ add_task(async function api_permissions() {
   );
 
   deepEqual(
-    getPermissionWarnings(manifestPermissions),
-    [
+    getPermissionWarnings(manifestPermissions).warnings,
+    l10n.formatValuesSync([
       // Host permissions first, with wildcards on top.
-      bundle.formatStringFromName("webextPerms.hostDescription.wildcard", [
-        "x",
-      ]),
-      bundle.formatStringFromName("webextPerms.hostDescription.wildcard", [
-        "tld",
-      ]),
-      bundle.formatStringFromName("webextPerms.hostDescription.oneSite", ["x"]),
+      { id: "webext-perms-host-description-wildcard", args: { domain: "x" } },
+      { id: "webext-perms-host-description-wildcard", args: { domain: "tld" } },
+      { id: "webext-perms-host-description-one-site", args: { domain: "x" } },
       // nativeMessaging permission warning first of all permissions.
-      bundle.formatStringFromName("webextPerms.description.nativeMessaging", [
-        DUMMY_APP_NAME,
-      ]),
+      "webext-perms-description-nativeMessaging",
       // Other permissions in alphabetical order.
       // Note: activeTab has no permission warning string.
-      bundle.GetStringFromName("webextPerms.description.tabs"),
-      bundle.GetStringFromName("webextPerms.description.webNavigation"),
-    ],
+      "webext-perms-description-tabs",
+      "webext-perms-description-webNavigation",
+    ]),
     "Expected warnings"
   );
 });
 
+add_task(async function nativeMessaging_permission() {
+  let manifestPermissions = await getManifestPermissions({
+    // isPrivileged: false, by default.
+    manifest: {
+      permissions: ["nativeMessaging"],
+    },
+  });
+
+  if (IS_NATIVE_MESSAGING_PRIVILEGED) {
+    // The behavior of nativeMessaging for unprivileged extensions on Android
+    // is covered in
+    // mobile/shared/components/extensions/test/xpcshell/test_ext_native_messaging_permissions.js
+    deepEqual(
+      manifestPermissions,
+      { origins: [], permissions: [] },
+      "nativeMessaging perm ignored for unprivileged extensions on Android"
+    );
+  } else {
+    deepEqual(
+      manifestPermissions,
+      { origins: [], permissions: ["nativeMessaging"] },
+      "nativeMessaging permission recognized for unprivileged extensions"
+    );
+  }
+});
+
+add_task(
+  { pref_set: [["extensions.dnr.enabled", true]] },
+  async function declarativeNetRequest_permission_with_warning() {
+    let manifestPermissions = await getManifestPermissions({
+      manifest: {
+        manifest_version: 3,
+        permissions: ["declarativeNetRequest", "declarativeNetRequestFeedback"],
+      },
+    });
+
+    deepEqual(
+      manifestPermissions,
+      {
+        origins: [],
+        permissions: ["declarativeNetRequest", "declarativeNetRequestFeedback"],
+      },
+      "Expected origins and permissions"
+    );
+
+    deepEqual(
+      getPermissionWarnings(manifestPermissions).warnings,
+      l10n.formatValuesSync([
+        "webext-perms-description-declarativeNetRequest",
+        "webext-perms-description-declarativeNetRequestFeedback",
+      ]),
+      "Expected warnings"
+    );
+  }
+);
+
+add_task(
+  { pref_set: [["extensions.dnr.enabled", true]] },
+  async function declarativeNetRequest_permission_without_warning() {
+    let manifestPermissions = await getManifestPermissions({
+      manifest: {
+        manifest_version: 3,
+        permissions: ["declarativeNetRequestWithHostAccess"],
+      },
+    });
+
+    deepEqual(
+      manifestPermissions,
+      { origins: [], permissions: ["declarativeNetRequestWithHostAccess"] },
+      "Expected origins and permissions"
+    );
+
+    deepEqual(
+      getPermissionWarnings(manifestPermissions).warnings,
+      [],
+      "No warnings"
+    );
+  }
+);
+
 // Tests that the expected permission warnings are generated for a mix of host
 // permissions and API permissions, for a privileged extension that uses the
 // mozillaAddons permission.
@@ -444,8 +574,8 @@ add_task(async function privileged_with_mozillaAddons() {
   );
 
   deepEqual(
-    getPermissionWarnings(manifestPermissions),
-    [bundle.GetStringFromName("webextPerms.hostDescription.allUrls")],
+    getPermissionWarnings(manifestPermissions).warnings,
+    [l10n.formatValueSync("webext-perms-host-description-all-urls")],
     "Expected warnings for privileged add-on with mozillaAddons permission."
   );
 });
@@ -475,15 +605,19 @@ add_task(async function unprivileged_with_mozillaAddons() {
   );
 
   deepEqual(
-    getPermissionWarnings(manifestPermissions),
-    [bundle.formatStringFromName("webextPerms.hostDescription.oneSite", ["a"])],
+    getPermissionWarnings(manifestPermissions).warnings,
+    [
+      l10n.formatValueSync("webext-perms-host-description-one-site", {
+        domain: "a",
+      }),
+    ],
     "Expected warnings for unprivileged add-on with mozillaAddons permission."
   );
 });
 
 // Tests that an update with less permissions has no warning.
 add_task(async function update_drop_permission() {
-  let warnings = await getPermissionWarningsForUpdate(
+  let { warnings } = await getPermissionWarningsForUpdate(
     {
       manifest: {
         permissions: ["<all_urls>", "https://a/", "http://b/"],
@@ -509,7 +643,7 @@ add_task(async function update_drop_permission() {
 // Tests that an update that switches from "*://*/*" to "<all_urls>" does not
 // result in additional permission warnings.
 add_task(async function update_all_urls_permission() {
-  let warnings = await getPermissionWarningsForUpdate(
+  let { warnings } = await getPermissionWarningsForUpdate(
     {
       manifest: {
         permissions: ["*://*/*"],
@@ -531,7 +665,7 @@ add_task(async function update_all_urls_permission() {
 // Tests that an update where a new permission whose domain overlaps with
 // an existing permission does not result in additional permission warnings.
 add_task(async function update_change_permissions() {
-  let warnings = await getPermissionWarningsForUpdate(
+  let { warnings } = await getPermissionWarningsForUpdate(
     {
       manifest: {
         permissions: ["https://a/", "http://*.b/", "http://c/", "http://f/"],
@@ -563,14 +697,10 @@ add_task(async function update_change_permissions() {
   );
   deepEqual(
     warnings,
-    [
-      bundle.formatStringFromName("webextPerms.hostDescription.wildcard", [
-        "c",
-      ]),
-      bundle.formatStringFromName("webextPerms.description.proxy", [
-        DUMMY_APP_NAME,
-      ]),
-    ],
+    l10n.formatValuesSync([
+      { id: "webext-perms-host-description-wildcard", args: { domain: "c" } },
+      "webext-perms-description-proxy",
+    ]),
     "Expected permission warnings for new permissions only"
   );
 });
@@ -578,7 +708,7 @@ add_task(async function update_change_permissions() {
 // Tests that a privileged extension with the mozillaAddons permission can be
 // updated without errors.
 add_task(async function update_privileged_with_mozillaAddons() {
-  let warnings = await getPermissionWarningsForUpdate(
+  let { warnings } = await getPermissionWarningsForUpdate(
     {
       isPrivileged: true,
       manifest: {
@@ -594,7 +724,11 @@ add_task(async function update_privileged_with_mozillaAddons() {
   );
   deepEqual(
     warnings,
-    [bundle.formatStringFromName("webextPerms.hostDescription.oneSite", ["b"])],
+    [
+      l10n.formatValueSync("webext-perms-host-description-one-site", {
+        domain: "b",
+      }),
+    ],
     "Expected permission warnings for new host only"
   );
 });
@@ -603,7 +737,7 @@ add_task(async function update_privileged_with_mozillaAddons() {
 // through an update.
 add_task(async function update_unprivileged_with_mozillaAddons() {
   // Unprivileged
-  let warnings = await getPermissionWarningsForUpdate(
+  let { warnings } = await getPermissionWarningsForUpdate(
     {
       manifest: {
         permissions: ["mozillaAddons", "resource://a/"],
@@ -623,27 +757,29 @@ add_task(async function update_unprivileged_with_mozillaAddons() {
 });
 
 // Tests that invalid permission warning for privileged permissions requested
-// without the privilged signature are emitted by the Extension class instance
-// but not for the ExtensionData instances (on which the signature is not
-// available and the warning would be emitted even for the ones signed correctly).
+// are not emitted for privileged extensions, only for unprivileged extensions.
 add_task(
   async function test_invalid_permission_warning_on_privileged_permission() {
     await AddonTestUtils.promiseStartupManager();
 
+    const MANIFEST_WARNINGS = [
+      "Reading manifest: Invalid extension permission: mozillaAddons",
+      "Reading manifest: Invalid extension permission: resource://x/",
+      "Reading manifest: Invalid extension permission: about:reader*",
+    ];
+
     async function testInvalidPermissionWarning({ isPrivileged }) {
       let id = isPrivileged
         ? "privileged-addon@mochi.test"
         : "nonprivileged-addon@mochi.test";
 
-      let expectedWarnings = isPrivileged
-        ? []
-        : ["Reading manifest: Invalid extension permission: mozillaAddons"];
+      let expectedWarnings = isPrivileged ? [] : MANIFEST_WARNINGS;
 
       const ext = ExtensionTestUtils.loadExtension({
         useAddonManager: "permanent",
         manifest: {
-          permissions: ["mozillaAddons"],
-          applications: { gecko: { id } },
+          permissions: ["mozillaAddons", "resource://x/", "about:reader*"],
+          browser_specific_settings: { gecko: { id } },
         },
         background() {},
       });
@@ -666,28 +802,69 @@ add_task(
     // ExtensionData instance created below).
     let generatedExt = ExtensionTestCommon.generate({
       manifest: {
-        permissions: ["mozillaAddons"],
-        applications: { gecko: { id: "extension-data@mochi.test" } },
+        permissions: ["mozillaAddons", "resource://x/", "about:reader*"],
+        browser_specific_settings: {
+          gecko: { id: "extension-data@mochi.test" },
+        },
       },
     });
 
-    // Verify that XPIInstall.jsm will not collect the warning for the
+    // Verify that XPIInstall.sys.mjs will not collect the warning for the
     // privileged permission as expected.
-    const extData = new ExtensionData(generatedExt.rootURI);
-    await extData.loadManifest();
+    async function getWarningsFromExtensionData({ isPrivileged }) {
+      let extData;
+      if (typeof isPrivileged == "function") {
+        // isPrivileged expected to be computed asynchronously.
+        extData = await ExtensionData.constructAsync({
+          rootURI: generatedExt.rootURI,
+          checkPrivileged: isPrivileged,
+        });
+      } else {
+        extData = new ExtensionData(generatedExt.rootURI, isPrivileged);
+      }
+      await extData.loadManifest();
+
+      // This assertion is just meant to prevent the test to pass if there were
+      // no warnings because some errors prevented the warnings to be
+      // collected).
+      Assert.deepEqual(
+        extData.errors,
+        [],
+        "No errors collected by the ExtensionData instance"
+      );
+      return extData.warnings;
+    }
+
     Assert.deepEqual(
-      extData.warnings,
-      [],
-      "No warnings for mozillaAddons permission collected for the ExtensionData instance"
+      await getWarningsFromExtensionData({ isPrivileged: undefined }),
+      MANIFEST_WARNINGS,
+      "Got warnings about privileged permissions by default"
     );
 
-    // This assertion is just meant to prevent the test to pass if there were no warnings
-    // because some errors prevented the warnings to be collected).
     Assert.deepEqual(
-      extData.errors,
-      [],
-      "No errors collected by the ExtensionData instance"
+      await getWarningsFromExtensionData({ isPrivileged: false }),
+      MANIFEST_WARNINGS,
+      "Got warnings about privileged permissions for non-privileged extensions"
     );
+
+    Assert.deepEqual(
+      await getWarningsFromExtensionData({ isPrivileged: true }),
+      [],
+      "No warnings about privileged permissions on privileged extensions"
+    );
+
+    Assert.deepEqual(
+      await getWarningsFromExtensionData({ isPrivileged: async () => false }),
+      MANIFEST_WARNINGS,
+      "Got warnings about privileged permissions for non-privileged extensions (async)"
+    );
+
+    Assert.deepEqual(
+      await getWarningsFromExtensionData({ isPrivileged: async () => true }),
+      [],
+      "No warnings about privileged permissions on privileged extensions (async)"
+    );
+
     // Cleanup the generated xpi file.
     await generatedExt.cleanupGeneratedFile();
 
