@@ -19,13 +19,9 @@ add_task(async function setup() {
 });
 
 add_task(async function test_contentscript_storage_sync() {
-  return runWithPrefs([[STORAGE_SYNC_PREF, true]], () =>
-    test_contentscript_storage("sync")
-  );
+  await test_contentscript_storage("sync");
 });
 
 add_task(async function test_contentscript_bytes_in_use_sync() {
-  return runWithPrefs([[STORAGE_SYNC_PREF, true]], () =>
-    test_contentscript_storage_area_with_bytes_in_use("sync", true)
-  );
+  await test_contentscript_storage_area_with_bytes_in_use("sync", true);
 });
