@@ -1,33 +1,40 @@
 "use strict";
 
-const { AsyncShutdown } = ChromeUtils.import(
-  "resource://gre/modules/AsyncShutdown.jsm"
+const { AsyncShutdown } = ChromeUtils.importESModule(
+  "resource://gre/modules/AsyncShutdown.sys.mjs"
 );
-const { NativeManifests } = ChromeUtils.import(
-  "resource://gre/modules/NativeManifests.jsm"
+const { NativeManifests } = ChromeUtils.importESModule(
+  "resource://gre/modules/NativeManifests.sys.mjs"
 );
-const { FileUtils } = ChromeUtils.import(
-  "resource://gre/modules/FileUtils.jsm"
+const { FileUtils } = ChromeUtils.importESModule(
+  "resource://gre/modules/FileUtils.sys.mjs"
 );
-const { Schemas } = ChromeUtils.import("resource://gre/modules/Schemas.jsm");
-const { Subprocess, SubprocessImpl } = ChromeUtils.import(
-  "resource://gre/modules/Subprocess.jsm",
-  null
+const { Schemas } = ChromeUtils.importESModule(
+  "resource://gre/modules/Schemas.sys.mjs"
 );
-const { NativeApp } = ChromeUtils.import(
-  "resource://gre/modules/NativeMessaging.jsm"
+const { Subprocess } = ChromeUtils.importESModule(
+  "resource://gre/modules/Subprocess.sys.mjs"
+);
+const { NativeApp } = ChromeUtils.importESModule(
+  "resource://gre/modules/NativeMessaging.sys.mjs"
 );
-const { OS } = ChromeUtils.import("resource://gre/modules/osfile.jsm");
 
 let registry = null;
 if (AppConstants.platform == "win") {
-  var { MockRegistry } = ChromeUtils.import(
-    "resource://testing-common/MockRegistry.jsm"
+  var { MockRegistry } = ChromeUtils.importESModule(
+    "resource://testing-common/MockRegistry.sys.mjs"
   );
   registry = new MockRegistry();
   registerCleanupFunction(() => {
     registry.shutdown();
   });
+  ChromeUtils.defineESModuleGetters(this, {
+    SubprocessImpl: "resource://gre/modules/subprocess/subprocess_win.sys.mjs",
+  });
+} else {
+  ChromeUtils.defineESModuleGetters(this, {
+    SubprocessImpl: "resource://gre/modules/subprocess/subprocess_unix.sys.mjs",
+  });
 }
 
 const REGPATH = "Software\\Mozilla\\NativeMessagingHosts";
@@ -50,8 +57,10 @@ let globalDir = dir.clone();
 globalDir.append("global");
 globalDir.create(Ci.nsIFile.DIRECTORY_TYPE, FileUtils.PERMS_DIRECTORY);
 
-OS.File.makeDir(OS.Path.join(userDir.path, TYPE_SLUG));
-OS.File.makeDir(OS.Path.join(globalDir.path, TYPE_SLUG));
+add_setup(async function setup() {
+  await IOUtils.makeDirectory(PathUtils.join(userDir.path, TYPE_SLUG));
+  await IOUtils.makeDirectory(PathUtils.join(globalDir.path, TYPE_SLUG));
+});
 
 let dirProvider = {
   getFile(property) {
@@ -75,18 +84,15 @@ function writeManifest(path, manifest) {
   if (typeof manifest != "string") {
     manifest = JSON.stringify(manifest);
   }
-  return OS.File.writeAtomic(path, manifest);
+  return IOUtils.writeUTF8(path, manifest);
 }
 
 let PYTHON;
 add_task(async function setup() {
   await Schemas.load(BASE_SCHEMA);
 
-  const env = Cc["@mozilla.org/process/environment;1"].getService(
-    Ci.nsIEnvironment
-  );
   try {
-    PYTHON = await Subprocess.pathSearch(env.get("PYTHON"));
+    PYTHON = await Subprocess.pathSearch(Services.env.get("PYTHON"));
   } catch (e) {
     notEqual(
       PYTHON,
@@ -144,6 +150,100 @@ function lookupApplication(app, ctx) {
   return NativeManifests.lookupManifest("stdio", app, ctx);
 }
 
+add_task(async function test_parse_good_manifest() {
+  let manifest = await NativeManifests.parseManifest(
+    "stdio",
+    "/some/path",
+    "test",
+    context,
+    templateManifest
+  );
+  deepEqual(
+    manifest,
+    templateManifest,
+    "parseManifest returns the manifest contents"
+  );
+});
+
+add_task(async function test_parse_invalid_manifest() {
+  function matchLastConsoleMessage(regex) {
+    ok(Services.console.getMessageArray().pop().message.match(regex));
+  }
+
+  equal(
+    null,
+    await NativeManifests.parseManifest(
+      "pkcs11",
+      "/some/path",
+      "test",
+      context,
+      templateManifest
+    )
+  );
+  matchLastConsoleMessage(
+    /Native manifest \/some\/path has type property stdio \(expected pkcs11\)/
+  );
+
+  equal(
+    null,
+    await NativeManifests.parseManifest(
+      "stdio",
+      "/some/path",
+      "foobar",
+      context,
+      templateManifest
+    )
+  );
+  matchLastConsoleMessage(
+    /Native manifest \/some\/path has name property test \(expected foobar\)/
+  );
+
+  const incompleteManifest = { ...templateManifest };
+  delete incompleteManifest.description;
+  equal(
+    null,
+    await NativeManifests.parseManifest(
+      "stdio",
+      "/some/path",
+      "test",
+      context,
+      incompleteManifest
+    )
+  );
+  matchLastConsoleMessage(/Value must either: match the pattern/);
+
+  const unauthorizedManifest = { ...templateManifest };
+  unauthorizedManifest.allowed_extensions = [];
+  equal(
+    null,
+    await NativeManifests.parseManifest(
+      "stdio",
+      "/some/path",
+      "test",
+      context,
+      unauthorizedManifest
+    )
+  );
+  matchLastConsoleMessage(
+    /Value must either: .allowed_extensions must have at least 1 items/
+  );
+
+  unauthorizedManifest.allowed_extensions = ["unauthorized@tests.mozilla.org"];
+  equal(
+    null,
+    await NativeManifests.parseManifest(
+      "stdio",
+      "/some/path",
+      "test",
+      context,
+      unauthorizedManifest
+    )
+  );
+  matchLastConsoleMessage(
+    /This extension does not have permission to use native manifest \/some\/path/
+  );
+});
+
 add_task(async function test_nonexistent_manifest() {
   let result = await lookupApplication("test", context);
   equal(
@@ -153,7 +253,7 @@ add_task(async function test_nonexistent_manifest() {
   );
 });
 
-const USER_TEST_JSON = OS.Path.join(userDir.path, TYPE_SLUG, "test.json");
+const USER_TEST_JSON = PathUtils.join(userDir.path, TYPE_SLUG, "test.json");
 
 add_task(async function test_nonexistent_manifest_with_registry_entry() {
   if (registry) {
@@ -165,7 +265,7 @@ add_task(async function test_nonexistent_manifest_with_registry_entry() {
     );
   }
 
-  await OS.File.remove(USER_TEST_JSON);
+  await IOUtils.remove(USER_TEST_JSON);
   let { messages, result } = await promiseConsoleOutput(() =>
     lookupApplication("test", context)
   );
@@ -221,10 +321,106 @@ add_task(async function test_good_manifest() {
   );
 });
 
+add_task(
+  { skip_if: () => AppConstants.platform != "win" },
+  async function test_forward_slashes_instead_of_backslashes_in_registry() {
+    Assert.ok(USER_TEST_JSON.includes("\\"), `Path has \\: ${USER_TEST_JSON}`);
+    const manifest = { ...templateManifest, name: "testslash" };
+    await writeManifest(USER_TEST_JSON, manifest);
+    registry.setValue(
+      Ci.nsIWindowsRegKey.ROOT_KEY_CURRENT_USER,
+      `${REGPATH}\\testslash`,
+      "",
+      USER_TEST_JSON.replaceAll("\\", "/")
+    );
+
+    let result = await lookupApplication("testslash", context);
+    notEqual(result, null, "lookupApplication finds the manifest despite /");
+    equal(
+      result.path,
+      USER_TEST_JSON,
+      "lookupApplication returns the correct path with platform-native slash"
+    );
+    // Side note: manifest.path does not contain a platform-native path,
+    // but it is normalized when used in NativeMessaging.sys.mjs.
+    deepEqual(
+      result.manifest,
+      manifest,
+      "lookupApplication returns the manifest contents"
+    );
+  }
+);
+
+add_task(async function test_manifest_with_utf8_byte_order_mark() {
+  const manifest = { ...templateManifest, description: "had BOM at start" };
+  const manifestString = JSON.stringify(manifest);
+
+  // "123" to have a placeholder where we'll fill in the 3 BOM bytes.
+  const manifestBytes = new TextEncoder().encode("123" + manifestString);
+  manifestBytes.set([0xef, 0xbb, 0xbf]);
+
+  // Sanity check: verify that the bytes prepended above have the special
+  // meaning of being a UTF-8 BOM. That is, when parsed as UTF-8, the bytes can
+  // be removed without loss of meaning.
+  equal(
+    new TextDecoder().decode(manifestBytes),
+    manifestString,
+    "Sanity check: input bytes has UTF-8 BOM that is ordinarily stripped"
+  );
+
+  await IOUtils.write(USER_TEST_JSON, manifestBytes);
+  if (registry) {
+    registry.setValue(
+      Ci.nsIWindowsRegKey.ROOT_KEY_CURRENT_USER,
+      `${REGPATH}\\test`,
+      "",
+      USER_TEST_JSON
+    );
+  }
+  let result = await lookupApplication("test", context);
+  notEqual(result, null, "lookupApplication finds a good manifest despite BOM");
+  deepEqual(
+    result.manifest,
+    manifest,
+    "lookupApplication returns the manifest contents"
+  );
+});
+
+add_task(async function test_manifest_with_invalid_utf_8() {
+  const manifest = { ...templateManifest, description: "bad bytes" };
+  const manifestString = JSON.stringify(manifest);
+  const manifestBytes = Uint8Array.from(manifestString, c => c.charCodeAt(0));
+  // manifestString ends with `bad bytes"}`. Replace the `s` with a bad byte:
+  manifestBytes.set([0xff], manifestBytes.byteLength - 3);
+
+  await IOUtils.write(USER_TEST_JSON, manifestBytes);
+  if (registry) {
+    registry.setValue(
+      Ci.nsIWindowsRegKey.ROOT_KEY_CURRENT_USER,
+      `${REGPATH}\\test`,
+      "",
+      USER_TEST_JSON
+    );
+  }
+  let { messages, result } = await promiseConsoleOutput(() =>
+    lookupApplication("test", context)
+  );
+  equal(result, null, "lookupApplication should reject file with invalid UTF8");
+  let errorPattern =
+    /NotReadableError: Could not read `.*': file is not UTF-8 encoded/;
+  let utf8Errors = messages.filter(({ message }) => errorPattern.test(message));
+  equal(utf8Errors.length, 1, "lookupApplication logs error about UTF-8");
+});
+
 add_task(async function test_invalid_json() {
   await writeManifest(USER_TEST_JSON, "this is not valid json");
-  let result = await lookupApplication("test", context);
+  let { messages, result } = await promiseConsoleOutput(() =>
+    lookupApplication("test", context)
+  );
   equal(result, null, "lookupApplication ignores bad json");
+  let errorPattern = /Error parsing native manifest .*test.json: JSON\.parse:/;
+  let jsonErrors = messages.filter(({ message }) => errorPattern.test(message));
+  equal(jsonErrors.length, 1, "lookupApplication logs JSON error");
 });
 
 add_task(async function test_invalid_name() {
@@ -280,12 +476,12 @@ add_task(async function test_no_allowed_extensions() {
   );
 });
 
-const GLOBAL_TEST_JSON = OS.Path.join(globalDir.path, TYPE_SLUG, "test.json");
+const GLOBAL_TEST_JSON = PathUtils.join(globalDir.path, TYPE_SLUG, "test.json");
 let globalManifest = Object.assign({}, templateManifest);
 globalManifest.description = "This manifest is from the systemwide directory";
 
 add_task(async function good_manifest_system_dir() {
-  await OS.File.remove(USER_TEST_JSON);
+  await IOUtils.remove(USER_TEST_JSON);
   await writeManifest(GLOBAL_TEST_JSON, globalManifest);
   if (registry) {
     registry.setValue(
@@ -376,8 +572,8 @@ while True:
     stdout.write(msg)
 `;
 
-  let scriptPath = OS.Path.join(userDir.path, TYPE_SLUG, "wontdie.py");
-  let manifestPath = OS.Path.join(userDir.path, TYPE_SLUG, "wontdie.json");
+  let scriptPath = PathUtils.join(userDir.path, TYPE_SLUG, "wontdie.py");
+  let manifestPath = PathUtils.join(userDir.path, TYPE_SLUG, "wontdie.json");
 
   const ID = "native@tests.mozilla.org";
   let manifest = {
@@ -388,12 +584,12 @@ while True:
   };
 
   if (AppConstants.platform == "win") {
-    await OS.File.writeAtomic(scriptPath, SCRIPT);
+    await IOUtils.writeUTF8(scriptPath, SCRIPT);
 
-    let batPath = OS.Path.join(userDir.path, TYPE_SLUG, "wontdie.bat");
+    let batPath = PathUtils.join(userDir.path, TYPE_SLUG, "wontdie.bat");
     let batBody = `@ECHO OFF\n${PYTHON} -u "${scriptPath}" %*\n`;
-    await OS.File.writeAtomic(batPath, batBody);
-    await OS.File.setPermissions(batPath, { unixMode: 0o755 });
+    await IOUtils.writeUTF8(batPath, batBody);
+    await IOUtils.setPermissions(batPath, 0o755);
 
     manifest.path = batPath;
     await writeManifest(manifestPath, manifest);
@@ -405,8 +601,8 @@ while True:
       manifestPath
     );
   } else {
-    await OS.File.writeAtomic(scriptPath, `#!${PYTHON} -u\n${SCRIPT}`);
-    await OS.File.setPermissions(scriptPath, { unixMode: 0o755 });
+    await IOUtils.writeUTF8(scriptPath, `#!${PYTHON} -u\n${SCRIPT}`);
+    await IOUtils.setPermissions(scriptPath, 0o755);
     manifest.path = scriptPath;
     await writeManifest(manifestPath, manifest);
   }
@@ -426,7 +622,7 @@ while True:
   });
 
   let buffer = NativeApp.encodeMessage(mockContext, MSG);
-  app.send(new StructuredCloneHolder(buffer));
+  app.send(new StructuredCloneHolder("", null, buffer));
   await recvPromise;
 
   app._cleanup();
