@@ -8,11 +8,9 @@
 
 #include "nsFind.h"
 #include "mozilla/Likely.h"
-#include "nsContentCID.h"
 #include "nsIContent.h"
 #include "nsINode.h"
 #include "nsIFrame.h"
-#include "nsITextControlFrame.h"
 #include "nsIFormControl.h"
 #include "nsTextFragment.h"
 #include "nsString.h"
@@ -22,6 +20,7 @@
 #include "nsUnicodeProperties.h"
 #include "nsCRT.h"
 #include "nsRange.h"
+#include "nsReadableUtils.h"
 #include "nsContentUtils.h"
 #include "mozilla/DebugOnly.h"
 #include "mozilla/TextEditor.h"
@@ -31,6 +30,8 @@
 #include "mozilla/dom/HTMLOptionElement.h"
 #include "mozilla/dom/HTMLSelectElement.h"
 #include "mozilla/dom/Text.h"
+#include "mozilla/intl/Segmenter.h"
+#include "mozilla/intl/UnicodeProperties.h"
 #include "mozilla/StaticPrefs_browser.h"
 
 using namespace mozilla;
@@ -56,23 +57,15 @@ NS_IMPL_CYCLE_COLLECTING_RELEASE(nsFind)
 
 NS_IMPL_CYCLE_COLLECTION(nsFind)
 
-nsFind::nsFind()
-    : mFindBackward(false),
-      mCaseSensitive(false),
-      mMatchDiacritics(false),
-      mWordBreaker(nullptr) {}
-
-nsFind::~nsFind() = default;
-
 #ifdef DEBUG_FIND
 #  define DEBUG_FIND_PRINTF(...) printf(__VA_ARGS__)
 #else
 #  define DEBUG_FIND_PRINTF(...) /* nothing */
 #endif
 
-static nsIContent& AnonymousSubtreeRootParent(const nsINode& aNode) {
+static nsIContent& AnonymousSubtreeRootParentOrHost(const nsINode& aNode) {
   MOZ_ASSERT(aNode.IsInNativeAnonymousSubtree());
-  return *aNode.GetClosestNativeAnonymousSubtreeRootParent();
+  return *aNode.GetClosestNativeAnonymousSubtreeRootParentOrHost();
 }
 
 static void DumpNode(const nsINode* aNode) {
@@ -106,7 +99,15 @@ static bool IsBlockNode(const nsIContent* aContent) {
   }
 
   nsIFrame* frame = aContent->GetPrimaryFrame();
-  return frame && frame->StyleDisplay()->IsBlockOutsideStyle();
+  if (!frame) {
+    return false;
+  }
+
+  const auto& disp = *frame->StyleDisplay();
+  // We also treat internal table frames as "blocks" for the purpose of
+  // locating boundaries for searches (see
+  // https://bugzilla.mozilla.org/show_bug.cgi?id=1645990).
+  return disp.IsBlockOutsideStyle() || disp.IsInternalTableStyleExceptCell();
 }
 
 static bool IsDisplayedNode(const nsINode* aNode) {
@@ -122,7 +123,22 @@ static bool IsDisplayedNode(const nsINode* aNode) {
   return aNode->IsElement() && aNode->AsElement()->IsDisplayContents();
 }
 
-static bool IsVisibleNode(const nsINode* aNode) {
+static bool IsRubyAnnotationNode(const nsINode* aNode) {
+  if (!aNode->IsContent()) {
+    return false;
+  }
+
+  nsIFrame* frame = aNode->AsContent()->GetPrimaryFrame();
+  if (!frame) {
+    return false;
+  }
+
+  StyleDisplay display = frame->StyleDisplay()->mDisplay;
+  return StyleDisplay::RubyText == display ||
+         StyleDisplay::RubyTextContainer == display;
+}
+
+static bool IsFindableNode(const nsINode* aNode) {
   if (!IsDisplayedNode(aNode)) {
     return false;
   }
@@ -133,14 +149,21 @@ static bool IsVisibleNode(const nsINode* aNode) {
     return true;
   }
 
+  if (frame->StyleUI()->IsInert() ||
+      frame->HidesContent(nsIFrame::IncludeContentVisibility::Hidden) ||
+      frame->IsHiddenByContentVisibilityOnAnyAncestor(
+          nsIFrame::IncludeContentVisibility::Hidden)) {
+    return false;
+  }
+
   return frame->StyleVisibility()->IsVisible();
 }
 
 static bool ShouldFindAnonymousContent(const nsIContent& aContent) {
   MOZ_ASSERT(aContent.IsInNativeAnonymousSubtree());
 
-  nsIContent& parent = AnonymousSubtreeRootParent(aContent);
-  if (nsCOMPtr<nsIFormControl> formControl = do_QueryInterface(&parent)) {
+  nsIContent& host = AnonymousSubtreeRootParentOrHost(aContent);
+  if (const auto* formControl = nsIFormControl::FromNode(&host)) {
     if (formControl->IsTextControl(/* aExcludePassword = */ true)) {
       // Only editable NAC in textfields should be findable. That is, we want to
       // find "bar" in `<input value="bar">`, but not in `<input
@@ -183,6 +206,13 @@ static bool SkipNode(const nsIContent* aContent) {
       }
     }
 
+    if (StaticPrefs::browser_find_ignore_ruby_annotations() &&
+        IsRubyAnnotationNode(content)) {
+      DEBUG_FIND_PRINTF("Skipping node: ");
+      DumpNode(content);
+      return true;
+    }
+
     if (content->IsInNativeAnonymousSubtree() &&
         !ShouldFindAnonymousContent(*content)) {
       DEBUG_FIND_PRINTF("Skipping node: ");
@@ -224,7 +254,7 @@ static bool ForceBreakBetweenText(const Text& aPrevious, const Text& aNext) {
 }
 
 struct nsFind::State final {
-  State(bool aFindBackward, nsIContent& aRoot, const nsRange& aStartPoint)
+  State(bool aFindBackward, nsIContent& aRoot, const RangeBoundary& aStartPoint)
       : mFindBackward(aFindBackward),
         mInitialized(false),
         mFoundBreak(false),
@@ -317,7 +347,7 @@ struct nsFind::State final {
   TreeIterator<StyleChildrenIterator> mIterator;
 
   // These are only needed for the first GetNextNode() call.
-  const nsRange& mStartPoint;
+  const RangeBoundary& mStartPoint;
 };
 
 void nsFind::State::Advance(Initializing aInitializing, bool aAlreadyMatching) {
@@ -347,12 +377,10 @@ void nsFind::State::Initialize() {
   mInitialized = true;
   mIterOffset = mFindBackward ? -1 : 0;
 
-  nsINode* container = mFindBackward ? mStartPoint.GetStartContainer()
-                                     : mStartPoint.GetEndContainer();
+  nsINode* container = mStartPoint.GetContainer();
 
   // Set up ourselves at the first node we want to start searching at.
-  nsIContent* beginning = mFindBackward ? mStartPoint.GetChildAtStartOffset()
-                                        : mStartPoint.GetChildAtEndOffset();
+  nsIContent* beginning = mStartPoint.GetChildAtOffset();
   if (beginning) {
     mIterator.Seek(*beginning);
     // If the start point is pointing to a node, when looking backwards we'd
@@ -384,8 +412,8 @@ void nsFind::State::Initialize() {
     return;
   }
 
-  mIterOffset =
-      mFindBackward ? mStartPoint.StartOffset() : mStartPoint.EndOffset();
+  mIterOffset = int(
+      *mStartPoint.Offset(RangeBoundary::OffsetFilter::kValidOrInvalidOffsets));
 }
 
 class MOZ_STACK_CLASS nsFind::StateRestorer final {
@@ -449,13 +477,13 @@ NS_IMETHODIMP
 nsFind::GetEntireWord(bool* aEntireWord) {
   if (!aEntireWord) return NS_ERROR_NULL_POINTER;
 
-  *aEntireWord = !!mWordBreaker;
+  *aEntireWord = mWordStartBounded && mWordEndBounded;
   return NS_OK;
 }
 
 NS_IMETHODIMP
 nsFind::SetEntireWord(bool aEntireWord) {
-  mWordBreaker = aEntireWord ? nsContentUtils::WordBreaker() : nullptr;
+  mWordStartBounded = mWordEndBounded = aEntireWord;
   return NS_OK;
 }
 
@@ -503,26 +531,14 @@ char32_t nsFind::DecodeChar(const char16_t* t2b, int32_t* index) const {
   return c;
 }
 
-bool nsFind::BreakInBetween(char32_t x, char32_t y) const {
-  char16_t x16[2], y16[2];
-  int32_t x16len, y16len;
-  if (IS_IN_BMP(x)) {
-    x16[0] = (char16_t)x;
-    x16len = 1;
-  } else {
-    x16[0] = H_SURROGATE(x);
-    x16[1] = L_SURROGATE(x);
-    x16len = 2;
-  }
-  if (IS_IN_BMP(y)) {
-    y16[0] = (char16_t)y;
-    y16len = 1;
-  } else {
-    y16[0] = H_SURROGATE(y);
-    y16[1] = L_SURROGATE(y);
-    y16len = 2;
-  }
-  return mWordBreaker->BreakInBetween(x16, x16len, y16, y16len);
+bool nsFind::BreakInBetween(char32_t x, char32_t y) {
+  nsAutoStringN<4> text;
+  AppendUCS4ToUTF16(x, text);
+  const uint32_t x16Len = text.Length();
+  AppendUCS4ToUTF16(y, text);
+  mWordBreakIter.Reset(text);
+
+  return *mWordBreakIter.Seek(x16Len - 1) == x16Len;
 }
 
 char32_t nsFind::PeekNextChar(State& aState, bool aAlreadyMatching) const {
@@ -556,7 +572,16 @@ char32_t nsFind::PeekNextChar(State& aState, bool aAlreadyMatching) const {
 }
 
 #define NBSP_CHARCODE (CHAR_TO_UNICHAR(160))
-#define IsSpace(c) (nsCRT::IsAsciiSpace(c) || (c) == NBSP_CHARCODE)
+// Bug 1943846: The char16_t version of nsCRT::IsAsciiSpace is not working as
+// expected for characters with UTF32 > 0xFFFF. Add a check for UTF32 characters
+
+static bool IsSpace(char16_t aChar) {
+  return nsCRT::IsAsciiSpace(aChar) || aChar == NBSP_CHARCODE;
+}
+static bool IsSpace(char32_t aChar) {
+  return aChar <= 0xffff && IsSpace(char16_t(aChar));
+}
+
 #define OVERFLOW_PINDEX (mFindBackward ? pindex < 0 : pindex > patLen)
 #define DONE_WITH_PINDEX (mFindBackward ? pindex <= 0 : pindex >= patLen)
 
@@ -575,14 +600,49 @@ nsFind::Find(const nsAString& aPatText, nsRange* aSearchRange,
   NS_ENSURE_ARG(aEndPoint);
   NS_ENSURE_ARG_POINTER(aRangeRet);
 
-  Document* document =
-      aStartPoint->GetRoot() ? aStartPoint->GetRoot()->OwnerDoc() : nullptr;
-  NS_ENSURE_ARG(document);
+  *aRangeRet = nullptr;
+
+  if (RefPtr findResult = FindFromRangeBoundaries(
+          aPatText, aStartPoint->StartRef(), aEndPoint->EndRef())) {
+    findResult.forget(aRangeRet);
+  }
+  return NS_OK;
+}
+already_AddRefed<nsRange> nsFind::FindFromRangeBoundaries(
+    const nsAString& aPatText, const mozilla::RangeBoundary& aStartPoint,
+    const mozilla::RangeBoundary& aEndPoint) {
+  if (!aStartPoint.IsSetAndInComposedDoc() ||
+      !aEndPoint.IsSetAndInComposedDoc()) {
+    return nullptr;
+  }
+  MOZ_ASSERT(aStartPoint.IsSetAndValid());
+  MOZ_ASSERT(aEndPoint.IsSetAndValid());
+  // set up node index cache. If there is no cache given from the outside
+  // (for repeated calls to `Find()`), set up a local one.
+  nsContentUtils::NodeIndexCache localCache;
+  AutoRestore<nsContentUtils::NodeIndexCache*> restoreCache(mNodeIndexCache);
+  if (!mNodeIndexCache) {
+    mNodeIndexCache = &localCache;
+  }
+#if MOZ_DIAGNOSTIC_ASSERT_ENABLED
+  auto cmp =
+      nsContentUtils::ComparePoints(aStartPoint, aEndPoint, mNodeIndexCache);
+  MOZ_DIAGNOSTIC_ASSERT(cmp, "Start and end points in different trees?");
+  MOZ_DIAGNOSTIC_ASSERT(*cmp != 1, "Start point must not be after end point");
+#endif
+  if (MOZ_UNLIKELY(aStartPoint == aEndPoint)) {
+    return nullptr;
+  }
+
+  const Document* document = aStartPoint.GetContainer()->GetComposedDoc();
+  if (!document) {
+    return nullptr;
+  }
 
   Element* root = document->GetRootElement();
-  NS_ENSURE_ARG(root);
-
-  *aRangeRet = 0;
+  if (!root) {
+    return nullptr;
+  }
 
   nsAutoString patAutoStr(aPatText);
   if (!mCaseSensitive) {
@@ -593,15 +653,15 @@ nsFind::Find(const nsAString& aPatText, nsRange* aSearchRange,
   }
 
   // Ignore soft hyphens in the pattern
-  static const char kShy[] = {char(CH_SHY), 0};
+  static const char16_t kShy[] = {CH_SHY, 0};
   patAutoStr.StripChars(kShy);
 
   const char16_t* patStr = patAutoStr.get();
-  int32_t patLen = patAutoStr.Length() - 1;
+  const int32_t patLen = int(patAutoStr.Length()) - 1;
 
   // If this function is called with an empty string, we should early exit.
   if (patLen < 0) {
-    return NS_OK;
+    return nullptr;
   }
 
   const int32_t patternStart = mFindBackward ? patLen : 0;
@@ -613,7 +673,7 @@ nsFind::Find(const nsAString& aPatText, nsRange* aSearchRange,
   int32_t findex = 0;
 
   // Direction to move pindex and ptr*
-  int incr = mFindBackward ? -1 : 1;
+  const int incr = mFindBackward ? -1 : 1;
 
   const nsTextFragment* frag = nullptr;
   int32_t fragLen = 0;
@@ -632,14 +692,13 @@ nsFind::Find(const nsAString& aPatText, nsRange* aSearchRange,
   char32_t matchAnchorChar = 0;
 
   // Get the end point, so we know when to end searches:
-  nsINode* endNode = aEndPoint->GetEndContainer();
-  uint32_t endOffset = aEndPoint->EndOffset();
+  const RangeBoundary& endPoint = mFindBackward ? aStartPoint : aEndPoint;
 
   char32_t c = 0;
   char32_t patc = 0;
   char32_t prevCharInMatch = 0;
 
-  State state(mFindBackward, *root, *aStartPoint);
+  State state(mFindBackward, *root, mFindBackward ? aEndPoint : aStartPoint);
   Text* current = nullptr;
 
   auto EndPartialMatch = [&]() -> bool {
@@ -688,7 +747,7 @@ nsFind::Find(const nsAString& aPatText, nsRange* aSearchRange,
         if (EndPartialMatch()) {
           continue;
         }
-        return NS_OK;
+        return nullptr;
       }
 
       // We have a new text content. See if we need to force a break due to
@@ -704,7 +763,7 @@ nsFind::Find(const nsAString& aPatText, nsRange* aSearchRange,
       }
 
       frag = &current->TextFragment();
-      fragLen = frag->GetLength();
+      fragLen = int32_t(frag->GetLength());
 
       // Set our starting point in this node. If we're going back to the anchor
       // node, which means that we just ended a partial match, use the saved
@@ -766,11 +825,13 @@ nsFind::Find(const nsAString& aPatText, nsRange* aSearchRange,
 
     // Have we gone past the endpoint yet? If we have, and we're not in the
     // middle of a match, return.
-    if (state.GetCurrentNode() == endNode &&
-        ((mFindBackward && findex < static_cast<int32_t>(endOffset)) ||
-         (!mFindBackward && findex > static_cast<int32_t>(endOffset)))) {
-      DEBUG_FIND_PRINTF("Reached the end and not in the middle of a match\n");
-      return NS_OK;
+    if (auto cmp = nsContentUtils::ComparePoints(
+            RawRangeBoundary(state.GetCurrentNode(), findex), endPoint,
+            mNodeIndexCache)) {
+      if ((mFindBackward && *cmp < 0) || (!mFindBackward && *cmp > 0)) {
+        DEBUG_FIND_PRINTF("Reached the end and not in the middle of a match\n");
+        return nullptr;
+      }
     }
 
     // Save the previous character for word boundary detection
@@ -780,7 +841,7 @@ nsFind::Find(const nsAString& aPatText, nsRange* aSearchRange,
     // already guaranteed to not be a combining diacritical mark.)
     c = (t2b ? DecodeChar(t2b, &findex) : CHAR_TO_UNICHAR(t1b[findex]));
     if (!mMatchDiacritics && IsCombiningDiacritic(c) &&
-        !IsMathOrMusicSymbol(prevChar)) {
+        !intl::UnicodeProperties::IsMathOrMusicSymbol(prevChar)) {
       continue;
     }
     patc = DecodeChar(patStr, &pindex);
@@ -839,9 +900,10 @@ nsFind::Find(const nsAString& aPatText, nsRange* aSearchRange,
       }
     }
 
-    // Figure whether the previous char is a word-breaking one.
-    bool wordBreakPrev = false;
-    if (mWordBreaker) {
+    // Figure whether the previous char is a word-breaking one,
+    // if we care about word boundaries.
+    bool wordBreakPrev = true;
+    if (mWordStartBounded && prevChar) {
       if (prevChar == NBSP_CHARCODE) {
         prevChar = CHAR_TO_UNICHAR(' ');
       }
@@ -854,7 +916,8 @@ nsFind::Find(const nsAString& aPatText, nsRange* aSearchRange,
     // b) a match has already been stored
     // c) the previous character is a different "class" than the current
     // character.
-    if ((c == patc && (!mWordBreaker || matchAnchorNode || wordBreakPrev)) ||
+    if ((c == patc && (!(mWordStartBounded || mWordEndBounded) ||
+                       matchAnchorNode || wordBreakPrev)) ||
         (inWhitespace && IsSpace(c))) {
       prevCharInMatch = c;
       if (inWhitespace) {
@@ -881,10 +944,10 @@ nsFind::Find(const nsAString& aPatText, nsRange* aSearchRange,
 
         // Make the range:
         // Check for word break (if necessary)
-        if (mWordBreaker || inWhitespace) {
+        if (mWordEndBounded || inWhitespace) {
           int32_t nextfindex = findex + incr;
 
-          char16_t nextChar;
+          char32_t nextChar;
           // If still in array boundaries, get nextChar.
           if (mFindBackward ? (nextfindex >= 0) : (nextfindex < fragLen)) {
             if (t2b) {
@@ -902,7 +965,7 @@ nsFind::Find(const nsAString& aPatText, nsRange* aSearchRange,
           }
 
           // If a word break isn't there when it needs to be, reset search.
-          if (mWordBreaker && !BreakInBetween(c, nextChar)) {
+          if (mWordEndBounded && nextChar && !BreakInBetween(c, nextChar)) {
             matchAnchorNode = nullptr;
             continue;
           }
@@ -933,16 +996,15 @@ nsFind::Find(const nsAString& aPatText, nsRange* aSearchRange,
         }
 
         RefPtr<nsRange> range = nsRange::Create(current);
-        if (startParent && endParent && IsVisibleNode(startParent) &&
-            IsVisibleNode(endParent)) {
+        if (startParent && endParent && IsFindableNode(startParent) &&
+            IsFindableNode(endParent)) {
           IgnoredErrorResult rv;
           range->SetStart(*startParent, matchStartOffset, rv);
           if (!rv.Failed()) {
             range->SetEnd(*endParent, matchEndOffset, rv);
           }
           if (!rv.Failed()) {
-            range.forget(aRangeRet);
-            return NS_OK;
+            return range.forget();
           }
         }
 
@@ -967,6 +1029,4 @@ nsFind::Find(const nsAString& aPatText, nsRange* aSearchRange,
     DEBUG_FIND_PRINTF("NOT: %c == %c\n", c, patc);
     EndPartialMatch();
   }
-
-  return NS_OK;
 }
