@@ -3,28 +3,22 @@
 
 "use strict";
 
-const { RemoteSettings } = ChromeUtils.import(
-  "resource://services-settings/remote-settings.js"
+const { RemoteSettings } = ChromeUtils.importESModule(
+  "resource://services-settings/remote-settings.sys.mjs"
 );
 const {
   _ExperimentFeature: ExperimentFeature,
-  NimbusFeatures,
+
   ExperimentAPI,
-} = ChromeUtils.import("resource://nimbus/ExperimentAPI.jsm");
-const { ExperimentTestUtils } = ChromeUtils.import(
-  "resource://testing-common/NimbusTestUtils.jsm"
+} = ChromeUtils.importESModule("resource://nimbus/ExperimentAPI.sys.mjs");
+const { ExperimentTestUtils } = ChromeUtils.importESModule(
+  "resource://testing-common/NimbusTestUtils.sys.mjs"
 );
-const { ExperimentManager } = ChromeUtils.import(
-  "resource://nimbus/lib/ExperimentManager.jsm"
+const { ExperimentManager } = ChromeUtils.importESModule(
+  "resource://nimbus/lib/ExperimentManager.sys.mjs"
 );
-const { RemoteSettingsExperimentLoader } = ChromeUtils.import(
-  "resource://nimbus/lib/RemoteSettingsExperimentLoader.jsm"
-);
-const { BrowserTestUtils } = ChromeUtils.import(
-  "resource://testing-common/BrowserTestUtils.jsm"
-);
-const { TelemetryEnvironment } = ChromeUtils.import(
-  "resource://gre/modules/TelemetryEnvironment.jsm"
+const { RemoteSettingsExperimentLoader } = ChromeUtils.importESModule(
+  "resource://nimbus/lib/RemoteSettingsExperimentLoader.sys.mjs"
 );
 
 const FOO_FAKE_FEATURE_MANIFEST = {
@@ -71,9 +65,8 @@ const REMOTE_CONFIGURATION_FOO = ExperimentFakes.recipe("foo-rollout", {
       features: [
         {
           featureId: "foo",
-          enabled: true,
           isEarlyStartup: true,
-          value: { remoteValue: 42 },
+          value: { remoteValue: 42, enabled: true },
         },
       ],
     },
@@ -89,9 +82,8 @@ const REMOTE_CONFIGURATION_BAR = ExperimentFakes.recipe("bar-rollout", {
       features: [
         {
           featureId: "bar",
-          enabled: true,
           isEarlyStartup: true,
-          value: { remoteValue: 3 },
+          value: { remoteValue: 3, enabled: true },
         },
       ],
     },
@@ -101,28 +93,34 @@ const REMOTE_CONFIGURATION_BAR = ExperimentFakes.recipe("bar-rollout", {
 
 const SYNC_DEFAULTS_PREF_BRANCH = "nimbus.syncdefaultsstore.";
 
+add_setup(function () {
+  const client = RemoteSettings("nimbus-desktop-experiments");
+  sinon.stub(client, "get").resolves([]);
+
+  registerCleanupFunction(() => client.get.restore());
+});
+
 async function setup(configuration) {
   const client = RemoteSettings("nimbus-desktop-experiments");
-  await client.db.importChanges(
-    {},
-    Date.now(),
-    configuration || [REMOTE_CONFIGURATION_FOO, REMOTE_CONFIGURATION_BAR],
-    {
-      clear: true,
-    }
+  client.get.resolves(
+    configuration ?? [REMOTE_CONFIGURATION_FOO, REMOTE_CONFIGURATION_BAR]
   );
 
-  registerCleanupFunction(async () => {
-    await client.db.clear();
-  });
-
-  return client;
+  // Simulate a state where no experiment exists.
+  const cleanup = () => client.get.resolves([]);
+  return { client, cleanup };
 }
 
 add_task(async function test_remote_fetch_and_ready() {
-  const sandbox = sinon.createSandbox();
   const fooInstance = new ExperimentFeature("foo", FOO_FAKE_FEATURE_MANIFEST);
   const barInstance = new ExperimentFeature("bar", BAR_FAKE_FEATURE_MANIFEST);
+
+  const cleanupTestFeatures = ExperimentTestUtils.addTestFeatures(
+    fooInstance,
+    barInstance
+  );
+
+  const sandbox = sinon.createSandbox();
   const setExperimentActiveStub = sandbox.stub(
     TelemetryEnvironment,
     "setExperimentActive"
@@ -132,10 +130,6 @@ add_task(async function test_remote_fetch_and_ready() {
     "setExperimentInactive"
   );
 
-  registerCleanupFunction(
-    ExperimentTestUtils.addTestFeatures(fooInstance, barInstance)
-  );
-
   Assert.equal(
     fooInstance.getVariable("remoteValue"),
     undefined,
@@ -149,11 +143,11 @@ add_task(async function test_remote_fetch_and_ready() {
 
   await ExperimentAPI.ready();
 
-  let rsClient = await setup();
+  let { cleanup } = await setup();
 
   // Fake being initialized so we can update recipes
   // we don't need to start any timers
-  RemoteSettingsExperimentLoader._initialized = true;
+  RemoteSettingsExperimentLoader._enabled = true;
   await RemoteSettingsExperimentLoader.updateRecipes(
     "browser_rsel_remote_defaults"
   );
@@ -162,7 +156,6 @@ add_task(async function test_remote_fetch_and_ready() {
   // async to evaluate targeting
   await Promise.all([fooUpdate, barUpdate]);
 
-  Assert.ok(fooInstance.isEnabled(), "Enabled by remote defaults");
   Assert.equal(
     fooInstance.getVariable("remoteValue"),
     REMOTE_CONFIGURATION_FOO.branches[0].features[0].value.remoteValue,
@@ -192,7 +185,6 @@ add_task(async function test_remote_fetch_and_ready() {
       REMOTE_CONFIGURATION_FOO.branches[0].slug,
       {
         type: "nimbus-rollout",
-        enrollmentId: sinon.match.string,
       }
     ),
     "should call setExperimentActive with `foo` feature"
@@ -203,7 +195,6 @@ add_task(async function test_remote_fetch_and_ready() {
       REMOTE_CONFIGURATION_BAR.branches[0].slug,
       {
         type: "nimbus-rollout",
-        enrollmentId: sinon.match.string,
       }
     ),
     "should call setExperimentActive with `bar` feature"
@@ -225,7 +216,7 @@ add_task(async function test_remote_fetch_and_ready() {
   Assert.equal(barInstance.getVariable("remoteValue"), 3, "Has rollout value");
 
   // Clear RS db and load again. No configurations so should clear the cache.
-  await rsClient.db.clear();
+  await cleanup();
   await RemoteSettingsExperimentLoader.updateRecipes(
     "browser_rsel_remote_defaults"
   );
@@ -261,11 +252,14 @@ add_task(async function test_remote_fetch_and_ready() {
   );
   Assert.ok(!barInstance.getVariable("remoteValue"), "Should be missing");
 
-  ExperimentAPI._store._deleteForTests("foo");
-  ExperimentAPI._store._deleteForTests("bar");
-  ExperimentAPI._store._deleteForTests(REMOTE_CONFIGURATION_FOO.slug);
-  ExperimentAPI._store._deleteForTests(REMOTE_CONFIGURATION_BAR.slug);
+  ExperimentAPI._manager.store._deleteForTests("foo");
+  ExperimentAPI._manager.store._deleteForTests("bar");
+  ExperimentAPI._manager.store._deleteForTests(REMOTE_CONFIGURATION_FOO.slug);
+  ExperimentAPI._manager.store._deleteForTests(REMOTE_CONFIGURATION_BAR.slug);
   sandbox.restore();
+
+  cleanupTestFeatures();
+  await cleanup();
 });
 
 add_task(async function test_remote_fetch_on_updateRecipes() {
@@ -282,12 +276,13 @@ add_task(async function test_remote_fetch_on_updateRecipes() {
   );
 
   // This will un-register the timer
-  RemoteSettingsExperimentLoader._initialized = true;
-  RemoteSettingsExperimentLoader.uninit();
+  RemoteSettingsExperimentLoader._enabled = true;
+  RemoteSettingsExperimentLoader.disable();
   Services.prefs.clearUserPref(
     "app.update.lastUpdateTime.rs-experiment-loader-timer"
   );
 
+  RemoteSettingsExperimentLoader._enabled = true;
   RemoteSettingsExperimentLoader.setTimer();
 
   await BrowserTestUtils.waitForCondition(
@@ -299,8 +294,7 @@ add_task(async function test_remote_fetch_on_updateRecipes() {
   Assert.equal(updateRecipesStub.firstCall.args[0], "timer", "Called by timer");
   sandbox.restore();
   // This will un-register the timer
-  RemoteSettingsExperimentLoader._initialized = true;
-  RemoteSettingsExperimentLoader.uninit();
+  RemoteSettingsExperimentLoader.disable();
   Services.prefs.clearUserPref(
     "app.update.lastUpdateTime.rs-experiment-loader-timer"
   );
@@ -309,36 +303,45 @@ add_task(async function test_remote_fetch_on_updateRecipes() {
 add_task(async function test_finalizeRemoteConfigs_cleanup() {
   const featureFoo = new ExperimentFeature("foo", {
     description: "mochitests",
-    variables: {},
+    variables: {
+      foo: { type: "boolean" },
+    },
   });
   const featureBar = new ExperimentFeature("bar", {
     description: "mochitests",
-    variables: {},
+    variables: {
+      bar: { type: "boolean" },
+    },
   });
 
-  registerCleanupFunction(
-    ExperimentTestUtils.addTestFeatures(featureFoo, featureBar)
+  const cleanupTestFeatures = ExperimentTestUtils.addTestFeatures(
+    featureFoo,
+    featureBar
   );
 
-  let fooCleanup = await ExperimentFakes.enrollWithRollout(
+  let fooCleanup = await ExperimentFakes.enrollWithFeatureConfig(
     {
       featureId: "foo",
-      enabled: true,
       isEarlyStartup: true,
       value: { foo: true },
     },
     {
+      slug: "foo-rollout",
+      branchSlug: REMOTE_CONFIGURATION_FOO.branches[0].slug,
+      isRollout: true,
       source: "rs-loader",
     }
   );
-  await ExperimentFakes.enrollWithRollout(
+  await ExperimentFakes.enrollWithFeatureConfig(
     {
       featureId: "bar",
-      enabled: true,
       isEarlyStartup: true,
       value: { bar: true },
     },
     {
+      slug: "bar-rollout",
+      branchSlug: REMOTE_CONFIGURATION_BAR.branches[0].slug,
+      isRollout: true,
       source: "rs-loader",
     }
   );
@@ -348,6 +351,19 @@ add_task(async function test_finalizeRemoteConfigs_cleanup() {
   featureBar.onUpdate(stubBar);
   let cleanupPromise = new Promise(resolve => featureBar.onUpdate(resolve));
 
+  // stubFoo and stubBar will be called because the store is ready. We are not interested in these calls.
+  // Reset call history and check calls stats after cleanup.
+  Assert.ok(
+    stubFoo.called,
+    "feature foo update triggered becuase store is ready"
+  );
+  Assert.ok(
+    stubBar.called,
+    "feature bar update triggered because store is ready"
+  );
+  stubFoo.resetHistory();
+  stubBar.resetHistory();
+
   Services.prefs.setStringPref(
     `${SYNC_DEFAULTS_PREF_BRANCH}foo`,
     JSON.stringify({ foo: true, branch: { feature: { featureId: "foo" } } })
@@ -357,8 +373,25 @@ add_task(async function test_finalizeRemoteConfigs_cleanup() {
     JSON.stringify({ bar: true, branch: { feature: { featureId: "bar" } } })
   );
 
-  await setup([REMOTE_CONFIGURATION_FOO]);
-  RemoteSettingsExperimentLoader._initialized = true;
+  const remoteConfiguration = {
+    ...REMOTE_CONFIGURATION_FOO,
+    branches: [
+      {
+        ...REMOTE_CONFIGURATION_FOO.branches[0],
+        features: [
+          {
+            ...REMOTE_CONFIGURATION_FOO.branches[0].features[0],
+            value: {
+              foo: true,
+            },
+          },
+        ],
+      },
+    ],
+  };
+
+  const { cleanup } = await setup([remoteConfiguration]);
+  RemoteSettingsExperimentLoader._enabled = true;
   await RemoteSettingsExperimentLoader.updateRecipes();
   await cleanupPromise;
 
@@ -376,19 +409,22 @@ add_task(async function test_finalizeRemoteConfigs_cleanup() {
     "Pref was cleared"
   );
 
-  await fooCleanup();
+  fooCleanup();
   // This will also remove the inactive recipe from the store
   // the previous update (from recipe not seen code path)
   // only sets the recipe as inactive
-  ExperimentAPI._store._deleteForTests("bar-rollout");
-  ExperimentAPI._store._deleteForTests("foo-rollout");
+  ExperimentAPI._manager.store._deleteForTests("bar-rollout");
+  ExperimentAPI._manager.store._deleteForTests("foo-rollout");
+
+  cleanupTestFeatures();
+  cleanup();
 });
 
 // If the remote config data returned from the store is not modified
 // this test should not throw
 add_task(async function remote_defaults_no_mutation() {
   let sandbox = sinon.createSandbox();
-  sandbox.stub(ExperimentAPI._store, "getRolloutForFeature").returns(
+  sandbox.stub(ExperimentAPI._manager.store, "getRolloutForFeature").returns(
     Cu.cloneInto(
       {
         featureIds: ["foo"],
@@ -410,8 +446,8 @@ add_task(async function remote_defaults_no_mutation() {
 });
 
 add_task(async function remote_defaults_active_remote_defaults() {
-  ExperimentAPI._store._deleteForTests("foo");
-  ExperimentAPI._store._deleteForTests("bar");
+  ExperimentAPI._manager.store._deleteForTests("foo");
+  ExperimentAPI._manager.store._deleteForTests("bar");
   let barFeature = new ExperimentFeature("bar", {
     description: "mochitest",
     variables: { enabled: { type: "boolean" } },
@@ -420,6 +456,12 @@ add_task(async function remote_defaults_active_remote_defaults() {
     description: "mochitest",
     variables: { enabled: { type: "boolean" } },
   });
+
+  const cleanupTestFeatures = ExperimentTestUtils.addTestFeatures(
+    barFeature,
+    fooFeature
+  );
+
   let rollout1 = ExperimentFakes.recipe("bar", {
     branches: [
       {
@@ -457,29 +499,35 @@ add_task(async function remote_defaults_active_remote_defaults() {
   });
 
   // Order is important, rollout2 won't match at first
-  await setup([rollout2, rollout1]);
+  const { cleanup } = await setup([rollout2, rollout1]);
   let updatePromise = new Promise(resolve => barFeature.onUpdate(resolve));
-  RemoteSettingsExperimentLoader._initialized = true;
+  RemoteSettingsExperimentLoader._enabled = true;
   await RemoteSettingsExperimentLoader.updateRecipes("mochitest");
 
   await updatePromise;
 
-  Assert.ok(barFeature.isEnabled(), "Enabled on first sync");
-  Assert.ok(!fooFeature.isEnabled(), "Targeting doesn't match");
+  Assert.ok(barFeature.getVariable("enabled"), "Enabled on first sync");
+  Assert.ok(!fooFeature.getVariable("enabled"), "Targeting doesn't match");
 
   let featureUpdate = new Promise(resolve => fooFeature.onUpdate(resolve));
   await RemoteSettingsExperimentLoader.updateRecipes("mochitest");
   await featureUpdate;
 
-  Assert.ok(fooFeature.isEnabled(), "Targeting should match");
-  ExperimentAPI._store._deleteForTests("foo");
-  ExperimentAPI._store._deleteForTests("bar");
+  Assert.ok(fooFeature.getVariable("enabled"), "Targeting should match");
+  ExperimentAPI._manager.store._deleteForTests("foo");
+  ExperimentAPI._manager.store._deleteForTests("bar");
+
+  cleanup();
+  cleanupTestFeatures();
 });
 
 add_task(async function remote_defaults_variables_storage() {
   let barFeature = new ExperimentFeature("bar", {
     description: "mochitest",
     variables: {
+      enabled: {
+        type: "boolean",
+      },
       storage: {
         type: "int",
       },
@@ -502,12 +550,14 @@ add_task(async function remote_defaults_variables_storage() {
     enabled: true,
   };
 
-  let doCleanup = await ExperimentFakes.enrollWithRollout({
-    featureId: "bar",
-    enabled: true,
-    isEarlyStartup: true,
-    value: rolloutValue,
-  });
+  let doCleanup = await ExperimentFakes.enrollWithFeatureConfig(
+    {
+      featureId: "bar",
+      isEarlyStartup: true,
+      value: rolloutValue,
+    },
+    { isRollout: true }
+  );
 
   Assert.ok(
     Services.prefs.getStringPref(`${SYNC_DEFAULTS_PREF_BRANCH}bar`, ""),
@@ -528,7 +578,7 @@ add_task(async function remote_defaults_variables_storage() {
     "Test types are returned correctly"
   );
 
-  await doCleanup();
+  doCleanup();
 
   Assert.equal(
     Services.prefs.getIntPref(`${SYNC_DEFAULTS_PREF_BRANCH}bar.storage`, -1),
@@ -536,6 +586,6 @@ add_task(async function remote_defaults_variables_storage() {
     "Variable pref is cleared"
   );
   Assert.ok(!barFeature.getVariable("string"), "Variable is no longer defined");
-  ExperimentAPI._store._deleteForTests("bar");
-  ExperimentAPI._store._deleteForTests("bar-rollout");
+  ExperimentAPI._manager.store._deleteForTests("bar");
+  ExperimentAPI._manager.store._deleteForTests("bar-rollout");
 });
