@@ -1,58 +1,30 @@
 "use strict";
 
-const { ExperimentAPI } = ChromeUtils.import(
-  "resource://nimbus/ExperimentAPI.jsm"
+const { ExperimentAPI } = ChromeUtils.importESModule(
+  "resource://nimbus/ExperimentAPI.sys.mjs"
 );
-const { ExperimentFakes } = ChromeUtils.import(
-  "resource://testing-common/NimbusTestUtils.jsm"
+const { ExperimentFakes } = ChromeUtils.importESModule(
+  "resource://testing-common/NimbusTestUtils.sys.mjs"
 );
-const { TestUtils } = ChromeUtils.import(
-  "resource://testing-common/TestUtils.jsm"
+const { NimbusTelemetry } = ChromeUtils.importESModule(
+  "resource://nimbus/lib/Telemetry.sys.mjs"
+);
+const { TestUtils } = ChromeUtils.importESModule(
+  "resource://testing-common/TestUtils.sys.mjs"
 );
 const COLLECTION_ID_PREF = "messaging-system.rsexperimentloader.collection_id";
 
 /**
  * #getExperiment
  */
-add_task(async function test_getExperiment_fromChild_slug() {
-  const sandbox = sinon.createSandbox();
-  const manager = ExperimentFakes.manager();
-  const expected = ExperimentFakes.experiment("foo");
-
-  await manager.onStartup();
-
-  sandbox.stub(ExperimentAPI, "_store").get(() => ExperimentFakes.childStore());
-
-  await manager.store.addEnrollment(expected);
-
-  // Wait to sync to child
-  await TestUtils.waitForCondition(
-    () => ExperimentAPI.getExperiment({ slug: "foo" }),
-    "Wait for child to sync"
-  );
-
-  Assert.equal(
-    ExperimentAPI.getExperiment({ slug: "foo" }).slug,
-    expected.slug,
-    "should return an experiment by slug"
-  );
-
-  Assert.deepEqual(
-    ExperimentAPI.getExperiment({ slug: "foo" }).branch,
-    expected.branch,
-    "should return the right branch by slug"
-  );
-
-  sandbox.restore();
-});
-
 add_task(async function test_getExperiment_fromParent_slug() {
   const sandbox = sinon.createSandbox();
   const manager = ExperimentFakes.manager();
   const expected = ExperimentFakes.experiment("foo");
 
+  sandbox.stub(ExperimentAPI, "_manager").get(() => manager);
+
   await manager.onStartup();
-  sandbox.stub(ExperimentAPI, "_store").get(() => manager.store);
   await ExperimentAPI.ready();
 
   await manager.store.addEnrollment(expected);
@@ -70,10 +42,11 @@ add_task(async function test_getExperimentMetaData() {
   const sandbox = sinon.createSandbox();
   const manager = ExperimentFakes.manager();
   const expected = ExperimentFakes.experiment("foo");
-  let exposureStub = sandbox.stub(ExperimentAPI, "recordExposureEvent");
+  let exposureStub = sandbox.stub(NimbusTelemetry, "recordExposure");
+
+  sandbox.stub(ExperimentAPI, "_manager").get(() => manager);
 
   await manager.onStartup();
-  sandbox.stub(ExperimentAPI, "_store").get(() => manager.store);
   await ExperimentAPI.ready();
 
   await manager.store.addEnrollment(expected);
@@ -93,6 +66,9 @@ add_task(async function test_getExperimentMetaData() {
 
   Assert.ok(exposureStub.notCalled, "Not called for this method");
 
+  manager.unenroll(expected.slug);
+  assertEmptyStore(manager.store);
+
   sandbox.restore();
 });
 
@@ -100,10 +76,11 @@ add_task(async function test_getRolloutMetaData() {
   const sandbox = sinon.createSandbox();
   const manager = ExperimentFakes.manager();
   const expected = ExperimentFakes.rollout("foo");
-  let exposureStub = sandbox.stub(ExperimentAPI, "recordExposureEvent");
+  let exposureStub = sandbox.stub(NimbusTelemetry, "recordExposure");
+
+  sandbox.stub(ExperimentAPI, "_manager").get(() => manager);
 
   await manager.onStartup();
-  sandbox.stub(ExperimentAPI, "_store").get(() => manager.store);
   await ExperimentAPI.ready();
 
   await manager.store.addEnrollment(expected);
@@ -123,15 +100,20 @@ add_task(async function test_getRolloutMetaData() {
 
   Assert.ok(exposureStub.notCalled, "Not called for this method");
 
+  manager.unenroll(expected.slug);
+  assertEmptyStore(manager.store);
+
   sandbox.restore();
 });
 
 add_task(function test_getExperimentMetaData_safe() {
   const sandbox = sinon.createSandbox();
-  let exposureStub = sandbox.stub(ExperimentAPI, "recordExposureEvent");
+  let exposureStub = sandbox.stub(NimbusTelemetry, "recordExposure");
 
-  sandbox.stub(ExperimentAPI._store, "get").throws();
-  sandbox.stub(ExperimentAPI._store, "getExperimentForFeature").throws();
+  sandbox.stub(ExperimentAPI._manager.store, "get").throws();
+  sandbox
+    .stub(ExperimentAPI._manager.store, "getExperimentForFeature")
+    .throws();
 
   try {
     let metadata = ExperimentAPI.getExperimentMetaData({ slug: "foo" });
@@ -140,7 +122,7 @@ add_task(function test_getExperimentMetaData_safe() {
     Assert.ok(false, "Error should be caught in ExperimentAPI");
   }
 
-  Assert.ok(ExperimentAPI._store.get.calledOnce, "Sanity check");
+  Assert.ok(ExperimentAPI._manager.store.get.calledOnce, "Sanity check");
 
   try {
     let metadata = ExperimentAPI.getExperimentMetaData({ featureId: "foo" });
@@ -150,7 +132,7 @@ add_task(function test_getExperimentMetaData_safe() {
   }
 
   Assert.ok(
-    ExperimentAPI._store.getExperimentForFeature.calledOnce,
+    ExperimentAPI._manager.store.getExperimentForFeature.calledOnce,
     "Sanity check"
   );
 
@@ -159,54 +141,11 @@ add_task(function test_getExperimentMetaData_safe() {
   sandbox.restore();
 });
 
-add_task(async function test_getExperiment_feature() {
-  const sandbox = sinon.createSandbox();
-  const manager = ExperimentFakes.manager();
-  const expected = ExperimentFakes.experiment("foo", {
-    branch: {
-      slug: "treatment",
-      features: [{ featureId: "cfr", enabled: true, value: null }],
-      feature: {
-        featureId: "unused-feature-id-for-legacy-support",
-        enabled: false,
-        value: {},
-      },
-    },
-  });
-
-  await manager.onStartup();
-
-  sandbox.stub(ExperimentAPI, "_store").get(() => ExperimentFakes.childStore());
-  let exposureStub = sandbox.stub(ExperimentAPI, "recordExposureEvent");
-
-  await manager.store.addEnrollment(expected);
-
-  // Wait to sync to child
-  await TestUtils.waitForCondition(
-    () => ExperimentAPI.getExperiment({ featureId: "cfr" }),
-    "Wait for child to sync"
-  );
-
-  Assert.equal(
-    ExperimentAPI.getExperiment({ featureId: "cfr" }).slug,
-    expected.slug,
-    "should return an experiment by featureId"
-  );
-
-  Assert.deepEqual(
-    ExperimentAPI.getExperiment({ featureId: "cfr" }).branch,
-    expected.branch,
-    "should return the right branch by featureId"
-  );
-
-  Assert.ok(exposureStub.notCalled, "Not called by default");
-
-  sandbox.restore();
-});
-
 add_task(async function test_getExperiment_safe() {
   const sandbox = sinon.createSandbox();
-  sandbox.stub(ExperimentAPI._store, "getExperimentForFeature").throws();
+  sandbox
+    .stub(ExperimentAPI._manager.store, "getExperimentForFeature")
+    .throws();
 
   try {
     Assert.equal(
@@ -231,7 +170,7 @@ add_task(async function test_getExperiment_featureAccess() {
     },
   });
   const stub = sandbox
-    .stub(ExperimentAPI._store, "getExperimentForFeature")
+    .stub(ExperimentAPI._manager.store, "getExperimentForFeature")
     .returns(expected);
 
   let { branch } = ExperimentAPI.getExperiment({ featureId: "cfr" });
@@ -253,7 +192,7 @@ add_task(async function test_getExperiment_featureAccess_backwardsCompat() {
     },
   });
   const stub = sandbox
-    .stub(ExperimentAPI._store, "getExperimentForFeature")
+    .stub(ExperimentAPI._manager.store, "getExperimentForFeature")
     .returns(expected);
 
   let { branch } = ExperimentAPI.getExperiment({ featureId: "cfr" });
@@ -350,8 +289,9 @@ add_task(async function test_getAllBranches_backwardsCompat() {
   delete RECIPE.branches[1].features;
   let feature = {
     featureId: "backwardsCompat",
-    enabled: true,
-    value: null,
+    value: {
+      enabled: true,
+    },
   };
   RECIPE.branches[0].feature = feature;
   RECIPE.branches[1].feature = feature;
@@ -385,116 +325,128 @@ add_task(async function test_getAllBranches_Failure() {
 });
 
 /**
- * #on
- * #off
+ * Store events
  */
 add_task(async function test_addEnrollment_eventEmit_add() {
   const sandbox = sinon.createSandbox();
-  const slugStub = sandbox.stub();
   const featureStub = sandbox.stub();
   const experiment = ExperimentFakes.experiment("foo", {
     branch: {
       slug: "variant",
-      features: [{ featureId: "purple", enabled: true, value: null }],
+      ratio: 1,
+      features: [{ featureId: "purple", value: {} }],
     },
   });
-  const store = ExperimentFakes.store();
-  sandbox.stub(ExperimentAPI, "_store").get(() => store);
+  const manager = ExperimentFakes.manager();
+  const store = manager.store;
+  sandbox.stub(ExperimentAPI, "_manager").get(() => manager);
 
   await store.init();
   await ExperimentAPI.ready();
 
-  ExperimentAPI.on("update", { slug: "foo" }, slugStub);
-  ExperimentAPI.on("update", { featureId: "purple" }, featureStub);
+  store.on("featureUpdate:purple", featureStub);
 
   await store.addEnrollment(experiment);
 
-  Assert.equal(
-    slugStub.callCount,
-    1,
-    "should call 'update' callback for slug when experiment is added"
-  );
-  Assert.equal(slugStub.firstCall.args[1].slug, experiment.slug);
   Assert.equal(
     featureStub.callCount,
     1,
-    "should call 'update' callback for featureId when an experiment is added"
+    "should call 'featureUpdate' callback for featureId when an experiment is added"
   );
-  Assert.equal(featureStub.firstCall.args[1].slug, experiment.slug);
+  Assert.equal(featureStub.firstCall.args[0], "featureUpdate:purple");
+  Assert.equal(featureStub.firstCall.args[1], "experiment-updated");
+
+  store.off("featureUpdate:purple", featureStub);
+
+  manager.unenroll(experiment.slug);
+  assertEmptyStore(store);
+
+  sandbox.restore();
 });
 
 add_task(async function test_updateExperiment_eventEmit_add_and_update() {
   const sandbox = sinon.createSandbox();
-  const slugStub = sandbox.stub();
   const featureStub = sandbox.stub();
   const experiment = ExperimentFakes.experiment("foo", {
     branch: {
       slug: "variant",
-      features: [{ featureId: "purple", enabled: true, value: null }],
+      ratio: 1,
+      features: [{ featureId: "purple", value: {} }],
     },
   });
-  const store = ExperimentFakes.store();
-  sandbox.stub(ExperimentAPI, "_store").get(() => store);
+  const manager = ExperimentFakes.manager();
+  const store = manager.store;
+  sandbox.stub(ExperimentAPI, "_manager").get(() => manager);
 
   await store.init();
   await ExperimentAPI.ready();
 
   await store.addEnrollment(experiment);
 
-  ExperimentAPI.on("update", { slug: "foo" }, slugStub);
-  ExperimentAPI.on("update", { featureId: "purple" }, featureStub);
+  store._onFeatureUpdate("purple", featureStub);
 
   store.updateExperiment(experiment.slug, experiment);
 
   await TestUtils.waitForCondition(
-    () => slugStub.callCount == 2,
+    () => featureStub.callCount == 2,
     "Wait for `on` method to notify callback about the `add` event."
   );
   // Called twice, once when attaching the event listener (because there is an
   // existing experiment with that name) and 2nd time for the update event
-  Assert.equal(slugStub.firstCall.args[1].slug, experiment.slug);
   Assert.equal(featureStub.callCount, 2, "Called twice for feature");
-  Assert.equal(featureStub.firstCall.args[1].slug, experiment.slug);
+  Assert.equal(featureStub.firstCall.args[0], "featureUpdate:purple");
+  Assert.equal(featureStub.firstCall.args[1], "experiment-updated");
+
+  store._offFeatureUpdate("featureUpdate:purple", featureStub);
+
+  manager.unenroll(experiment.slug);
+  assertEmptyStore(store);
 });
 
 add_task(async function test_updateExperiment_eventEmit_off() {
   const sandbox = sinon.createSandbox();
-  const slugStub = sandbox.stub();
   const featureStub = sandbox.stub();
   const experiment = ExperimentFakes.experiment("foo", {
     branch: {
       slug: "variant",
-      features: [{ featureId: "purple", enabled: true, value: null }],
+      ratio: 1,
+      features: [{ featureId: "purple", value: {} }],
     },
   });
-  const store = ExperimentFakes.store();
-  sandbox.stub(ExperimentAPI, "_store").get(() => store);
+  const manager = ExperimentFakes.manager();
+  const store = manager.store;
+  sandbox.stub(ExperimentAPI, "_manager").get(() => manager);
 
   await store.init();
   await ExperimentAPI.ready();
 
-  ExperimentAPI.on("update", { slug: "foo" }, slugStub);
-  ExperimentAPI.on("update", { featureId: "purple" }, featureStub);
+  store.on("featureUpdate:purple", featureStub);
 
   await store.addEnrollment(experiment);
 
-  ExperimentAPI.off("update:foo", slugStub);
-  ExperimentAPI.off("update:purple", featureStub);
+  store.off("featureUpdate:purple", featureStub);
 
   store.updateExperiment(experiment.slug, experiment);
 
-  Assert.equal(slugStub.callCount, 1, "Called only once before `off`");
   Assert.equal(featureStub.callCount, 1, "Called only once before `off`");
+
+  manager.unenroll(experiment.slug);
+  assertEmptyStore(store);
+
+  sandbox.restore();
 });
 
-add_task(async function test_activateBranch() {
+add_task(async function test_getActiveBranch() {
   const sandbox = sinon.createSandbox();
-  const store = ExperimentFakes.store();
-  sandbox.stub(ExperimentAPI, "_store").get(() => store);
+  const manager = ExperimentFakes.manager();
+  const store = manager.store;
+
+  sandbox.stub(ExperimentAPI, "_manager").get(() => manager);
   const experiment = ExperimentFakes.experiment("foo", {
     branch: {
       slug: "variant",
-      features: [{ featureId: "green", enabled: true, value: null }],
+      ratio: 1,
+      features: [{ featureId: "green", value: {} }],
     },
   });
 
@@ -502,21 +454,26 @@ add_task(async function test_activateBranch() {
   await store.addEnrollment(experiment);
 
   Assert.deepEqual(
-    ExperimentAPI.activateBranch({ featureId: "green" }),
+    ExperimentAPI.getActiveBranch({ featureId: "green" }),
     experiment.branch,
     "Should return feature of active experiment"
   );
 
+  manager.unenroll(experiment.slug);
+  assertEmptyStore(store);
+
   sandbox.restore();
 });
 
-add_task(async function test_activateBranch_safe() {
+add_task(async function test_getActiveBranch_safe() {
   const sandbox = sinon.createSandbox();
-  sandbox.stub(ExperimentAPI._store, "getAllActive").throws();
+  sandbox
+    .stub(ExperimentAPI._manager.store, "getAllActiveExperiments")
+    .throws();
 
   try {
     Assert.equal(
-      ExperimentAPI.activateBranch({ featureId: "green" }),
+      ExperimentAPI.getActiveBranch({ featureId: "green" }),
       null,
       "Should not throw"
     );
@@ -527,14 +484,16 @@ add_task(async function test_activateBranch_safe() {
   sandbox.restore();
 });
 
-add_task(async function test_activateBranch_storeFailure() {
-  const store = ExperimentFakes.store();
+add_task(async function test_getActiveBranch_storeFailure() {
   const sandbox = sinon.createSandbox();
-  sandbox.stub(ExperimentAPI, "_store").get(() => store);
+  const manager = ExperimentFakes.manager();
+  const store = manager.store;
+  sandbox.stub(ExperimentAPI, "_manager").get(() => manager);
   const experiment = ExperimentFakes.experiment("foo", {
     branch: {
       slug: "variant",
-      features: [{ featureId: "green", enabled: true }],
+      ratio: 1,
+      features: [{ featureId: "green", value: {} }],
     },
   });
 
@@ -542,26 +501,32 @@ add_task(async function test_activateBranch_storeFailure() {
   await store.addEnrollment(experiment);
   // Adding stub later because `addEnrollment` emits update events
   const stub = sandbox.stub(store, "emit");
-  // Call activateBranch to trigger an activation event
-  sandbox.stub(store, "getAllActive").throws();
+  // Call getActiveBranch to trigger an activation event
+  sandbox.stub(store, "getAllActiveExperiments").throws();
   try {
-    ExperimentAPI.activateBranch({ featureId: "green" });
+    ExperimentAPI.getActiveBranch({ featureId: "green" });
   } catch (e) {
     /* This is expected */
   }
 
   Assert.equal(stub.callCount, 0, "Not called if store somehow fails");
+
+  manager.unenroll(experiment.slug);
+  assertEmptyStore(store);
+
   sandbox.restore();
 });
 
-add_task(async function test_activateBranch_noActivationEvent() {
-  const store = ExperimentFakes.store();
+add_task(async function test_getActiveBranch_noActivationEvent() {
+  const manager = ExperimentFakes.manager();
+  const store = manager.store;
   const sandbox = sinon.createSandbox();
-  sandbox.stub(ExperimentAPI, "_store").get(() => store);
+  sandbox.stub(ExperimentAPI, "_manager").get(() => manager);
   const experiment = ExperimentFakes.experiment("foo", {
     branch: {
       slug: "variant",
-      features: [{ featureId: "green", enabled: true }],
+      ratio: 1,
+      features: [{ featureId: "green", value: {} }],
     },
   });
 
@@ -569,9 +534,13 @@ add_task(async function test_activateBranch_noActivationEvent() {
   await store.addEnrollment(experiment);
   // Adding stub later because `addEnrollment` emits update events
   const stub = sandbox.stub(store, "emit");
-  // Call activateBranch to trigger an activation event
-  ExperimentAPI.activateBranch({ featureId: "green" });
+  // Call getActiveBranch to trigger an activation event
+  ExperimentAPI.getActiveBranch({ featureId: "green" });
 
   Assert.equal(stub.callCount, 0, "Not called: sendExposureEvent is false");
+
+  manager.unenroll(experiment.slug);
+  assertEmptyStore(store);
+
   sandbox.restore();
 });
