@@ -1,17 +1,13 @@
 "use strict";
 
-const {
-  ExperimentAPI,
-  _ExperimentFeature: ExperimentFeature,
-} = ChromeUtils.import("resource://nimbus/ExperimentAPI.jsm");
+const { ExperimentAPI, _ExperimentFeature: ExperimentFeature } =
+  ChromeUtils.importESModule("resource://nimbus/ExperimentAPI.sys.mjs");
 
-const { JsonSchema } = ChromeUtils.import(
-  "resource://gre/modules/JsonSchema.jsm"
+const { JsonSchema } = ChromeUtils.importESModule(
+  "resource://gre/modules/JsonSchema.sys.mjs"
 );
 
-Cu.importGlobalProperties(["fetch"]);
-
-XPCOMUtils.defineLazyGetter(this, "fetchSchema", () => {
+ChromeUtils.defineLazyGetter(this, "fetchSchema", () => {
   return fetch("resource://nimbus/schemas/NimbusEnrollment.schema.json", {
     credentials: "omit",
   }).then(rsp => rsp.json());
@@ -21,10 +17,11 @@ const NON_MATCHING_ROLLOUT = Object.freeze(
   ExperimentFakes.rollout("non-matching-rollout", {
     branch: {
       slug: "slug",
+      ratio: 1,
       features: [
         {
           featureId: "aboutwelcome",
-          value: { skipFocus: false, enabled: false },
+          value: { enabled: false },
         },
       ],
     },
@@ -34,10 +31,11 @@ const MATCHING_ROLLOUT = Object.freeze(
   ExperimentFakes.rollout("matching-rollout", {
     branch: {
       slug: "slug",
+      ratio: 1,
       features: [
         {
           featureId: "aboutwelcome",
-          value: { skipFocus: false, enabled: true },
+          value: { enabled: false },
         },
       ],
     },
@@ -58,9 +56,6 @@ const AW_FAKE_MANIFEST = {
     enabled: {
       type: "boolean",
     },
-    skipFocus: {
-      type: "boolean",
-    },
   },
 };
 
@@ -70,7 +65,7 @@ async function setupForExperimentFeature() {
 
   await manager.onStartup();
 
-  sandbox.stub(ExperimentAPI, "_store").get(() => manager.store);
+  sandbox.stub(ExperimentAPI, "_manager").get(() => manager);
 
   return { sandbox, manager };
 }
@@ -94,13 +89,16 @@ add_task(async function readyCallAfterStore_with_remote_value() {
   let { sandbox, manager } = await setupForExperimentFeature();
   let feature = new ExperimentFeature("aboutwelcome");
 
-  Assert.ok(feature.getVariable("skipFocus"), "Feature is true by default");
+  Assert.ok(feature.getVariable("enabled"), "Feature is true by default");
 
   await manager.store.addEnrollment(MATCHING_ROLLOUT);
 
-  Assert.ok(!feature.getVariable("skipFocus"), "Loads value from store");
-  manager.store._deleteForTests("aboutwelcome");
+  Assert.ok(!feature.getVariable("enabled"), "Loads value from store");
+
+  manager.unenroll(MATCHING_ROLLOUT.slug, "test-cleanup");
+
   sandbox.restore();
+  assertEmptyStore(manager.store);
 });
 
 add_task(async function has_sync_value_before_ready() {
@@ -128,7 +126,11 @@ add_task(async function has_sync_value_before_ready() {
 
   Assert.equal(feature.getVariable("remoteValue"), true, "Sync load from pref");
 
-  manager.store._deleteForTests("aboutwelcome");
+  Services.prefs.clearUserPref("nimbus.syncdefaultsstore.aboutwelcome");
+  Services.prefs.clearUserPref(
+    "nimbus.syncdefaultsstore.aboutwelcome.remoteValue"
+  );
+  assertEmptyStore(manager.store);
 });
 
 add_task(async function update_remote_defaults_onUpdate() {
@@ -144,8 +146,10 @@ add_task(async function update_remote_defaults_onUpdate() {
   Assert.equal(stub.callCount, 1, "Called once for remote configs");
   Assert.equal(stub.firstCall.args[1], "rollout-updated", "Correct reason");
 
-  manager.store._deleteForTests("aboutwelcome");
+  manager.unenroll(MATCHING_ROLLOUT.slug, "test-cleanup");
+
   sandbox.restore();
+  assertEmptyStore(manager.store);
 });
 
 add_task(async function test_features_over_feature() {
@@ -155,6 +159,7 @@ add_task(async function test_features_over_feature() {
     ExperimentFakes.rollout("matching-rollout", {
       branch: {
         slug: "slug",
+        ratio: 1,
         feature: {
           featureId: "aboutwelcome",
           value: { enabled: false },
@@ -162,7 +167,7 @@ add_task(async function test_features_over_feature() {
         features: [
           {
             featureId: "aboutwelcome",
-            value: { skipFocus: false, enabled: true },
+            value: { enabled: true },
           },
         ],
       },
@@ -172,6 +177,7 @@ add_task(async function test_features_over_feature() {
     ExperimentFakes.rollout("matching-rollout", {
       branch: {
         slug: "slug",
+        ratio: 1,
         feature: {
           featureId: "aboutwelcome",
           value: { enabled: false },
@@ -197,6 +203,9 @@ add_task(async function test_features_over_feature() {
 
   manager.store._deleteForTests("aboutwelcome");
   manager.store._deleteForTests("matching-rollout");
+
+  assertEmptyStore(manager.store);
+
   sandbox.restore();
 });
 
@@ -215,7 +224,8 @@ add_task(async function update_remote_defaults_readyPromise() {
     "Update called after enrollment processed."
   );
 
-  manager.store._deleteForTests("aboutwelcome");
+  manager.unenroll(MATCHING_ROLLOUT.slug, "test-cleanup");
+  assertEmptyStore(manager.store);
   sandbox.restore();
 });
 
@@ -224,7 +234,7 @@ add_task(async function update_remote_defaults_enabled() {
   let feature = new ExperimentFeature("aboutwelcome");
 
   Assert.equal(
-    feature.isEnabled(),
+    feature.getVariable("enabled"),
     true,
     "Feature is enabled by manifest.variables.enabled"
   );
@@ -232,11 +242,12 @@ add_task(async function update_remote_defaults_enabled() {
   await manager.store.addEnrollment(NON_MATCHING_ROLLOUT);
 
   Assert.ok(
-    !feature.isEnabled(),
+    !feature.getVariable("enabled"),
     "Feature is disabled by remote configuration"
   );
 
-  manager.store._deleteForTests("aboutwelcome");
+  manager.unenroll(NON_MATCHING_ROLLOUT.slug, "test-cleanup");
+  assertEmptyStore(manager.store);
   sandbox.restore();
 });
 
@@ -259,6 +270,7 @@ add_task(async function test_getVariable_no_mutation() {
 
   Assert.ok(feature.getVariable("mochitest"), "Got back the expected feature");
 
+  assertEmptyStore(manager.store);
   sandbox.restore();
 });
 
@@ -267,6 +279,7 @@ add_task(async function remote_isEarlyStartup_config() {
   let rollout = ExperimentFakes.rollout("password-autocomplete", {
     branch: {
       slug: "remote-config-isEarlyStartup",
+      ratio: 1,
       features: [
         {
           featureId: "password-autocomplete",
@@ -291,4 +304,7 @@ add_task(async function remote_isEarlyStartup_config() {
   Services.prefs.clearUserPref(
     "nimbus.syncdefaultsstore.password-autocomplete"
   );
+
+  manager.unenroll(rollout.slug, "test-cleanup");
+  assertEmptyStore(manager.store);
 });
