@@ -39,66 +39,6 @@ add_task(async function test_usageBeforeInitialization() {
   );
 });
 
-add_task(async function test_event_add_experiment() {
-  const sandbox = sinon.createSandbox();
-  const store = ExperimentFakes.store();
-  const expected = ExperimentFakes.experiment("foo");
-  const updateEventCbStub = sandbox.stub();
-
-  // Setup ExperimentManager and child store for ExperimentAPI
-  await store.init();
-
-  // Set update cb
-  store.on("update:foo", updateEventCbStub);
-
-  // Add some data
-  store.addEnrollment(expected);
-
-  Assert.equal(updateEventCbStub.callCount, 1, "Called once for add");
-
-  store.off("update:foo", updateEventCbStub);
-});
-
-add_task(async function test_event_updates_main() {
-  const sandbox = sinon.createSandbox();
-  const store = ExperimentFakes.store();
-  const experiment = ExperimentFakes.experiment("foo");
-  const updateEventCbStub = sandbox.stub();
-
-  // Setup ExperimentManager and child store for ExperimentAPI
-  await store.init();
-
-  // Set update cb
-  store.on(
-    `featureUpdate:${experiment.branch.features[0].featureId}`,
-    updateEventCbStub
-  );
-
-  store.addEnrollment(experiment);
-  store.updateExperiment("foo", { active: false });
-
-  Assert.equal(
-    updateEventCbStub.callCount,
-    2,
-    "Should be called twice: add, update"
-  );
-  Assert.equal(
-    updateEventCbStub.firstCall.args[1],
-    "experiment-updated",
-    "Should be called with updated experiment status"
-  );
-  Assert.equal(
-    updateEventCbStub.secondCall.args[1],
-    "experiment-updated",
-    "Should be called with updated experiment status"
-  );
-
-  store.off(
-    `featureUpdate:${experiment.branch.features[0].featureId}`,
-    updateEventCbStub
-  );
-});
-
 add_task(async function test_getExperimentForGroup() {
   const store = ExperimentFakes.store();
   const experiment = ExperimentFakes.experiment("foo", {
@@ -481,13 +421,9 @@ add_task(async function test_getRolloutForFeature_fromSyncCache() {
       features: [{ featureId: "aboutwelcome", value: { enabled: true } }],
     },
   });
-  let updatePromise = new Promise(resolve =>
-    store.on(`update:${rollout.slug}`, resolve)
-  );
 
   await store.init();
   store.addEnrollment(rollout);
-  await updatePromise;
   // New uninitialized store will return data from sync cache
   // before init
   store = ExperimentFakes.store();
@@ -518,28 +454,18 @@ add_task(async function test_remoteRollout() {
     },
   });
   let featureUpdateStub = sinon.stub();
-  let updatePromise = new Promise(resolve =>
-    store.on(`update:${rollout.slug}`, resolve)
-  );
   store.on("featureUpdate:aboutwelcome", featureUpdateStub);
 
   await store.init();
   store.addEnrollment(rollout);
-  await updatePromise;
 
   Assert.ok(
     Services.prefs.getStringPref(`${SYNC_DEFAULTS_PREF_BRANCH}aboutwelcome`),
     "Sync cache is set"
   );
 
-  updatePromise = new Promise(resolve =>
-    store.on(`update:${rollout.slug}`, resolve)
-  );
   store.updateExperiment(rollout.slug, { active: false });
 
-  // wait for it to be removed
-  await updatePromise;
-
   Assert.ok(featureUpdateStub.calledTwice, "Called for add and remove");
   Assert.ok(
     store.get(rollout.slug),
@@ -811,46 +737,59 @@ add_task(async function test_storeValuePerPref_returnsSameValue_allTypes() {
 });
 
 add_task(async function test_cleanupOldRecipes() {
-  let store = ExperimentFakes.store();
-  let sandbox = sinon.createSandbox();
-  let stub = sandbox.stub(store, "_removeEntriesByKeys");
-  const experiment1 = ExperimentFakes.experiment("foo", {
-    branch: {
-      slug: "variant",
-      features: [{ featureId: "purple" }],
-    },
+  const store = ExperimentFakes.store();
+  const sandbox = sinon.createSandbox();
+  const stub = sandbox.stub(store, "_removeEntriesByKeys");
+
+  const NOW = Date.now();
+  const SIX_HOURS = 6 * 3600 * 1000;
+  const ONE_DAY = 4 * SIX_HOURS;
+  const ONE_YEAR = 365.25 * 24 * 3600 * 1000;
+  const ONE_MONTH = Math.floor(ONE_YEAR / 12);
+
+  const active = ExperimentFakes.experiment("active-6hrs", {
+    active: true,
+    lastSeen: new Date(NOW - SIX_HOURS),
   });
-  const experiment2 = ExperimentFakes.experiment("bar", {
-    branch: {
-      slug: "variant",
-      features: [{ featureId: "purple" }],
-    },
+
+  const inactiveToday = ExperimentFakes.experiment("inactive-recent", {
+    active: false,
+    lastSeen: new Date(NOW - SIX_HOURS),
   });
-  const experiment3 = ExperimentFakes.experiment("baz", {
-    branch: {
-      slug: "variant",
-      features: [{ featureId: "purple" }],
-    },
+
+  const inactiveSixMonths = ExperimentFakes.experiment("inactive-6mo", {
+    active: false,
+    lastSeen: new Date(NOW - 6 * ONE_MONTH),
   });
-  const experiment4 = ExperimentFakes.experiment("faz", {
-    branch: {
-      slug: "variant",
-      features: [{ featureId: "purple" }],
-    },
+
+  const inactiveUnderTwelveMonths = ExperimentFakes.experiment(
+    "inactive-under-12mo",
+    {
+      active: false,
+      lastSeen: new Date(NOW - ONE_YEAR + ONE_DAY),
+    }
+  );
+
+  const inactiveOverTwelveMonths = ExperimentFakes.experiment(
+    "inactive-over-12mo",
+    {
+      active: false,
+      lastSeen: new Date(NOW - ONE_YEAR - ONE_DAY),
+    }
+  );
+
+  const inactiveNoLastSeen = ExperimentFakes.experiment("inactive-unknown", {
+    active: false,
   });
-  // Exp 2 is kept because it's recent (even though it's not active)
-  // Exp 4 is kept because it's active
-  experiment2.lastSeen = new Date().toISOString();
-  experiment2.active = false;
-  experiment1.lastSeen = new Date("2020-01-01").toISOString();
-  experiment1.active = false;
-  experiment3.active = false;
-  delete experiment3.lastSeen;
+  delete inactiveNoLastSeen.lastSeen;
+
   store._data = {
-    foo: experiment1,
-    bar: experiment2,
-    baz: experiment3,
-    faz: experiment4,
+    [active.slug]: active,
+    [inactiveToday.slug]: inactiveToday,
+    [inactiveSixMonths.slug]: inactiveSixMonths,
+    [inactiveUnderTwelveMonths.slug]: inactiveUnderTwelveMonths,
+    [inactiveOverTwelveMonths.slug]: inactiveOverTwelveMonths,
+    [inactiveNoLastSeen.slug]: inactiveNoLastSeen,
   };
 
   store._cleanupOldRecipes();
@@ -863,12 +802,12 @@ add_task(async function test_cleanupOldRecipes() {
   );
   Assert.equal(
     stub.firstCall.args[0][0],
-    experiment1.slug,
+    inactiveOverTwelveMonths.slug,
     "Should remove expired enrollment"
   );
   Assert.equal(
     stub.firstCall.args[0][1],
-    experiment3.slug,
+    inactiveNoLastSeen.slug,
     "Should remove invalid enrollment"
   );
 });
