@@ -6,12 +6,12 @@ const { ExperimentAPI, _ExperimentFeature: ExperimentFeature } =
 const { ExperimentFakes, ExperimentTestUtils } = ChromeUtils.importESModule(
   "resource://testing-common/NimbusTestUtils.sys.mjs"
 );
+const { MatchStatus } = ChromeUtils.importESModule(
+  "resource://nimbus/lib/RemoteSettingsExperimentLoader.sys.mjs"
+);
 const { TelemetryTestUtils } = ChromeUtils.importESModule(
   "resource://testing-common/TelemetryTestUtils.sys.mjs"
 );
-const { TelemetryEvents } = ChromeUtils.importESModule(
-  "resource://normandy/lib/TelemetryEvents.sys.mjs"
-);
 
 const LOCALIZATIONS = {
   "en-US": {
@@ -122,7 +122,6 @@ add_setup(function setup() {
   do_get_profile();
 
   Services.fog.initializeFOG();
-  TelemetryEvents.init();
 
   registerCleanupFunction(ExperimentTestUtils.addTestFeatures(FEATURE));
   registerCleanupFunction(resetTelemetry);
@@ -210,7 +209,6 @@ add_task(async function test_getLocalizedValue() {
   const manager = ExperimentFakes.manager();
 
   sandbox.stub(ExperimentAPI, "_manager").get(() => manager);
-  sandbox.stub(ExperimentAPI, "_store").get(() => manager.store);
 
   await manager.onStartup();
   await manager.store.ready();
@@ -219,6 +217,7 @@ add_task(async function test_getLocalizedValue() {
     branches: [
       {
         slug: "control",
+        ratio: 1,
         features: [
           {
             featureId: FEATURE_ID,
@@ -230,9 +229,8 @@ add_task(async function test_getLocalizedValue() {
     localizations: LOCALIZATIONS,
   });
 
-  const { enrollmentPromise, doExperimentCleanup } =
-    ExperimentFakes.enrollmentHelper(experiment);
-  await enrollmentPromise;
+  const doExperimentCleanup =
+    await ExperimentFakes.enrollmentHelper(experiment);
 
   const enrollment = manager.store.getExperimentForFeature(FEATURE_ID);
 
@@ -254,7 +252,7 @@ add_task(async function test_getLocalizedValue() {
     "_getLocalizedValue() with a nested localization"
   );
 
-  await doExperimentCleanup();
+  doExperimentCleanup();
   await cleanupStore(manager.store);
   sandbox.reset();
 });
@@ -266,7 +264,6 @@ add_task(async function test_getLocalizedValue_unenroll_missingEntry() {
   const manager = ExperimentFakes.manager();
 
   sandbox.stub(ExperimentAPI, "_manager").get(() => manager);
-  sandbox.stub(ExperimentAPI, "_store").get(() => manager.store);
 
   await manager.onStartup();
   await manager.store.ready();
@@ -275,6 +272,7 @@ add_task(async function test_getLocalizedValue_unenroll_missingEntry() {
     branches: [
       {
         slug: "control",
+        ratio: 1,
         features: [
           {
             featureId: FEATURE_ID,
@@ -294,7 +292,7 @@ add_task(async function test_getLocalizedValue_unenroll_missingEntry() {
     localizations: LOCALIZATIONS,
   });
 
-  await ExperimentFakes.enrollmentHelper(experiment).enrollmentPromise;
+  await ExperimentFakes.enrollmentHelper(experiment);
 
   const enrollment = manager.store.getExperimentForFeature(FEATURE_ID);
 
@@ -310,7 +308,7 @@ add_task(async function test_getLocalizedValue_unenroll_missingEntry() {
     "Experiment should be unenrolled"
   );
 
-  const gleanEvents = Glean.nimbusEvents.unenrollment.testGetValue();
+  const gleanEvents = Glean.nimbusEvents.unenrollment.testGetValue("events");
   Assert.equal(gleanEvents.length, 1, "Should be one unenrollment event");
   Assert.equal(
     gleanEvents[0].extra.reason,
@@ -348,7 +346,6 @@ add_task(async function test_getLocalizedValue_unenroll_missingEntry() {
   const manager = ExperimentFakes.manager();
 
   sandbox.stub(ExperimentAPI, "_manager").get(() => manager);
-  sandbox.stub(ExperimentAPI, "_store").get(() => manager.store);
 
   await manager.onStartup();
   await manager.store.ready();
@@ -357,6 +354,7 @@ add_task(async function test_getLocalizedValue_unenroll_missingEntry() {
     branches: [
       {
         slug: "control",
+        ratio: 1,
         features: [
           {
             featureId: FEATURE_ID,
@@ -378,7 +376,7 @@ add_task(async function test_getLocalizedValue_unenroll_missingEntry() {
     },
   });
 
-  await ExperimentFakes.enrollmentHelper(experiment).enrollmentPromise;
+  await ExperimentFakes.enrollmentHelper(experiment);
 
   const enrollment = manager.store.getExperimentForFeature(FEATURE_ID);
 
@@ -394,7 +392,7 @@ add_task(async function test_getLocalizedValue_unenroll_missingEntry() {
     "Experiment should be unenrolled"
   );
 
-  const gleanEvents = Glean.nimbusEvents.unenrollment.testGetValue();
+  const gleanEvents = Glean.nimbusEvents.unenrollment.testGetValue("events");
   Assert.equal(gleanEvents.length, 1, "Should be one unenrollment event");
   Assert.equal(
     gleanEvents[0].extra.reason,
@@ -430,7 +428,6 @@ add_task(async function test_getVariables() {
   const manager = ExperimentFakes.manager();
 
   sandbox.stub(ExperimentAPI, "_manager").get(() => manager);
-  sandbox.stub(ExperimentAPI, "_store").get(() => manager.store);
 
   await manager.onStartup();
   await manager.store.ready();
@@ -439,6 +436,7 @@ add_task(async function test_getVariables() {
     branches: [
       {
         slug: "control",
+        ratio: 1,
         features: [
           {
             featureId: FEATURE_ID,
@@ -450,9 +448,8 @@ add_task(async function test_getVariables() {
     localizations: LOCALIZATIONS,
   });
 
-  const { enrollmentPromise, doExperimentCleanup } =
-    ExperimentFakes.enrollmentHelper(experiment);
-  await enrollmentPromise;
+  const doExperimentCleanup =
+    await ExperimentFakes.enrollmentHelper(experiment);
 
   Assert.deepEqual(
     FEATURE.getAllVariables(),
@@ -484,7 +481,7 @@ add_task(async function test_getVariables() {
     "getVariable() returns substitutions inside arrays"
   );
 
-  await doExperimentCleanup();
+  doExperimentCleanup();
   await cleanupStore(manager.store);
   sandbox.reset();
 });
@@ -494,7 +491,6 @@ add_task(async function test_getVariables_fallback() {
   const manager = ExperimentFakes.manager();
 
   sandbox.stub(ExperimentAPI, "_manager").get(() => manager);
-  sandbox.stub(ExperimentAPI, "_store").get(() => manager.store);
 
   await manager.onStartup();
   await manager.store.ready();
@@ -513,6 +509,7 @@ add_task(async function test_getVariables_fallback() {
       branches: [
         {
           slug: "control",
+          ratio: 1,
           features: [
             {
               featureId: FEATURE_ID,
@@ -535,6 +532,7 @@ add_task(async function test_getVariables_fallback() {
       branches: [
         {
           slug: "control",
+          ratio: 1,
           features: [
             {
               featureId: FEATURE_ID,
@@ -584,13 +582,7 @@ add_task(async function test_getVariables_fallback() {
   );
 
   // Enroll in the rollout.
-  {
-    const { enrollmentPromise, doExperimentCleanup } =
-      ExperimentFakes.enrollmentHelper(recipes.rollout);
-    await enrollmentPromise;
-
-    cleanup.rollout = doExperimentCleanup;
-  }
+  cleanup.rollout = await ExperimentFakes.enrollmentHelper(recipes.rollout);
 
   Assert.deepEqual(
     FEATURE.getAllVariables({ defaultValues: { waldo: ["default-value"] } }),
@@ -620,13 +612,9 @@ add_task(async function test_getVariables_fallback() {
   );
 
   // Enroll in the experiment.
-  {
-    const { enrollmentPromise, doExperimentCleanup } =
-      ExperimentFakes.enrollmentHelper(recipes.experiment);
-    await enrollmentPromise;
-
-    cleanup.experiment = doExperimentCleanup;
-  }
+  cleanup.experiment = await ExperimentFakes.enrollmentHelper(
+    recipes.experiment
+  );
 
   Assert.deepEqual(
     FEATURE.getAllVariables({ defaultValues: { waldo: ["default-value"] } }),
@@ -729,7 +717,6 @@ add_task(async function test_getVariables_fallback_unenroll() {
   const manager = ExperimentFakes.manager();
 
   sandbox.stub(ExperimentAPI, "_manager").get(() => manager);
-  sandbox.stub(ExperimentAPI, "_store").get(() => manager.store);
 
   await manager.onStartup();
   await manager.store.ready();
@@ -756,6 +743,7 @@ add_task(async function test_getVariables_fallback_unenroll() {
       branches: [
         {
           slug: "control",
+          ratio: 1,
           features: [
             {
               featureId: FEATURE_ID,
@@ -774,6 +762,7 @@ add_task(async function test_getVariables_fallback_unenroll() {
       branches: [
         {
           slug: "control",
+          ratio: 1,
           features: [
             {
               featureId: FEATURE_ID,
@@ -791,7 +780,7 @@ add_task(async function test_getVariables_fallback_unenroll() {
   ];
 
   for (const recipe of recipes) {
-    await ExperimentFakes.enrollmentHelper(recipe).enrollmentPromise;
+    await ExperimentFakes.enrollmentHelper(recipe);
   }
 
   Assert.deepEqual(FEATURE.getAllVariables(), {
@@ -813,7 +802,7 @@ add_task(async function test_getVariables_fallback_unenroll() {
     "Rollout should be unenrolled"
   );
 
-  const gleanEvents = Glean.nimbusEvents.unenrollment.testGetValue();
+  const gleanEvents = Glean.nimbusEvents.unenrollment.testGetValue("events");
   Assert.equal(gleanEvents.length, 2, "Should be two unenrollment events");
   Assert.equal(
     gleanEvents[0].extra.reason,
@@ -866,10 +855,13 @@ add_task(async function test_updateRecipes() {
 
   loader.manager = manager;
   sandbox.stub(ExperimentAPI, "_manager").get(() => manager);
-  sandbox.stub(ExperimentAPI, "_store").get(() => manager.store);
   sandbox.stub(manager, "onRecipe");
 
   const recipe = ExperimentFakes.recipe("foo", {
+    bucketConfig: {
+      ...ExperimentFakes.recipe.bucketConfig,
+      count: 1000,
+    },
     branches: [
       {
         slug: "control",
@@ -885,15 +877,22 @@ add_task(async function test_updateRecipes() {
     localizations: LOCALIZATIONS,
   });
 
-  await loader.init();
-
   await manager.onStartup();
   await manager.store.ready();
+  await loader.enable();
 
-  sandbox.stub(loader.remoteSettingsClient, "get").resolves([recipe]);
+  sandbox
+    .stub(loader.remoteSettingsClients.experiments, "get")
+    .resolves([recipe]);
   await loader.updateRecipes();
 
-  Assert.ok(manager.onRecipe.calledOnce, "Enrolled");
+  Assert.ok(
+    manager.onRecipe.calledOnceWith(recipe, "rs-loader", {
+      ok: true,
+      status: MatchStatus.TARGETING_AND_BUCKETING,
+    }),
+    "would enroll"
+  );
 
   await cleanupStore(manager.store);
   sandbox.reset();
@@ -901,7 +900,6 @@ add_task(async function test_updateRecipes() {
 
 async function test_updateRecipes_missingLocale({
   featureValidationOptOut = false,
-  validationEnabled = true,
 } = {}) {
   resetTelemetry();
 
@@ -911,11 +909,14 @@ async function test_updateRecipes_missingLocale({
 
   loader.manager = manager;
   sandbox.stub(ExperimentAPI, "_manager").get(() => manager);
-  sandbox.stub(ExperimentAPI, "_store").get(() => manager.store);
-  sandbox.stub(manager, "onRecipe");
-  sandbox.spy(manager, "onFinalize");
+  sandbox.spy(manager, "onRecipe");
+  sandbox.stub(manager, "enroll");
 
   const recipe = ExperimentFakes.recipe("foo", {
+    bucketConfig: {
+      ...ExperimentFakes.recipe.bucketConfig,
+      count: 1000,
+    },
     branches: [
       {
         slug: "control",
@@ -932,30 +933,27 @@ async function test_updateRecipes_missingLocale({
     featureValidationOptOut,
   });
 
-  await loader.init();
-
   await manager.onStartup();
   await manager.store.ready();
+  await loader.enable();
 
-  sandbox.stub(loader.remoteSettingsClient, "get").resolves([recipe]);
+  sandbox
+    .stub(loader.remoteSettingsClients.experiments, "get")
+    .resolves([recipe]);
   await loader.updateRecipes();
 
-  Assert.ok(!manager.onRecipe.called, "Did not enroll in the recipe");
   Assert.ok(
-    onFinalizeCalled(manager.onFinalize, "rs-loader", {
-      recipeMismatches: [],
-      invalidRecipes: [],
-      invalidBranches: new Map(),
-      invalidFeatures: new Map(),
-      missingLocale: ["foo"],
-      missingL10nIds: new Map(),
+    manager.onRecipe.calledOnceWith(recipe, "rs-loader", {
+      ok: false,
+      reason: "l10n-missing-locale",
       locale: "en-US",
-      validationEnabled,
     }),
-    "should call .onFinalize with missing locale"
+    "Called onRecipe with missing locale"
   );
+  Assert.ok(manager.enroll.notCalled, "Did not enroll");
 
-  const gleanEvents = Glean.nimbusEvents.validationFailed.testGetValue();
+  const gleanEvents =
+    Glean.nimbusEvents.validationFailed.testGetValue("events");
   Assert.equal(gleanEvents.length, 1, "Should be one validationFailed event");
   Assert.equal(
     gleanEvents[0].extra.experiment,
@@ -997,9 +995,8 @@ add_task(async function test_updateRecipes_missingEntry() {
 
   loader.manager = manager;
   sandbox.stub(ExperimentAPI, "_manager").get(() => manager);
-  sandbox.stub(ExperimentAPI, "_store").get(() => manager.store);
-  sandbox.stub(manager, "onRecipe");
-  sandbox.spy(manager, "onFinalize");
+  sandbox.spy(manager, "onRecipe");
+  sandbox.stub(manager, "enroll");
 
   const recipe = ExperimentFakes.recipe("foo", {
     branches: [
@@ -1019,30 +1016,37 @@ add_task(async function test_updateRecipes_missingEntry() {
     },
   });
 
-  await loader.init();
-
   await manager.onStartup();
   await manager.store.ready();
+  await loader.enable();
 
-  sandbox.stub(loader.remoteSettingsClient, "get").resolves([recipe]);
+  sandbox
+    .stub(loader.remoteSettingsClients.experiments, "get")
+    .resolves([recipe]);
   await loader.updateRecipes();
 
-  Assert.ok(!manager.onRecipe.called, "Did not enroll in the recipe");
   Assert.ok(
-    onFinalizeCalled(manager.onFinalize, "rs-loader", {
-      recipeMismatches: [],
-      invalidRecipes: [],
-      invalidBranches: new Map(),
-      invalidFeatures: new Map(),
-      missingLocale: [],
-      missingL10nIds: new Map([["foo", ["foo", "qux", "grault", "waldo"]]]),
-      locale: "en-US",
-      validationEnabled: true,
-    }),
-    "should call .onFinalize with missing locale"
+    manager.onRecipe.calledOnceWith(
+      recipe,
+      "rs-loader",
+      sinon.match({
+        ok: false,
+        reason: "l10n-missing-entry",
+        locale: "en-US",
+        missingL10nIds: sinon.match.array.contains([
+          "foo",
+          "qux",
+          "grault",
+          "waldo",
+        ]),
+      })
+    ),
+    "Called onRecipe with missing l10n ids"
   );
+  Assert.ok(manager.enroll.notCalled, "Did not enroll");
 
-  const gleanEvents = Glean.nimbusEvents.validationFailed.testGetValue();
+  const gleanEvents =
+    Glean.nimbusEvents.validationFailed.testGetValue("events");
   Assert.equal(gleanEvents.length, 1, "Should be one validationFailed event");
   Assert.equal(
     gleanEvents[0].extra.experiment,
@@ -1088,7 +1092,7 @@ add_task(async function test_updateRecipes_validationDisabled_pref() {
 
   Services.prefs.setBoolPref("nimbus.validation.enabled", false);
 
-  await test_updateRecipes_missingLocale({ validationEnabled: false });
+  await test_updateRecipes_missingLocale();
 
   Services.prefs.clearUserPref("nimbus.validation.enabled");
 });
@@ -1108,10 +1112,8 @@ add_task(async function test_updateRecipes_unenroll_missingEntry() {
 
   loader.manager = manager;
   sandbox.stub(ExperimentAPI, "_manager").get(() => manager);
-  sandbox.stub(ExperimentAPI, "_store").get(() => manager.store);
-  sandbox.spy(manager, "onRecipe");
-  sandbox.spy(manager, "onFinalize");
-  sandbox.spy(manager, "unenroll");
+  sandbox.spy(manager, "updateEnrollment");
+  sandbox.spy(manager, "_unenroll");
 
   const recipe = ExperimentFakes.recipe("foo", {
     branches: [
@@ -1129,14 +1131,11 @@ add_task(async function test_updateRecipes_unenroll_missingEntry() {
     localizations: LOCALIZATIONS,
   });
 
-  await loader.init();
-
   await manager.onStartup();
   await manager.store.ready();
+  await loader.enable();
 
-  await ExperimentFakes.enrollmentHelper(recipe, {
-    source: "rs-loader",
-  }).enrollmentPromise;
+  await manager.enroll(recipe, "rs-loader");
   Assert.ok(
     !!manager.store.getExperimentForFeature(FEATURE_ID),
     "Should be enrolled in the experiment"
@@ -1144,27 +1143,37 @@ add_task(async function test_updateRecipes_unenroll_missingEntry() {
 
   const badRecipe = { ...recipe, localizations: { "en-US": {} } };
 
-  sandbox.stub(loader.remoteSettingsClient, "get").resolves([badRecipe]);
+  sandbox
+    .stub(loader.remoteSettingsClients.experiments, "get")
+    .resolves([badRecipe]);
 
   await loader.updateRecipes();
 
   Assert.ok(
-    onFinalizeCalled(manager.onFinalize, "rs-loader", {
-      recipeMismatches: [],
-      invalidRecipes: [],
-      invalidBranches: new Map(),
-      invalidFeatures: new Map(),
-      missingLocale: [],
-      missingL10nIds: new Map([
-        [recipe.slug, ["foo", "qux", "grault", "waldo"]],
-      ]),
-      locale: "en-US",
-      validationEnabled: true,
-    }),
-    "should call .onFinalize with missing l10n entry"
+    manager.updateEnrollment.calledOnceWith(
+      sinon.match({ slug: recipe.slug }),
+      badRecipe,
+      "rs-loader",
+      sinon.match({
+        ok: false,
+        reason: "l10n-missing-entry",
+        locale: "en-US",
+        missingL10nIds: sinon.match.array.contains([
+          "foo",
+          "qux",
+          "grault",
+          "waldo",
+        ]),
+      })
+    ),
+    "Should call .onRecipe with the missing l10n entries"
   );
 
-  Assert.ok(manager.unenroll.calledWith(recipe.slug, "l10n-missing-entry"));
+  Assert.ok(
+    manager._unenroll.calledOnceWith(sinon.match({ slug: recipe.slug }), {
+      reason: "l10n-missing-entry",
+    })
+  );
 
   Assert.equal(
     manager.store.getExperimentForFeature(FEATURE_ID),
@@ -1172,7 +1181,7 @@ add_task(async function test_updateRecipes_unenroll_missingEntry() {
     "Should no longer be enrolled in the experiment"
   );
 
-  const unenrollEvents = Glean.nimbusEvents.unenrollment.testGetValue();
+  const unenrollEvents = Glean.nimbusEvents.unenrollment.testGetValue("events");
   Assert.equal(unenrollEvents.length, 1, "Should be one unenroll event");
   Assert.equal(
     unenrollEvents[0].extra.experiment,
@@ -1186,7 +1195,7 @@ add_task(async function test_updateRecipes_unenroll_missingEntry() {
   );
 
   const validationFailedEvents =
-    Glean.nimbusEvents.validationFailed.testGetValue();
+    Glean.nimbusEvents.validationFailed.testGetValue("events");
   Assert.equal(
     validationFailedEvents.length,
     1,
@@ -1261,10 +1270,8 @@ add_task(async function test_updateRecipes_unenroll_missingLocale() {
 
   loader.manager = manager;
   sandbox.stub(ExperimentAPI, "_manager").get(() => manager);
-  sandbox.stub(ExperimentAPI, "_store").get(() => manager.store);
-  sandbox.spy(manager, "onRecipe");
-  sandbox.spy(manager, "onFinalize");
-  sandbox.spy(manager, "unenroll");
+  sandbox.spy(manager, "updateEnrollment");
+  sandbox.spy(manager, "_unenroll");
 
   const recipe = ExperimentFakes.recipe("foo", {
     branches: [
@@ -1282,14 +1289,11 @@ add_task(async function test_updateRecipes_unenroll_missingLocale() {
     localizations: LOCALIZATIONS,
   });
 
-  await loader.init();
-
   await manager.onStartup();
   await manager.store.ready();
+  await loader.enable();
 
-  await ExperimentFakes.enrollmentHelper(recipe, {
-    source: "rs-loader",
-  }).enrollmentPromise;
+  await manager.enroll(recipe, "rs-loader");
   Assert.ok(
     !!manager.store.getExperimentForFeature(FEATURE_ID),
     "Should be enrolled in the experiment"
@@ -1300,25 +1304,31 @@ add_task(async function test_updateRecipes_unenroll_missingLocale() {
     localizations: {},
   };
 
-  sandbox.stub(loader.remoteSettingsClient, "get").resolves([badRecipe]);
+  sandbox
+    .stub(loader.remoteSettingsClients.experiments, "get")
+    .resolves([badRecipe]);
 
   await loader.updateRecipes();
 
   Assert.ok(
-    onFinalizeCalled(manager.onFinalize, "rs-loader", {
-      recipeMismatches: [],
-      invalidRecipes: [],
-      invalidBranches: new Map(),
-      invalidFeatures: new Map(),
-      missingLocale: ["foo"],
-      missingL10nIds: new Map(),
-      locale: "en-US",
-      validationEnabled: true,
-    }),
-    "should call .onFinalize with missing locale"
+    manager.updateEnrollment.calledOnceWith(
+      sinon.match({ slug: recipe.slug }),
+      badRecipe,
+      "rs-loader",
+      {
+        ok: false,
+        reason: "l10n-missing-locale",
+        locale: "en-US",
+      }
+    ),
+    "Should call .onFinal with missing-locale"
   );
 
-  Assert.ok(manager.unenroll.calledWith(recipe.slug, "l10n-missing-locale"));
+  Assert.ok(
+    manager._unenroll.calledWith(sinon.match({ slug: recipe.slug }), {
+      reason: "l10n-missing-locale",
+    })
+  );
 
   Assert.equal(
     manager.store.getExperimentForFeature(FEATURE_ID),
@@ -1326,7 +1336,7 @@ add_task(async function test_updateRecipes_unenroll_missingLocale() {
     "Should no longer be enrolled in the experiment"
   );
 
-  const unenrollEvents = Glean.nimbusEvents.unenrollment.testGetValue();
+  const unenrollEvents = Glean.nimbusEvents.unenrollment.testGetValue("events");
   Assert.equal(unenrollEvents.length, 1, "Should be one unenroll event");
   Assert.equal(
     unenrollEvents[0].extra.experiment,
@@ -1340,7 +1350,7 @@ add_task(async function test_updateRecipes_unenroll_missingLocale() {
   );
 
   const validationFailedEvents =
-    Glean.nimbusEvents.validationFailed.testGetValue();
+    Glean.nimbusEvents.validationFailed.testGetValue("events");
   Assert.equal(
     validationFailedEvents.length,
     1,
