@@ -1,6 +1,6 @@
 const PAGEURI = NetUtil.newURI("http://deliciousbacon.com/");
 
-add_task(async function() {
+add_task(async function () {
   // First, add a history entry or else Places can't save a favicon.
   await PlacesTestUtils.addVisits({
     uri: PAGEURI,
@@ -8,22 +8,25 @@ add_task(async function() {
     visitDate: Date.now() * 1000,
   });
 
-  await new Promise(resolve => {
-    function onSetComplete(aURI, aDataLen, aData, aMimeType, aWidth) {
-      equal(aURI.spec, SMALLSVG_DATA_URI.spec, "setFavicon aURI check");
-      equal(aDataLen, 263, "setFavicon aDataLen check");
-      equal(aMimeType, "image/svg+xml", "setFavicon aMimeType check");
-      dump(aWidth);
-      resolve();
-    }
+  await PlacesUtils.favicons.setFaviconForPage(
+    PAGEURI,
+    SMALLSVG_DATA_URI,
+    SMALLSVG_DATA_URI
+  );
 
-    PlacesUtils.favicons.setAndFetchFaviconForPage(
+  await new Promise(resolve => {
+    PlacesUtils.favicons.getFaviconDataForPage(
       PAGEURI,
-      SMALLSVG_DATA_URI,
-      false,
-      PlacesUtils.favicons.FAVICON_LOAD_NON_PRIVATE,
-      onSetComplete,
-      Services.scriptSecurityManager.getSystemPrincipal()
+      function (aURI, aDataLen, aData, aMimeType) {
+        Assert.equal(
+          aURI.spec,
+          SMALLSVG_DATA_URI.spec,
+          "setFavicon aURI check"
+        );
+        Assert.equal(aDataLen, 263, "setFavicon aDataLen check");
+        Assert.equal(aMimeType, "image/svg+xml", "setFavicon aMimeType check");
+        resolve();
+      }
     );
   });
 
