@@ -7,7 +7,6 @@
 module.exports = {
   rules: {
     "no-inner-declarations": "error",
-    "no-shadow": "error",
-    "no-unused-vars": ["error", { vars: "all", args: "none" }],
+    "no-unused-vars": ["error", { vars: "all", argsIgnorePattern: "^_" }],
   },
 };
