@@ -1,58 +1,51 @@
 /* -*- indent-tabs-mode: nil; js-indent-level: 2 -*- */
 /* vim:set ts=2 sw=2 sts=2 et: */
 
-const { XPCOMUtils } = ChromeUtils.import(
-  "resource://gre/modules/XPCOMUtils.jsm"
-);
-
-XPCOMUtils.defineLazyModuleGetters(this, {
-  clearTimeout: "resource://gre/modules/Timer.jsm",
-  FileUtils: "resource://gre/modules/FileUtils.jsm",
-  PromiseUtils: "resource://gre/modules/PromiseUtils.jsm",
-  Region: "resource://gre/modules/Region.jsm",
-  RemoteSettings: "resource://services-settings/remote-settings.js",
-  RemoteSettingsClient: "resource://services-settings/RemoteSettingsClient.jsm",
-  SearchSettings: "resource://gre/modules/SearchSettings.jsm",
-  SearchEngineSelector: "resource://gre/modules/SearchEngineSelector.jsm",
-  SearchService: "resource://gre/modules/SearchService.jsm",
-  SearchTestUtils: "resource://testing-common/SearchTestUtils.jsm",
-  Services: "resource://gre/modules/Services.jsm",
-  setTimeout: "resource://gre/modules/Timer.jsm",
-  TestUtils: "resource://testing-common/TestUtils.jsm",
-  SearchUtils: "resource://gre/modules/SearchUtils.jsm",
-  sinon: "resource://testing-common/Sinon.jsm",
+ChromeUtils.defineESModuleGetters(this, {
+  AddonTestUtils: "resource://testing-common/AddonTestUtils.sys.mjs",
+  clearTimeout: "resource://gre/modules/Timer.sys.mjs",
+  EnterprisePolicyTesting:
+    "resource://testing-common/EnterprisePolicyTesting.sys.mjs",
+  ExtensionTestUtils:
+    "resource://testing-common/ExtensionXPCShellUtils.sys.mjs",
+  FileUtils: "resource://gre/modules/FileUtils.sys.mjs",
+  HttpServer: "resource://testing-common/httpd.sys.mjs",
+  Region: "resource://gre/modules/Region.sys.mjs",
+  RemoteSettings: "resource://services-settings/remote-settings.sys.mjs",
+  RemoteSettingsClient:
+    "resource://services-settings/RemoteSettingsClient.sys.mjs",
+  SearchEngineClassification: "resource://gre/modules/RustSearch.sys.mjs",
+  SearchEngineSelector: "resource://gre/modules/SearchEngineSelector.sys.mjs",
+  SearchService: "resource://gre/modules/SearchService.sys.mjs",
+  SearchSettings: "resource://gre/modules/SearchSettings.sys.mjs",
+  SearchTestUtils: "resource://testing-common/SearchTestUtils.sys.mjs",
+  SearchUtils: "resource://gre/modules/SearchUtils.sys.mjs",
+  TestUtils: "resource://testing-common/TestUtils.sys.mjs",
+  updateAppInfo: "resource://testing-common/AppInfo.sys.mjs",
+  Utils: "resource://services-settings/Utils.sys.mjs",
+  setTimeout: "resource://gre/modules/Timer.sys.mjs",
+  sinon: "resource://testing-common/Sinon.sys.mjs",
 });
 
-var { HttpServer } = ChromeUtils.import("resource://testing-common/httpd.js");
-var { AddonTestUtils } = ChromeUtils.import(
-  "resource://testing-common/AddonTestUtils.jsm"
-);
-const { ExtensionTestUtils } = ChromeUtils.import(
-  "resource://testing-common/ExtensionXPCShellUtils.jsm"
-);
+// Expose Remote Settings utils with an explicit name.
+const RemoteSettingsUtils = Utils;
+
+// We need Services.appinfo.name set up to allow the hashes to work with a
+// consistent name.
+// Note: the name and versions here match those in ExtensionXPCShellUtils.sys.mjs.
+updateAppInfo({ name: "XPCShell", version: "48", platformVersion: "48" });
+
+// We generally also need a profile set-up, for saving search settings etc.
+do_get_profile();
 
 SearchTestUtils.init(this);
 
 const SETTINGS_FILENAME = "search.json.mozlz4";
 
-// nsSearchService.js uses Services.appinfo.name to build a salt for a hash.
-// eslint-disable-next-line mozilla/use-services
-var XULRuntime = Cc["@mozilla.org/xre/runtime;1"].getService(Ci.nsIXULRuntime);
-
 // Expand the amount of information available in error logs
 Services.prefs.setBoolPref("browser.search.log", true);
 Services.prefs.setBoolPref("browser.region.log", true);
 
-Services.prefs.setBoolPref("browser.search.modernConfig", true);
-
-AddonTestUtils.init(this, false);
-AddonTestUtils.createAppInfo(
-  "xpcshell@tests.mozilla.org",
-  "XPCShell",
-  "42",
-  "42"
-);
-
 // Allow telemetry probes which may otherwise be disabled for some applications (e.g. Thunderbird)
 Services.prefs.setBoolPref(
   "toolkit.telemetry.testing.overrideProductsCheck",
@@ -65,13 +58,13 @@ SearchSettings.SETTNGS_INVALIDATION_DELAY = 250;
 
 async function promiseSettingsData() {
   let path = PathUtils.join(PathUtils.profileDir, SETTINGS_FILENAME);
-  return JSON.parse(await IOUtils.readUTF8(path, { decompress: true }));
+  return IOUtils.readJSON(path, { decompress: true });
 }
 
 function promiseSaveSettingsData(data) {
-  return IOUtils.write(
+  return IOUtils.writeJSON(
     PathUtils.join(PathUtils.profileDir, SETTINGS_FILENAME),
-    new TextEncoder().encode(JSON.stringify(data)),
+    data,
     { compress: true }
   );
 }
@@ -146,6 +139,7 @@ const kTestEngineName = "Test search engine";
 
 /**
  * Waits for the settings file to be saved.
+ *
  * @returns {Promise} Resolved when the settings file is saved.
  */
 function promiseAfterSettings() {
@@ -203,8 +197,10 @@ async function readJSONFile(file) {
  * on actualObj.
  *
  * @param {object} expectedObj
+ *   The source object that we expect to match
  * @param {object} actualObj
- * @param {function} skipProp
+ *   The object to check against the source
+ * @param {Function} skipProp
  *   A function that is called with the property name and its value, to see if
  *   testing that property should be skipped or not.
  */
@@ -231,24 +227,22 @@ function isSubObjectOf(expectedObj, actualObj, skipProp) {
 }
 
 /**
- * After useHttpServer() is called, this string contains the URL of the "data"
- * directory, including the final slash.
+ * After useHttpServer() is called, this string contains the URL test directory,
+ * excluding the final slash.
  */
-var gDataUrl;
+var gHttpURL;
 
 /**
  * Initializes the HTTP server and ensures that it is terminated when tests end.
  *
- * @param {string} dir
- *   The test sub-directory to use for the engines.
  * @returns {HttpServer}
  *   The HttpServer object in case further customization is needed.
  */
-function useHttpServer(dir = "data") {
+function useHttpServer() {
   let httpServer = new HttpServer();
   httpServer.start(-1);
   httpServer.registerDirectory("/", do_get_cwd());
-  gDataUrl = `http://localhost:${httpServer.identity.primaryPort}/${dir}/`;
+  gHttpURL = `http://localhost:${httpServer.identity.primaryPort}`;
   registerCleanupFunction(async function cleanup_httpServer() {
     await new Promise(resolve => {
       httpServer.stop(resolve);
@@ -286,27 +280,115 @@ function checkCountryResultTelemetry(aExpectedValue) {
 }
 
 /**
- * Provides a basic set of remote settings for use in tests.
+ * Reads the specified file from the data directory and returns its contents as
+ * an Uint8Array.
+ *
+ * @param {string} filename
+ *   The name of the file to read.
+ * @returns {Promise<Uint8Array>}
+ *   The contents of the file in an Uint8Array.
  */
-async function setupRemoteSettings() {
-  const settings = await RemoteSettings("hijack-blocklists");
-  sinon.stub(settings, "get").returns([
-    {
-      id: "load-paths",
-      matches: ["[other]addEngineWithDetails:searchignore@mozilla.com"],
-      _status: "synced",
+async function getFileDataBuffer(filename) {
+  return IOUtils.read(PathUtils.join(do_get_cwd().path, "icons", filename));
+}
+
+/**
+ * Creates a mock attachment record for use in remote settings related tests.
+ *
+ * @param {object} item
+ *   An object containing the details of the attachment.
+ * @param {string} item.filename
+ *   The name of the attachmnet file in the data directory.
+ * @param {string[]} item.engineIdentifiers
+ *  The engine identifiers for the attachment.
+ * @param {number} item.imageSize
+ *  The size of the image.
+ * @param {string} [item.id]
+ *   The ID to use for the record. If not provided, a new UUID will be generated.
+ * @param {number} [item.lastModified]
+ *   The last modified time for the record. Defaults to the current time.
+ * @returns {object}
+ *   An object containing the record and attachment.
+ */
+async function mockRecordWithAttachment({
+  filename,
+  engineIdentifiers,
+  imageSize,
+  id = Services.uuid.generateUUID().toString(),
+  lastModified = Date.now(),
+}) {
+  let buffer = await getFileDataBuffer(filename);
+
+  // Generate a hash.
+  let hasher = Cc["@mozilla.org/security/hash;1"].createInstance(
+    Ci.nsICryptoHash
+  );
+  hasher.init(Ci.nsICryptoHash.SHA256);
+  hasher.update(buffer, buffer.length);
+
+  let hash = hasher.finish(false);
+  hash = Array.from(hash, (_, i) =>
+    ("0" + hash.charCodeAt(i).toString(16)).slice(-2)
+  ).join("");
+
+  // Mapping file extensions to corresponding MIME types.
+  const mimetypeFromExtension = {
+    ico: "image/x-icon",
+    svg: "image/svg+xml",
+    png: "image/png",
+  };
+
+  const extension = filename.split(".").pop().toLowerCase();
+
+  let record = {
+    id,
+    engineIdentifiers,
+    imageSize,
+    attachment: {
+      hash,
+      location: `${filename}`,
+      filename,
+      size: buffer.byteLength,
+      mimetype: mimetypeFromExtension[extension] || "",
     },
-    {
-      id: "submission-urls",
-      matches: ["ignore=true"],
-      _status: "synced",
-    },
-  ]);
+    last_modified: lastModified,
+  };
+
+  let attachment = {
+    record,
+    blob: new Blob([buffer]),
+  };
+
+  return { record, attachment };
+}
+
+/**
+ * Inserts an attachment record into the remote settings collection.
+ *
+ * @param {RemoteSettingsClient} client
+ *   The remote settings client to use.
+ * @param {object} item
+ *   An object containing the details of the attachment - see mockRecordWithAttachment.
+ * @param {boolean} [addAttachmentToCache]
+ *   Whether to add the attachment file to the cache. Defaults to true.
+ */
+async function insertRecordIntoCollection(
+  client,
+  item,
+  addAttachmentToCache = true
+) {
+  let { record, attachment } = await mockRecordWithAttachment(item);
+  await client.db.create(record);
+  if (addAttachmentToCache) {
+    await client.attachments.cacheImpl.set(record.id, attachment);
+  }
+  await client.db.importChanges({}, record.last_modified);
 }
 
 /**
  * Helper function that sets up a server and respnds to region
  * fetch requests.
+ *
  * @param {string} region
  *   The region that the server will respond with.
  * @param {Promise|null} waitToRespond
@@ -332,16 +414,23 @@ function useCustomGeoServer(region, waitToRespond = Promise.resolve()) {
 /**
  * @typedef {object} TelemetryDetails
  * @property {string} engineId
+ *   The telemetry ID for the search engine.
  * @property {string} [displayName]
+ *   The search engine's display name.
  * @property {string} [loadPath]
+ *   The load path for the search engine.
  * @property {string} [submissionUrl]
- * @property {string} [verified].
+ *   The submission URL for the search engine.
+ * @property {string} [verified]
+ *   Whether the search engine is verified.
+ */
 
 /**
  * Asserts that default search engine telemetry has been correctly reported
  * to Glean.
  *
  * @param {object} expected
+ *   An object containing telemetry details for normal and private engines.
  * @param {TelemetryDetails} expected.normal
  *   An object with the expected details for the normal search engine.
  * @param {TelemetryDetails} [expected.private]
@@ -362,12 +451,7 @@ async function assertGleanDefaultEngine(expected) {
     "Should have set the correct telemetry id for the private engine"
   );
 
-  for (let property of [
-    "displayName",
-    "loadPath",
-    "submissionUrl",
-    "verified",
-  ]) {
+  for (let property of ["displayName", "loadPath", "submissionUrl"]) {
     if (property in expected.normal) {
       Assert.equal(
         Glean.searchEngineDefault[property].testGetValue(),
@@ -385,19 +469,54 @@ async function assertGleanDefaultEngine(expected) {
   }
 }
 
+/**
+ * Loads a new enterprise policy, and re-initialise the search service
+ * with the new policy. Also waits for the search service to write the settings
+ * file to disk.
+ *
+ * @param {object} policy
+ *   The enterprise policy to use.
+ */
+async function setupPolicyEngineWithJson(policy) {
+  Services.search.wrappedJSObject.reset();
+
+  await this.EnterprisePolicyTesting.setupPolicyEngineWithJson(policy);
+
+  let settingsWritten = SearchTestUtils.promiseSearchNotification(
+    "write-settings-to-disk-complete"
+  );
+  await Services.search.init();
+  await settingsWritten;
+}
+
+/**
+ * Makes Services.policies.isEnterprise return true by loading an enterprise
+ * policy and re-initialise the search service with the new policy. Also waits
+ * for the search service to write the settings file to disk.
+ */
+async function enableEnterprise() {
+  await setupPolicyEngineWithJson({
+    // Use any policy.
+    policies: {
+      BlockAboutSupport: true,
+    },
+  });
+  Assert.ok(Services.policies.isEnterprise, "isEnterprise");
+}
+
 /**
  * A simple observer to ensure we get only the expected notifications.
  */
 class SearchObserver {
   constructor(expectedNotifications, returnEngineForNotification = false) {
     this.observer = this.observer.bind(this);
-    this.deferred = PromiseUtils.defer();
+    this.deferred = Promise.withResolvers();
     this.expectedNotifications = expectedNotifications;
     this.returnEngineForNotification = returnEngineForNotification;
 
     Services.obs.addObserver(this.observer, SearchUtils.TOPIC_ENGINE_MODIFIED);
 
-    this.timeout = setTimeout(this.handleTimeout.bind(this), 1000);
+    this.timeout = setTimeout(this.handleTimeout.bind(this), 5000);
   }
 
   get promise() {
@@ -462,29 +581,117 @@ registerCleanupFunction(async () => {
 });
 
 let consoleAllowList = [
+  // Harness issues.
   'property "localProfileDir" is non-configurable and can\'t be deleted',
   'property "profileDir" is non-configurable and can\'t be deleted',
 ];
 
-function observe(subject) {
-  let msg = subject.wrappedJSObject;
-  let messageContents = msg.arguments[0]?.message || msg.arguments[0];
-  if (
-    msg.level == "error" &&
-    !consoleAllowList.some(e => messageContents.includes(e))
-  ) {
-    Assert.ok(false, "Unexpected console message: " + messageContents);
-  }
-}
-
-const ConsoleAPIStorage = Cc["@mozilla.org/consoleAPI-storage;1"].getService(
-  Ci.nsIConsoleAPIStorage
-);
-ConsoleAPIStorage.addLogEventListener(
-  observe,
-  Cc["@mozilla.org/systemprincipal;1"].createInstance(Ci.nsIPrincipal)
-);
+let endConsoleListening = TestUtils.listenForConsoleMessages();
 
 registerCleanupFunction(async () => {
-  ConsoleAPIStorage.removeLogEventListener(observe);
+  let msgs = await endConsoleListening();
+  for (let msg of msgs) {
+    msg = msg.wrappedJSObject;
+    if (msg.level != "error") {
+      continue;
+    }
+
+    if (!msg.arguments?.length) {
+      Assert.ok(
+        false,
+        "Unexpected console message received during test: " + msg
+      );
+    } else {
+      let firstArg = msg.arguments[0];
+      // Use the appropriate message depending on the object supplied to
+      // the first argument.
+      let message = firstArg.messageContents ?? firstArg.message ?? firstArg;
+      if (!consoleAllowList.some(e => message.includes(e))) {
+        Assert.ok(
+          false,
+          "Unexpected console message received during test: " + message
+        );
+      }
+    }
+  }
 });
+
+/**
+ * This function asserts if the actual engines returned equals the expected
+ * engines.
+ *
+ * @param {SearchEngineSelector} engineSelector
+ *   The search engine selector to use for the test.
+ * @param {object} config
+ *   A fake search config containing engines.
+ * @param {object} userEnv
+ *   A fake user's environment including locale and region, experiment, etc.
+ * @param {Array} expectedEngines
+ *   The array of expected engines to be returned from the fake config.
+ * @param {string} message
+ *   The assertion message.
+ */
+async function assertSelectorEnginesEqualsExpected(
+  engineSelector,
+  config,
+  userEnv,
+  expectedEngines,
+  message
+) {
+  engineSelector._configuration = null;
+  SearchTestUtils.setRemoteSettingsConfig(config);
+
+  if (expectedEngines.length) {
+    let { engines } = await engineSelector.fetchEngineConfiguration(userEnv);
+
+    if (SearchUtils.rustSelectorFeatureGate) {
+      // Add default parameters to match the selector output.
+      for (let i = 0; i < expectedEngines.length; i++) {
+        expectedEngines[i] = {
+          aliases: [],
+          charset: "UTF-8",
+          optional: false,
+          partnerCode: "",
+          telemetrySuffix: "",
+          orderHint: null,
+          clickUrl: null,
+          ...expectedEngines[i],
+        };
+        expectedEngines[i].classification =
+          expectedEngines[i].classification == "general"
+            ? SearchEngineClassification.GENERAL
+            : SearchEngineClassification.UNKNOWN;
+
+        expectedEngines[i].urls = {
+          suggestions: null,
+          trending: null,
+          searchForm: null,
+          ...expectedEngines[i].urls,
+        };
+        expectedEngines[i].urls.search = {
+          method: "GET",
+          ...expectedEngines[i].urls.search,
+        };
+        if (!expectedEngines[i].urls.search.params) {
+          expectedEngines[i].urls.search.params = [];
+        }
+        for (let j = 0; j < expectedEngines[i].urls.search.params.length; j++) {
+          expectedEngines[i].urls.search.params[j] = {
+            enterpriseValue: null,
+            experimentConfig: null,
+            value: null,
+            ...expectedEngines[i].urls.search.params[j],
+          };
+        }
+      }
+    }
+
+    Assert.deepEqual(engines, expectedEngines, message);
+  } else {
+    await Assert.rejects(
+      engineSelector.fetchEngineConfiguration(userEnv),
+      /Could not find any engines in the filtered configuration/,
+      message
+    );
+  }
+}
