@@ -3,42 +3,16 @@
 
 "use strict";
 
-XPCOMUtils.defineLazyModuleGetters(this, {
-  SearchEngineSelector: "resource://gre/modules/SearchEngineSelector.jsm",
-  SearchService: "resource://gre/modules/SearchService.jsm",
-});
-
 const tests = [];
 
-// Bing should be default everywhere for Acer
-for (let [locale, region] of [
-  ["en-US", "US"],
-  ["pl", "PL"],
-  ["be", "BY"],
-  ["ru", "RU"],
-  ["zh-CN", "CN"],
-]) {
-  for (let distribution of ["acer-003", "acer-004"]) {
-    tests.push({
-      distribution,
-      locale,
-      region,
-      test: engines =>
-        hasParams(engines, "Bing", "searchbar", "pc=MOZX") &&
-        hasDefault(engines, "Bing") &&
-        hasEnginesFirst(engines, ["Bing"]),
-    });
-  }
-}
-
 for (let canonicalId of ["canonical", "canonical-001"]) {
   tests.push({
     locale: "en-US",
     region: "US",
     distribution: canonicalId,
     test: engines =>
-      hasParams(engines, "Google", "searchbar", "client=ubuntu") &&
-      hasParams(engines, "Google", "searchbar", "channel=fs") &&
+      hasParams(engines, "Google", "client=ubuntu") &&
+      hasParams(engines, "Google", "channel=fs") &&
       hasTelemetryId(engines, "Google", "google-canonical"),
   });
 
@@ -47,25 +21,30 @@ for (let canonicalId of ["canonical", "canonical-001"]) {
     region: "GB",
     distribution: canonicalId,
     test: engines =>
-      hasParams(engines, "Google", "searchbar", "client=ubuntu") &&
-      hasParams(engines, "Google", "searchbar", "channel=fs") &&
+      hasParams(engines, "Google", "client=ubuntu") &&
+      hasParams(engines, "Google", "channel=fs") &&
       hasTelemetryId(engines, "Google", "google-canonical"),
   });
 }
 
 tests.push({
-  locale: "zh-CN",
-  region: "CN",
-  distribution: "MozillaOnline",
+  locale: "en-US",
+  region: "US",
+  distribution: "canonical-002",
   test: engines =>
-    hasParams(engines, "百度", "searchbar", "tn=monline_4_dg") &&
-    hasParams(engines, "百度", "suggestions", "tn=monline_4_dg") &&
-    hasParams(engines, "百度", "homepage", "tn=monline_3_dg") &&
-    hasParams(engines, "百度", "newtab", "tn=monline_3_dg") &&
-    hasParams(engines, "百度", "contextmenu", "tn=monline_4_dg") &&
-    hasParams(engines, "百度", "keyword", "tn=monline_4_dg") &&
-    hasDefault(engines, "百度") &&
-    hasEnginesFirst(engines, ["百度", "Bing", "Google", "亚马逊", "维基百科"]),
+    hasParams(engines, "Google", "client=ubuntu-sn") &&
+    hasParams(engines, "Google", "channel=fs") &&
+    hasTelemetryId(engines, "Google", "google-ubuntu-sn"),
+});
+
+tests.push({
+  locale: "en-US",
+  region: "GB",
+  distribution: "canonical-002",
+  test: engines =>
+    hasParams(engines, "Google", "client=ubuntu-sn") &&
+    hasParams(engines, "Google", "channel=fs") &&
+    hasTelemetryId(engines, "Google", "google-ubuntu-sn"),
 });
 
 tests.push({
@@ -73,20 +52,14 @@ tests.push({
   region: "CN",
   distribution: "MozillaOnline",
   test: engines =>
-    hasParams(engines, "亚马逊", "searchbar", "ie=UTF8") &&
-    hasParams(engines, "亚马逊", "suggestions", "tag=mozilla") &&
-    hasParams(engines, "亚马逊", "homepage", "camp=536") &&
-    hasParams(engines, "亚马逊", "homepage", "creative=3200") &&
-    hasParams(engines, "亚马逊", "homepage", "index=aps") &&
-    hasParams(engines, "亚马逊", "homepage", "linkCode=ur2") &&
-    hasEnginesFirst(engines, ["百度", "Bing", "Google", "亚马逊", "维基百科"]),
+    hasEnginesFirst(engines, ["百度", "Bing", "Google", "维基百科"]),
 });
 
 tests.push({
   locale: "fr",
   distribution: "qwant-001",
   test: engines =>
-    hasParams(engines, "Qwant", "searchbar", "client=firefoxqwant") &&
+    hasParams(engines, "Qwant", "client=firefoxqwant") &&
     hasDefault(engines, "Qwant") &&
     hasEnginesFirst(engines, ["Qwant", "Qwant Junior"]),
 });
@@ -94,15 +67,14 @@ tests.push({
 tests.push({
   locale: "fr",
   distribution: "qwant-001",
-  test: engines =>
-    hasParams(engines, "Qwant Junior", "searchbar", "client=firefoxqwant"),
+  test: engines => hasParams(engines, "Qwant Junior", "client=firefoxqwant"),
 });
 
 tests.push({
   locale: "fr",
   distribution: "qwant-002",
   test: engines =>
-    hasParams(engines, "Qwant", "searchbar", "client=firefoxqwant") &&
+    hasParams(engines, "Qwant", "client=firefoxqwant") &&
     hasDefault(engines, "Qwant") &&
     hasEnginesFirst(engines, ["Qwant", "Qwant Junior"]),
 });
@@ -110,87 +82,15 @@ tests.push({
 tests.push({
   locale: "fr",
   distribution: "qwant-002",
-  test: engines =>
-    hasParams(engines, "Qwant Junior", "searchbar", "client=firefoxqwant"),
+  test: engines => hasParams(engines, "Qwant Junior", "client=firefoxqwant"),
 });
 
-tests.push({
-  locale: "cs",
-  distribution: "seznam",
-  test: engines =>
-    hasParams(engines, "Seznam", "searchbar", "sourceid=FF_3") &&
-    hasDefault(engines, "Seznam") &&
-    hasEnginesFirst(engines, ["Seznam"]),
-});
-
-for (const locale of ["en-US", "en-GB", "fr", "de"]) {
-  tests.push({
-    locale,
-    distribution: "sweetlabs-b-oem1",
-    test: engines =>
-      hasParams(engines, "Bing", "searchbar", "pc=MZSL01") &&
-      hasParams(engines, "Bing", "searchbar", "ptag=MOZZ0000000020") &&
-      hasDefault(engines, "Bing") &&
-      hasEnginesFirst(engines, ["Bing"]),
-  });
-
-  tests.push({
-    locale,
-    distribution: "sweetlabs-b-r-oem1",
-    test: engines =>
-      hasParams(engines, "Bing", "searchbar", "pc=MZSL01") &&
-      hasParams(engines, "Bing", "searchbar", "ptag=MOZZ0000000020") &&
-      hasDefault(engines, "Bing") &&
-      hasEnginesFirst(engines, ["Bing"]),
-  });
-
-  tests.push({
-    locale,
-    distribution: "sweetlabs-b-oem2",
-    test: engines =>
-      hasParams(engines, "Bing", "searchbar", "pc=MZSL02") &&
-      hasParams(engines, "Bing", "searchbar", "ptag=MOZZ0000000020") &&
-      hasDefault(engines, "Bing") &&
-      hasEnginesFirst(engines, ["Bing"]),
-  });
-
-  tests.push({
-    locale,
-    distribution: "sweetlabs-b-r-oem2",
-    test: engines =>
-      hasParams(engines, "Bing", "searchbar", "pc=MZSL02") &&
-      hasParams(engines, "Bing", "searchbar", "ptag=MOZZ0000000020") &&
-      hasDefault(engines, "Bing") &&
-      hasEnginesFirst(engines, ["Bing"]),
-  });
-
-  tests.push({
-    locale,
-    distribution: "sweetlabs-b-oem3",
-    test: engines =>
-      hasParams(engines, "Bing", "searchbar", "pc=MZSL03") &&
-      hasParams(engines, "Bing", "searchbar", "ptag=MOZZ0000000020") &&
-      hasDefault(engines, "Bing") &&
-      hasEnginesFirst(engines, ["Bing"]),
-  });
-
-  tests.push({
-    locale,
-    distribution: "sweetlabs-b-r-oem3",
-    test: engines =>
-      hasParams(engines, "Bing", "searchbar", "pc=MZSL03") &&
-      hasParams(engines, "Bing", "searchbar", "ptag=MOZZ0000000020") &&
-      hasDefault(engines, "Bing") &&
-      hasEnginesFirst(engines, ["Bing"]),
-  });
-}
-
 for (const locale of ["en-US", "de"]) {
   tests.push({
     locale,
     distribution: "1und1",
     test: engines =>
-      hasParams(engines, "1&1 Suche", "searchbar", "enc=UTF-8") &&
+      hasParams(engines, "1&1 Suche", "enc=UTF-8") &&
       hasDefault(engines, "1&1 Suche") &&
       hasEnginesFirst(engines, ["1&1 Suche"]),
   });
@@ -199,7 +99,7 @@ for (const locale of ["en-US", "de"]) {
     locale,
     distribution: "gmx",
     test: engines =>
-      hasParams(engines, "GMX Suche", "searchbar", "enc=UTF-8") &&
+      hasParams(engines, "GMX Suche", "enc=UTF-8") &&
       hasDefault(engines, "GMX Suche") &&
       hasEnginesFirst(engines, ["GMX Suche"]),
   });
@@ -207,15 +107,14 @@ for (const locale of ["en-US", "de"]) {
   tests.push({
     locale,
     distribution: "gmx",
-    test: engines =>
-      hasParams(engines, "GMX Shopping", "searchbar", "origin=br_osd"),
+    test: engines => hasParams(engines, "GMX Shopping", "origin=br_osd"),
   });
 
   tests.push({
     locale,
     distribution: "mail.com",
     test: engines =>
-      hasParams(engines, "mail.com search", "searchbar", "enc=UTF-8") &&
+      hasParams(engines, "mail.com search", "enc=UTF-8") &&
       hasDefault(engines, "mail.com search") &&
       hasEnginesFirst(engines, ["mail.com search"]),
   });
@@ -224,7 +123,7 @@ for (const locale of ["en-US", "de"]) {
     locale,
     distribution: "webde",
     test: engines =>
-      hasParams(engines, "WEB.DE Suche", "searchbar", "enc=UTF-8") &&
+      hasParams(engines, "WEB.DE Suche", "enc=UTF-8") &&
       hasDefault(engines, "WEB.DE Suche") &&
       hasEnginesFirst(engines, ["WEB.DE Suche"]),
   });
@@ -245,7 +144,7 @@ tests.push({
       engines,
       "GMX Search",
       "https://go.gmx.co.uk/br/moz_search_web/?enc=UTF-8&q=test",
-      "https://suggestplugin.gmx.co.uk/s?q=test&brand=gmxcouk&origin=moz_splugin_ff&enc=UTF-8"
+      "https://suggestplugin.gmx.co.uk/s?brand=gmxcouk&origin=moz_splugin_ff&enc=UTF-8&q=test"
     ) &&
     hasDefault(engines, "GMX Search") &&
     hasEnginesFirst(engines, ["GMX Search"]),
@@ -266,10 +165,10 @@ tests.push({
       engines,
       "GMX - Búsqueda web",
       "https://go.gmx.es/br/moz_search_web/?enc=UTF-8&q=test",
-      "https://suggestplugin.gmx.es/s?q=test&brand=gmxes&origin=moz_splugin_ff&enc=UTF-8"
+      "https://suggestplugin.gmx.es/s?brand=gmxes&origin=moz_splugin_ff&enc=UTF-8&q=test"
     ) &&
-    hasDefault(engines, "GMX Search") &&
-    hasEnginesFirst(engines, ["GMX Search"]),
+    hasDefault(engines, "GMX - Búsqueda web") &&
+    hasEnginesFirst(engines, ["GMX - Búsqueda web"]),
 });
 
 tests.push({
@@ -287,10 +186,10 @@ tests.push({
       engines,
       "GMX - Recherche web",
       "https://go.gmx.fr/br/moz_search_web/?enc=UTF-8&q=test",
-      "https://suggestplugin.gmx.fr/s?q=test&brand=gmxfr&origin=moz_splugin_ff&enc=UTF-8"
+      "https://suggestplugin.gmx.fr/s?brand=gmxfr&origin=moz_splugin_ff&enc=UTF-8&q=test"
     ) &&
-    hasDefault(engines, "GMX Search") &&
-    hasEnginesFirst(engines, ["GMX Search"]),
+    hasDefault(engines, "GMX - Recherche web") &&
+    hasEnginesFirst(engines, ["GMX - Recherche web"]),
 });
 
 tests.push({
@@ -305,7 +204,8 @@ tests.push({
   region: "US",
   distribution: "mint-001",
   test: engines =>
-    hasParams(engines, "Google", "searchbar", "client=firefox-b-1-lm") &&
+    hasParams(engines, "DuckDuckGo", "t=lm") &&
+    hasParams(engines, "Google", "client=firefox-b-1-lm") &&
     hasDefault(engines, "Google") &&
     hasEnginesFirst(engines, ["Google"]) &&
     hasTelemetryId(engines, "Google", "google-b-1-lm"),
@@ -316,14 +216,61 @@ tests.push({
   region: "GB",
   distribution: "mint-001",
   test: engines =>
-    hasParams(engines, "Google", "searchbar", "client=firefox-b-lm") &&
+    hasParams(engines, "DuckDuckGo", "t=lm") &&
+    hasParams(engines, "Google", "client=firefox-b-lm") &&
     hasDefault(engines, "Google") &&
     hasEnginesFirst(engines, ["Google"]) &&
     hasTelemetryId(engines, "Google", "google-b-lm"),
 });
 
+// This distribution is used on mobile, but we can test it here as it only
+// needs the distribution id referencing.
+tests.push({
+  locale: "en-GB",
+  region: "GB",
+  distribution: "vivo-001",
+  application: "firefox-android",
+  test: engines =>
+    hasParams(engines, "Bing", "pc=MZCP") &&
+    hasParams(engines, "Bing", "form=MZVIVO") &&
+    hasTelemetryId(engines, "Bing", "bing-MZCP") &&
+    hasParams(engines, "Google", "client=firefox-b-vv") &&
+    hasDefault(engines, "Google") &&
+    hasEnginesFirst(engines, ["Google"]) &&
+    hasTelemetryId(engines, "Google", "google-b-vv"),
+});
+
+// This distribution is used on mobile, but we can test it here as it only
+// needs the distribution id referencing.
+tests.push({
+  locale: "en-GB",
+  region: "GB",
+  distribution: "dt-001",
+  application: "firefox-android",
+  test: engines =>
+    hasParams(engines, "Google", "client=firefox-b-tf") &&
+    hasDefault(engines, "Google") &&
+    hasEnginesFirst(engines, ["Google"]) &&
+    hasTelemetryId(engines, "Google", "google-b-tf"),
+});
+
+// This distribution is used on mobile, but we can test it here as it only
+// needs the distribution id referencing.
+tests.push({
+  locale: "en-GB",
+  region: "GB",
+  distribution: "dt-001",
+  application: "firefox-android",
+  test: engines =>
+    hasParams(engines, "Bing", "pc=MZTOF") &&
+    hasParams(engines, "Bing", "form=MZTOFO") &&
+    hasDefault(engines, "Google") &&
+    hasEnginesFirst(engines, ["Google"]) &&
+    hasTelemetryId(engines, "Bing", "bing-MZTOF"),
+});
+
 function hasURLs(engines, engineName, url, suggestURL) {
-  let engine = engines.find(e => e._name === engineName);
+  let engine = engines.find(e => e.name === engineName);
   Assert.ok(engine, `Should be able to find ${engineName}`);
 
   let submission = engine.getSubmission("test", "text/html");
@@ -339,13 +286,13 @@ function hasURLs(engines, engineName, url, suggestURL) {
     suggestURL,
     `Should have the correct suggestion url for ${engineName}`
   );
+  return true;
 }
 
-function hasParams(engines, engineName, purpose, param) {
-  let engine = engines.find(e => e._name === engineName);
+function hasParams(engines, engineName, param) {
+  let engine = engines.find(e => e.name === engineName);
   Assert.ok(engine, `Should be able to find ${engineName}`);
-
-  let submission = engine.getSubmission("test", "text/html", purpose);
+  let submission = engine.getSubmission("test", "text/html");
   let queries = submission.uri.query.split("&");
 
   let paramNames = new Set();
@@ -364,7 +311,7 @@ function hasParams(engines, engineName, purpose, param) {
 }
 
 function hasTelemetryId(engines, engineName, telemetryId) {
-  let engine = engines.find(e => e._name === engineName);
+  let engine = engines.find(e => e.name === engineName);
   Assert.ok(engine, `Should be able to find ${engineName}`);
 
   Assert.equal(
@@ -392,40 +339,42 @@ function hasEnginesFirst(engines, expectedEngines) {
       `Should have the expected engine in position ${i}`
     );
   }
+  return true;
 }
 
-engineSelector = new SearchEngineSelector();
-
-AddonTestUtils.init(GLOBAL_SCOPE);
-AddonTestUtils.createAppInfo(
-  "xpcshell@tests.mozilla.org",
-  "XPCShell",
-  "42",
-  "42"
-);
-
-add_task(async function setup() {
-  await AddonTestUtils.promiseStartupManager();
+add_setup(async function () {
+  updateAppInfo({
+    name: "firefox",
+    ID: "xpcshell@tests.mozilla.org",
+    version: "128",
+    platformVersion: "128",
+  });
 
   await maybeSetupConfig();
 });
 
 add_task(async function test_expected_distribution_engines() {
-  let searchService = new SearchService();
-  for (const { distribution, locale = "en-US", region = "US", test } of tests) {
+  let engineSelector = new SearchEngineSelector();
+
+  for (const {
+    application = "firefox",
+    distribution,
+    locale = "en-US",
+    region = "US",
+    test,
+  } of tests) {
     let config = await engineSelector.fetchEngineConfiguration({
       locale,
       region,
       distroID: distribution,
+      appName: application,
     });
+
     let engines = await SearchTestUtils.searchConfigToEngines(config.engines);
-    searchService._engines = engines;
-    searchService._searchDefault = {
-      id: config.engines[0].webExtension.id,
-      locale:
-        config.engines[0]?.webExtension?.locale ?? SearchUtils.DEFAULT_TAG,
-    };
-    engines = searchService._sortEnginesByDefaults(engines);
+    engines = SearchUtils.sortEnginesByDefaults({
+      engines,
+      appDefaultEngine: engines[0],
+    });
     test(engines);
   }
 });
