@@ -3,24 +3,27 @@
 /* eslint-disable mozilla/no-arbitrary-setTimeout */
 
 /**
- * Testing search suggestions from SearchSuggestionController.jsm.
+ * Testing search suggestions from SearchSuggestionController.sys.mjs.
  */
 
 "use strict";
 
-const { FormHistory } = ChromeUtils.import(
-  "resource://gre/modules/FormHistory.jsm"
+const { FormHistory } = ChromeUtils.importESModule(
+  "resource://gre/modules/FormHistory.sys.mjs"
 );
-const { SearchSuggestionController } = ChromeUtils.import(
-  "resource://gre/modules/SearchSuggestionController.jsm"
+const { SearchSuggestionController } = ChromeUtils.importESModule(
+  "resource://gre/modules/SearchSuggestionController.sys.mjs"
 );
-const { PromiseUtils } = ChromeUtils.import(
-  "resource://gre/modules/PromiseUtils.jsm"
+const { TelemetryTestUtils } = ChromeUtils.importESModule(
+  "resource://testing-common/TelemetryTestUtils.sys.mjs"
 );
 
+const ENGINE_NAME = "other";
+const SEARCH_TELEMETRY_LATENCY = "SEARCH_SUGGESTIONS_LATENCY_MS";
+
 // We must make sure the FormHistoryStartup component is
 // initialized in order for it to respond to FormHistory
-// requests from nsFormAutoComplete.js.
+// requests from FormHistoryAutoComplete.sys.mjs.
 var formHistoryStartup = Cc[
   "@mozilla.org/satchel/form-history-startup;1"
 ].getService(Ci.nsIObserver);
@@ -28,30 +31,28 @@ formHistoryStartup.observe(null, "profile-after-change", null);
 
 var getEngine, postEngine, unresolvableEngine, alternateJSONEngine;
 
-add_task(async function setup() {
+add_setup(async function () {
   Services.prefs.setBoolPref("browser.search.suggest.enabled", true);
+  // These tests intentionally test broken connections.
+  consoleAllowList = consoleAllowList.concat([
+    "Non-200 status or empty HTTP response: 404",
+    "Non-200 status or empty HTTP response: 500",
+    "SearchSuggestionController found an unexpected string value",
+    "HTTP request timeout",
+    "HTTP error",
+  ]);
 
   let server = useHttpServer();
   server.registerContentType("sjs", "sjs");
 
-  await AddonTestUtils.promiseStartupManager();
-
-  registerCleanupFunction(async () => {
-    // Remove added form history entries
-    await updateSearchHistory("remove", null);
-    Services.prefs.clearUserPref("browser.search.suggest.enabled");
-  });
-});
-
-add_task(async function add_test_engines() {
   let getEngineData = {
-    baseURL: gDataUrl,
+    baseURL: `${gHttpURL}/sjs/`,
     name: "GET suggestion engine",
     method: "GET",
   };
 
   let postEngineData = {
-    baseURL: gDataUrl,
+    baseURL: `${gHttpURL}/sjs/`,
     name: "POST suggestion engine",
     method: "POST",
   };
@@ -63,70 +64,57 @@ add_task(async function add_test_engines() {
   };
 
   let alternateJSONSuggestEngineData = {
-    baseURL: gDataUrl,
+    baseURL: `${gHttpURL}/sjs/`,
     name: "Alternative JSON suggestion type",
     method: "GET",
     alternativeJSONType: true,
   };
 
-  getEngine = await SearchTestUtils.promiseNewSearchEngine(
-    `${gDataUrl}engineMaker.sjs?${JSON.stringify(getEngineData)}`
-  );
-  postEngine = await SearchTestUtils.promiseNewSearchEngine(
-    `${gDataUrl}engineMaker.sjs?${JSON.stringify(postEngineData)}`
-  );
-  unresolvableEngine = await SearchTestUtils.promiseNewSearchEngine(
-    `${gDataUrl}engineMaker.sjs?${JSON.stringify(unresolvableEngineData)}`
-  );
-  alternateJSONEngine = await SearchTestUtils.promiseNewSearchEngine(
-    `${gDataUrl}engineMaker.sjs?${JSON.stringify(
+  getEngine = await SearchTestUtils.installOpenSearchEngine({
+    url: `${gHttpURL}/sjs/engineMaker.sjs?${JSON.stringify(getEngineData)}`,
+  });
+  postEngine = await SearchTestUtils.installOpenSearchEngine({
+    url: `${gHttpURL}/sjs/engineMaker.sjs?${JSON.stringify(postEngineData)}`,
+  });
+  unresolvableEngine = await SearchTestUtils.installOpenSearchEngine({
+    url: `${gHttpURL}/sjs/engineMaker.sjs?${JSON.stringify(
+      unresolvableEngineData
+    )}`,
+  });
+  alternateJSONEngine = await SearchTestUtils.installOpenSearchEngine({
+    url: `${gHttpURL}/sjs/engineMaker.sjs?${JSON.stringify(
       alternateJSONSuggestEngineData
-    )}`
-  );
+    )}`,
+  });
+
+  registerCleanupFunction(async () => {
+    // Remove added form history entries
+    await updateSearchHistory("remove", null);
+    Services.prefs.clearUserPref("browser.search.suggest.enabled");
+  });
 });
 
 // Begin tests
 
-add_task(async function simple_no_result_callback() {
-  await new Promise(resolve => {
-    let controller = new SearchSuggestionController(result => {
-      Assert.equal(result.term, "no remote");
-      Assert.equal(result.local.length, 0);
-      Assert.equal(result.remote.length, 0);
-      resolve();
-    });
-
-    controller.fetch("no remote", false, getEngine);
-  });
-});
-
-add_task(async function simple_no_result_callback_and_promise() {
-  // Make sure both the callback and promise get results
-  let deferred = PromiseUtils.defer();
-  let controller = new SearchSuggestionController(result => {
-    Assert.equal(result.term, "no results");
-    Assert.equal(result.local.length, 0);
-    Assert.equal(result.remote.length, 0);
-    deferred.resolve();
-  });
-
-  let result = await controller.fetch("no results", false, getEngine);
-  Assert.equal(result.term, "no results");
-  Assert.equal(result.local.length, 0);
-  Assert.equal(result.remote.length, 0);
-
-  await deferred.promise;
-});
-
 add_task(async function simple_no_result_promise() {
+  let histogram = TelemetryTestUtils.getAndClearKeyedHistogram(
+    SEARCH_TELEMETRY_LATENCY
+  );
+
   let controller = new SearchSuggestionController();
   let result = await controller.fetch("no remote", false, getEngine);
   Assert.equal(result.term, "no remote");
   Assert.equal(result.local.length, 0);
   Assert.equal(result.remote.length, 0);
+
+  assertLatencyHistogram(histogram, true);
 });
 
 add_task(async function simple_remote_no_local_result() {
+  let histogram = TelemetryTestUtils.getAndClearKeyedHistogram(
+    SEARCH_TELEMETRY_LATENCY
+  );
+
   let controller = new SearchSuggestionController();
   let result = await controller.fetch("mo", false, getEngine);
   Assert.equal(result.term, "mo");
@@ -135,26 +123,8 @@ add_task(async function simple_remote_no_local_result() {
   Assert.equal(result.remote[0].value, "Mozilla");
   Assert.equal(result.remote[1].value, "modern");
   Assert.equal(result.remote[2].value, "mom");
-});
 
-add_task(async function simple_remote_no_local_result_telemetry() {
-  Services.telemetry.clearScalars();
-
-  let controller = new SearchSuggestionController();
-  await controller.fetch("mo", false, getEngine);
-
-  let scalars = {};
-  const key = "browser.search.data_transferred";
-
-  await TestUtils.waitForCondition(() => {
-    scalars =
-      Services.telemetry.getSnapshotForKeyedScalars("main", false).parent || {};
-    return key in scalars;
-  }, "should have the expected keyed scalars");
-
-  const scalar = scalars[key];
-  Assert.ok("sggt-other" in scalar, "correct telemetry category");
-  Assert.notEqual(scalar["sggt-other"], 0, "bandwidth logged");
+  assertLatencyHistogram(histogram, true);
 });
 
 add_task(async function simple_remote_no_local_result_alternative_type() {
@@ -350,6 +320,10 @@ add_task(async function tail_offset_index() {
 });
 
 add_task(async function fetch_twice_in_a_row() {
+  let histogram = TelemetryTestUtils.getAndClearKeyedHistogram(
+    SEARCH_TELEMETRY_LATENCY
+  );
+
   // Two entries since the first will match the first fetch but not the second.
   await updateSearchHistory("bump", "delay local");
   await updateSearchHistory("bump", "delayed local");
@@ -367,32 +341,10 @@ add_task(async function fetch_twice_in_a_row() {
   Assert.equal(result.local[0].value, "delayed local");
   Assert.equal(result.remote.length, 1);
   Assert.equal(result.remote[0].value, "delayed ");
-});
 
-add_task(async function fetch_twice_subset_reuse_formHistoryResult() {
-  // This tests if we mess up re-using the cached form history result.
-  // Two entries since the first will match the first fetch but not the second.
-  await updateSearchHistory("bump", "delay local");
-  await updateSearchHistory("bump", "delayed local");
-
-  let controller = new SearchSuggestionController();
-  let result = await controller.fetch("delay", false, getEngine);
-  Assert.equal(result.term, "delay");
-  Assert.equal(result.local.length, 2);
-  Assert.equal(result.local[0].value, "delay local");
-  Assert.equal(result.local[1].value, "delayed local");
-  Assert.equal(result.remote.length, 1);
-  Assert.equal(result.remote[0].value, "delay");
-
-  // Remove the entry from the DB but it should remain in the cached formHistoryResult.
-  await updateSearchHistory("remove", "delayed local");
-
-  let result2 = await controller.fetch("delayed ", false, getEngine);
-  Assert.equal(result2.term, "delayed ");
-  Assert.equal(result2.local.length, 1);
-  Assert.equal(result2.local[0].value, "delayed local");
-  Assert.equal(result2.remote.length, 1);
-  Assert.equal(result2.remote[0].value, "delayed ");
+  // Only the second fetch's latency should be recorded since the first fetch
+  // was aborted and latencies for aborted fetches are not recorded.
+  assertLatencyHistogram(histogram, true);
 });
 
 add_task(async function both_identical_with_more_than_max_results() {
@@ -431,6 +383,10 @@ add_task(async function both_identical_with_more_than_max_results() {
 });
 
 add_task(async function noremote_maxLocal() {
+  let histogram = TelemetryTestUtils.getAndClearKeyedHistogram(
+    SEARCH_TELEMETRY_LATENCY
+  );
+
   let controller = new SearchSuggestionController();
   controller.maxLocalResults = 2; // (should be ignored because no remote results)
   controller.maxRemoteResults = 0;
@@ -444,9 +400,15 @@ add_task(async function noremote_maxLocal() {
     );
   }
   Assert.equal(result.remote.length, 0);
+
+  assertLatencyHistogram(histogram, false);
 });
 
 add_task(async function someremote_maxLocal() {
+  let histogram = TelemetryTestUtils.getAndClearKeyedHistogram(
+    SEARCH_TELEMETRY_LATENCY
+  );
+
   let controller = new SearchSuggestionController();
   controller.maxLocalResults = 2;
   controller.maxRemoteResults = 4;
@@ -467,6 +429,8 @@ add_task(async function someremote_maxLocal() {
       "letter " + String.fromCharCode("C".charCodeAt() + i)
     );
   }
+
+  assertLatencyHistogram(histogram, true);
 });
 
 add_task(async function one_of_each() {
@@ -482,6 +446,10 @@ add_task(async function one_of_each() {
 });
 
 add_task(async function local_result_returned_remote_result_disabled() {
+  let histogram = TelemetryTestUtils.getAndClearKeyedHistogram(
+    SEARCH_TELEMETRY_LATENCY
+  );
+
   Services.prefs.setBoolPref("browser.search.suggest.enabled", false);
   let controller = new SearchSuggestionController();
   controller.maxLocalResults = 1;
@@ -496,11 +464,15 @@ add_task(async function local_result_returned_remote_result_disabled() {
     );
   }
   Assert.equal(result.remote.length, 0);
+  assertLatencyHistogram(histogram, false);
   Services.prefs.setBoolPref("browser.search.suggest.enabled", true);
 });
 
 add_task(
   async function local_result_returned_remote_result_disabled_after_creation_of_controller() {
+    let histogram = TelemetryTestUtils.getAndClearKeyedHistogram(
+      SEARCH_TELEMETRY_LATENCY
+    );
     let controller = new SearchSuggestionController();
     controller.maxLocalResults = 1;
     controller.maxRemoteResults = 1;
@@ -515,12 +487,17 @@ add_task(
       );
     }
     Assert.equal(result.remote.length, 0);
+    assertLatencyHistogram(histogram, false);
     Services.prefs.setBoolPref("browser.search.suggest.enabled", true);
   }
 );
 
 add_task(
   async function one_of_each_disabled_before_creation_enabled_after_creation_of_controller() {
+    let histogram = TelemetryTestUtils.getAndClearKeyedHistogram(
+      SEARCH_TELEMETRY_LATENCY
+    );
+
     Services.prefs.setBoolPref("browser.search.suggest.enabled", false);
     let controller = new SearchSuggestionController();
     controller.maxLocalResults = 1;
@@ -533,11 +510,16 @@ add_task(
     Assert.equal(result.remote.length, 1);
     Assert.equal(result.remote[0].value, "letter B");
 
+    assertLatencyHistogram(histogram, true);
+
     Services.prefs.setBoolPref("browser.search.suggest.enabled", true);
   }
 );
 
 add_task(async function one_local_zero_remote() {
+  let histogram = TelemetryTestUtils.getAndClearKeyedHistogram(
+    SEARCH_TELEMETRY_LATENCY
+  );
   let controller = new SearchSuggestionController();
   controller.maxLocalResults = 1;
   controller.maxRemoteResults = 0;
@@ -551,9 +533,13 @@ add_task(async function one_local_zero_remote() {
     );
   }
   Assert.equal(result.remote.length, 0);
+  assertLatencyHistogram(histogram, false);
 });
 
 add_task(async function zero_local_one_remote() {
+  let histogram = TelemetryTestUtils.getAndClearKeyedHistogram(
+    SEARCH_TELEMETRY_LATENCY
+  );
   let controller = new SearchSuggestionController();
   controller.maxLocalResults = 0;
   controller.maxRemoteResults = 1;
@@ -562,63 +548,135 @@ add_task(async function zero_local_one_remote() {
   Assert.equal(result.local.length, 0);
   Assert.equal(result.remote.length, 1);
   Assert.equal(result.remote[0].value, "letter A");
+  assertLatencyHistogram(histogram, true);
 });
 
 add_task(async function stop_search() {
-  let controller = new SearchSuggestionController(result => {
-    do_throw("The callback shouldn't be called after stop()");
-  });
+  let histogram = TelemetryTestUtils.getAndClearKeyedHistogram(
+    SEARCH_TELEMETRY_LATENCY
+  );
+  let controller = new SearchSuggestionController();
   let resultPromise = controller.fetch("mo", false, getEngine);
   controller.stop();
   await resultPromise.then(result => {
     Assert.equal(null, result);
   });
+  assertLatencyHistogram(histogram, false);
 });
 
 add_task(async function empty_searchTerm() {
   // Empty searches don't go to the server but still get form history.
+  let histogram = TelemetryTestUtils.getAndClearKeyedHistogram(
+    SEARCH_TELEMETRY_LATENCY
+  );
   let controller = new SearchSuggestionController();
   let result = await controller.fetch("", false, getEngine);
   Assert.equal(result.term, "");
   Assert.ok(!!result.local.length);
   Assert.equal(result.remote.length, 0);
+  assertLatencyHistogram(histogram, false);
 });
 
 add_task(async function slow_timeout() {
-  let d = PromiseUtils.defer();
-  function check_result(result) {
-    Assert.equal(result.term, "slow ");
+  let histogram = TelemetryTestUtils.getAndClearKeyedHistogram(
+    SEARCH_TELEMETRY_LATENCY
+  );
+
+  // Make the server return suggestions on a delay longer than the timeout of
+  // the suggestion controller.
+  let delayMs = 3 * SearchSuggestionController.REMOTE_TIMEOUT_DEFAULT;
+  let searchString = `delay${delayMs} `;
+
+  // Add a local result.
+  let localValue = searchString + " local result";
+  await updateSearchHistory("bump", localValue);
+
+  // Do a search. The remote fetch should time out but the local result should
+  // be returned.
+  let controller = new SearchSuggestionController();
+  let result = await controller.fetch(searchString, false, getEngine);
+  Assert.equal(result.term, searchString);
+  Assert.equal(result.local.length, 1);
+  Assert.equal(result.local[0].value, localValue);
+  Assert.equal(result.remote.length, 0);
+
+  // The remote fetch isn't done yet, so the latency histogram should not be
+  // updated.
+  assertLatencyHistogram(histogram, false);
+
+  // Wait for the remote fetch to finish.
+  await new Promise(r => setTimeout(r, delayMs));
+
+  // Now the latency histogram should be updated.
+  assertLatencyHistogram(histogram, true);
+});
+
+add_task(async function slow_timeout_2() {
+  let histogram = TelemetryTestUtils.getAndClearKeyedHistogram(
+    SEARCH_TELEMETRY_LATENCY
+  );
+
+  // Make the server return suggestions on a delay longer the timeout of the
+  // suggestion controller.
+  let delayMs = 3 * SearchSuggestionController.REMOTE_TIMEOUT_DEFAULT;
+  let searchString = `delay${delayMs} `;
+
+  // Add a local result.
+  let localValue = searchString + " local result";
+  await updateSearchHistory("bump", localValue);
+
+  // Do two searches using the same controller. Both times, the remote fetches
+  // should time out and only the local result should be returned. The second
+  // search should abort the remote fetch of the first search, and the remote
+  // fetch of the second search should be ongoing when the second search
+  // finishes.
+  let controller = new SearchSuggestionController();
+  for (let i = 0; i < 2; i++) {
+    let result = await controller.fetch(searchString, false, getEngine);
+    Assert.equal(result.term, searchString);
     Assert.equal(result.local.length, 1);
-    Assert.equal(result.local[0].value, "slow local result");
+    Assert.equal(result.local[0].value, localValue);
     Assert.equal(result.remote.length, 0);
   }
-  await updateSearchHistory("bump", "slow local result");
 
-  let controller = new SearchSuggestionController();
-  setTimeout(function check_timeout() {
-    // The HTTP response takes 10 seconds so check that we already have results after 2 seconds.
-    check_result(result);
-    d.resolve();
-  }, 2000);
-  let result = await controller.fetch("slow ", false, getEngine);
-  check_result(result);
-  await d.promise;
+  // The remote fetch of the second search isn't done yet, so the latency
+  // histogram should not be updated.
+  assertLatencyHistogram(histogram, false);
+
+  // Wait for the second remote fetch to finish.
+  await new Promise(r => setTimeout(r, delayMs));
+
+  // Now the latency histogram should be updated, and only the remote fetch of
+  // the second search should be recorded.
+  assertLatencyHistogram(histogram, true);
 });
 
 add_task(async function slow_stop() {
-  let d = PromiseUtils.defer();
-  let controller = new SearchSuggestionController();
-  let resultPromise = controller.fetch("slow ", false, getEngine);
-  setTimeout(function check_timeout() {
-    // The HTTP response takes 10 seconds but we timeout in less than a second so just use 0.
-    controller.stop();
-    d.resolve();
-  }, 0);
-  await resultPromise.then(result => {
-    Assert.equal(null, result);
-  });
+  let histogram = TelemetryTestUtils.getAndClearKeyedHistogram(
+    SEARCH_TELEMETRY_LATENCY
+  );
 
-  await d.promise;
+  // Make the server return suggestions on a delay longer the timeout of the
+  // suggestion controller.
+  let delayMs = 3 * SearchSuggestionController.REMOTE_TIMEOUT_DEFAULT;
+  let searchString = `delay${delayMs} `;
+
+  // Do a search but stop it before it finishes. Wait a tick before stopping it
+  // to better simulate the real world.
+  let controller = new SearchSuggestionController();
+  let resultPromise = controller.fetch(searchString, false, getEngine);
+  await TestUtils.waitForTick();
+  controller.stop();
+  let result = await resultPromise;
+  Assert.equal(result, null, "No result should be returned");
+
+  // The remote fetch should have been aborted by stopping the controller, but
+  // wait for the timeout period just to make sure it's done.
+  await new Promise(r => setTimeout(r, delayMs));
+
+  // Since the latencies of aborted fetches are not recorded, the latency
+  // histogram should not be updated.
+  assertLatencyHistogram(histogram, false);
 });
 
 // Error handling
@@ -626,39 +684,82 @@ add_task(async function slow_stop() {
 add_task(async function remote_term_mismatch() {
   await updateSearchHistory("bump", "Query Mismatch Entry");
 
+  let histogram = TelemetryTestUtils.getAndClearKeyedHistogram(
+    SEARCH_TELEMETRY_LATENCY
+  );
+
   let controller = new SearchSuggestionController();
   let result = await controller.fetch("Query Mismatch", false, getEngine);
   Assert.equal(result.term, "Query Mismatch");
   Assert.equal(result.local.length, 1);
   Assert.equal(result.local[0].value, "Query Mismatch Entry");
   Assert.equal(result.remote.length, 0);
+
+  assertLatencyHistogram(histogram, true);
 });
 
 add_task(async function http_404() {
   await updateSearchHistory("bump", "HTTP 404 Entry");
 
+  let histogram = TelemetryTestUtils.getAndClearKeyedHistogram(
+    SEARCH_TELEMETRY_LATENCY
+  );
+
   let controller = new SearchSuggestionController();
   let result = await controller.fetch("HTTP 404", false, getEngine);
   Assert.equal(result.term, "HTTP 404");
   Assert.equal(result.local.length, 1);
   Assert.equal(result.local[0].value, "HTTP 404 Entry");
   Assert.equal(result.remote.length, 0);
+
+  assertLatencyHistogram(histogram, true);
 });
 
 add_task(async function http_500() {
   await updateSearchHistory("bump", "HTTP 500 Entry");
 
+  let histogram = TelemetryTestUtils.getAndClearKeyedHistogram(
+    SEARCH_TELEMETRY_LATENCY
+  );
+
   let controller = new SearchSuggestionController();
   let result = await controller.fetch("HTTP 500", false, getEngine);
   Assert.equal(result.term, "HTTP 500");
   Assert.equal(result.local.length, 1);
   Assert.equal(result.local[0].value, "HTTP 500 Entry");
   Assert.equal(result.remote.length, 0);
+
+  assertLatencyHistogram(histogram, true);
+});
+
+add_task(async function invalid_response_does_not_throw() {
+  let controller = new SearchSuggestionController();
+  // Although the server will return invalid json, the error is handled by
+  // the suggestion controller, and so we receive no results.
+  let result = await controller.fetch("invalidJSON", false, getEngine);
+  Assert.equal(result.term, "invalidJSON");
+  Assert.equal(result.local.length, 0);
+  Assert.equal(result.remote.length, 0);
+});
+
+add_task(async function invalid_content_type_treated_as_json() {
+  let controller = new SearchSuggestionController();
+  // An invalid content type is overridden as we expect all the responses to
+  // be JSON.
+  let result = await controller.fetch("invalidContentType", false, getEngine);
+  Assert.equal(result.term, "invalidContentType");
+  Assert.equal(result.local.length, 0);
+  Assert.equal(result.remote.length, 1);
+  Assert.equal(result.remote[0].value, "invalidContentType response");
 });
 
 add_task(async function unresolvable_server() {
   await updateSearchHistory("bump", "Unresolvable Server Entry");
 
+  let histogram = TelemetryTestUtils.getAndClearKeyedHistogram(
+    SEARCH_TELEMETRY_LATENCY
+  );
+
   let controller = new SearchSuggestionController();
   let result = await controller.fetch(
     "Unresolvable Server",
@@ -669,6 +770,8 @@ add_task(async function unresolvable_server() {
   Assert.equal(result.local.length, 1);
   Assert.equal(result.local[0].value, "Unresolvable Server Entry");
   Assert.equal(result.remote.length, 0);
+
+  assertLatencyHistogram(histogram, true);
 });
 
 // Exception handling
@@ -713,40 +816,58 @@ add_task(async function minus_one_results_requested() {
 
 add_task(async function test_userContextId() {
   let controller = new SearchSuggestionController();
-  controller._fetchRemote = function(
+  controller._fetchRemote = function (
     searchTerm,
     engine,
     privateMode,
     userContextId
   ) {
     Assert.equal(userContextId, 1);
-    return PromiseUtils.defer();
+    return Promise.withResolvers();
   };
 
   controller.fetch("test", false, getEngine, 1);
 });
 
+// Non-English characters
+
+add_task(async function suggestions_contain_escaped_unicode() {
+  let controller = new SearchSuggestionController();
+  let result = await controller.fetch("stü", false, getEngine);
+  Assert.equal(result.term, "stü");
+  Assert.equal(result.local.length, 0);
+  Assert.equal(result.remote.length, 2);
+  Assert.equal(result.remote[0].value, "stühle");
+  Assert.equal(result.remote[1].value, "stüssy");
+});
+
 // Helpers
 
 function updateSearchHistory(operation, value) {
-  return new Promise((resolve, reject) => {
-    FormHistory.update(
-      {
-        op: operation,
-        fieldname: "searchbar-history",
-        value,
-      },
-      {
-        handleError(error) {
-          do_throw("Error occurred updating form history: " + error);
-          reject(error);
-        },
-        handleCompletion(reason) {
-          if (!reason) {
-            resolve();
-          }
-        },
-      }
-    );
+  return FormHistory.update({
+    op: operation,
+    fieldname: "searchbar-history",
+    value,
   });
 }
+
+function assertLatencyHistogram(histogram, shouldRecord) {
+  let snapshot = histogram.snapshot();
+  info("Checking latency snapshot: " + JSON.stringify(snapshot));
+
+  // Build a map from engine ID => number of non-zero values recorded for it.
+  let valueCountByEngineId = Object.entries(snapshot).reduce(
+    (memo, [key, data]) => {
+      memo[key] = Object.values(data.values).filter(v => v != 0);
+      return memo;
+    },
+    {}
+  );
+
+  let expected = shouldRecord ? { [ENGINE_NAME]: [1] } : {};
+  Assert.deepEqual(
+    valueCountByEngineId,
+    expected,
+    shouldRecord ? "Latency histogram updated" : "Latency histogram not updated"
+  );
+}
