@@ -2,15 +2,15 @@
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
 import contextlib
-import mozinstall
 import os
-import pytest
 import re
 import sys
 import textwrap
 import time
 
-from marionette_driver import By, keys
+import mozinstall
+import pytest
+from marionette_driver import keys
 from marionette_driver.addons import Addons
 from marionette_driver.errors import MarionetteException
 from marionette_driver.marionette import Marionette
@@ -19,6 +19,7 @@ from six import reraise
 from telemetry_harness.ping_server import PingServer
 
 CANARY_CLIENT_ID = "c0ffeec0-ffee-c0ff-eec0-ffeec0ffeec0"
+CANARY_PROFILE_GROUP_ID = "decafdec-afde-cafd-ecaf-decafdecafde"
 SERVER_ROOT = "toolkit/components/telemetry/tests/marionette/harness/www"
 UUID_PATTERN = re.compile(
     r"^[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{12}$"
@@ -84,7 +85,9 @@ def fixture_marionette(binary, ping_server):
 @pytest.fixture(name="ping_server")
 def fixture_ping_server():
     """Run a ping server on localhost on a free port assigned by the OS"""
-    server = PingServer(SERVER_ROOT, "http://localhost:0")
+    server = PingServer(
+        os.path.join(build.topsrcdir, SERVER_ROOT), "http://localhost:0"
+    )
     server.start()
     yield server
     server.stop()
@@ -102,23 +105,6 @@ class Browser(object):
         )
         self.marionette.set_pref("datareporting.healthreport.uploadEnabled", False)
 
-    def enable_search_events(self):
-        """
-        Event Telemetry categories are disabled by default.
-        Search events are in the "navigation" category and are not enabled by
-        default in builds of Firefox, so we enable them here.
-        """
-
-        script = """\
-        let { Services } = ChromeUtils.import(
-          "resource://gre/modules/Services.jsm"
-        );
-        Services.telemetry.setEventRecordingEnabled("navigation", true);
-        """
-
-        with self.marionette.using_context(self.marionette.CONTEXT_CHROME):
-            self.marionette.execute_script(textwrap.dedent(script))
-
     def enable_telemetry(self):
         self.marionette.instance.profile.set_persistent_preferences(
             {"datareporting.healthreport.uploadEnabled": True}
@@ -130,13 +116,25 @@ class Browser(object):
         with self.marionette.using_context(self.marionette.CONTEXT_CHROME):
             return self.marionette.execute_script(
                 """\
-                const { ClientID } = ChromeUtils.import(
-                  "resource://gre/modules/ClientID.jsm"
+                const { ClientID } = ChromeUtils.importESModule(
+                  "resource://gre/modules/ClientID.sys.mjs"
                 );
                 return ClientID.getCachedClientID();
             """
             )
 
+    def get_profile_group_id(self):
+        """Return the group ID of the current client."""
+        with self.marionette.using_context(self.marionette.CONTEXT_CHROME):
+            return self.marionette.execute_script(
+                """\
+                const { ClientID } = ChromeUtils.importESModule(
+                  "resource://gre/modules/ClientID.sys.mjs"
+                );
+                return ClientID.getCachedProfileGroupID();
+            """
+            )
+
     def get_default_search_engine(self):
         """Retrieve the identifier of the default search engine.
 
@@ -168,8 +166,8 @@ class Browser(object):
             # triggers an "environment-change" ping.
             script = """\
                     let [resolve] = arguments;
-            const { TelemetryEnvironment } = ChromeUtils.import(
-              "resource://gre/modules/TelemetryEnvironment.jsm"
+            const { TelemetryEnvironment } = ChromeUtils.importESModule(
+              "resource://gre/modules/TelemetryEnvironment.sys.mjs"
             );
             TelemetryEnvironment.onInitialized().then(resolve);
             """
@@ -225,7 +223,7 @@ class Browser(object):
 
             return new_tab
 
-    def quit(self, in_app=False):
+    def quit(self, in_app=True):
         self.marionette.quit(in_app=in_app)
 
     def restart(self):
@@ -236,7 +234,7 @@ class Browser(object):
 
         with self.marionette.using_context(self.marionette.CONTEXT_CHROME):
             self.marionette.execute_script("gURLBar.select();")
-            urlbar = self.marionette.find_element(By.ID, "urlbar-input")
+            urlbar = self.marionette.execute_script("return gURLBar.inputField")
             urlbar.send_keys(keys.Keys.DELETE)
             urlbar.send_keys(text + keys.Keys.ENTER)
 
@@ -284,6 +282,7 @@ class Helpers(object):
         assert value is not None
         assert value != ""
         assert value != CANARY_CLIENT_ID
+        assert value != CANARY_PROFILE_GROUP_ID
         assert re.match(UUID_PATTERN, value) is not None
 
     def wait_for_ping(self, action_func, ping_filter):
