@@ -1,26 +1,24 @@
 /* Any copyright is dedicated to the Public Domain.
    http://creativecommons.org/publicdomain/zero/1.0/ */
 
-const { XPCOMUtils } = ChromeUtils.import(
-  "resource://gre/modules/XPCOMUtils.jsm"
+const { XPCOMUtils } = ChromeUtils.importESModule(
+  "resource://gre/modules/XPCOMUtils.sys.mjs"
+);
+const { AppConstants } = ChromeUtils.importESModule(
+  "resource://gre/modules/AppConstants.sys.mjs"
 );
 
-XPCOMUtils.defineLazyModuleGetters(this, {
-  TelemetryScheduler: "resource://gre/modules/TelemetryScheduler.jsm",
-  TelemetryController: "resource://gre/modules/TelemetryController.jsm",
-  TelemetryUtils: "resource://gre/modules/TelemetryUtils.jsm",
-  Services: "resource://gre/modules/Services.jsm",
-  AddonTestUtils: "resource://testing-common/AddonTestUtils.jsm",
-  AppConstants: "resource://gre/modules/AppConstants.jsm",
-  FileUtils: "resource://gre/modules/FileUtils.jsm",
-  Log: "resource://gre/modules/Log.jsm",
-  HttpServer: "resource://testing-common/httpd.js",
-  NetUtil: "resource://gre/modules/NetUtil.jsm",
-  OS: "resource://gre/modules/osfile.jsm",
-  Preferences: "resource://gre/modules/Preferences.jsm",
-  PromiseUtils: "resource://gre/modules/PromiseUtils.jsm",
-  TelemetrySend: "resource://gre/modules/TelemetrySend.jsm",
-  TelemetryStorage: "resource://gre/modules/TelemetryStorage.jsm",
+ChromeUtils.defineESModuleGetters(this, {
+  AddonTestUtils: "resource://testing-common/AddonTestUtils.sys.mjs",
+  FileUtils: "resource://gre/modules/FileUtils.sys.mjs",
+  HttpServer: "resource://testing-common/httpd.sys.mjs",
+  Log: "resource://gre/modules/Log.sys.mjs",
+  NetUtil: "resource://gre/modules/NetUtil.sys.mjs",
+  TelemetryController: "resource://gre/modules/TelemetryController.sys.mjs",
+  TelemetryScheduler: "resource://gre/modules/TelemetryScheduler.sys.mjs",
+  TelemetrySend: "resource://gre/modules/TelemetrySend.sys.mjs",
+  TelemetryStorage: "resource://gre/modules/TelemetryStorage.sys.mjs",
+  TelemetryUtils: "resource://gre/modules/TelemetryUtils.sys.mjs",
 });
 
 const gIsWindows = AppConstants.platform == "win";
@@ -37,14 +35,15 @@ const MILLISECONDS_PER_MINUTE = 60 * 1000;
 const MILLISECONDS_PER_HOUR = 60 * MILLISECONDS_PER_MINUTE;
 const MILLISECONDS_PER_DAY = 24 * MILLISECONDS_PER_HOUR;
 
-const UUID_REGEX = /^[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{12}$/i;
+const UUID_REGEX =
+  /^[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{12}$/i;
 
 var gGlobalScope = this;
 
 const PingServer = {
   _httpServer: null,
   _started: false,
-  _defers: [PromiseUtils.defer()],
+  _defers: [Promise.withResolvers()],
   _currentDeferred: 0,
   _logger: null,
 
@@ -77,13 +76,13 @@ const PingServer = {
   },
 
   resetPingHandler() {
-    this.registerPingHandler((request, response) => {
+    this.registerPingHandler(request => {
       let r = request;
       this._log.trace(
         `defaultPingHandler() - ${r.method} ${r.scheme}://${r.host}:${r.port}${r.path}`
       );
       let deferred = this._defers[this._defers.length - 1];
-      this._defers.push(PromiseUtils.defer());
+      this._defers.push(Promise.withResolvers());
       deferred.resolve(request);
     });
   },
@@ -104,7 +103,7 @@ const PingServer = {
   },
 
   clearRequests() {
-    this._defers = [PromiseUtils.defer()];
+    this._defers = [Promise.withResolvers()];
     this._currentDeferred = 0;
   },
 
@@ -202,16 +201,18 @@ function decodeRequestPayload(request) {
       payload.clientId,
       `Known clientId shouldn't appear in a "${payload.type}" ping on the server.`
     );
+
+    Assert.ok(
+      "profileGroupId" in payload,
+      "Pings with a clientId must also contain a profileGroupId"
+    );
   }
 
   return payload;
 }
 
 function checkPingFormat(aPing, aType, aHasClientId, aHasEnvironment) {
-  const APP_VERSION = "1";
-  const APP_NAME = "XPCShell";
   const PING_FORMAT_VERSION = 4;
-  const PLATFORM_VERSION = "1.9.2";
   const MANDATORY_PING_FIELDS = [
     "type",
     "id",
@@ -291,7 +292,7 @@ async function loadAddonManager(...args) {
 
   // As we're not running in application, we need to setup the features directory
   // used by system add-ons.
-  const distroDir = FileUtils.getDir("ProfD", ["sysfeatures", "app0"], true);
+  const distroDir = FileUtils.getDir("ProfD", ["sysfeatures", "app0"]);
   AddonTestUtils.registerDirectory("XREAppFeat", distroDir);
   await AddonTestUtils.overrideBuiltIns({
     system: ["tel-system-xpi@tests.mozilla.org"],
@@ -306,10 +307,10 @@ function finishAddonManagerStartup() {
 var gAppInfo = null;
 
 function createAppInfo(
-  ID = "xpcshell@tests.mozilla.org",
-  name = "XPCShell",
-  version = "1.0",
-  platformVersion = "1.0"
+  ID = APP_ID,
+  name = APP_NAME,
+  version = APP_VERSION,
+  platformVersion = PLATFORM_VERSION
 ) {
   AddonTestUtils.createAppInfo(ID, name, version, platformVersion);
   gAppInfo = AddonTestUtils.appInfo;
@@ -317,8 +318,8 @@ function createAppInfo(
 
 // Fake the timeout functions for the TelemetryScheduler.
 function fakeSchedulerTimer(set, clear) {
-  const { Policy } = ChromeUtils.import(
-    "resource://gre/modules/TelemetryScheduler.jsm"
+  const { Policy } = ChromeUtils.importESModule(
+    "resource://gre/modules/TelemetryScheduler.sys.mjs"
   );
   Policy.setSchedulerTickTimeout = set;
   Policy.clearSchedulerTickTimeout = clear;
@@ -338,13 +339,25 @@ function fakeSchedulerTimer(set, clear) {
 function fakeNow(...args) {
   const date = new Date(...args);
   const modules = [
-    ChromeUtils.import("resource://gre/modules/TelemetrySession.jsm"),
-    ChromeUtils.import("resource://gre/modules/TelemetryEnvironment.jsm"),
-    ChromeUtils.import("resource://gre/modules/TelemetryControllerParent.jsm"),
-    ChromeUtils.import("resource://gre/modules/TelemetryStorage.jsm"),
-    ChromeUtils.import("resource://gre/modules/TelemetrySend.jsm"),
-    ChromeUtils.import("resource://gre/modules/TelemetryReportingPolicy.jsm"),
-    ChromeUtils.import("resource://gre/modules/TelemetryScheduler.jsm"),
+    ChromeUtils.importESModule(
+      "resource://gre/modules/TelemetrySession.sys.mjs"
+    ),
+    ChromeUtils.importESModule(
+      "resource://gre/modules/TelemetryEnvironment.sys.mjs"
+    ),
+    ChromeUtils.importESModule(
+      "resource://gre/modules/TelemetryControllerParent.sys.mjs"
+    ),
+    ChromeUtils.importESModule(
+      "resource://gre/modules/TelemetryStorage.sys.mjs"
+    ),
+    ChromeUtils.importESModule("resource://gre/modules/TelemetrySend.sys.mjs"),
+    ChromeUtils.importESModule(
+      "resource://gre/modules/TelemetryReportingPolicy.sys.mjs"
+    ),
+    ChromeUtils.importESModule(
+      "resource://gre/modules/TelemetryScheduler.sys.mjs"
+    ),
   ];
 
   for (let m of modules) {
@@ -355,8 +368,8 @@ function fakeNow(...args) {
 }
 
 function fakeMonotonicNow(ms) {
-  const { Policy } = ChromeUtils.import(
-    "resource://gre/modules/TelemetrySession.jsm"
+  const { Policy } = ChromeUtils.importESModule(
+    "resource://gre/modules/TelemetrySession.sys.mjs"
   );
   Policy.monotonicNow = () => ms;
   return ms;
@@ -364,8 +377,8 @@ function fakeMonotonicNow(ms) {
 
 // Fake the timeout functions for TelemetryController sending.
 function fakePingSendTimer(set, clear) {
-  const { Policy } = ChromeUtils.import(
-    "resource://gre/modules/TelemetrySend.jsm"
+  const { Policy } = ChromeUtils.importESModule(
+    "resource://gre/modules/TelemetrySend.sys.mjs"
   );
   let obj = Cu.cloneInto({ set, clear }, TelemetrySend, {
     cloneFunctions: true,
@@ -375,22 +388,22 @@ function fakePingSendTimer(set, clear) {
 }
 
 function fakeMidnightPingFuzzingDelay(delayMs) {
-  const { Policy } = ChromeUtils.import(
-    "resource://gre/modules/TelemetrySend.jsm"
+  const { Policy } = ChromeUtils.importESModule(
+    "resource://gre/modules/TelemetrySend.sys.mjs"
   );
   Policy.midnightPingFuzzingDelay = () => delayMs;
 }
 
 function fakeGeneratePingId(func) {
-  const { Policy } = ChromeUtils.import(
-    "resource://gre/modules/TelemetryControllerParent.jsm"
+  const { Policy } = ChromeUtils.importESModule(
+    "resource://gre/modules/TelemetryControllerParent.sys.mjs"
   );
   Policy.generatePingId = func;
 }
 
 function fakeCachedClientId(uuid) {
-  const { Policy } = ChromeUtils.import(
-    "resource://gre/modules/TelemetryControllerParent.jsm"
+  const { Policy } = ChromeUtils.importESModule(
+    "resource://gre/modules/TelemetryControllerParent.sys.mjs"
   );
   Policy.getCachedClientID = () => uuid;
 }
@@ -398,19 +411,19 @@ function fakeCachedClientId(uuid) {
 // Fake the gzip compression for the next ping to be sent out
 // and immediately reset to the original function.
 function fakeGzipCompressStringForNextPing(length) {
-  const { Policy, gzipCompressString } = ChromeUtils.import(
-    "resource://gre/modules/TelemetrySend.jsm"
+  const { Policy, gzipCompressString } = ChromeUtils.importESModule(
+    "resource://gre/modules/TelemetrySend.sys.mjs"
   );
   let largePayload = generateString(length);
-  Policy.gzipCompressString = data => {
+  Policy.gzipCompressString = () => {
     Policy.gzipCompressString = gzipCompressString;
     return largePayload;
   };
 }
 
 function fakeIntlReady() {
-  const { Policy } = ChromeUtils.import(
-    "resource://gre/modules/TelemetryEnvironment.jsm"
+  const { Policy } = ChromeUtils.importESModule(
+    "resource://gre/modules/TelemetryEnvironment.sys.mjs"
   );
   Policy._intlLoaded = true;
   // Dispatch the observer event in case the promise has been registered already.
@@ -419,13 +432,13 @@ function fakeIntlReady() {
 
 // Override the uninstall ping file names
 function fakeUninstallPingPath(aPathFcn) {
-  const { Policy } = ChromeUtils.import(
-    "resource://gre/modules/TelemetryStorage.jsm"
+  const { Policy } = ChromeUtils.importESModule(
+    "resource://gre/modules/TelemetryStorage.sys.mjs"
   );
   Policy.getUninstallPingPath =
     aPathFcn ||
     (id => ({
-      directory: new FileUtils.File(OS.Constants.Path.profileDir),
+      directory: new FileUtils.File(PathUtils.profileDir),
       file: `uninstall_ping_0123456789ABCDEF_${id}.json`,
     }));
 }
@@ -475,8 +488,8 @@ function getSnapshot(histogramId) {
 
 // Helper for setting an empty list of Environment preferences to watch.
 function setEmptyPrefWatchlist() {
-  const { TelemetryEnvironment } = ChromeUtils.import(
-    "resource://gre/modules/TelemetryEnvironment.jsm"
+  const { TelemetryEnvironment } = ChromeUtils.importESModule(
+    "resource://gre/modules/TelemetryEnvironment.sys.mjs"
   );
   return TelemetryEnvironment.onInitialized().then(() =>
     TelemetryEnvironment.testWatchPreferences(new Map())
@@ -535,7 +548,7 @@ if (runningInParent) {
   }
 
   fakePingSendTimer(
-    (callback, timeout) => {
+    callback => {
       Services.tm.dispatchToMainThread(() => callback());
     },
     () => {}
