@@ -1,27 +1,23 @@
 /* Any copyright is dedicated to the Public Domain.
  * http://creativecommons.org/publicdomain/zero/1.0/ */
 
-const { AddonManager, AddonManagerPrivate } = ChromeUtils.import(
-  "resource://gre/modules/AddonManager.jsm"
+const { AddonManager, AddonManagerPrivate } = ChromeUtils.importESModule(
+  "resource://gre/modules/AddonManager.sys.mjs"
 );
-const { TelemetryEnvironment } = ChromeUtils.import(
-  "resource://gre/modules/TelemetryEnvironment.jsm"
+const { TelemetryEnvironment } = ChromeUtils.importESModule(
+  "resource://gre/modules/TelemetryEnvironment.sys.mjs"
 );
-const { ContentTaskUtils } = ChromeUtils.import(
-  "resource://testing-common/ContentTaskUtils.jsm"
+const { SearchTestUtils } = ChromeUtils.importESModule(
+  "resource://testing-common/SearchTestUtils.sys.mjs"
 );
-const { SearchTestUtils } = ChromeUtils.import(
-  "resource://testing-common/SearchTestUtils.jsm"
-);
-const { TelemetryEnvironmentTesting } = ChromeUtils.import(
-  "resource://testing-common/TelemetryEnvironmentTesting.jsm"
+const { TelemetryEnvironmentTesting } = ChromeUtils.importESModule(
+  "resource://testing-common/TelemetryEnvironmentTesting.sys.mjs"
 );
 
-ChromeUtils.defineModuleGetter(
-  this,
-  "ExtensionTestUtils",
-  "resource://testing-common/ExtensionXPCShellUtils.jsm"
-);
+ChromeUtils.defineESModuleGetters(this, {
+  ExtensionTestUtils:
+    "resource://testing-common/ExtensionXPCShellUtils.sys.mjs",
+});
 
 async function installXPIFromURL(url) {
   let install = await AddonManager.getInstallForURL(url);
@@ -44,7 +40,7 @@ MockAddonWrapper.prototype = {
   },
 
   get type() {
-    return "service";
+    return this.addon.type;
   },
 
   get appDisabled() {
@@ -140,11 +136,11 @@ add_task(async function setup() {
   do_get_profile();
 
   // We need to ensure FOG is initialized, otherwise we will panic trying to get test values.
-  let FOG = Cc["@mozilla.org/toolkit/glean;1"].createInstance(Ci.nsIFOG);
-  FOG.initializeFOG();
+  Services.fog.initializeFOG();
 
   // The system add-on must be installed before AddonManager is started.
-  const distroDir = FileUtils.getDir("ProfD", ["sysfeatures", "app0"], true);
+  const distroDir = FileUtils.getDir("ProfD", ["sysfeatures", "app0"]);
+  distroDir.create(Ci.nsIFile.DIRECTORY_TYPE, FileUtils.PERMS_DIRECTORY);
   do_get_file("system.xpi").copyTo(
     distroDir,
     "tel-system-xpi@tests.mozilla.org.xpi"
@@ -165,9 +161,6 @@ add_task(async function setup() {
   await AddonTestUtils.overrideBuiltIns({ system: [] });
   AddonTestUtils.addonStartup.remove(true);
   await AddonTestUtils.promiseStartupManager();
-  // Override ExtensionXPCShellUtils.jsm's overriding of the pref as the
-  // search service needs it.
-  Services.prefs.clearUserPref("services.settings.default_bucket");
 
   // Setup a webserver to serve Addons, etc.
   gHttpServer = new HttpServer();
@@ -182,12 +175,16 @@ add_task(async function setup() {
   // The attribution functionality only exists in Firefox.
   if (AppConstants.MOZ_BUILD_APP == "browser") {
     TelemetryEnvironmentTesting.spoofAttributionData();
-    registerCleanupFunction(TelemetryEnvironmentTesting.cleanupAttributionData);
+    registerCleanupFunction(async function () {
+      await TelemetryEnvironmentTesting.cleanupAttributionData;
+    });
   }
 
   await TelemetryEnvironmentTesting.spoofProfileReset();
   await TelemetryEnvironment.delayedInit();
-  await SearchTestUtils.useTestEngines("data", "search-extensions");
+  // The environment needs the search service initialised, so use a dummy
+  // configuration.
+  await SearchTestUtils.setRemoteSettingsConfig([{ identifier: "unused" }]);
 });
 
 add_task(async function test_checkEnvironment() {
@@ -257,12 +254,12 @@ add_task(async function test_prefWatchPolicies() {
     ],
   ]);
 
-  Preferences.set(PREF_TEST_4, expectedValue);
-  Preferences.set(PREF_TEST_5, expectedValue);
+  Services.prefs.setStringPref(PREF_TEST_4, expectedValue);
+  Services.prefs.setStringPref(PREF_TEST_5, expectedValue);
 
   // Set the Environment preferences to watch.
   await TelemetryEnvironment.testWatchPreferences(PREFS_TO_WATCH);
-  let deferred = PromiseUtils.defer();
+  let deferred = Promise.withResolvers();
 
   // Check that the pref values are missing or present as expected
   Assert.strictEqual(
@@ -285,9 +282,9 @@ add_task(async function test_prefWatchPolicies() {
   let oldEnvironmentData = TelemetryEnvironment.currentEnvironment;
 
   // Trigger a change in the watched preferences.
-  Preferences.set(PREF_TEST_1, expectedValue);
-  Preferences.set(PREF_TEST_2, false);
-  Preferences.set(PREF_TEST_5, unexpectedValue);
+  Services.prefs.setStringPref(PREF_TEST_1, expectedValue);
+  Services.prefs.setBoolPref(PREF_TEST_2, false);
+  Services.prefs.setStringPref(PREF_TEST_5, unexpectedValue);
   let eventEnvironmentData = await deferred.promise;
 
   // Unregister the listener.
@@ -325,11 +322,11 @@ add_task(async function test_prefWatch_prefReset() {
   ]);
 
   // Set the preference to a non-default value.
-  Preferences.set(PREF_TEST, false);
+  Services.prefs.setBoolPref(PREF_TEST, false);
 
   // Set the Environment preferences to watch.
   await TelemetryEnvironment.testWatchPreferences(PREFS_TO_WATCH);
-  let deferred = PromiseUtils.defer();
+  let deferred = Promise.withResolvers();
   TelemetryEnvironment.registerChangeListener(
     "testWatchPrefs_reset",
     deferred.resolve
@@ -341,7 +338,7 @@ add_task(async function test_prefWatch_prefReset() {
   );
 
   // Trigger a change in the watched preferences.
-  Preferences.reset(PREF_TEST);
+  Services.prefs.clearUserPref(PREF_TEST);
   await deferred.promise;
 
   Assert.strictEqual(
@@ -431,7 +428,7 @@ add_task(async function test_addonsWatch_InterestingChange() {
     return new Promise(resolve =>
       TelemetryEnvironment.registerChangeListener(
         "testWatchAddons_Changes" + aExpected,
-        (reason, data) => {
+        reason => {
           Assert.equal(reason, "addons-changed");
           receivedNotifications++;
           resolve();
@@ -491,12 +488,16 @@ add_task(async function test_addonsWatch_InterestingChange() {
 });
 
 add_task(async function test_addonsWatch_NotInterestingChange() {
-  // We are not interested to dictionary addons changes.
-  const DICTIONARY_ADDON_INSTALL_URL = gDataRoot + "dictionary.xpi";
-  const INTERESTING_ADDON_INSTALL_URL = gDataRoot + "restartless.xpi";
+  // Plugins from GMPProvider are listed separately in addons.activeGMPlugins.
+  // We simulate the "plugin" type in this test and verify that it is excluded.
+  const PLUGIN_ID = "tel-fake-gmp-plugin@tests.mozilla.org";
+  // "theme" type is already covered by addons.theme, so we aren't interested.
+  const THEME_ID = "tel-theme@tests.mozilla.org";
+  // "dictionary" type should be in addon.activeAddons.
+  const DICT_ID = "tel-dict@tests.mozilla.org";
 
   let receivedNotification = false;
-  let deferred = PromiseUtils.defer();
+  let deferred = Promise.withResolvers();
   TelemetryEnvironment.registerChangeListener("testNotInteresting", () => {
     Assert.ok(
       !receivedNotification,
@@ -506,23 +507,53 @@ add_task(async function test_addonsWatch_NotInterestingChange() {
     deferred.resolve();
   });
 
-  let dictionaryAddon = await installXPIFromURL(DICTIONARY_ADDON_INSTALL_URL);
-  let interestingAddon = await installXPIFromURL(INTERESTING_ADDON_INSTALL_URL);
+  // "plugin" type, to verify that non-XPIProvider types such as the "plugin"
+  // type from GMPProvider are not included in activeAddons.
+  let fakePluginProvider = createMockAddonProvider("Fake GMPProvider");
+  AddonManagerPrivate.registerProvider(fakePluginProvider);
+  fakePluginProvider.addAddon({
+    id: PLUGIN_ID,
+    name: "Fake plugin",
+    version: "1",
+    type: "plugin",
+  });
+
+  // "theme" type.
+  let themeXpi = AddonTestUtils.createTempWebExtensionFile({
+    manifest: {
+      theme: {},
+      browser_specific_settings: { gecko: { id: THEME_ID } },
+    },
+  });
+  let themeAddon = (await AddonTestUtils.promiseInstallFile(themeXpi)).addon;
+
+  let dictXpi = AddonTestUtils.createTempWebExtensionFile({
+    manifest: {
+      dictionaries: {},
+      browser_specific_settings: { gecko: { id: DICT_ID } },
+    },
+  });
+  let dictAddon = (await AddonTestUtils.promiseInstallFile(dictXpi)).addon;
 
   await deferred.promise;
   Assert.ok(
-    !(
-      "telemetry-dictionary@tests.mozilla.org" in
-      TelemetryEnvironment.currentEnvironment.addons.activeAddons
-    ),
-    "Dictionaries should not appear in active addons."
+    !(PLUGIN_ID in TelemetryEnvironment.currentEnvironment.addons.activeAddons),
+    "GMP plugins should not appear in active addons."
+  );
+  Assert.ok(
+    !(THEME_ID in TelemetryEnvironment.currentEnvironment.addons.activeAddons),
+    "Themes should not appear in active addons."
+  );
+  Assert.ok(
+    DICT_ID in TelemetryEnvironment.currentEnvironment.addons.activeAddons,
+    "Dictionaries should appear in active addons."
   );
 
   TelemetryEnvironment.unregisterChangeListener("testNotInteresting");
 
-  dictionaryAddon.uninstall();
-  await interestingAddon.startupPromise;
-  interestingAddon.uninstall();
+  AddonManagerPrivate.unregisterProvider(fakePluginProvider);
+  await themeAddon.uninstall();
+  await dictAddon.uninstall();
 });
 
 add_task(async function test_addons() {
@@ -546,6 +577,10 @@ add_task(async function test_addons() {
     isSystem: false,
     isWebExtension: true,
     multiprocessCompatible: true,
+    quarantineIgnoredByUser: false,
+    // quarantineIgnoredByApp expected to be true because
+    // the test addon is signed as privileged (see signedState).
+    quarantineIgnoredByApp: true,
   };
   const SYSTEM_ADDON_ID = "tel-system-xpi@tests.mozilla.org";
   const EXPECTED_SYSTEM_ADDON_DATA = {
@@ -565,6 +600,10 @@ add_task(async function test_addons() {
     isSystem: true,
     isWebExtension: true,
     multiprocessCompatible: true,
+    quarantineIgnoredByUser: false,
+    // quarantineIgnoredByApp expected to be true because
+    // the test addon is a system addon (see isSystem).
+    quarantineIgnoredByApp: true,
   };
 
   const WEBEXTENSION_ADDON_ID = "tel-webextension-xpi@tests.mozilla.org";
@@ -586,21 +625,23 @@ add_task(async function test_addons() {
     isSystem: false,
     isWebExtension: true,
     multiprocessCompatible: true,
+    quarantineIgnoredByUser: false,
+    // quarantineIgnoredByApp expected to be true because
+    // the test addon is signed as privileged (see signedState).
+    quarantineIgnoredByApp: true,
   };
 
-  let deferred = PromiseUtils.defer();
-  TelemetryEnvironment.registerChangeListener(
-    "test_WebExtension",
-    (reason, data) => {
-      Assert.equal(reason, "addons-changed");
-      deferred.resolve();
-    }
-  );
+  let deferred = Promise.withResolvers();
+  TelemetryEnvironment.registerChangeListener("test_WebExtension", reason => {
+    Assert.equal(reason, "addons-changed");
+    deferred.resolve();
+  });
 
   // Install an add-on so we have some data.
   let addon = await installXPIFromURL(ADDON_INSTALL_URL);
 
   // Install a webextension as well.
+  // Note: all addons are webextensions, so doing this again is redundant...
   ExtensionTestUtils.init(this);
 
   let webextension = ExtensionTestUtils.loadExtension({
@@ -609,7 +650,7 @@ add_task(async function test_addons() {
       name: "XPI Telemetry WebExtension Add-on Test",
       description: "A webextension addon.",
       version: "1.0",
-      applications: {
+      browser_specific_settings: {
         gecko: {
           id: WEBEXTENSION_ADDON_ID,
         },
@@ -673,19 +714,79 @@ add_task(async function test_addons() {
   await addon.uninstall();
 });
 
+add_task(async function test_signedTheme() {
+  AddonTestUtils.useRealCertChecks = true;
+
+  const { PKCS7_WITH_SHA1, COSE_WITH_SHA256 } = Ci.nsIAppSignatureInfo;
+
+  const ADDON_THEME_INSTALL_URL = gDataRoot + "webext-implicit-id.xpi";
+  const ADDON_THEME_ID = "{46607a7b-1b2a-40ce-9afe-91cda52c46a6}";
+
+  // Install the theme.
+  let deferred = Promise.withResolvers();
+  TelemetryEnvironment.registerChangeListener(
+    "test_signedAddon",
+    deferred.resolve
+  );
+  let theme = await installXPIFromURL(ADDON_THEME_INSTALL_URL);
+  await theme.enable();
+  ok(theme.isActive, "Theme should be active");
+
+  // Install an extension to force the telemetry environment to be
+  // updated (currently theme add-ons changes do not seem to be
+  // notified as changes, see EnvironmentAddonBuilder _updateAddons
+  // method for how changes to the environment.addons property are
+  // being detected).
+  const ADDON_INSTALL_URL = gDataRoot + "amosigned.xpi";
+  let addon = await installXPIFromURL(ADDON_INSTALL_URL);
+
+  await deferred.promise;
+  TelemetryEnvironment.unregisterChangeListener("test_signedAddon");
+
+  let data = TelemetryEnvironment.currentEnvironment;
+  TelemetryEnvironmentTesting.checkEnvironmentData(data);
+
+  // Check signedState and signedTypes on active theme data
+  // (NOTE: other properties of active theme are technically
+  // not covered by any other test task in this xpcshell test).
+  Assert.equal(
+    data.addons.theme.id,
+    ADDON_THEME_ID,
+    "Theme should be in the environment."
+  );
+  Assert.equal(
+    data.addons.theme.signedState,
+    AddonManager.SIGNEDSTATE_SIGNED,
+    "Got expected signedState on activeTheme"
+  );
+  Assert.equal(
+    data.addons.theme.signedTypes,
+    JSON.stringify([COSE_WITH_SHA256, PKCS7_WITH_SHA1]),
+    "Got expected signedTypes on activeTheme"
+  );
+
+  AddonTestUtils.useRealCertChecks = false;
+  await addon.startupPromise;
+  await addon.uninstall();
+  await theme.startupPromise;
+  await theme.uninstall();
+});
+
 add_task(async function test_signedAddon() {
   AddonTestUtils.useRealCertChecks = true;
 
-  const ADDON_INSTALL_URL = gDataRoot + "signed-webext.xpi";
-  const ADDON_ID = "tel-signed-webext@tests.mozilla.org";
+  const { PKCS7_WITH_SHA1, COSE_WITH_SHA256 } = Ci.nsIAppSignatureInfo;
+
+  const ADDON_INSTALL_URL = gDataRoot + "amosigned.xpi";
+  const ADDON_ID = "amosigned-xpi@tests.mozilla.org";
   const ADDON_INSTALL_DATE = truncateToDays(Date.now());
   const EXPECTED_ADDON_DATA = {
     blocklisted: false,
-    description: "A signed webextension",
-    name: "XPI Telemetry Signed Test",
+    description: null,
+    name: "XPI Test",
     userDisabled: false,
     appDisabled: false,
-    version: "1.0",
+    version: "2.2",
     scope: 1,
     type: "extension",
     foreignInstall: false,
@@ -693,9 +794,15 @@ add_task(async function test_signedAddon() {
     installDay: ADDON_INSTALL_DATE,
     updateDay: ADDON_INSTALL_DATE,
     signedState: AddonManager.SIGNEDSTATE_SIGNED,
+    signedTypes: JSON.stringify([COSE_WITH_SHA256, PKCS7_WITH_SHA1]),
+    quarantineIgnoredByUser: false,
+    // quarantineIgnoredByApp expected to be false because
+    // the test addon is signed as a non-privileged (see signedState),
+    // and it doesn't include any recommendations.
+    quarantineIgnoredByApp: false,
   };
 
-  let deferred = PromiseUtils.defer();
+  let deferred = Promise.withResolvers();
   TelemetryEnvironment.registerChangeListener(
     "test_signedAddon",
     deferred.resolve
@@ -725,6 +832,28 @@ add_task(async function test_signedAddon() {
     );
   }
 
+  // Make sure quarantineIgnoredByUser property is updated also in the
+  // telemetry environment in response to the user changing it.
+  deferred = Promise.withResolvers();
+  TelemetryEnvironment.registerChangeListener(
+    "test_quarantineIgnoreByUser_changed",
+    deferred.resolve
+  );
+
+  addon.quarantineIgnoredByUser = true;
+  await deferred.promise;
+  // Unregister the listener.
+  TelemetryEnvironment.unregisterChangeListener(
+    "test_quarantineIgnoreByUser_changed"
+  );
+
+  Assert.equal(
+    TelemetryEnvironment.currentEnvironment.addons.activeAddons[ADDON_ID]
+      .quarantineIgnoredByUser,
+    true,
+    "Expect quarantineIgnoredByUser to be set to true"
+  );
+
   AddonTestUtils.useRealCertChecks = false;
   await addon.startupPromise;
   await addon.uninstall();
@@ -735,7 +864,7 @@ add_task(async function test_addonsFieldsLimit() {
   const ADDON_ID = "tel-longfields-webext@tests.mozilla.org";
 
   // Install the addon and wait for the TelemetryEnvironment to pick it up.
-  let deferred = PromiseUtils.defer();
+  let deferred = Promise.withResolvers();
   TelemetryEnvironment.registerChangeListener(
     "test_longFieldsAddon",
     deferred.resolve
@@ -812,7 +941,7 @@ add_task(async function test_collectionWithbrokenAddonData() {
     return new Promise(resolve =>
       TelemetryEnvironment.registerChangeListener(
         "testBrokenAddon_collection" + aExpected,
-        (reason, data) => {
+        reason => {
           Assert.equal(reason, "addons-changed");
           receivedNotifications++;
           resolve();
@@ -844,8 +973,8 @@ add_task(async function test_collectionWithbrokenAddonData() {
   await checkpointPromise;
   assertCheckpoint(2);
 
-  // Check that the new environment contains the Social addon installed with the broken
-  // manifest and the rest of the data.
+  // Check that the new environment contains the info from the broken provider,
+  // despite the addon missing some details.
   let data = TelemetryEnvironment.currentEnvironment;
   TelemetryEnvironmentTesting.checkEnvironmentData(data, {
     expectBrokenAddons: true,
@@ -894,6 +1023,10 @@ add_task(
     );
 
     Services.obs.notifyObservers(null, "sessionstore-windows-restored");
+    // Session restore triggers search service init asynchronously.
+    // If this completes during shutdown, it throws an exception.
+    // Await the search service init to make this deterministic (bug 1885310).
+    await Services.search.promiseInitialized;
 
     environmentData = TelemetryEnvironment.currentEnvironment;
     TelemetryEnvironmentTesting.checkEnvironmentData(environmentData);
@@ -912,17 +1045,17 @@ add_task(
     const PREFS_TO_WATCH = new Map([
       [PREF_TEST, { what: TelemetryEnvironment.RECORD_PREF_STATE }],
     ]);
-    Preferences.reset(PREF_TEST);
+    Services.prefs.clearUserPref(PREF_TEST);
 
     // Watch the test preference.
     await TelemetryEnvironment.testWatchPreferences(PREFS_TO_WATCH);
-    let deferred = PromiseUtils.defer();
+    let deferred = Promise.withResolvers();
     TelemetryEnvironment.registerChangeListener(
       "testDefaultBrowser_pref",
       deferred.resolve
     );
     // Trigger an environment change.
-    Preferences.set(PREF_TEST, 1);
+    Services.prefs.setIntPref(PREF_TEST, 1);
     await deferred.promise;
     TelemetryEnvironment.unregisterChangeListener("testDefaultBrowser_pref");
 
@@ -983,7 +1116,7 @@ add_task(async function test_experimentsAPI() {
   const EXPERIMENT2 = "experiment-2";
   const EXPERIMENT2_BRANCH = "other-branch";
 
-  let checkExperiment = (environmentData, id, branch, type = null) => {
+  let checkExperiment = (environmentData, id, branch) => {
     Assert.ok(
       "experiments" in environmentData,
       "The current environment must report the experiment annotations."
@@ -1012,7 +1145,7 @@ add_task(async function test_experimentsAPI() {
   );
 
   // Add a change listener and add an experiment annotation.
-  let deferred = PromiseUtils.defer();
+  let deferred = Promise.withResolvers();
   TelemetryEnvironment.registerChangeListener(
     "test_experimentsAPI",
     (reason, env) => {
@@ -1037,7 +1170,7 @@ add_task(async function test_experimentsAPI() {
   TelemetryEnvironment.unregisterChangeListener("test_experimentsAPI");
 
   // Add a second annotation and check that both experiments are there.
-  deferred = PromiseUtils.defer();
+  deferred = Promise.withResolvers();
   TelemetryEnvironment.registerChangeListener(
     "test_experimentsAPI2",
     (reason, env) => {
@@ -1079,7 +1212,7 @@ add_task(async function test_experimentsAPI() {
 
   // Check that removing a known experiment leaves the other in place and triggers
   // a change.
-  deferred = PromiseUtils.defer();
+  deferred = Promise.withResolvers();
   TelemetryEnvironment.registerChangeListener(
     "test_experimentsAPI4",
     (reason, env) => {
@@ -1135,7 +1268,7 @@ add_task(async function test_experimentsAPI_limits() {
   );
 
   // Add a change listener and wait for the annotation to happen.
-  let deferred = PromiseUtils.defer();
+  let deferred = Promise.withResolvers();
   TelemetryEnvironment.registerChangeListener("test_experimentsAPI", () =>
     deferred.resolve()
   );
@@ -1210,6 +1343,18 @@ if (gIsWindows) {
       "boolean",
       "isWowARM64 must be a boolean."
     );
+    Assert.equal(
+      typeof data.system.hasWinPackageId,
+      "boolean",
+      "hasWinPackageId must be a boolean."
+    );
+    // This is only sent for Mozilla produced MSIX packages
+    Assert.ok(
+      !("winPackageFamilyName" in data.system) ||
+        data.system.winPackageFamilyName === null ||
+        typeof data.system.winPackageFamilyName === "string",
+      "winPackageFamilyName must be a string if non null"
+    );
     // These should be numbers if they are not null
     for (let f of [
       "count",
@@ -1261,7 +1406,10 @@ add_task(
       // Test the 'yes to both' case.
 
       // This makes the weave service return that the usere is definitely a sync user
-      Preferences.set("services.sync.username", "c00lperson123@example.com");
+      Services.prefs.setStringPref(
+        "services.sync.username",
+        "c00lperson123@example.com"
+      );
       let calledFxa = false;
       cache._getFxaSignedInUser = () => {
         calledFxa = true;
@@ -1279,7 +1427,7 @@ add_task(
       });
 
       // Test the fxa-but-not-sync case.
-      Preferences.reset("services.sync.username");
+      Services.prefs.clearUserPref("services.sync.username");
       // We don't actually inspect the returned object, just t
       cache._getFxaSignedInUser = async () => {
         return {};
@@ -1306,7 +1454,7 @@ add_task(
       equal(cache.currentEnvironment.services, null);
     } finally {
       cache._getFxaSignedInUser = oldGetFxaSignedInUser;
-      Preferences.reset("services.sync.username");
+      Services.prefs.clearUserPref("services.sync.username");
     }
   }
 );
@@ -1347,7 +1495,7 @@ add_task(async function test_environmentShutdown() {
   const PREFS_TO_WATCH = new Map([
     [PREF_TEST, { what: TelemetryEnvironment.RECORD_PREF_STATE }],
   ]);
-  Preferences.reset(PREF_TEST);
+  Services.prefs.clearUserPref(PREF_TEST);
 
   // Set up the preferences and listener, then the trigger shutdown
   await TelemetryEnvironment.testWatchPreferences(PREFS_TO_WATCH);
@@ -1361,7 +1509,7 @@ add_task(async function test_environmentShutdown() {
   TelemetryEnvironment.shutdown();
 
   // Flipping  the test preference after shutdown should not trigger the listener
-  Preferences.set(PREF_TEST, 1);
+  Services.prefs.setIntPref(PREF_TEST, 1);
 
   // Unregister the listener.
   TelemetryEnvironment.unregisterChangeListener(
