@@ -3,11 +3,6 @@
 
 const INT_MAX = 0x7fffffff;
 
-const { Services } = ChromeUtils.import("resource://gre/modules/Services.jsm");
-const { TelemetryUtils } = ChromeUtils.import(
-  "resource://gre/modules/TelemetryUtils.jsm"
-);
-
 // Return an array of numbers from lower up to, excluding, upper
 function numberRange(lower, upper) {
   let a = [];
@@ -17,29 +12,7 @@ function numberRange(lower, upper) {
   return a;
 }
 
-function expect_fail(f) {
-  let failed = false;
-  try {
-    f();
-    failed = false;
-  } catch (e) {
-    failed = true;
-  }
-  Assert.ok(failed);
-}
-
-function expect_success(f) {
-  let succeeded = false;
-  try {
-    f();
-    succeeded = true;
-  } catch (e) {
-    succeeded = false;
-  }
-  Assert.ok(succeeded);
-}
-
-function check_histogram(histogram_type, name, min, max, bucket_count) {
+function check_histogram(histogram_type, name, min, max) {
   var h = Telemetry.getHistogramById(name);
   h.add(0);
   var s = h.snapshot();
@@ -156,8 +129,10 @@ add_task(async function test_noSerialization() {
   // Instantiate the storage for this histogram and make sure it doesn't
   // get reflected into JS, as it has no interesting data in it.
   Telemetry.getHistogramById("NEWTAB_PAGE_PINNED_SITES_COUNT");
-  let histograms = Telemetry.getSnapshotForHistograms("main", false /* clear */)
-    .parent;
+  let histograms = Telemetry.getSnapshotForHistograms(
+    "main",
+    false /* clear */
+  ).parent;
   Assert.equal(false, "NEWTAB_PAGE_PINNED_SITES_COUNT" in histograms);
 });
 
@@ -313,9 +288,8 @@ add_task(async function test_add_error_behaviour() {
 
   // Check that |add| doesn't throw for plain histograms.
   for (let hist of PLAIN_HISTOGRAMS_TO_TEST) {
-    const returnValue = Telemetry.getHistogramById(hist).add(
-      "unexpected-value"
-    );
+    const returnValue =
+      Telemetry.getHistogramById(hist).add("unexpected-value");
     Assert.strictEqual(
       returnValue,
       undefined,
@@ -533,7 +507,7 @@ add_task(async function test_keyed_boolean_histogram() {
     sum: 1,
     values: { 0: 0, 1: 1, 2: 0 },
   };
-  let testHistograms = numberRange(0, 3).map(i =>
+  let testHistograms = numberRange(0, 3).map(() =>
     JSON.parse(JSON.stringify(histogramBase))
   );
   let testKeys = [];
@@ -584,7 +558,7 @@ add_task(async function test_keyed_count_histogram() {
     sum: 0,
     values: { 0: 1, 1: 0 },
   };
-  let testHistograms = numberRange(0, 5).map(i =>
+  let testHistograms = numberRange(0, 5).map(() =>
     JSON.parse(JSON.stringify(histogramBase))
   );
   let testKeys = [];
@@ -760,156 +734,6 @@ add_task(async function test_keyed_histogram_recording() {
   Assert.equal(h.snapshot()[TEST_KEY].sum, 1);
 });
 
-add_task(async function test_histogram_recording_enabled() {
-  Telemetry.canRecordBase = true;
-  Telemetry.canRecordExtended = true;
-
-  // Check that a "normal" histogram respects recording-enabled on/off
-  var h = Telemetry.getHistogramById("TELEMETRY_TEST_COUNT");
-  var orig = h.snapshot();
-
-  h.add(1);
-  Assert.equal(orig.sum + 1, h.snapshot().sum, "add should record by default.");
-
-  // Check that when recording is disabled - add is ignored
-  Telemetry.setHistogramRecordingEnabled("TELEMETRY_TEST_COUNT", false);
-  h.add(1);
-  Assert.equal(
-    orig.sum + 1,
-    h.snapshot().sum,
-    "When recording is disabled add should not record."
-  );
-
-  // Check that we're back to normal after recording is enabled
-  Telemetry.setHistogramRecordingEnabled("TELEMETRY_TEST_COUNT", true);
-  h.add(1);
-  Assert.equal(
-    orig.sum + 2,
-    h.snapshot().sum,
-    "When recording is re-enabled add should record."
-  );
-
-  // Check that we're correctly accumulating values other than 1.
-  h.clear();
-  h.add(3);
-  Assert.equal(
-    3,
-    h.snapshot().sum,
-    "Recording counts greater than 1 should work."
-  );
-
-  // Check that a histogram with recording disabled by default behaves correctly
-  h = Telemetry.getHistogramById("TELEMETRY_TEST_COUNT_INIT_NO_RECORD");
-  orig = h.snapshot();
-
-  h.add(1);
-  Assert.equal(
-    orig.sum,
-    h.snapshot().sum,
-    "When recording is disabled by default, add should not record by default."
-  );
-
-  Telemetry.setHistogramRecordingEnabled(
-    "TELEMETRY_TEST_COUNT_INIT_NO_RECORD",
-    true
-  );
-  h.add(1);
-  Assert.equal(
-    orig.sum + 1,
-    h.snapshot().sum,
-    "When recording is enabled add should record."
-  );
-
-  // Restore to disabled
-  Telemetry.setHistogramRecordingEnabled(
-    "TELEMETRY_TEST_COUNT_INIT_NO_RECORD",
-    false
-  );
-  h.add(1);
-  Assert.equal(
-    orig.sum + 1,
-    h.snapshot().sum,
-    "When recording is disabled add should not record."
-  );
-});
-
-add_task(async function test_keyed_histogram_recording_enabled() {
-  Telemetry.canRecordBase = true;
-  Telemetry.canRecordExtended = true;
-
-  // Check RecordingEnabled for keyed histograms which are recording by default
-  const TEST_KEY = "record_foo";
-  let h = Telemetry.getKeyedHistogramById(
-    "TELEMETRY_TEST_KEYED_RELEASE_OPTOUT"
-  );
-
-  h.clear();
-  h.add(TEST_KEY, 1);
-  Assert.equal(
-    h.snapshot()[TEST_KEY].sum,
-    1,
-    "Keyed histogram add should record by default"
-  );
-
-  Telemetry.setHistogramRecordingEnabled(
-    "TELEMETRY_TEST_KEYED_RELEASE_OPTOUT",
-    false
-  );
-  h.add(TEST_KEY, 1);
-  Assert.equal(
-    h.snapshot()[TEST_KEY].sum,
-    1,
-    "Keyed histogram add should not record when recording is disabled"
-  );
-
-  Telemetry.setHistogramRecordingEnabled(
-    "TELEMETRY_TEST_KEYED_RELEASE_OPTOUT",
-    true
-  );
-  h.clear();
-  h.add(TEST_KEY, 1);
-  Assert.equal(
-    h.snapshot()[TEST_KEY].sum,
-    1,
-    "Keyed histogram add should record when recording is re-enabled"
-  );
-
-  // Check that a histogram with recording disabled by default behaves correctly
-  h = Telemetry.getKeyedHistogramById(
-    "TELEMETRY_TEST_KEYED_COUNT_INIT_NO_RECORD"
-  );
-  h.clear();
-
-  h.add(TEST_KEY, 1);
-  Assert.ok(
-    !(TEST_KEY in h.snapshot()),
-    "Keyed histogram add should not record by default for histograms which don't record by default"
-  );
-
-  Telemetry.setHistogramRecordingEnabled(
-    "TELEMETRY_TEST_KEYED_COUNT_INIT_NO_RECORD",
-    true
-  );
-  h.add(TEST_KEY, 1);
-  Assert.equal(
-    h.snapshot()[TEST_KEY].sum,
-    1,
-    "Keyed histogram add should record when recording is enabled"
-  );
-
-  // Restore to disabled
-  Telemetry.setHistogramRecordingEnabled(
-    "TELEMETRY_TEST_KEYED_COUNT_INIT_NO_RECORD",
-    false
-  );
-  h.add(TEST_KEY, 1);
-  Assert.equal(
-    h.snapshot()[TEST_KEY].sum,
-    1,
-    "Keyed histogram add should not record when recording is disabled"
-  );
-});
-
 add_task(async function test_histogramSnapshots() {
   let keyed = Telemetry.getKeyedHistogramById("TELEMETRY_TEST_KEYED_COUNT");
   keyed.add("a", 1);
@@ -1013,8 +837,10 @@ add_task(async function test_keyed_keys() {
   Assert.ok(!("not-allowed" in snap), "'not-allowed' must not be recorded.");
 
   // Check that these failures were correctly tracked.
-  const parentScalars = Telemetry.getSnapshotForKeyedScalars("main", false)
-    .parent;
+  const parentScalars = Telemetry.getSnapshotForKeyedScalars(
+    "main",
+    false
+  ).parent;
   const scalarName = "telemetry.accumulate_unknown_histogram_keys";
   Assert.ok(
     scalarName in parentScalars,
@@ -1359,8 +1185,10 @@ add_task(async function test_productsOverride() {
   multiproduct_histo.add(1);
   mobile_histo.add(1);
 
-  let histograms = Telemetry.getSnapshotForHistograms("main", false /* clear */)
-    .parent;
+  let histograms = Telemetry.getSnapshotForHistograms(
+    "main",
+    false /* clear */
+  ).parent;
 
   Assert.ok(
     DEFAULT_PRODUCTS_HISTOGRAM in histograms,
@@ -1395,16 +1223,20 @@ add_task(
     let snapshot;
 
     // The first snapshot should be empty, nothing recorded.
-    snapshot = Telemetry.getSnapshotForHistograms("main", false /* clear */)
-      .parent;
+    snapshot = Telemetry.getSnapshotForHistograms(
+      "main",
+      false /* clear */
+    ).parent;
     Assert.ok(!(COUNT in snapshot));
 
     // After recording into a histogram, the data should be in the snapshot. Don't delete it.
     h.add(1);
 
     Assert.equal(h.snapshot().sum, 1);
-    snapshot = Telemetry.getSnapshotForHistograms("main", false /* clear */)
-      .parent;
+    snapshot = Telemetry.getSnapshotForHistograms(
+      "main",
+      false /* clear */
+    ).parent;
     Assert.ok(COUNT in snapshot);
     Assert.equal(snapshot[COUNT].sum, 1);
 
@@ -1413,63 +1245,23 @@ add_task(
     h.add(41);
 
     Assert.equal(h.snapshot().sum, 42);
-    snapshot = Telemetry.getSnapshotForHistograms("main", true /* clear */)
-      .parent;
+    snapshot = Telemetry.getSnapshotForHistograms(
+      "main",
+      true /* clear */
+    ).parent;
     Assert.ok(COUNT in snapshot);
     Assert.equal(snapshot[COUNT].sum, 42);
 
     // Finally, no data should be in the snapshot.
     Assert.equal(h.snapshot().sum, 0);
-    snapshot = Telemetry.getSnapshotForHistograms("main", false /* clear */)
-      .parent;
+    snapshot = Telemetry.getSnapshotForHistograms(
+      "main",
+      false /* clear */
+    ).parent;
     Assert.ok(!(COUNT in snapshot));
   }
 );
 
-add_task(async function test_valid_os_smoketest() {
-  let nonExistingProbe;
-  let existingProbe;
-
-  switch (AppConstants.platform) {
-    case "linux":
-      nonExistingProbe = "TELEMETRY_TEST_OS_ANDROID_ONLY";
-      existingProbe = "TELEMETRY_TEST_OS_LINUX_ONLY";
-      break;
-    case "macosx":
-      nonExistingProbe = "TELEMETRY_TEST_OS_ANDROID_ONLY";
-      existingProbe = "TELEMETRY_TEST_OS_MAC_ONLY";
-      break;
-    case "win":
-      nonExistingProbe = "TELEMETRY_TEST_OS_ANDROID_ONLY";
-      existingProbe = "TELEMETRY_TEST_OS_WIN_ONLY";
-      break;
-    case "android":
-      nonExistingProbe = "TELEMETRY_TEST_OS_LINUX_ONLY";
-      existingProbe = "TELEMETRY_TEST_OS_ANDROID_ONLY";
-      break;
-    default:
-      /* Unknown OS. Let's not test OS-specific probes */
-      return;
-  }
-
-  Assert.throws(
-    () => Telemetry.getHistogramById(nonExistingProbe),
-    /NS_ERROR_FAILURE/,
-    `Should throw on ${nonExistingProbe} probe that's not available on ${AppConstants.platform}`
-  );
-
-  let h = Telemetry.getHistogramById(existingProbe);
-  h.clear();
-  h.add(1);
-  let snapshot = Telemetry.getSnapshotForHistograms("main", false /* clear */)
-    .parent;
-  Assert.ok(
-    existingProbe in snapshot,
-    `${existingProbe} should be recorded on ${AppConstants.platform}`
-  );
-  Assert.equal(snapshot[existingProbe].sum, 1);
-});
-
 add_task(async function test_multistore_individual_histogram() {
   Telemetry.canRecordExtended = true;
 
@@ -1606,8 +1398,10 @@ add_task(async function test_multistore_main_snapshot() {
   Assert.ok(!(id in snapshot), `${id} should not be in a main store snapshot`);
 
   // Data should still be in, getting snapshot and clearing
-  snapshot = Telemetry.getSnapshotForHistograms("main", /* clear */ true)
-    .parent;
+  snapshot = Telemetry.getSnapshotForHistograms(
+    "main",
+    /* clear */ true
+  ).parent;
   id = "TELEMETRY_TEST_MAIN_ONLY";
   Assert.ok(id in snapshot, `${id} should be in a main store snapshot`);
   id = "TELEMETRY_TEST_MULTIPLE_STORES";
@@ -1616,8 +1410,10 @@ add_task(async function test_multistore_main_snapshot() {
   Assert.ok(!(id in snapshot), `${id} should not be in a main store snapshot`);
 
   // Should be empty after clearing
-  snapshot = Telemetry.getSnapshotForHistograms("main", /* clear */ false)
-    .parent;
+  snapshot = Telemetry.getSnapshotForHistograms(
+    "main",
+    /* clear */ false
+  ).parent;
   id = "TELEMETRY_TEST_MAIN_ONLY";
   Assert.ok(!(id in snapshot), `${id} should not be in a main store snapshot`);
   id = "TELEMETRY_TEST_MULTIPLE_STORES";
@@ -1644,16 +1440,20 @@ add_task(async function test_multistore_main_snapshot() {
   Assert.ok(!(id in snapshot), `${id} should not be in a main store snapshot`);
 
   // Data should still be in, getting snapshot and clearing
-  snapshot = Telemetry.getSnapshotForKeyedHistograms("main", /* clear */ true)
-    .parent;
+  snapshot = Telemetry.getSnapshotForKeyedHistograms(
+    "main",
+    /* clear */ true
+  ).parent;
   id = "TELEMETRY_TEST_KEYED_MULTIPLE_STORES";
   Assert.ok(id in snapshot, `${id} should be in a main store snapshot`);
   id = "TELEMETRY_TEST_KEYED_SYNC_ONLY";
   Assert.ok(!(id in snapshot), `${id} should not be in a main store snapshot`);
 
   // Should be empty after clearing
-  snapshot = Telemetry.getSnapshotForKeyedHistograms("main", /* clear */ false)
-    .parent;
+  snapshot = Telemetry.getSnapshotForKeyedHistograms(
+    "main",
+    /* clear */ false
+  ).parent;
   id = "TELEMETRY_TEST_KEYED_MULTIPLE_STORES";
   Assert.ok(!(id in snapshot), `${id} should not be in a main store snapshot`);
   id = "TELEMETRY_TEST_KEYED_SYNC_ONLY";
@@ -1832,8 +1632,10 @@ add_task(async function test_multistore_sync_snapshot() {
   hist.add(1);
 
   // Getting snapshot and clearing
-  snapshot = Telemetry.getSnapshotForHistograms("main", /* clear */ true)
-    .parent;
+  snapshot = Telemetry.getSnapshotForHistograms(
+    "main",
+    /* clear */ true
+  ).parent;
   id = "TELEMETRY_TEST_MAIN_ONLY";
   Assert.ok(id in snapshot, `${id} should be in a main store snapshot`);
   id = "TELEMETRY_TEST_MULTIPLE_STORES";
@@ -1841,8 +1643,10 @@ add_task(async function test_multistore_sync_snapshot() {
   id = "TELEMETRY_TEST_SYNC_ONLY";
   Assert.ok(!(id in snapshot), `${id} should not be in a main store snapshot`);
 
-  snapshot = Telemetry.getSnapshotForHistograms("sync", /* clear */ true)
-    .parent;
+  snapshot = Telemetry.getSnapshotForHistograms(
+    "sync",
+    /* clear */ true
+  ).parent;
   id = "TELEMETRY_TEST_MAIN_ONLY";
   Assert.ok(!(id in snapshot), `${id} should not be in a sync store snapshot`);
   id = "TELEMETRY_TEST_MULTIPLE_STORES";
@@ -1877,8 +1681,10 @@ add_task(async function test_multistore_keyed_sync_snapshot() {
   hist.add("key-1", 1);
 
   // Getting snapshot and clearing
-  snapshot = Telemetry.getSnapshotForKeyedHistograms("main", /* clear */ true)
-    .parent;
+  snapshot = Telemetry.getSnapshotForKeyedHistograms(
+    "main",
+    /* clear */ true
+  ).parent;
   id = "TELEMETRY_TEST_KEYED_LINEAR";
   Assert.ok(id in snapshot, `${id} should be in a main store snapshot`);
   id = "TELEMETRY_TEST_KEYED_MULTIPLE_STORES";
@@ -1886,8 +1692,10 @@ add_task(async function test_multistore_keyed_sync_snapshot() {
   id = "TELEMETRY_TEST_KEYED_SYNC_ONLY";
   Assert.ok(!(id in snapshot), `${id} should not be in a main store snapshot`);
 
-  snapshot = Telemetry.getSnapshotForKeyedHistograms("sync", /* clear */ true)
-    .parent;
+  snapshot = Telemetry.getSnapshotForKeyedHistograms(
+    "sync",
+    /* clear */ true
+  ).parent;
   id = "TELEMETRY_TEST_KEYED_LINEAR";
   Assert.ok(!(id in snapshot), `${id} should not be in a sync store snapshot`);
   id = "TELEMETRY_TEST_KEYED_MULTIPLE_STORES";
