@@ -11,7 +11,7 @@
 #include "nsThreadUtils.h"
 #include "mozilla/Components.h"
 #include "mozilla/EndianUtils.h"
-#include "mozilla/Telemetry.h"
+#include "mozilla/glean/UrlClassifierMetrics.h"
 #include "mozilla/IntegerPrintfMacros.h"
 #include "mozilla/LazyIdleThread.h"
 #include "mozilla/Logging.h"
@@ -29,13 +29,13 @@ extern mozilla::LazyLogModule gUrlClassifierDbServiceLog;
 #define LOG_ENABLED() \
   MOZ_LOG_TEST(gUrlClassifierDbServiceLog, mozilla::LogLevel::Debug)
 
-#define STORE_DIRECTORY NS_LITERAL_CSTRING("safebrowsing")
-#define TO_DELETE_DIR_SUFFIX NS_LITERAL_CSTRING("-to_delete")
-#define BACKUP_DIR_SUFFIX NS_LITERAL_CSTRING("-backup")
-#define UPDATING_DIR_SUFFIX NS_LITERAL_CSTRING("-updating")
+#define STORE_DIRECTORY "safebrowsing"_ns
+#define TO_DELETE_DIR_SUFFIX "-to_delete"_ns
+#define BACKUP_DIR_SUFFIX "-backup"_ns
+#define UPDATING_DIR_SUFFIX "-updating"_ns
 
-#define V4_METADATA_SUFFIX NS_LITERAL_CSTRING(".metadata")
-#define V2_METADATA_SUFFIX NS_LITERAL_CSTRING(".sbstore")
+#define V4_METADATA_SUFFIX ".metadata"_ns
+#define V2_METADATA_SUFFIX ".sbstore"_ns
 
 // The amount of time, in milliseconds, that our IO thread will stay alive after
 // the last event it processes.
@@ -56,19 +56,9 @@ void Classifier::SplitTables(const nsACString& str,
                              nsTArray<nsCString>& tables) {
   tables.Clear();
 
-  nsACString::const_iterator begin, iter, end;
-  str.BeginReading(begin);
-  str.EndReading(end);
-  while (begin != end) {
-    iter = begin;
-    FindCharInReadable(',', iter, end);
-    nsDependentCSubstring table = Substring(begin, iter);
+  for (const auto& table : str.Split(',')) {
     if (!table.IsEmpty()) {
-      tables.AppendElement(Substring(begin, iter));
-    }
-    begin = iter;
-    if (begin != end) {
-      begin++;
+      tables.AppendElement(table);
     }
   }
 
@@ -83,7 +73,7 @@ nsresult Classifier::GetPrivateStoreDirectory(
     const nsACString& aProvider, nsIFile** aPrivateStoreDirectory) {
   NS_ENSURE_ARG_POINTER(aPrivateStoreDirectory);
 
-  if (!StringEndsWith(aTableName, NS_LITERAL_CSTRING("-proto"))) {
+  if (!StringEndsWith(aTableName, "-proto"_ns)) {
     // Only V4 table names (ends with '-proto') would be stored
     // to per-provider sub-directory.
     nsCOMPtr<nsIFile>(aRootStoreDirectory).forget(aPrivateStoreDirectory);
@@ -134,12 +124,13 @@ nsresult Classifier::GetPrivateStoreDirectory(
 
 Classifier::Classifier()
     : mIsTableRequestResultOutdated(true),
+      mAsyncUpdateInProgress(false),
       mUpdateInterrupted(true),
       mIsClosed(false) {
   // Make a lazy thread for any IO
-  mUpdateThread = new LazyIdleThread(
-      DEFAULT_THREAD_TIMEOUT_MS, NS_LITERAL_CSTRING("Classifier Update"),
-      LazyIdleThread::ShutdownMethod::ManualShutdown);
+  mUpdateThread =
+      new LazyIdleThread(DEFAULT_THREAD_TIMEOUT_MS, "Classifier Update",
+                         LazyIdleThread::ShutdownMethod::ManualShutdown);
 }
 
 Classifier::~Classifier() {
@@ -224,13 +215,10 @@ nsresult Classifier::ClearLegacyFiles() {
   }
 
   nsTArray<nsLiteralCString> tables = {
-      NS_LITERAL_CSTRING("test-phish-simple"),
-      NS_LITERAL_CSTRING("test-malware-simple"),
-      NS_LITERAL_CSTRING("test-unwanted-simple"),
-      NS_LITERAL_CSTRING("test-harmful-simple"),
-      NS_LITERAL_CSTRING("test-track-simple"),
-      NS_LITERAL_CSTRING("test-trackwhite-simple"),
-      NS_LITERAL_CSTRING("test-block-simple"),
+      "test-phish-simple"_ns,    "test-malware-simple"_ns,
+      "test-unwanted-simple"_ns, "test-harmful-simple"_ns,
+      "test-track-simple"_ns,    "test-trackwhite-simple"_ns,
+      "test-block-simple"_ns,
   };
 
   const auto fnFindAndRemove = [](nsIFile* aRootDirectory,
@@ -262,10 +250,8 @@ nsresult Classifier::ClearLegacyFiles() {
 
   for (const auto& table : tables) {
     // Remove both .sbstore and .vlpse if .sbstore exists
-    if (fnFindAndRemove(mRootStoreDirectory,
-                        table + NS_LITERAL_CSTRING(".sbstore"))) {
-      fnFindAndRemove(mRootStoreDirectory,
-                      table + NS_LITERAL_CSTRING(".vlpset"));
+    if (fnFindAndRemove(mRootStoreDirectory, table + ".sbstore"_ns)) {
+      fnFindAndRemove(mRootStoreDirectory, table + ".vlpset"_ns);
     }
   }
 
@@ -712,6 +698,8 @@ void Classifier::FlushAndDisableAsyncUpdate() {
 
   mUpdateThread->Shutdown();
   mUpdateThread = nullptr;
+  mPendingUpdates.Clear();
+  mAsyncUpdateInProgress = false;
 }
 
 nsresult Classifier::AsyncApplyUpdates(const TableUpdateArray& aUpdates,
@@ -723,6 +711,22 @@ nsresult Classifier::AsyncApplyUpdates(const TableUpdateArray& aUpdates,
     return NS_ERROR_FAILURE;
   }
 
+  if (mAsyncUpdateInProgress) {
+    mPendingUpdates.AppendElement(NS_NewRunnableFunction(
+        "safebrowsing::Classifier::AsyncApplyUpdates",
+        [self = RefPtr{this}, aUpdates = aUpdates.Clone(),
+         aCallback]() mutable {
+          nsresult rv = self->AsyncApplyUpdates(aUpdates, aCallback);
+
+          // Calling the callback if we got an failure here to notify update
+          // observers.
+          if (NS_FAILED(rv)) {
+            aCallback(rv);
+          }
+        }));
+    return NS_OK;
+  }
+
   //         Caller thread      |       Update thread
   // --------------------------------------------------------
   //                            |    ApplyUpdatesBackground
@@ -733,6 +737,7 @@ nsresult Classifier::AsyncApplyUpdates(const TableUpdateArray& aUpdates,
   MOZ_ASSERT(mNewLookupCaches.IsEmpty(),
              "There should be no leftovers from a previous update.");
 
+  mAsyncUpdateInProgress = true;
   mUpdateInterrupted = false;
   nsresult rv =
       mRootStoreDirectory->Clone(getter_AddRefs(mRootStoreDirectoryForUpdate));
@@ -788,6 +793,8 @@ nsresult Classifier::AsyncApplyUpdates(const TableUpdateArray& aUpdates,
 
               LOG(("Step 3. Updates applied! Fire callback."));
               aCallback(rv);
+
+              classifier->AsyncUpdateFinished();
             });
 
         callerThread->Dispatch(fgRunnable, NS_DISPATCH_NORMAL);
@@ -796,6 +803,22 @@ nsresult Classifier::AsyncApplyUpdates(const TableUpdateArray& aUpdates,
   return mUpdateThread->Dispatch(bgRunnable, NS_DISPATCH_NORMAL);
 }
 
+void Classifier::AsyncUpdateFinished() {
+  MOZ_ASSERT(!OnUpdateThread(),
+             "AsyncUpdateFinished() MUST NOT be called on update thread");
+  MOZ_ASSERT(!NS_IsMainThread(),
+             "AsyncUpdateFinished() must be called on the worker thread");
+
+  mAsyncUpdateInProgress = false;
+
+  // If there are pending updates, run the first one.
+  if (!mPendingUpdates.IsEmpty()) {
+    auto& runnable = mPendingUpdates.ElementAt(0);
+    runnable->Run();
+    mPendingUpdates.RemoveElementAt(0);
+  }
+}
+
 nsresult Classifier::ApplyUpdatesBackground(
     TableUpdateArray& aUpdates, nsTArray<nsCString>& aFailedTableNames) {
   // |mUpdateInterrupted| is guaranteed to have been unset.
@@ -816,8 +839,8 @@ nsresult Classifier::ApplyUpdatesBackground(
   // Assume all TableUpdate objects should have the same provider.
   urlUtil->GetTelemetryProvider(aUpdates[0]->TableName(), provider);
 
-  Telemetry::AutoTimer<Telemetry::URLCLASSIFIER_CL_KEYED_UPDATE_TIME>
-      keyedTimer(provider);
+  auto keyedTimer =
+      glean::urlclassifier::cl_keyed_update_time.Get(provider).Measure();
 
   PRIntervalTime clockStart = 0;
   if (LOG_ENABLED()) {
@@ -952,8 +975,7 @@ nsresult Classifier::RegenActiveTables() {
 
   // The extension of V2 and V4 prefix files is .vlpset
   // We still check .pset here for legacy load.
-  nsTArray<nsCString> exts = {NS_LITERAL_CSTRING(".vlpset"),
-                              NS_LITERAL_CSTRING(".pset")};
+  nsTArray<nsCString> exts = {".vlpset"_ns, ".pset"_ns};
   nsTArray<nsCString> foundTables;
   nsresult rv = ScanStoreDir(mRootStoreDirectory, exts, foundTables);
   Unused << NS_WARN_IF(NS_FAILED(rv));
@@ -986,13 +1008,10 @@ nsresult Classifier::RegenActiveTables() {
 
 nsresult Classifier::AddMozEntries(nsTArray<nsCString>& aTables) {
   nsTArray<nsLiteralCString> tables = {
-      NS_LITERAL_CSTRING("moztest-phish-simple"),
-      NS_LITERAL_CSTRING("moztest-malware-simple"),
-      NS_LITERAL_CSTRING("moztest-unwanted-simple"),
-      NS_LITERAL_CSTRING("moztest-harmful-simple"),
-      NS_LITERAL_CSTRING("moztest-track-simple"),
-      NS_LITERAL_CSTRING("moztest-trackwhite-simple"),
-      NS_LITERAL_CSTRING("moztest-block-simple"),
+      "moztest-phish-simple"_ns,    "moztest-malware-simple"_ns,
+      "moztest-unwanted-simple"_ns, "moztest-harmful-simple"_ns,
+      "moztest-track-simple"_ns,    "moztest-trackwhite-simple"_ns,
+      "moztest-block-simple"_ns,
   };
 
   for (const auto& table : tables) {
@@ -1062,80 +1081,6 @@ nsresult Classifier::CleanToDelete() {
   return NS_OK;
 }
 
-#ifdef MOZ_SAFEBROWSING_DUMP_FAILED_UPDATES
-
-already_AddRefed<nsIFile> Classifier::GetFailedUpdateDirectroy() {
-  nsCString failedUpdatekDirName = STORE_DIRECTORY + nsCString("-failedupdate");
-
-  nsCOMPtr<nsIFile> failedUpdatekDirectory;
-  if (NS_FAILED(
-          mCacheDirectory->Clone(getter_AddRefs(failedUpdatekDirectory))) ||
-      NS_FAILED(failedUpdatekDirectory->AppendNative(failedUpdatekDirName))) {
-    LOG(("Failed to init failedUpdatekDirectory."));
-    return nullptr;
-  }
-
-  return failedUpdatekDirectory.forget();
-}
-
-nsresult Classifier::DumpRawTableUpdates(const nsACString& aRawUpdates) {
-  LOG(("Dumping raw table updates..."));
-
-  DumpFailedUpdate();
-
-  nsCOMPtr<nsIFile> failedUpdatekDirectory = GetFailedUpdateDirectroy();
-
-  // Create tableupdate.bin and dump raw table update data.
-  nsCOMPtr<nsIFile> rawTableUpdatesFile;
-  nsCOMPtr<nsIOutputStream> outputStream;
-  if (NS_FAILED(
-          failedUpdatekDirectory->Clone(getter_AddRefs(rawTableUpdatesFile))) ||
-      NS_FAILED(
-          rawTableUpdatesFile->AppendNative(nsCString("tableupdates.bin"))) ||
-      NS_FAILED(NS_NewLocalFileOutputStream(
-          getter_AddRefs(outputStream), rawTableUpdatesFile,
-          PR_WRONLY | PR_TRUNCATE | PR_CREATE_FILE))) {
-    LOG(("Failed to create file to dump raw table updates."));
-    return NS_ERROR_FAILURE;
-  }
-
-  // Write out the data.
-  uint32_t written;
-  nsresult rv = outputStream->Write(aRawUpdates.BeginReading(),
-                                    aRawUpdates.Length(), &written);
-  NS_ENSURE_SUCCESS(rv, rv);
-  NS_ENSURE_TRUE(written == aRawUpdates.Length(), NS_ERROR_FAILURE);
-
-  return rv;
-}
-
-nsresult Classifier::DumpFailedUpdate() {
-  LOG(("Dumping failed update..."));
-
-  nsCOMPtr<nsIFile> failedUpdatekDirectory = GetFailedUpdateDirectroy();
-
-  // Remove the "failed update" directory no matter it exists or not.
-  // Failure is fine because the directory may not exist.
-  failedUpdatekDirectory->Remove(true);
-
-  nsCString failedUpdatekDirName;
-  nsresult rv = failedUpdatekDirectory->GetNativeLeafName(failedUpdatekDirName);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  // Copy the in-use directory to a clean "failed update" directory.
-  nsCOMPtr<nsIFile> inUseDirectory;
-  if (NS_FAILED(mRootStoreDirectory->Clone(getter_AddRefs(inUseDirectory))) ||
-      NS_FAILED(inUseDirectory->CopyToNative(nullptr, failedUpdatekDirName))) {
-    LOG(("Failed to move in-use to the \"failed update\" directory %s",
-         failedUpdatekDirName.get()));
-    return NS_ERROR_FAILURE;
-  }
-
-  return rv;
-}
-
-#endif  // MOZ_SAFEBROWSING_DUMP_FAILED_UPDATES
-
 /**
  * This function copies the files one by one to the destination folder.
  * Before copying a file, it checks ::ShouldAbort and returns
@@ -1174,7 +1119,7 @@ nsresult Classifier::CopyDirectoryInterruptible(nsCOMPtr<nsIFile>& aDestDir,
       rv = CopyDirectoryInterruptible(dest, source);
       NS_ENSURE_SUCCESS(rv, rv);
     } else {
-      rv = source->CopyToNative(aDestDir, EmptyCString());
+      rv = source->CopyToNative(aDestDir, ""_ns);
       NS_ENSURE_SUCCESS(rv, rv);
     }
   }
@@ -1277,13 +1222,13 @@ bool Classifier::CheckValidUpdate(TableUpdateArray& aUpdates,
 nsCString Classifier::GetProvider(const nsACString& aTableName) {
   nsUrlClassifierUtils* urlUtil = nsUrlClassifierUtils::GetInstance();
   if (NS_WARN_IF(!urlUtil)) {
-    return EmptyCString();
+    return ""_ns;
   }
 
   nsCString provider;
   nsresult rv = urlUtil->GetProvider(aTableName, provider);
 
-  return NS_SUCCEEDED(rv) ? provider : EmptyCString();
+  return NS_SUCCEEDED(rv) ? provider : ""_ns;
 }
 
 /*
@@ -1458,6 +1403,10 @@ nsresult Classifier::UpdateTableV4(TableUpdateArray& aUpdates,
       // get prefixes from the lookup cache first.
       if (prefixes1.IsEmpty() && prefixes2.IsEmpty()) {
         lookupCacheV4->GetPrefixes(prefixes1);
+
+        // Bug 1911932: Temporary move the ApplyUpdate call here to verify the
+        // issue.
+        rv = lookupCacheV4->ApplyUpdate(updateV4, *input, *output);
       } else {
         MOZ_ASSERT(prefixes1.IsEmpty() ^ prefixes2.IsEmpty());
 
@@ -1466,9 +1415,12 @@ nsresult Classifier::UpdateTableV4(TableUpdateArray& aUpdates,
         // output should always point to the empty prefix set.
         input = prefixes1.IsEmpty() ? &prefixes2 : &prefixes1;
         output = prefixes1.IsEmpty() ? &prefixes1 : &prefixes2;
+
+        // Bug 1911932: Temporary move the ApplyUpdate call here to verify the
+        // issue.
+        rv = lookupCacheV4->ApplyUpdate(updateV4, *input, *output);
       }
 
-      rv = lookupCacheV4->ApplyUpdate(updateV4, *input, *output);
       if (NS_FAILED(rv)) {
         return rv;
       }
@@ -1576,7 +1528,7 @@ RefPtr<LookupCache> Classifier::GetLookupCache(const nsACString& aTable,
     }
   }
 
-  if (StringEndsWith(aTable, NS_LITERAL_CSTRING("-proto"))) {
+  if (StringEndsWith(aTable, "-proto"_ns)) {
     cache = new LookupCacheV4(aTable, provider, rootStoreDirectory);
   } else {
     cache = new LookupCacheV2(aTable, provider, rootStoreDirectory);
@@ -1619,25 +1571,16 @@ nsresult Classifier::ReadNoiseEntries(const Prefix& aPrefix,
                                       const nsACString& aTableName,
                                       uint32_t aCount,
                                       PrefixArray& aNoiseEntries) {
-  FallibleTArray<uint32_t> prefixes;
-  nsresult rv;
-
   RefPtr<LookupCache> cache = GetLookupCache(aTableName);
   if (!cache) {
     return NS_ERROR_FAILURE;
   }
 
   RefPtr<LookupCacheV2> cacheV2 = LookupCache::Cast<LookupCacheV2>(cache);
-  if (cacheV2) {
-    rv = cacheV2->GetPrefixes(prefixes);
-  } else {
-    rv = LookupCache::Cast<LookupCacheV4>(cache)->GetFixedLengthPrefixes(
-        prefixes);
-  }
+  RefPtr<LookupCacheV4> cacheV4 = LookupCache::Cast<LookupCacheV4>(cache);
+  MOZ_ASSERT_IF(cacheV2, !cacheV4);
 
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  if (prefixes.Length() == 0) {
+  if (cache->PrefixLength() == 0) {
     NS_WARNING("Could not find prefix in PrefixSet during noise lookup");
     return NS_ERROR_FAILURE;
   }
@@ -1653,20 +1596,36 @@ nsresult Classifier::ReadNoiseEntries(const Prefix& aPrefix,
   // for.
   // http://en.wikipedia.org/wiki/Linear_congruential_generator
 
-  uint32_t m = prefixes.Length();
+  uint32_t m = cache->PrefixLength();
   uint32_t a = aCount % m;
   uint32_t idx = aPrefix.ToUint32() % m;
 
   for (size_t i = 0; i < aCount; i++) {
     idx = (a * idx + a) % m;
 
+    uint32_t hash;
+
+    nsresult rv;
+    if (cacheV2) {
+      rv = cacheV2->GetPrefixByIndex(idx, &hash);
+    } else {
+      // We don't add noises for variable length prefix because of simplicity,
+      // so we will only get fixed length prefix (4 bytes).
+      rv = cacheV4->GetFixedLengthPrefixByIndex(idx, &hash);
+    }
+
+    if (NS_FAILED(rv)) {
+      NS_WARNING(
+          "Could not find the target prefix in PrefixSet during noise lookup");
+      return NS_ERROR_FAILURE;
+    }
+
     Prefix newPrefix;
-    uint32_t hash = prefixes[idx];
     // In the case V4 little endian, we did swapping endian when converting from
     // char* to int, should revert endian to make sure we will send hex string
     // correctly See https://bugzilla.mozilla.org/show_bug.cgi?id=1283007#c23
     if (!cacheV2 && !bool(MOZ_BIG_ENDIAN())) {
-      hash = NativeEndian::swapFromBigEndian(prefixes[idx]);
+      hash = NativeEndian::swapFromBigEndian(hash);
     }
 
     newPrefix.FromUint32(hash);
@@ -1706,7 +1665,7 @@ nsresult Classifier::LoadHashStore(nsIFile* aDirectory, nsACString& aResult,
     ChunkSet& subs = store.SubChunks();
 
     // Open HashStore will always succeed even that is not a v2 table.
-    // So skip tables without add and sub chunks.
+    // So exception tables without add and sub chunks.
     if (adds.Length() == 0 && subs.Length() == 0) {
       continue;
     }
@@ -1758,8 +1717,10 @@ nsresult Classifier::LoadMetadata(nsIFile* aDirectory, nsACString& aResult,
 
     nsCString state, sha256;
     rv = lookupCacheV4->LoadMetadata(state, sha256);
-    Telemetry::Accumulate(Telemetry::URLCLASSIFIER_VLPS_METADATA_CORRUPT,
-                          rv == NS_ERROR_FILE_CORRUPTED);
+    glean::urlclassifier::vlps_metadata_corrupt
+        .EnumGet(static_cast<glean::urlclassifier::VlpsMetadataCorruptLabel>(
+            rv == NS_ERROR_FILE_CORRUPTED))
+        .Add();
     if (NS_FAILED(rv)) {
       LOG(("Failed to get metadata for v4 table %s", table.get()));
       aFailedTableNames.AppendElement(table);
@@ -1791,7 +1752,7 @@ nsresult Classifier::LoadMetadata(nsIFile* aDirectory, nsACString& aResult,
 
 bool Classifier::ShouldAbort() const {
   return mIsClosed || nsUrlClassifierDBService::ShutdownHasStarted() ||
-         (mUpdateInterrupted && (NS_GetCurrentThread() == mUpdateThread));
+         (mUpdateInterrupted && mUpdateThread->IsOnCurrentThread());
 }
 
 }  // namespace safebrowsing
