@@ -35,7 +35,6 @@
 #include "nsCheckSummedOutputStream.h"
 #include "prio.h"
 #include "mozilla/Logging.h"
-#include "mozilla/IntegerPrintfMacros.h"
 #include "zlib.h"
 #include "Classifier.h"
 #include "nsUrlClassifierDBService.h"
@@ -157,15 +156,7 @@ void TableUpdateV4::NewPrefixes(int32_t aSize, const nsACString& aPrefixes) {
 
   int numOfPrefixes = aPrefixes.Length() / aSize;
 
-  if (aSize > 4) {
-    // TODO Bug 1364043 we may have a better API to record multiple samples into
-    // histograms with one call
-#ifdef NIGHTLY_BUILD
-    for (int i = 0; i < std::min(20, numOfPrefixes); i++) {
-      Telemetry::Accumulate(Telemetry::URLCLASSIFIER_VLPS_LONG_PREFIXES, aSize);
-    }
-#endif
-  } else if (LOG_ENABLED()) {
+  if (aSize <= 4 && LOG_ENABLED()) {
     const uint32_t* p =
         reinterpret_cast<const uint32_t*>(ToNewCString(aPrefixes));
 
@@ -182,7 +173,7 @@ void TableUpdateV4::NewPrefixes(int32_t aSize, const nsACString& aPrefixes) {
       LOG(("%.2X%.2X%.2X%.2X", c[0], c[1], c[2], c[3]));
     }
 
-    LOG(("---- %u fixed-length prefixes in total.",
+    LOG(("---- %zu fixed-length prefixes in total.",
          aPrefixes.Length() / aSize));
   }
 
