@@ -4,11 +4,12 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "LookupCache.h"
+#include "LookupCacheV4.h"
 #include "HashStore.h"
 #include "nsIFileStreams.h"
 #include "nsISeekableStream.h"
 #include "mozilla/ArrayUtils.h"
-#include "mozilla/Telemetry.h"
+#include "mozilla/glean/UrlClassifierMetrics.h"
 #include "mozilla/Logging.h"
 #include "nsNetUtil.h"
 #include "nsCheckSummedOutputStream.h"
@@ -16,6 +17,12 @@
 #include "prprf.h"
 #include "Classifier.h"
 #include "nsUrlClassifierInfo.h"
+#include "nsUrlClassifierUtils.h"
+#include "nsUrlClassifierDBService.h"
+
+#ifdef DEBUG
+#  include "nsPrintfCString.h"
+#endif
 
 // We act as the main entry point for all the real lookups,
 // so note that those are not done to the actual HashStore.
@@ -83,8 +90,37 @@ struct ValueTraits<nsACString> {
   }
 };
 
+void CStringToHexString(const nsACString& aIn, nsACString& aOut) {
+  static const char* const lut = "0123456789ABCDEF";
+
+  size_t len = aIn.Length();
+  MOZ_ASSERT(len <= COMPLETE_SIZE);
+
+  aOut.SetCapacity(2 * len);
+  for (size_t i = 0; i < aIn.Length(); ++i) {
+    const char c = static_cast<char>(aIn[i]);
+    aOut.Append(lut[(c >> 4) & 0x0F]);
+    aOut.Append(lut[c & 15]);
+  }
+}
+
+#ifdef DEBUG
+nsCString GetFormattedTimeString(int64_t aCurTimeSec) {
+  PRExplodedTime pret;
+  PR_ExplodeTime(aCurTimeSec * PR_USEC_PER_SEC, PR_GMTParameters, &pret);
+
+  return nsPrintfCString("%04d-%02d-%02d %02d:%02d:%02d UTC", pret.tm_year,
+                         pret.tm_month + 1, pret.tm_mday, pret.tm_hour,
+                         pret.tm_min, pret.tm_sec);
+}
+#endif
+
+}  // end of unnamed namespace.
+////////////////////////////////////////////////////////////////////////
+
 template <typename T>
-static nsresult WriteValue(nsIOutputStream* aOutputStream, const T& aValue) {
+nsresult LookupCache::WriteValue(nsIOutputStream* aOutputStream,
+                                 const T& aValue) {
   uint32_t writeLength = ValueTraits<T>::Length(aValue);
   MOZ_ASSERT(writeLength <= LookupCacheV4::MAX_METADATA_VALUE_LENGTH,
              "LookupCacheV4::MAX_METADATA_VALUE_LENGTH is too small.");
@@ -107,7 +143,7 @@ static nsresult WriteValue(nsIOutputStream* aOutputStream, const T& aValue) {
 }
 
 template <typename T>
-static nsresult ReadValue(nsIInputStream* aInputStream, T& aValue) {
+nsresult LookupCache::ReadValue(nsIInputStream* aInputStream, T& aValue) {
   nsresult rv;
 
   uint32_t readLength;
@@ -137,33 +173,13 @@ static nsresult ReadValue(nsIInputStream* aInputStream, T& aValue) {
   return rv;
 }
 
-void CStringToHexString(const nsACString& aIn, nsACString& aOut) {
-  static const char* const lut = "0123456789ABCDEF";
-
-  size_t len = aIn.Length();
-  MOZ_ASSERT(len <= COMPLETE_SIZE);
-
-  aOut.SetCapacity(2 * len);
-  for (size_t i = 0; i < aIn.Length(); ++i) {
-    const char c = static_cast<char>(aIn[i]);
-    aOut.Append(lut[(c >> 4) & 0x0F]);
-    aOut.Append(lut[c & 15]);
-  }
-}
-
-#ifdef DEBUG
-nsCString GetFormattedTimeString(int64_t aCurTimeSec) {
-  PRExplodedTime pret;
-  PR_ExplodeTime(aCurTimeSec * PR_USEC_PER_SEC, PR_GMTParameters, &pret);
-
-  return nsPrintfCString("%04d-%02d-%02d %02d:%02d:%02d UTC", pret.tm_year,
-                         pret.tm_month + 1, pret.tm_mday, pret.tm_hour,
-                         pret.tm_min, pret.tm_sec);
-}
-#endif
-
-}  // end of unnamed namespace.
-////////////////////////////////////////////////////////////////////////
+// These symbols are referenced from another compilation unit, but their
+// implementation depends on local symbols. Workaround this by forcing their
+// instantiation there.
+template nsresult mozilla::safebrowsing::LookupCache::WriteValue(
+    nsIOutputStream*, nsTSubstring<char> const&);
+template nsresult mozilla::safebrowsing::LookupCache::ReadValue(
+    nsIInputStream*, nsTSubstring<char>&);
 
 LookupCache::LookupCache(const nsACString& aTableName,
                          const nsACString& aProvider,
@@ -424,7 +440,7 @@ bool LookupCache::IsCanonicalizedIP(const nsACString& aHost) {
 // third.party.domain. This is to make sure we can find a match when a
 // exceptionlisted domain is eTLD.
 /* static */
-nsresult LookupCache::GetLookupEntitylistFragments(
+void LookupCache::GetLookupEntitylistFragments(
     const nsACString& aSpec, nsTArray<nsCString>* aFragments) {
   aFragments->Clear();
 
@@ -439,7 +455,8 @@ nsresult LookupCache::GetLookupEntitylistFragments(
   // "/?resoruce=" because this means the URL is not generated in
   // CreatePairwiseEntityListURI()
   if (!FindInReadable("/?resource="_ns, iter, iter_end)) {
-    return GetLookupFragments(aSpec, aFragments);
+    GetLookupFragments(aSpec, aFragments);
+    return;
   }
 
   const nsACString& topLevelURL = Substring(begin, iter++);
@@ -503,13 +520,11 @@ nsresult LookupCache::GetLookupEntitylistFragments(
       aFragments->AppendElement(key);
     }
   }
-
-  return NS_OK;
 }
 
 /* static */
-nsresult LookupCache::GetLookupFragments(const nsACString& aSpec,
-                                         nsTArray<nsCString>* aFragments)
+void LookupCache::GetLookupFragments(const nsACString& aSpec,
+                                     nsTArray<nsCString>* aFragments)
 
 {
   aFragments->Clear();
@@ -520,7 +535,7 @@ nsresult LookupCache::GetLookupFragments(const nsACString& aSpec,
 
   iter = begin;
   if (!FindCharInReadable('/', iter, end)) {
-    return NS_OK;
+    return;
   }
 
   const nsACString& host = Substring(begin, iter++);
@@ -608,8 +623,6 @@ nsresult LookupCache::GetLookupFragments(const nsACString& aSpec,
       aFragments->AppendElement(key);
     }
   }
-
-  return NS_OK;
 }
 
 nsresult LookupCache::LoadPrefixSet() {
@@ -702,7 +715,7 @@ nsresult LookupCache::StoreToFile(nsCOMPtr<nsIFile>& aFile) {
   // Preallocate the file storage
   {
     nsCOMPtr<nsIFileOutputStream> fos(do_QueryInterface(localOutFile));
-    Telemetry::AutoTimer<Telemetry::URLCLASSIFIER_VLPS_FALLOCATE_TIME> timer;
+    auto timer = glean::urlclassifier::vlps_fallocate_time.Measure();
 
     Unused << fos->Preallocate(fileSize);
   }
@@ -747,7 +760,7 @@ nsresult LookupCache::StoreToFile(nsCOMPtr<nsIFile>& aFile) {
 nsresult LookupCache::LoadFromFile(nsCOMPtr<nsIFile>& aFile) {
   NS_ENSURE_ARG_POINTER(aFile);
 
-  Telemetry::AutoTimer<Telemetry::URLCLASSIFIER_VLPS_FILELOAD_TIME> timer;
+  auto timer = glean::urlclassifier::vlps_fileload_time.Measure();
 
   nsCOMPtr<nsIInputStream> localInFile;
   nsresult rv = NS_NewLocalFileInputStream(getter_AddRefs(localInFile), aFile,
@@ -923,6 +936,13 @@ nsresult LookupCacheV2::GetPrefixes(FallibleTArray<uint32_t>& aAddPrefixes,
   return mVLPrefixSet->GetFixedLengthPrefixes(&aAddPrefixes, &aAddCompletes);
 }
 
+nsresult LookupCacheV2::GetPrefixByIndex(uint32_t aIndex,
+                                         uint32_t* aOutPrefix) const {
+  NS_ENSURE_ARG_POINTER(aOutPrefix);
+
+  return mVLPrefixSet->GetFixedLengthPrefixByIndex(aIndex, aOutPrefix);
+}
+
 void LookupCacheV2::AddGethashResultToCache(
     const AddCompleteArray& aAddCompletes, const MissPrefixArray& aMissPrefixes,
     int64_t aExpirySec) {
