@@ -41,8 +41,6 @@
 
 #include <vector>
 
-#include "nsThreadUtils.h"
-
 #include "linux/crash_generation/crash_generation_server.h"
 #include "linux/crash_generation/client_info.h"
 #include "linux/handler/exception_handler.h"
@@ -52,29 +50,23 @@
 #include "common/linux/safe_readlink.h"
 
 #if defined(MOZ_OXIDIZED_BREAKPAD)
-#include "mozilla/toolkit/crashreporter/rust_minidump_writer_linux_ffi_generated.h"
-#include "nsString.h"
+#  include "mozilla/toolkit/crashreporter/rust_minidump_writer_linux_ffi_generated.h"
 #endif
 
+#include "mozilla/Alignment.h"
+
 static const char kCommandQuit = 'x';
 
 namespace google_breakpad {
 
 CrashGenerationServer::CrashGenerationServer(
   const int listen_fd,
-  OnClientDumpRequestCallback dump_callback,
-  void* dump_context,
-  OnClientExitingCallback exit_callback,
-  void* exit_context,
-  bool generate_dumps,
+  std::function<OnClientDumpRequestCallback> dump_callback,
   const string* dump_path) :
     server_fd_(listen_fd),
-    dump_callback_(dump_callback),
-    dump_context_(dump_context),
-    exit_callback_(exit_callback),
-    exit_context_(exit_context),
-    generate_dumps_(generate_dumps),
-    started_(false)
+    dump_callback_(std::move(dump_callback)),
+    started_(false),
+    reserved_fds_{-1, -1}
 {
   if (dump_path)
     dump_dir_ = *dump_path;
@@ -109,8 +101,11 @@ CrashGenerationServer::Start()
   control_pipe_in_ = control_pipe[0];
   control_pipe_out_ = control_pipe[1];
 
-  if (pthread_create(&thread_, NULL,
-                     ThreadMain, reinterpret_cast<void*>(this)))
+  if (pthread_create(&thread_, nullptr,
+                    [](void* context) -> void* {
+                      reinterpret_cast<CrashGenerationServer*>(context)->Run();
+                      return nullptr;
+                    }, this))
     return false;
 
   started_ = true;
@@ -165,6 +160,8 @@ CrashGenerationServer::CreateReportChannel(int* server_fd, int* client_fd)
 void
 CrashGenerationServer::Run()
 {
+  ReserveFileDescriptors();
+
   struct pollfd pollfds[2];
   memset(&pollfds, 0, sizeof(pollfds));
 
@@ -214,7 +211,7 @@ CrashGenerationServer::ClientEvent(short revents)
 
   struct msghdr msg = {0};
   struct iovec iov[1];
-  char crash_context[kCrashContextSize];
+  MOZ_ALIGNED_DECL(16, char crash_context[kCrashContextSize]);
   char control[kControlMsgSize];
   const ssize_t expected_msg_size = sizeof(crash_context);
 
@@ -271,13 +268,50 @@ CrashGenerationServer::ClientEvent(short revents)
   if (!MakeMinidumpFilename(minidump_filename))
     return true;
 
+  // We won't re-reserve the file descriptors past this point. If we crash more
+  // than once we'll just accept that we might run out of file descriptors, but
+  // we don't want to make the situation worse by trying  to grab them again.
+  ReleaseFileDescriptors();
+
 #if defined(MOZ_OXIDIZED_BREAKPAD)
-  // Ignoring the return-value here for now.
-  // The function always creates an empty minidump file even in case of an error.
-  // So we'll report that as well via the callback-functions.
-  nsCString error_msg;
-  bool res = write_minidump_linux_with_context(minidump_filename.c_str(),
-                                    crashing_pid, crash_context, &error_msg);
+  ExceptionHandler::CrashContext* breakpad_cc =
+      reinterpret_cast<ExceptionHandler::CrashContext*>(crash_context);
+  char* error_msg = nullptr;
+  siginfo_t& si = breakpad_cc->siginfo;
+  signalfd_siginfo signalfd_si = {};
+  signalfd_si.ssi_signo = si.si_signo;
+  signalfd_si.ssi_errno = si.si_errno;
+  signalfd_si.ssi_code = si.si_code;
+
+  switch (si.si_signo) {
+    case SIGILL:
+    case SIGFPE:
+    case SIGSEGV:
+    case SIGBUS:
+    case SIGSYS:
+      signalfd_si.ssi_addr = reinterpret_cast<size_t>(si.si_addr);
+      break;
+  }
+
+  bool res = false;
+
+  MinidumpWriterContext* writer = minidump_writer_create(
+    minidump_filename.c_str(),
+    crashing_pid,
+    breakpad_cc->tid,
+    &error_msg
+  );
+  if (writer) {
+    const fpregset_t *float_state = nullptr;
+
+#  ifndef __arm__
+    float_state = reinterpret_cast<const fpregset_t *>(&breakpad_cc->float_state);
+#  endif
+
+    minidump_writer_set_crash_context(writer, &breakpad_cc->context, float_state, &signalfd_si);
+
+    res = minidump_writer_dump(writer, &error_msg);
+  }
 #else
   if (!google_breakpad::WriteMinidump(minidump_filename.c_str(),
                                       crashing_pid, crash_context,
@@ -294,15 +328,16 @@ CrashGenerationServer::ClientEvent(short revents)
   }
 #endif
   if (dump_callback_) {
-    dump_callback_(dump_context_, info, minidump_filename);
+    dump_callback_(info, minidump_filename);
   }
 
   // Send the done signal to the process: it can exit now.
   // (Closing this will make the child's sys_read unblock and return 0.)
   close(signal_fd);
 
-  if (exit_callback_) {
-    exit_callback_(exit_context_, info);
+  info.set_error_msg(nullptr);
+  if (error_msg) {
+    free_minidump_error_msg(error_msg);
   }
 
   return true;
@@ -346,13 +381,30 @@ CrashGenerationServer::MakeMinidumpFilename(string& outFilename)
   return true;
 }
 
-// static
-void*
-CrashGenerationServer::ThreadMain(void *arg)
-{
-  NS_SetCurrentThreadName("Breakpad Server");
-  reinterpret_cast<CrashGenerationServer*>(arg)->Run();
-  return NULL;
+void
+CrashGenerationServer::ReserveFileDescriptors() {
+  for (size_t i = 0; i < CrashGenerationServer::RESERVED_FDS_NUM; i++) {
+    assert(reserved_fds_[i] < 0);
+
+    // This fd is just taking up space in the file table, so it can be
+    // anything that's self-contained and simple to create.
+    int fds[2];
+    int rv = pipe2(fds, O_CLOEXEC);
+    if (rv == 0) {
+      close(fds[0]);
+      reserved_fds_[i] = fds[1];
+    }
+  }
+}
+
+void
+CrashGenerationServer::ReleaseFileDescriptors() {
+  for (size_t i = 0; i < CrashGenerationServer::RESERVED_FDS_NUM; i++) {
+    if (reserved_fds_[i] > 0) {
+      close(reserved_fds_[i]);
+      reserved_fds_[i] = -1;
+    }
+  }
 }
 
 }  // namespace google_breakpad
