@@ -364,7 +364,7 @@ void CrashGenerationServer::HandleConnectedState() {
   assert(server_state_ == IPC_SERVER_STATE_CONNECTED);
 
   DWORD bytes_count = 0;
-  memset(&msg_, 0, sizeof(msg_));
+  msg_.reset();
   bool success = ReadFile(pipe_,
                           &msg_,
                           sizeof(msg_),
@@ -938,7 +938,7 @@ bool CrashGenerationServer::GenerateDump(const ClientInfo& client,
     }
 
     // Allocate AppMemory instances for exception context.
-    for (int i = 0; i < kExceptionAppMemoryRegions; i++) {
+    for (size_t i = 0; i < kExceptionAppMemoryRegions; i++) {
       AppMemory app_memory;
       app_memory.ptr = reinterpret_cast<ULONG64>(nullptr);
       app_memory.length = 0;
