@@ -3,19 +3,19 @@
 
 "use strict";
 
-const { XPIInstall } = ChromeUtils.import(
-  "resource://gre/modules/addons/XPIInstall.jsm"
+const { XPIExports } = ChromeUtils.importESModule(
+  "resource://gre/modules/addons/XPIExports.sys.mjs"
 );
+
+// NOTE: Only constants can be extracted from XPIExports.
 const {
   XPIInternal: {
-    BuiltInLocation,
     KEY_APP_PROFILE,
+    KEY_APP_SYSTEM_BUILTINS,
     KEY_APP_SYSTEM_DEFAULTS,
     KEY_APP_SYSTEM_PROFILE,
-    TemporaryInstallLocation,
-    XPIStates,
   },
-} = ChromeUtils.import("resource://gre/modules/addons/XPIProvider.jsm");
+} = XPIExports;
 
 AddonTestUtils.init(this);
 AddonTestUtils.overrideCertDB();
@@ -37,33 +37,40 @@ Services.prefs.setIntPref(
 // test_builtin_system_location tests the (isSystem && isBuiltin) combination
 // (i.e. KEY_APP_SYSTEM_DEFAULTS). That location only exists if this directory
 // is found:
-const distroDir = FileUtils.getDir("ProfD", ["sysfeatures"], true);
+const distroDir = FileUtils.getDir("ProfD", ["sysfeatures"]);
+distroDir.create(Ci.nsIFile.DIRECTORY_TYPE, FileUtils.PERMS_DIRECTORY);
 registerDirectory("XREAppFeat", distroDir);
 
 function getInstallLocation({
   isBuiltin = false,
   isSystem = false,
   isTemporary = false,
+  isSystemBuiltin = false,
 }) {
   if (isTemporary) {
     // Temporary installation. Signatures will not be verified.
-    return TemporaryInstallLocation; // KEY_APP_TEMPORARY
+    return XPIExports.XPIInternal.TemporaryInstallLocation; // KEY_APP_TEMPORARY
   }
   let location;
   if (isSystem) {
     if (isBuiltin) {
-      // System location. Signatures will not be verified.
-      location = XPIStates.getLocation(KEY_APP_SYSTEM_DEFAULTS);
+      location = isSystemBuiltin
+        ? // System app bundled directory location. Signatures will not be verified.
+          XPIExports.XPIInternal.XPIStates.getLocation(KEY_APP_SYSTEM_BUILTINS)
+        : // System Defaults directory location. Signatures will not be verified.
+          XPIExports.XPIInternal.XPIStates.getLocation(KEY_APP_SYSTEM_DEFAULTS);
     } else {
       // Normandy installations. Signatures will be verified.
-      location = XPIStates.getLocation(KEY_APP_SYSTEM_PROFILE);
+      location = XPIExports.XPIInternal.XPIStates.getLocation(
+        KEY_APP_SYSTEM_PROFILE
+      );
     }
   } else if (isBuiltin) {
     // Packaged with the application. Signatures will not be verified.
-    location = BuiltInLocation; // KEY_APP_BUILTINS
+    location = XPIExports.XPIInternal.BuiltInLocation; // KEY_APP_BUILTINS
   } else {
     // By default - The profile directory. Signatures will be verified.
-    location = XPIStates.getLocation(KEY_APP_PROFILE);
+    location = XPIExports.XPIInternal.XPIStates.getLocation(KEY_APP_PROFILE);
   }
   // Sanity checks to make sure that the flags match the expected values.
   if (location.isSystem !== isSystem) {
@@ -88,14 +95,14 @@ async function testLoadManifest({ location, expectPrivileged }) {
     if (location.isTemporary && !expectPrivileged) {
       ExtensionTestUtils.failOnSchemaWarnings(false);
       await Assert.rejects(
-        XPIInstall.loadManifestFromFile(xpi, location),
+        XPIExports.XPIInstall.loadManifestFromFile(xpi, location),
         /Extension is invalid/,
         "load manifest failed with privileged permission"
       );
       ExtensionTestUtils.failOnSchemaWarnings(true);
       return;
     }
-    let addon = await XPIInstall.loadManifestFromFile(xpi, location);
+    let addon = await XPIExports.XPIInstall.loadManifestFromFile(xpi, location);
     actualPermissions = addon.userPermissions;
     equal(addon.isPrivileged, expectPrivileged, "addon.isPrivileged");
   });
@@ -182,7 +189,8 @@ add_task(async function test_system_location() {
   });
 });
 
-add_task(async function test_builtin_system_location() {
+// TODO(Bug 1949847): remove this test along with removing the app-system-defaults location.
+add_task(async function test_builtin_system_location_directory() {
   AddonTestUtils.usePrivilegedSignatures = false;
   await testLoadManifest({
     expectPrivileged: true,
@@ -190,6 +198,18 @@ add_task(async function test_builtin_system_location() {
   });
 });
 
+add_task(async function test_builtin_system_location() {
+  AddonTestUtils.usePrivilegedSignatures = false;
+  await testLoadManifest({
+    expectPrivileged: true,
+    location: getInstallLocation({
+      isSystem: true,
+      isBuiltin: true,
+      isSystemBuiltin: true,
+    }),
+  });
+});
+
 add_task(async function test_temporary_regular() {
   AddonTestUtils.usePrivilegedSignatures = false;
   Services.prefs.setBoolPref("extensions.experiments.enabled", false);
