@@ -4,8 +4,19 @@
 
 // This verifies that add-on update checks work
 
+// Enable SCOPE_APPLICATION for builtin testing.  Default in tests is only SCOPE_PROFILE.
+let scopes = AddonManager.SCOPE_PROFILE | AddonManager.SCOPE_APPLICATION;
+Services.prefs.setIntPref("extensions.enabledScopes", scopes);
+
 // The test extension uses an insecure update url.
 Services.prefs.setBoolPref(PREF_EM_CHECK_UPDATE_SECURITY, false);
+// This test uses add-on versions that follow the toolkit version but we
+// started to encourage the use of a simpler format in Bug 1793925. We disable
+// the pref below to avoid install errors.
+Services.prefs.setBoolPref(
+  "extensions.webextensions.warnings-as-errors",
+  false
+);
 
 const updateFile = "test_update.json";
 
@@ -45,7 +56,7 @@ testserver.registerDirectory("/data/", do_get_file("data"));
 
 const XPIS = {};
 
-add_task(async function setup() {
+add_setup(async function setup() {
   createAppInfo("xpcshell@tests.mozilla.org", "XPCShell", "1", "1");
 
   Services.locale.requestedLocales = ["fr-FR"];
@@ -55,7 +66,7 @@ add_task(async function setup() {
       manifest: {
         name: info.name,
         version: info.version,
-        applications: { gecko: { id: info.id } },
+        browser_specific_settings: { gecko: { id: info.id } },
       },
     });
     testserver.registerFile(`/addons/${name}.xpi`, XPIS[name]);
@@ -72,7 +83,7 @@ add_task(async function test_apply_update() {
     manifest: {
       name: "Test Addon 1",
       version: "1.0",
-      applications: {
+      browser_specific_settings: {
         gecko: {
           id: "addon1@tests.mozilla.org",
           update_url: `http://example.com/data/${updateFile}`,
@@ -116,9 +127,8 @@ add_task(async function test_apply_update() {
       installEvents: [{ event: "onNewInstall" }],
     },
     async () => {
-      ({
-        updateAvailable: install,
-      } = await AddonTestUtils.promiseFindAddonUpdates(a1));
+      ({ updateAvailable: install } =
+        await AddonTestUtils.promiseFindAddonUpdates(a1));
     }
   );
 
@@ -133,9 +143,8 @@ add_task(async function test_apply_update() {
   equal(install.releaseNotesURI.spec, "http://example.com/updateInfo.xhtml");
 
   // Verify that another update check returns the same AddonInstall
-  let {
-    updateAvailable: install2,
-  } = await AddonTestUtils.promiseFindAddonUpdates(a1);
+  let { updateAvailable: install2 } =
+    await AddonTestUtils.promiseFindAddonUpdates(a1);
 
   installs = await AddonManager.getAllInstalls();
   equal(installs.length, 1);
@@ -211,7 +220,7 @@ add_task(async function test_apply_update() {
   // Make sure that the extension lastModifiedTime was updated.
   let testFile = getAddonFile(a1);
   let difference = testFile.lastModifiedTime - startupTime;
-  ok(Math.abs(difference) < MAX_TIME_DIFFERENCE);
+  Assert.less(Math.abs(difference), MAX_TIME_DIFFERENCE);
 
   await a1.uninstall();
 });
@@ -222,7 +231,7 @@ add_task(async function test_compat_update() {
     manifest: {
       name: "Test Addon 2",
       version: "1.0",
-      applications: {
+      browser_specific_settings: {
         gecko: {
           id: "addon2@tests.mozilla.org",
           update_url: "http://example.com/data/" + updateFile,
@@ -264,7 +273,7 @@ add_task(async function test_no_compat() {
   await promiseInstallWebExtension({
     manifest: {
       name: "Test Addon 3",
-      applications: {
+      browser_specific_settings: {
         gecko: {
           id: "addon3@tests.mozilla.org",
           update_url: `http://example.com/data/${updateFile}`,
@@ -334,7 +343,7 @@ add_task(async function test_background_update() {
     manifest: {
       name: "Test Addon 1",
       version: "1.0",
-      applications: {
+      browser_specific_settings: {
         gecko: {
           id: "addon1@tests.mozilla.org",
           update_url: `http://example.com/data/${updateFile}`,
@@ -368,7 +377,7 @@ add_task(async function test_background_update() {
       ],
     },
     () => {
-      AddonManagerInternal.backgroundUpdateCheck();
+      AddonManagerPrivate.backgroundUpdateCheck();
     }
   );
 
@@ -380,6 +389,8 @@ add_task(async function test_background_update() {
   await a1.uninstall();
 });
 
+const STATE_BLOCKED = Ci.nsIBlocklistService.STATE_BLOCKED;
+
 const PARAMS =
   "?" +
   [
@@ -402,7 +413,7 @@ const PARAM_ADDONS = {
     manifest: {
       name: "Test Addon 1",
       version: "5.0",
-      applications: {
+      browser_specific_settings: {
         gecko: {
           id: "addon1@tests.mozilla.org",
           update_url: `http://example.com/data/param_test.json${PARAMS}`,
@@ -425,7 +436,7 @@ const PARAM_ADDONS = {
     manifest: {
       name: "Test Addon 2",
       version: "67.0.5b1",
-      applications: {
+      browser_specific_settings: {
         gecko: {
           id: "addon2@tests.mozilla.org",
           update_url: "http://example.com/data/param_test.json" + PARAMS,
@@ -452,7 +463,7 @@ const PARAM_ADDONS = {
     manifest: {
       name: "Test Addon 3",
       version: "1.3+",
-      applications: {
+      browser_specific_settings: {
         gecko: {
           id: "addon3@tests.mozilla.org",
           update_url: `http://example.com/data/param_test.json${PARAMS}`,
@@ -472,7 +483,7 @@ const PARAM_ADDONS = {
     manifest: {
       name: "Test Addon 4",
       version: "0.5ab6",
-      applications: {
+      browser_specific_settings: {
         gecko: {
           id: "addon4@tests.mozilla.org",
           update_url: `http://example.com/data/param_test.json${PARAMS}`,
@@ -495,7 +506,7 @@ const PARAM_ADDONS = {
     manifest: {
       name: "Test Addon 5",
       version: "1.0",
-      applications: {
+      browser_specific_settings: {
         gecko: {
           id: "addon5@tests.mozilla.org",
           update_url: `http://example.com/data/param_test.json${PARAMS}`,
@@ -519,7 +530,7 @@ const PARAM_ADDONS = {
     manifest: {
       name: "Test Addon 6",
       version: "1.0",
-      applications: {
+      browser_specific_settings: {
         gecko: {
           id: "addon6@tests.mozilla.org",
           update_url: `http://example.com/data/param_test.json${PARAMS}`,
@@ -538,35 +549,11 @@ const PARAM_ADDONS = {
     updateType: [AddonManager.UPDATE_WHEN_NEW_APP_INSTALLED],
   },
 
-  "blocklist1@tests.mozilla.org": {
-    manifest: {
-      name: "Test Addon 1",
-      version: "5.0",
-      applications: {
-        gecko: {
-          id: "blocklist1@tests.mozilla.org",
-          update_url: `http://example.com/data/param_test.json${PARAMS}`,
-          strict_min_version: "1",
-          strict_max_version: "2",
-        },
-      },
-    },
-    params: {
-      item_version: "5.0",
-      item_maxappversion: "2",
-      item_status: "userDisabled,softblocked",
-      app_version: "1",
-      update_type: "97",
-    },
-    updateType: [AddonManager.UPDATE_WHEN_USER_REQUESTED],
-    blocklistState: "STATE_SOFTBLOCKED",
-  },
-
   "blocklist2@tests.mozilla.org": {
     manifest: {
       name: "Test Addon 1",
       version: "5.0",
-      applications: {
+      browser_specific_settings: {
         gecko: {
           id: "blocklist2@tests.mozilla.org",
           update_url: `http://example.com/data/param_test.json${PARAMS}`,
@@ -583,7 +570,7 @@ const PARAM_ADDONS = {
       update_type: "97",
     },
     updateType: [AddonManager.UPDATE_WHEN_USER_REQUESTED],
-    blocklistState: "STATE_BLOCKED",
+    blocklistState: STATE_BLOCKED,
   },
 };
 
@@ -591,13 +578,14 @@ const PARAM_IDS = Object.keys(PARAM_ADDONS);
 
 // Verify the parameter escaping in update urls.
 add_task(async function test_params() {
-  let blocklistAddons = new Map();
+  let blocked = [];
   for (let [id, options] of Object.entries(PARAM_ADDONS)) {
-    if (options.blocklistState) {
-      blocklistAddons.set(id, Ci.nsIBlocklistService[options.blocklistState]);
+    if (options.blocklistState == STATE_BLOCKED) {
+      blocked.push(`${id}:${options.manifest.version}`);
     }
   }
-  let mockBlocklist = await AddonTestUtils.overrideBlocklist(blocklistAddons);
+  let extensionsMLBF = [{ stash: { blocked, unblocked: [] }, stash_time: 0 }];
+  await AddonTestUtils.loadBlocklistRawData({ extensionsMLBF });
 
   for (let [id, options] of Object.entries(PARAM_ADDONS)) {
     await promiseInstallWebExtension({ manifest: options.manifest });
@@ -611,10 +599,7 @@ add_task(async function test_params() {
   let resultsPromise = new Promise(resolve => {
     let results = new Map();
 
-    testserver.registerPathHandler("/data/param_test.json", function(
-      request,
-      response
-    ) {
+    testserver.registerPathHandler("/data/param_test.json", function (request) {
       let params = new URLSearchParams(request.queryString);
       let itemId = params.get("item_id");
       ok(
@@ -665,8 +650,6 @@ add_task(async function test_params() {
   for (let [, addon] of await getAddons(PARAM_IDS)) {
     await addon.uninstall();
   }
-
-  await mockBlocklist.unregister();
 });
 
 // Tests that if a manifest claims compatibility then the add-on will be
@@ -676,7 +659,7 @@ add_task(async function test_manifest_compat() {
     manifest: {
       name: "Test Addon 1",
       version: "5.0",
-      applications: {
+      browser_specific_settings: {
         gecko: {
           id: "addon4@tests.mozilla.org",
           update_url: `http://example.com/data/${updateFile}`,
@@ -718,7 +701,7 @@ add_task(async function test_no_auto_update() {
     manifest: {
       name: "Test Addon 1",
       version: "1.0",
-      applications: {
+      browser_specific_settings: {
         gecko: {
           id: "addon1@tests.mozilla.org",
           update_url: `http://example.com/data/${updateFile}`,
@@ -731,7 +714,7 @@ add_task(async function test_no_auto_update() {
     manifest: {
       name: "Test Addon 8",
       version: "1.0",
-      applications: {
+      browser_specific_settings: {
         gecko: {
           id: "addon8@tests.mozilla.org",
           update_url: `http://example.com/data/${updateFile}`,
@@ -764,11 +747,11 @@ add_task(async function test_no_auto_update() {
         equal(aInstall.existingAddon.id, "addon1@tests.mozilla.org");
       },
 
-      onDownloadFailed(aInstall) {
+      onDownloadFailed() {
         ok(false, "Should not have seen onDownloadFailed event");
       },
 
-      onDownloadCancelled(aInstall) {
+      onDownloadCancelled() {
         ok(false, "Should not have seen onDownloadCancelled event");
       },
 
@@ -782,16 +765,16 @@ add_task(async function test_no_auto_update() {
         resolve();
       },
 
-      onInstallFailed(aInstall) {
+      onInstallFailed() {
         ok(false, "Should not have seen onInstallFailed event");
       },
 
-      onInstallCancelled(aInstall) {
+      onInstallCancelled() {
         ok(false, "Should not have seen onInstallCancelled event");
       },
     };
     AddonManager.addInstallListener(listener);
-    AddonManagerInternal.backgroundUpdateCheck();
+    AddonManagerPrivate.backgroundUpdateCheck();
   });
   AddonManager.removeInstallListener(listener);
 
@@ -811,35 +794,62 @@ add_task(async function test_no_auto_update() {
 
 // Test that the update check returns nothing for addons in locked install
 // locations.
-add_task(async function run_test_locked_install() {
-  const lockedDir = gProfD.clone();
-  lockedDir.append("locked_extensions");
-  registerDirectory("XREAppFeat", lockedDir);
-
-  await promiseShutdownManager();
-
-  let xpi = await createTempWebExtensionFile({
+async function run_test_locked_install({ asBuiltIn = true } = {}) {
+  const ADDON_ID = "addon13@tests.mozilla.org";
+  const testExtensionProps = {
     manifest: {
       name: "Test Addon 13",
       version: "1.0",
-      applications: {
+      browser_specific_settings: {
         gecko: {
-          id: "addon13@tests.mozilla.org",
+          id: ADDON_ID,
           update_url: "http://example.com/data/test_update.json",
         },
       },
     },
-  });
-  xpi.copyTo(lockedDir, "addon13@tests.mozilla.org.xpi");
+  };
 
-  let validAddons = { system: ["addon13@tests.mozilla.org"] };
-  await overrideBuiltIns(validAddons);
+  await promiseShutdownManager();
+
+  if (!asBuiltIn) {
+    const lockedDir = gProfD.clone();
+    lockedDir.append("locked_extensions");
+    registerDirectory("XREAppFeat", lockedDir);
+
+    let xpi = await createTempWebExtensionFile(testExtensionProps);
+    xpi.copyTo(lockedDir, `${ADDON_ID}.xpi`);
+    await overrideBuiltIns({ system: [ADDON_ID] });
+  } else {
+    await setupBuiltinExtension(testExtensionProps, "builtin-addon13-ext");
+    await overrideBuiltIns({
+      builtins: [
+        {
+          addon_id: ADDON_ID,
+          addon_version: "1.0",
+          res_url: "resource://builtin-addon13-ext/",
+        },
+      ],
+    });
+  }
 
   await promiseStartupManager();
 
-  let a13 = await AddonManager.getAddonByID("addon13@tests.mozilla.org");
+  let a13 = await AddonManager.getAddonByID(ADDON_ID);
   notEqual(a13, null);
 
+  if (asBuiltIn) {
+    equal(
+      a13.getResourceURI().spec,
+      "resource://builtin-addon13-ext/",
+      "Expect addon root uri to be a resource:// URI"
+    );
+  } else {
+    ok(
+      a13.getResourceURI() instanceof Ci.nsIJARURI,
+      "Expect addon root uri to be a jar uri"
+    );
+  }
+
   let result = await AddonTestUtils.promiseFindAddonUpdates(a13);
   ok(
     !result.compatibilityUpdate,
@@ -849,4 +859,13 @@ add_task(async function run_test_locked_install() {
 
   let installs = await AddonManager.getAllInstalls();
   equal(installs.length, 0);
+}
+
+// TODO(Bug 1949847): remove this test along with removing the app-system-defaults location.
+add_task(async function test_locked_location_system_xpi() {
+  await run_test_locked_install({ asBuiltIn: false });
+});
+
+add_task(async function test_locked_location_system_builtin() {
+  await run_test_locked_install();
 });
